<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

// include configurations
require_once 'includes/constants.inc.php';
require_once 'includes/config_default.inc.php';

// include all core libs
require_once 'core/core_api.inc.php';
require_once 'core/core_db.inc.php';
require_once 'core/core_html.inc.php';

// includes setup
require_once 'includes/cmdb_setup.inc.php';

// security check: stop, if config exists 
if (
	is_file( CONFIG_INC_PHP )
	||
	is_link( CONFIG_INC_PHP )
)
{
	require_once 'includes/page_error.inc.php';
	
	html_error_page( 'Action Not Permitted' );
	exit();
}

// load language
require_once 'lang/catalog.inc.php';
$lang_file_name = lang_catalog::getFilename( 'english' );
require_once $lang_file_name;

// read arguments

$setup->db_form			= http_get_int( "db_form", 0 );
$setup->db_host			= http_get_string( "db_host", "" );
$setup->db_name			= http_get_string( "db_name", "" );
$setup->db_user			= http_get_string( "db_user", "" );
$setup->db_pass			= http_get_string( "db_pass", "" );
$setup->db_pass_verify	= http_get_string( "db_pass_verify", "" );
$setup->db_root_user	= http_get_string( "db_root_user", "" );
$setup->db_root_pass	= http_get_string( "db_root_pass", "" );
$setup->cmdb_user		= http_get_string( "cmdb_user", "" );
$setup->cmdb_pass		= http_get_string( "cmdb_pass", "" );
$setup->cmdb_pass_verify = http_get_string( "cmdb_pass_verify", "" );


// start html page
html_header( 'PHP-CMDB Setup' );

$rc_global = 0;
html_table_header( 50, 150, 600 );

?>
<tr><td colspan='3' class='title'>PHP-CMDB Setup</td></tr>
<?php

if ($setup->db_form == 0)
{
	// --------------------------------------------------------
	// step 1: check dependencies
	// --------------------------------------------------------
	if ($rc_global == 0)
	{
		$rc_global = cmdb_setup_step1();
	}
	
	// --------------------------------------------------------
	// step 2: setup formular
	// --------------------------------------------------------
	
	if ($rc_global == 0)
	{
		$rc_global = cmdb_setup_step2( $setup );
	}
}
else
{
	// --------------------------------------------------------
	// step 3: verify formular data
	// --------------------------------------------------------
	
	if ($rc_global == 0)
	{
		$rc_global = cmdb_setup_step3( $setup );
	}
	
	// --------------------------------------------------------
	// step 4: create database and user inside mysql
	// --------------------------------------------------------
	
	if ($rc_global == 0)
	{
		$rc_global = cmdb_setup_step4( $setup );
	}
	
	// --------------------------------------------------------
	// step 5: init schema and create CMDB initial user
	// --------------------------------------------------------
	
	if ($rc_global == 0)
	{
		$rc_global = cmdb_setup_step5( $setup );
	}

	// --------------------------------------------------------
	// step 6: write config
	// --------------------------------------------------------
	
	if ($rc_global == 0)
	{
		$rc_global = cmdb_setup_step6( $setup );
	}


	// --------------------------------------------------------
	// step 7: setup completed
	// --------------------------------------------------------
	
	if ($rc_global == 0)
	{
		html_table_ruler();
		html_status( STATUS_OK, 'Setup Completed', INDEX_PHP );
	}
}

html_table_footer();

html_footer();
