<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_date.inc.php 423 2010-04-11 13:48:37Z micha $
*/

/**
 * data type class for date
 */
class core_date
{
	protected $date_y		= 0;
	protected $date_m		= 0;
	protected $date_d		= 0;
	
	public function __construct ( $_new_date = "" )
	{
		if (strlen($_new_date)>0) $this->set( $_new_date );
		return;
	}

	// set methods

	public function set_today ()
	{
		$_new_date = date( "Y-m-d" );
		$this->set( $_new_date );
		return;
	}

	public function set ( $_new_date )
	{
//echo "DEBUG: core_date:set( $_new_date )<br>\n";
		$this->set_year( (int)substr( $_new_date, 0, 4 ) );
		$this->set_month( (int)substr( $_new_date, 5, 2 ) );
		$this->set_day( (int)substr( $_new_date, 8, 2 ) );
		return;
	}
	
	public function set_day ( $_day )
	{
//echo "DEBUG: core_date:set_day( $_day )<br>\n";
		if (($_day < 1) || ($_day > 31)) return;
		$this->date_d = $_day;
		return;
	}

	public function set_month ( $_month )
	{
//echo "DEBUG: core_date:set_month( $_month )<br>\n";
		if (($_month < 1) || ($_month > 12)) return;
		$this->date_m = $_month;
		return;
	}

	public function set_year ( $_year )
	{
//echo "DEBUG: core_date:set_year( $_year )<br>\n";
		if ($_year < 0) return;
		$this->date_y = $_year;
		if ($_year == 0)
		{
			$this->date_m = 0;
			$this->date_d = 0;
		}
		return;
	}

	// get methods

	public function get ()
	{
		$text = "-";
		if ($this->is_valid() &&  ! $this->is_empty())
		{
			$text = sprintf("%04d-%02d-%02d", $this->date_y, $this->date_m, $this->date_d );
		}
		return $text;
	}

	public function get_day ()
	{
		return $this->date_d;
	}
	
	public function get_month ()
	{
		return $this->date_m;
	}
	
	public function get_year ()
	{
		return $this->date_y;
	}
	
	public function get_str ()
	{
		$xx = "-";
		if ($this->is_valid() && ! $this->is_empty())
		{
			$xx = sprintf( "%02d.%02d.%04d",
				$this->date_d,
				$this->date_m,
				$this->date_y);
		} 
	
		return $xx;
	}

	function get_str2 ()
	{
		$xx = "-";
		if ($this->is_valid() && ! $this->is_empty())
		{
			$xx = sprintf( "%02d.%02d.%02d",
				$this->date_d,
				$this->date_m,
				$this->date_y);
		} 
	
		return $xx;
	}
	
	function get_str3 ()
	{
		$xx = "-";
		if ($this->is_valid() && ! $this->is_empty())
		{
			$xx = sprintf( "%04d/%02d",
				$this->date_y,
				$this->date_m);
		} 
	
		return $xx;
	}
	
	// operatoren

	public function is_empty ()
	{
		return	(
					($this->date_y == 0) &&
					($this->date_m == 0) &&
					($this->date_d == 0)
				);
	}

	public function is_valid ()
	{
		return (
				(
					($this->date_y >0) &&
					(($this->date_m>0) && ($this->date_m<13)) &&
					(($this->date_d>0) && ($this->date_d<32))
				) ||
				$this->is_empty()
				);
	}

	// html

	public function cat_year_segment ( $_pre_count, $_post_count, $arg_name, $default="", $form_name="" )
	{
		if (! $this->is_valid()) $this->set_today();
		if (($_year = $this->get_year()) == 0) $_year = date('Y');
		$_start = $_year - $_pre_count;
		$_end = $_year + $_post_count;

		if (strlen($form_name)==0)
		{
			printf( "<select name='%s'>", $arg_name	);
		}
		else
		{
			printf( "<select name='%s' onchange='%s.submit()'>", $arg_name, $form_name );
		}

		if (strlen($default)!=0) cat_option( 0, $this->get_year(), $default );

		for ($i=$_start; $i<=$_end; $i++)
		{
			cat_option(  $i, $this->get_year(), sprintf("%04d", $i) );
		}
		printf( "</select>"	);
		
		return;
	}

	public function cat_month_segment ( $arg_name, $all=FALSE, $default="", $form_name="" )
	{
		if (strlen($form_name)==0)
		{
			printf( "<select name='%s'>", $arg_name	);
		}
		else
		{
			printf( "<select name='%s' onchange='%s.submit()'>", $arg_name, $form_name );
		}
	
		if (strlen($default)!=0) cat_option( 0, $this->get_month(), $default );
	
		$month_array = get_enum_to_array("enum_months");
		foreach( $month_array as $key => $value )
		{
			if (($key < 13) || $all) cat_option( $key, $this->get_month(), $value );
		}
		printf( "</select>"	);
	
		return;
	}

	function cat_day_segment ( $arg_name, $default="", $form_name="" )
	{
		if (strlen($form_name)==0)
		{
			printf( "<select name='%s'>", $arg_name	);
		}
		else
		{
			printf( "<select name='%s' onchange='%s.submit()'>", $arg_name, $form_name );
		}
	
		if (strlen($default)!=0) cat_option( 0, $this->get_day(), $default );
	
		for ($d=1; $d<32; $d++)
		{
			cat_option( $d, $this->get_day(), $d );
		}
	
		printf( "</select>"	);
	
		return;
	}

} 
