<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_db_mysql.inc.php 338 2009-04-10 12:09:15Z micha $
*/

require_once 'core/core_exception.inc.php';

/**
 * excpetion class for database access
 */
class core_db_exception extends core_exception
{
	public function __construct ()
	{
		parent::__construct( mysql_error(), mysql_errno() );
	}
	
}	

/**
 * class to access MySQL databases
 */
class core_db_mysql 
{
	protected $db_link;		// db handle
	protected $db_host;
	protected $db_user;
	protected $db_pass;
	protected $db_name;

	/**
	 * constructor
	 * 
	 * initialisiert die instance und oeffnet eine connection
	 * 
	 * @param string	$_host		database host
	 * @param string	$_user		database user
	 * @param string	$_pass		password of database user
	 * @param string	$_name		database name
	 */
	public function __construct( $_host, $_user, $_pass, $_name )
	{
		// init fields
		//
		$this->db_link		= null;
		$this->db_host		= $_host;
		$this->db_user		= $_user;
		$this->db_pass		= $_pass;
		$this->db_name		= $_name;
		
		// open connect
		//
		$this->connect();

		return;
	}

	// operator methods
	
	/**
	 * reply TRUE, if connected to database, otherwise FALSE
	 * 
	 * @return	boolean
	 */
	public function is_connected ()
	{
		return (is_resource($this->db_link));
	}

	// action methods

	/**
	 * connects to database
	 * 
	 * @return	boolean
	 */
	protected function connect ()
	{
		if (is_resource($this->db_link)) return TRUE;

		$this->db_link = @mysql_pconnect( $this->db_host, $this->db_user, $this->db_pass);
		if (! is_resource($this->db_link))
		{
			throw new core_db_exception;
		}		
		if (!mysql_select_db( $this->db_name, $this->db_link))
		{
			throw new core_db_exception;
		}		

		return TRUE;		
	}

	// handling

	public function escape_string ( $string )
	{
		return @mysql_real_escape_string( $string, $this->db_link );
	}
	
	public function query ( $query )
	{
		$result = mysql_query( $query, $this->db_link );
		if (! $result)
		{
			throw new core_db_exception;
		} 
		return $result;
	}

	public function num_rows ( $result )
	{
		if( ! $result ) return 0;
		return @mysql_num_rows( $result );
	}
	
	public function affected_rows ()
	{
		return @mysql_affected_rows( $this->db_link );
	}
	
	public function fetch_object ( $result )
	{
		if ( ! $result ) return 0;
		return @mysql_fetch_object( $result );
	}
	
	public function fetch ( $result )
	{
		if( ! $result ) return 0;
		return( @mysql_fetch_assoc( $result ) );
	}

	public function free ( $result )
	{
		if( $result ) @mysql_free_result( $result );
	}

	public function id ()
	{
		return @mysql_insert_id( $this->db_link );
	}
	
	public function db_mangle ( $text )
	{
		return preg_replace( "/'/", '"', $text );
	}

	public function errno ( $result )
	{
		if ($result) return 0;
		if ($this->db_link != null)
		{
			syslog( LOG_ERR, sprintf( "MYSQL-ERROR: ERRNO=%d MSG='%s'\n",
				@mysql_errno( $this->db_link ), mysql_error( $this->db_link ) ) );
		}
		return -1;
	}

	public function errno_msg ( $result, &$db_rc, &$db_msg )
	{
		$db_rc  = 0;
		$db_msg = "";
		
		if ($result) return 0;
		if ($this->db_link != null)
		{
			$db_rc  = @mysql_errno( $this->db_link );
			$db_msg = @mysql_error( $this->db_link );
			syslog( LOG_ERR, sprintf( "MYSQL-ERROR: ERRNO=%d MSG='%s'",
				@mysql_errno( $this->db_link ), mysql_error( $this->db_link ) ) );
		}
		return -1;
	}

	public function result ( $result, $row )
	{
		return @mysql_result( $result, $row );
	}
}
