<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_html.inc.php 440 2010-11-12 16:22:23Z micha $
*/

// core html functions
require_once 'core/core_html_menu.inc.php';

// global auto refresh
$core_html_refresh_url	= "";
$core_html_table_cols	= 0;
$core_html_table_row	= 1;

/**
 * returns the CSS style name for the given return code
 * 
 * @param	int		$rc				return code
 * @return	string
 */
function get_rc_style( $rc )
{
	if ($rc == STATUS_ERROR) return 'error';
	if ($rc == STATUS_WARNING) return 'warning';
	return 'ok';
}

/**
 * returns the display-text for the given return code
 * 
 * @param	int		$rc				return code
 * @return	string
 */
function get_rc_text( $rc )
{
	if ($rc == STATUS_ERROR) return get_string( 'Status_ERROR' );
	if ($rc == STATUS_WARNING) return get_string( 'Status_WARNING' );
	return get_string( 'Status_OK' );
}

function html_table_header ()
{
	global $core_html_table_row;
	global $core_html_table_cols;

	// init
	$core_html_table_row    = 1;
	$table_width			= get_property_from_config( "page_table_width" );
	$table_cellspace		= get_property_from_config( "page_table_cellspace" );
	$table_cellpadding		= get_property_from_config( "page_table_cellpadding" );

	// table
	//
	echo sprintf( "<table width='%d' cellspacing='%d' cellpadding='%d' border='%d' summary='%s'>\n",
		$table_width, $table_cellspace, $table_cellpadding, 0, "" );

	// colgroup
	//
	echo sprintf( "<colgroup>" );
	$core_html_table_cols=func_num_args();

	// strict
	$i_sum=0;
	for ($i=0; $i < $core_html_table_cols; $i++ )
	{
		$i_str = func_get_arg( $i );
		echo sprintf( "<col width='%d'>", $i_str - $table_cellspace );
		$i_sum += (int)$i_str;
	}

	if ($i_sum != $table_width)
	{
		user_error( sprintf( "table-width(%d) not equal cell-sum(%d)",
			$table_width, $i_sum ), E_USER_WARNING );
	}

	echo sprintf( "</colgroup>\n" );

	return;
}

function html_table_footer ()
{
	global $core_html_table_row;
	global $core_html_table_cols;

?>
</table>
<?php
	$core_html_table_row = 0;
	$core_html_table_cols = 0;

	return;
}

function html_table_ruler ()
{
	global $core_html_table_cols;

	$tc = $core_html_table_cols;
	if ($tc == 0) html_table_header( 20, get_property_from_config( "page_table_width" ) - 20 );

	echo sprintf( "<tr><td colspan='%d'><hr></td></tr>\n", $core_html_table_cols );

	if ($tc == 0) html_table_footer();

	return;
}

function html_table_line ()
{
	global $core_html_table_cols;

//	echo sprintf( "<tr><td colspan='%d'><img src='imx/1x1_red.png' width='%d' height='%d' alt='--'></td></tr>\n",
//		$core_html_table_cols, get_property( "page_table_width" ) - get_property( "page_table_cellspace" ), 1 );

	// TODO: funktioniert so nicht mit dem IE
?>
<tr><td colspan='<?php echo $core_html_table_cols; ?>'><div class='line'></div></td></tr>
<?php

	return;
}

function html_table_row_reset ( $_style )
{
	global $core_html_table_row;
	global $core_html_table_row_style;
	
	$core_html_table_row = 1;
	$core_html_table_row_style = array();
	if (is_array($_style) and (count($_style)==2))
	{
		$core_html_table_row_style = $_style;
	}
	
	return;
}

function html_table_row ()
{
	global $core_html_table_row;

	$core_html_table_row *= -1;
	html_table_row_repeat();

	return;
}

function html_table_row_repeat ()
{
	global $core_html_table_row;
	global $core_html_table_row_style;

	if (count($core_html_table_row_style)==2)
	{
		$style1 = $core_html_table_row_style[0];
		$style2 = $core_html_table_row_style[1];
	}
	else
	{
		$style1 = "tr1";
		$style2 = "tr2";
	}
	if ($core_html_table_row==1)
	{
		echo sprintf( "<tr class='$style1'>" );
	}
	else
	{
		echo sprintf( "<tr class='$style2'>" );
	}

	return;
}

function html_table_section ( $text, &$menu )
{
	global $core_html_table_cols;

	echo sprintf( "<tr><td colspan='%d' class='section'>%s</td></tr>\n",
		$core_html_table_cols, $text );

	if ( is_object($menu) && ($menu->get_count() > 0) )
	{
		echo sprintf( "  <tr><td colspan='%d' class='menu'>&gt;&gt; %s</td></tr>\n",
			$core_html_table_cols, $menu->get_menu_string() );
	}

	return;
}

function html_table_icon_section ( $icon_url, $text, &$menu )
{
	global $core_html_table_cols;

	printf( "<tr><td rowspan='2'>%s</td><td colspan='%d' class='section'>%s</td></tr>\n",
		$icon_url, $core_html_table_cols - 1, $text );
	printf( "  <tr><td colspan='%d' class='menu'>",
		$core_html_table_cols - 1 );
	if ( $menu->get_count() > 0)
	{
		printf( "&gt;&gt; %s",
			$menu->get_menu_string() );
	}
	printf( "</td></tr>\n" );

	return;
}

function html_table_action ( $text, $url, $help="" )
{
	global $core_html_table_cols;

	$tc = $core_html_table_cols;
	if (($tc > 0) && ($tc < 3)) user_error( "table_action needs 3 or more cols" );
	if ($tc == 0) html_table_header( 20, 300, get_property_from_config( "page_table_width" ) - 320 );

	echo sprintf( "<tr>" .
		"<td></td>" .
		"<td colspan='%d' class='menu3'>&gt;&gt; <a href='%s'>%s</a></td>" .
		"<td class='tableinfo'>%s</td>" .
		"</tr>\n",
		$core_html_table_cols - 2, $url, $text,
		$help );

	if ($tc == 0) html_table_footer();

	return;
}

function html_oblig_row ( $flag, $text )
{
	echo sprintf( "<tr><td class='f_oblig'>%s</td><td class='f_attr'>%s :</td>", $flag, $text );

	return;
}

function html_form_checkbox ( $active, $arg_name, $form_name="" )
{
	$check="";
	if ($active) $check="checked='checked'";

	$submit="";
	if (strlen($form_name)>0) $submit="onchange='$form_name.submit()'"; 
	
	print "<input type='checkbox' name='$arg_name' value='1' $submit $check>";
}

function html_status_checkbox ( $active, $active_text="x", $passive_text="" )
{
	if ($active) return $active_text;
	return $passive_text;
}

function html_empty_row ( $num, $shift )
{
	global $core_html_table_cols;
	
	if ($core_html_table_cols == 0)
	{
		user_error( "html_empty_row: no table defined" );
		return;
	}

	if ($num == 0)
	{
		echo sprintf( "<tr>" );
		if ($shift != 0)
		{
			echo sprintf( "<td colspan='%d'></td>", $shift );
		}
		echo sprintf( "<td colspan='%d'>".get_string('NoEntries_inf')."</td></tr>\n",
			$core_html_table_cols - $shift );
	}

	return;
}

function html_question ( $question, $yes_text, $yes_url, $no_text, $no_url )
{
	global $core_html_table_cols;

	$tc = $core_html_table_cols;
	if ($tc == 0) html_table_header( get_property_from_config( "page_table_width" ) );
	
?>
<tr><td class="question"><?php echo $question;?></td></tr>
<tr><td class="command">
<a href="<?php echo $yes_url;?>"><?php echo $yes_text;?></a> | <a href="<?php echo $no_url;?>"><?php echo $no_text;?></a>
</td></tr>
<?php
	if ($tc == 0) html_table_footer();

	return;
}

function html_table_info ( $pre_cols, $free_cols, $info, $css_class="tableinfo" )
{
	global $core_html_table_cols;

	$tc = $core_html_table_cols;
	if ($tc == 0) html_table_header( get_property_from_config( "page_table_width" ) );
	
	if ($pre_cols == 0) echo sprintf( "<tr>" );

	$colspan = $free_cols - $pre_cols;
	if ($colspan != 0) echo sprintf( "<td colspan='%d'></td>", $colspan );
	
	$colspan = $core_html_table_cols - $free_cols;
	if ($colspan != 0) echo sprintf( "<td colspan='%d' class='%s'>%s</td>", $colspan, $css_class, $info );

	if ($tc == 0) html_table_footer();

	return;
}

function html_status ( $rc, $rc_message, $rc_url = "" )
{
	global $core_html_table_cols;

	$tc = $core_html_table_cols;
	if ($tc == 0) html_table_header( 100, get_property_from_config( 'page_table_width' ) - 100 );
?>
<tr>
<td class="attr <?php echo get_rc_style( $rc );?>"><?php echo get_rc_text( $rc );?>:</td>
<td colspan="<?php echo ($core_html_table_cols - 1);?>"><?php echo $rc_message;?></td>
</tr>
<?php
	if (strlen($rc_url)!=0)
	{
		echo sprintf( "<tr><td></td><td colspan='%d'><a href='%s'>%s...</a></td></tr>\n",
			$core_html_table_cols - 1, $rc_url, get_string('Continue') );
	}

	if ($tc == 0) html_table_footer();

	return;
}

function html_page_header ( &$top_menu, &$sub_menu, $show_mode = MODE_BROWSE )
{
	$_app_name = get_property_from_config( 'app_name' );
	$_app_logofile = get_property_from_config( 'app_logofile' );
	$session_menu = get_session_menu();
	$session_menu->set_selected_item( $top_menu->get_selected_item() );

	// html header
	html_header( $_app_name );
?>
<center>
<div id='header'>
<?php
	html_table_header( 200, get_property_from_config( "page_table_width" ) - 450, 250 );
?>
<!-- logo / top-menu / session-info --> 
<tr>
	<td class='title' colspan='1' rowspan='2'>
<?php
	if (strlen($_app_logofile))
	{
		printf("<a href='./'><img class='logo' src='%s' alt='logo'></a> ", $_app_logofile ); 
	}
	printf( "%s", $_app_name );
?>
	</td>
	<td class='session' colspan='2'><?php echo $session_menu->get_menu_string(); ?></td>
</tr>
<tr>
	<td class='topmenu' colspan='2'><?php echo $top_menu->get_menu_string();?></td>
</tr>
<tr>
	<td class='submenu' colspan='2'><?php echo $sub_menu->get_menu_string();?></td>
	<td class='modemenu'><?php
	switch ($show_mode)
	{
		case MODE_BROWSE:
		case MODE_REPORT:
		{
?><form action='<?php echo INDEX_PHP; ?>' method='post' name='s_form'><?PHP
	echo icon_display_url( INDEX_PHP, ICON_RELOAD, "reload last search result", 16, 16, "reload last search result" );
?><input type='text' class='c_search' name='s_text' size='20' maxlength='200' onchange='s_form.submit()'>
<input type='hidden' name='s_form' value='<?php echo SEARCH_MODE_SIMPLE; ?>'>
<?php
printf("<input type='submit' class='b_search' value=".get_string('Search_btn')."></form>\n");
			break;
		}
//		case MODE_DESIGN:
//		{
//			echo "Design";
//			break;
//		}
//		case MODE_MAINTAIN:
//		{
//			echo "Administration";
//			break;
//		}
		default:
		{
			printf( "&nbsp;" );
		}
	}
?>
</td>
</tr>
<?php
	html_table_footer();
?>
</div>
<?php
	return;
}

function html_page_footer ()
{
	$_app_version = get_property_from_config("app_version");
	$_app_url = get_property_from_config("app_url");
	$_app_copyright_logofile = get_property_from_config("app_copyright_logofile");
	$_app_copyright_holder = get_property_from_config("app_copyright_holder");
	$_app_copyright_period = get_property_from_config("app_copyright_period");
	$_db_name = get_property_from_config("db_name");
?>
<div id='footer'>
<?php
	html_table_header( 70, 330, get_property_from_config( "page_table_width" ) - 400 );
?>	
<tr>
	<td class='nofilesinfo'><a href='<?php echo $_app_url;?>'><img class='nofiles' src='<?php echo $_app_copyright_logofile;?>' width='59' alt='nofiles' title='nofiles'></a></td>
	<td class='nofilesinfo'>&copy; <?php echo $_app_copyright_period . ' ' . $_app_copyright_holder;?> <?php ?></td>
	<td class='versioninfo'><?php echo get_string('Version_ttl');?>: <?php echo $_app_version;?> / <?php echo get_string('DB_ttl');?>: <?php echo $_db_name;?></td>
</tr>
<?php
	html_table_footer();
?>
</div>
</center>
<?php
	html_footer();

	return;
}

function html_header (  $page_title = "PAGE-TITLE NOT SET" )
{
	global $core_html_refresh_url;

	// build page header
	header ("Content-type: text/html");
	header ("Expires: Wed, 17 Jul 1968 18:30:00 GMT");
	header ("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
	header ("Cache-Control: no-cache, must-revalidate");
	header ("Pragma: no-cache");
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Strict//EN">
<html>
<head>
	<meta http-equiv="content-type" content="text/html; charset=ISO-8859-1">
	<title><?php echo $page_title;?></title>
	<link rel="stylesheet" type="text/css" media="all" href="<?php echo get_property_from_config( "app_stylesheet" );?>">
<?php
	$_customer_stylesheet = get_property_from_config( "customer_stylesheet" );
	if (strlen( $_customer_stylesheet) > 0)
	{
?>
	<link rel="stylesheet" type="text/css" media="all" href="<?php echo $_customer_stylesheet;?>">
<?php
	}
?>
	<link rel="icon" href="<?php echo get_property_from_config("app_favicon" );?>" type="image/x-icon"> 
	<link rel="shortcut icon" href="<?php echo get_property_from_config("app_favicon" );?>" type="image/x-icon">
<?php
	if (strlen($core_html_refresh_url)!=0)
	{
?>
	<meta http-equiv="refresh" content="<?php echo sprintf("%d; URL=%s", get_property_from_config( "page_auto_refresh" ), $core_html_refresh_url );?>">
<?php
	}
?>
</head>
<body>
<?php
	return;
}

function html_footer()
{
?>
</body>
</html>
<?php
	return;
}	

function html_page_redirect ( $wait_sec, $url )
{
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 TRANSITIONAL//EN">
<html>
<head>
  <title><?php echo get_property_from_config( "app_name" );?> | Weiterleitung</title>
  <meta http-equiv="refresh" content="<?php echo $wait_sec;?>; URL=<?php echo $url;?>">
</head>
<body>
</body>
</html>
<?php
	return;
}
