<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_html_menu.inc.php 338 2009-04-10 12:09:15Z micha $
*/

require_once 'core/core_api.inc.php';
require_once 'core/core_arm.inc.php';

class html_menu 
{
	protected $items			= array();
	protected $selected_item	= "";
	
	public function __construct()
	{
		$this->items = array();
		return;
	}
	
	public function set_selected_item ( $_name )
	{
		$this->selected_item = $_name;
		return;
	}

	public function get_count()
	{
		return count($this->items);
	}
	
	public function get_selected_item ()
	{
		return $this->selected_item;
	}

	public function add_item ( $_name, $_url, $_title="" )
	{
		if (! core_arm::getInstance()->ac_permitted(get_url_to_ac($_url))) return;
		$this->items[ $_name ][0] = 0;
		$this->items[ $_name ][1] = $_url;
		$this->items[ $_name ][2] = $_title;
		return;
	}
	
	public function add_mailto_item ( $_name, $_url, $_title="" )
	{
		if (! core_arm::getInstance()->ac_permitted(get_url_to_ac($_url))) return;
		$this->items[ $_name ][0] = 1;
		$this->items[ $_name ][1] = $_url;
		$this->items[ $_name ][2] = $_title;
		return;
	}
	
	public function get_menu_string ()
	{
		$menu_str = "";
		if ($this->get_count()>0)
		{
			foreach( $this->items as $name => $data )
			{
				switch ($data[0])
				{
					case 0:
					{
						$menu_str = $menu_str . " | " . $this->get_item_string( $name );
						break;
					}
					case 1:
					{
						$menu_str = $menu_str . " | " . $this->get_mailto_string( $name );
						break;
					}
				}
			}
			$menu_str = trim( $menu_str, "| " );
		}
		return $menu_str;
	}

	protected function get_item_string ( $name )
	{
		$item_str = $name;
		if (
			(strcmp($name, $this->selected_item)!=0)
			&&		
			(strlen($this->items[ $name ][1])!=0)
			)
		{
			$item_str = sprintf( "<a href='%s' title='%s'>%s</a>",
				$this->items[ $name ][1], $this->items[ $name ][2], $name );
		}
		return $item_str;
	}

	protected function get_mailto_string ( $name, $data )
	{
		$item_str = $name;
		if (
			(strcmp($name, $this->selected_item)!=0)
			&&		
			(strlen($this->items[ $name ][1])!=0)
			)
		{
			$item_str = sprintf( "%s [<a href='mailto:%s' title='%s'>mailto</a>]",
				$name, $this->items[ $name ][1], $this->items[ $name ][2] );
		}
		return $item_str;
	}

}

function & get_session_menu ( $_selected_name="" )
{
	$_user_name = core_session::getInstance()->get_user_name();
	$_user_role = core_session::getInstance()->get_role_name();

	$session_menu = new html_menu();
	$session_menu->add_item( "$_user_name ($_user_role)", "", "" );
	$session_menu->add_item( get_string('MyAccount_ttl'), U_SETTINGS_PHP, get_string('MyAccount_dsc') );
	$session_menu->add_item( get_string('Logout_ttl'), S_LOGOUT_PHP, get_string('Logout_dsc') );
	$session_menu->set_selected_item( $_selected_name );

	return $session_menu;	
}

function & get_top_menu ( $_selected_name="" )
{
	// setup top-menu
	$top_menu = new html_menu();
	$top_menu->add_item( get_string('SearchBrowse_ttl'), INDEX_PHP, get_string('SearchBrowse_dsc') );
	$top_menu->add_item( get_string('Reports_ttl'), REPORT_INDEX_PHP, get_string('Reports_dsc') );
	$top_menu->add_item( get_string('Design_ttl'), DESIGN_INDEX_PHP, get_string('Design_dsc') );
	$top_menu->add_item( get_string('Administration_ttl'), MAINTAIN_INDEX_PHP, get_string('Administration_dsc') );
	$top_menu->set_selected_item( $_selected_name );

	return $top_menu;
}

function & get_empty_menu ()
{
	// setup sub-menu
	$menu = new html_menu();
	return $menu;
}
