<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_ipv4.inc.php 338 2009-04-10 12:09:15Z micha $
*/

/**
 * data type class for IP v4 addresses
 */
class core_ipv4
{
	protected $a		= -1;
	protected $b		= -1;
	protected $c		= -1;
	protected $d		= -1;
	
	public function __construct ( $_new_ipv4 = "" )
	{
		if (strlen($_new_ipv4)>0) $this->set( $_new_ipv4 );
		return;
	}

	// set methods

	public function set_host ()
	{
		$this->set( $_SERVER["SERVER_ADDR"] );
		return;
	}

	public function set ( $_new_ipv4 )
	{
		if (strlen($_new_ipv4)==0)
		{
			$this->set_a( -1 );
			return;
		}
		
		$ip_parts = explode( '.', $_new_ipv4 );
		if (count($ip_parts)!=4) return;

		$this->set_a( $ip_parts[0] );
		$this->set_b( $ip_parts[1] );
		$this->set_c( $ip_parts[2] );
		$this->set_d( $ip_parts[3] );
		return;
	}
	
	public function set_a ( $_a )
	{
		if (($_a < -1) || ($_a > 255)) return;
		$this->a = $_a;
		if ($_a == -1)
		{
			$this->b = -1;
			$this->c = -1;
			$this->d = -1;
		}
		return;
	}

	public function set_b ( $_b )
	{
		if (($_b < 0) || ($_b > 255)) return;
		$this->b = $_b;
		return;
	}

	public function set_c ( $_c )
	{
		if (($_c < 0) || ($_c > 255)) return;
		$this->c = $_c;
		return;
	}

	public function set_d ( $_d )
	{
		if (($_d < 0) || ($_d > 255)) return;
		$this->d = $_d;
		return;
	}

	// get methods

	public function get ()
	{
		$text = "-";
		if ($this->is_valid() &&  ! $this->is_empty())
		{
			$text = sprintf("%d.%d.%d.%d", $this->a, $this->b, $this->c, $this->d );
		}
		return $text;
	}

	public function get_a ()
	{
		return $this->a;
	}
	
	public function get_b ()
	{
		return $this->b;
	}
	
	public function get_c ()
	{
		return $this->c;
	}
	
	public function get_d ()
	{
		return $this->d;
	}
	
	// operatoren
	
	public function is_empty ()
	{
		return (
				($this->a == -1)
				&&
				($this->b == -1)
				&&
				($this->c == -1)
				&&
				($this->d == -1)
				);
	}

	public function is_valid ()
	{
		return (
					(
						(($this->a>=0) && ($this->a<256))
						&&
						(($this->b>=0) && ($this->b<256))
						&&
						(($this->c>=0) && ($this->c<256))
						&&
						(($this->d>=0) && ($this->d<256))
					)
					||
					$this->is_empty()
				);
	}

	// html


} 
