<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_session.inc.php 423 2010-04-11 13:48:37Z micha $
*/

/**
 * singleton class to manage the current session
 */
class core_session
{
	private static $instance = false;
	protected $s_session_id = "";
	protected $s_user_id = 0;
	protected $s_user_name = "";
	protected $s_role_id = 0;
	protected $s_role_name = "";
	protected $s_remote_addr = "";
	protected $s_user_agent = "";
	
	/**
	 * fabrik-method
	 * 
	 * @return	core_session
	 */
	static public function getInstance ()
	{
		if (self::$instance === false)
		{
			self::$instance = new core_session();
		}
		
		return self::$instance;
	}

	static public function destroyInstance()
	{
		if (self::$instance !== false)
		{
			self::$instance->destroy();
			self::$instance = false;
		}

		return;		
	}

	/**
	 * constructor
	 */
	protected function __construct ()
	{
		session_name("php-cmdb");
		session_start();
		$this->s_session_id	= session_id();
		$this->load();
		$this->security_check();

		return;
	}
	
	// get methoden
	
	public function get_user_id ()
	{
		return $this->s_user_id;
	}
	
	public function get_user_name ()
	{
		return $this->s_user_name;
	}
	
	public function get_role_id ()
	{
		return $this->s_role_id;
	}
	
	public function get_role_name ()
	{
		return $this->s_role_name;
	}
	
	public function get_session_id ()
	{
		return $this->s_session_id;
	}

	// set methoden

	public function set_user_id ( $_id )
	{
		if ($_id < 0) return FALSE;
		$this->s_user_id = $_id;
		return TRUE;
	}

	public function set_user_name ( $_name )
	{
		$this->s_user_name = $_name;
		return TRUE;
	}

	public function set_role_id ( $_id )
	{
		if ($_id < 0) return FALSE;
		$this->s_role_id = $_id;
		return TRUE;
	}

	public function set_role_name ( $_name )
	{
		$this->s_role_name = $_name;
		return TRUE;
	}

	// operators
	
	/**
	 * replay TRUE, if session is authenticated, otherwise FALSE
	 * 
	 * @return	boolean
	 */
	public function is_authenticated ()
	{
		return ($this->s_user_id != 0);
	}
	
	/**
	 * store all session data
	 */
	public function store ()
	{
		global $_SESSION;
		
		foreach ($this as $key => $value)
		{
			if (strcmp($key,'s_session_id')!=0)
			{
				$_SESSION[$key] = $this->$key;
			}
		}
		return;
	}
	
	/**
	 * reset all session data to default
	 */
	public function reset ()
	{
		// user_id loeschen
		$this->set_user_id( 0 );
		$this->set_user_name( "" );
		$this->set_role_id( 0 );
		$this->set_role_name( "" );

		// client info loeschen
		$this->s_remote_addr = "";
		$this->s_user_agent  = "";

		// speichern
		$this->store();

		return;
	}

	/**
	 * destroy session including all data
	 */
	public function destroy ()
	{
		global $_SESSION;
		$_SESSION = array();		

		// Falls die Session gelöscht werden soll, löschen Sie auch das
		// Session-Cookie.
		// Achtung: Damit wird die Session gelöscht, nicht nur die Session-Daten!
		if (ini_get("session.use_cookies")) {
		    $params = session_get_cookie_params();
		    setcookie(session_name(), '', time() - 42000, $params["path"],
		        $params["domain"], $params["secure"], $params["httponly"]
		    );
		}		

		session_unset();
		session_destroy();
		return;
	}

	// helper methods

	/**
	 * load session data
	 */
	protected function load ()
	{
		global $_SESSION;
		
		foreach ($this as $key => $value)
		{
			if (
				(strcmp($key,'s_session_id')!=0)
				&&
				(isset($_SESSION[$key]))
				)
			{
				$this->$key = $_SESSION[$key];
			}
		}
		
		return;
	}

	/**
	 * check session data to protect against security breaks, reset
	 * session if check fails
	 */
	protected function security_check ()
	{
		global $_SERVER;
		
		// check current remote addr
		if (strlen($this->s_remote_addr)!=0)
		{
			if (strcmp($this->s_remote_addr, $_SERVER['REMOTE_ADDR'] )!=0) $this->destroy();
		}
		else
		{
			if (isset($_SERVER['REMOTE_ADDR']))
				$this->s_remote_addr = $_SERVER['REMOTE_ADDR'];
		}
		
		// check current user agent
		if (strlen($this->s_user_agent)!=0)
		{
			if (strcmp($this->s_user_agent, $_SERVER['HTTP_USER_AGENT'] )!=0) $this->destroy();
		}
		else
		{
			if (isset($_SERVER['HTTP_USER_AGENT']))
				$this->s_user_agent = $_SERVER['HTTP_USER_AGENT'];
		}

		// check timeout

		return;
	}

}
