<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_time.inc.php 423 2010-04-11 13:48:37Z micha $
*/

/**
 * data type class for time
 */
class core_time
{
	protected $time_h		= -1;
	protected $time_m		= -1;
	protected $time_s		= -1;
	
	public function __construct ( $_new_time = "" )
	{
		if (strlen($_new_time)>0) $this->set( $_new_time );
		return;
	}

	// set methods

	public function set_now ()
	{
		$_new_time = date( "H:i:s" );
		$this->set( $_new_time );
		return;
	}

	public function set ( $_new_time )
	{
		$this->set_hour( (int)substr( $_new_time, 0, 2 ) );
		$this->set_min( (int)substr( $_new_time, 3, 2 ) );
		$this->set_sec( (int)substr( $_new_time, 6, 2 ) );
		return;
	}
	
	public function set_sec ( $_sec )
	{
		if (($_sec < 0) || ($_sec > 59)) return;
		$this->time_s = $_sec;
		return;
	}

	public function set_min ( $_min )
	{
		if (($_min < 0) || ($_min > 59)) return;
		$this->time_m = $_min;
		return;
	}

	public function set_hour ( $_hour )
	{
		if (($_hour < -1) || ($_hour > 23)) return;
		$this->time_h = $_hour;
		if ($_hour == -1)
		{
			$this->time_m = -1;
			$this->time_s = -1;
		}

		return;
	}

	// get methods

	public function get ()
	{
		$text = "-";
		if ($this->is_valid() &&  ! $this->is_empty())
		{
			$text = sprintf("%02d:%02d:%02d", $this->time_h, $this->time_m, $this->time_s );
		}
		return $text;
	}

	public function get_sec ()
	{
		return $this->time_s;
	}
	
	public function get_min ()
	{
		return $this->time_m;
	}
	
	public function get_hour ()
	{
		return $this->time_h;
	}
	
	// operatoren
	
	public function is_empty ()
	{
		return (
				($this->time_h == -1) &&
				($this->time_m == -1) &&
				($this->time_s == -1)
				);
	}

	public function is_valid ()
	{
		return (
					(
						(($this->time_h>=0) && ($this->time_h<24)) &&
						(($this->time_m>=0) && ($this->time_m<60)) &&
						(($this->time_s>=0) && ($this->time_s<60))
					) ||
					$this->is_empty()
				);
	}

	// html

	public function cat_hour_segment ( $arg_name, $default="", $form_name="" )
	{
		if (strlen($form_name)==0)
		{
			printf( "<select name='%s'>", $arg_name	);
		}
		else
		{
			printf( "<select name='%s' onchange='%s.submit()'>", $arg_name, $form_name );
		}

		if (strlen($default)!=0) cat_option( -1, $this->get_hour(), $default );

		for ($i=0; $i<24; $i++)
		{
			cat_option(  $i, $this->get_hour(), sprintf("%02d", $i) );
		}
		printf( "</select>"	);
		
		return;
	}

	public function cat_min_segment ( $arg_name, $default="", $form_name="" )
	{
		if (strlen($form_name)==0)
		{
			printf( "<select name='%s'>", $arg_name	);
		}
		else
		{
			printf( "<select name='%s' onchange='%s.submit()'>", $arg_name, $form_name );
		}
	
		if (strlen($default)!=0) cat_option( -1, $this->get_min(), $default );
	
		for ($i=0; $i<60; $i++)
		{
			cat_option(  $i, $this->get_min(), sprintf("%02d", $i) );
		}
		printf( "</select>"	);
	
		return;
	}

	function cat_sec_segment ( $arg_name, $default="", $form_name="" )
	{
		if (strlen($form_name)==0)
		{
			printf( "<select name='%s'>", $arg_name	);
		}
		else
		{
			printf( "<select name='%s' onchange='%s.submit()'>", $arg_name, $form_name );
		}
	
		if (strlen($default)!=0) cat_option( -1, $this->get_sec(), $default );
	
		for ($i=0; $i<60; $i++)
		{
			cat_option(  $i, $this->get_sec(), sprintf("%02d", $i) );
		}
		printf( "</select>"	);
	
		return;
	}

} 
