<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_date.inc.php';
require_once 'core/core_time.inc.php';
require_once 'core/core_ipv4.inc.php';
require_once 'core/core_processor.inc.php';
require_once 'tables/db_attr.inc.php';
require_once 'includes/cat_entry_list.inc.php';
require_once 'includes/ci_navi_list.inc.php';
require_once 'includes/ci_cit_navi_list.inc.php';
require_once 'includes/module_label_list.inc.php';

class attr extends core_db_processor
{

	public function __construct( $data )
	{
		$this->create_db_object( $data['a_id'] );
		// besser so:
//		$this->create_db_object( 0 );
//		$this->db_object->set_id( $data['a_id'] );
		$this->db_object->load_from_array( $data );
		return;
	}

	/**
	 * load value from http args
	 */
	public function load_from_args()
	{
		$_value = http_get_string( $this->get_arg_name(), $this->db_object->get_val_text() );
		$this->db_object->set_val_text( $_value );

		return;
	}

	/**
	 * load value from implicit function
	 */
	public function load_from_func( &$item_processor )
	{
		if ($this->db_object->get_input() == ATTR_INPUT_FUNC)
		{
			$_value = module_list::get_input_value( $this->db_object->get_label(), $item_processor);
			$this->db_object->set_val_text( $_value );
		}

		return;	
	}

	/**
	 * load value from array, delivered from CSV import
	 * 
	 * the array has ta_id => value mappings
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function load_from_csv( &$ta_val_map )
	{
		$_ta_id = $this->db_object->get_ta_id();
		if (array_key_exists( $_ta_id, $ta_val_map))
		{
			$_value = $ta_val_map[ $_ta_id ];
			$this->db_object->set_val_text( $_value );
		}
		return;
	}

	/**
	 * store value into csv array, at position ta_id
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function store_to_csv( &$ta_val_map )
	{
		$_ta_id = $this->db_object->get_ta_id();
		if (array_key_exists( $_ta_id, $ta_val_map))
		{
			$ta_val_map[ $_ta_id ] = $this->db_object->get_val_text();
		}
		return;
	}

	/**
	 * setzt den classtype im DB-Objekt
	 */
	public function set_classtype( $_classtype )
	{
		$this->db_object->set_classtype( $_classtype );
	}

	/**
	 * setzt die classtype ID im DB-Objekt
	 */
	public function set_classtype_id( $_id )
	{
		$this->db_object->set_classtype_id( $_id );
	}

	public function is_modified()
	{
		return $this->db_object->is_modified();
	}

	public function is_empty()
	{
		return $this->db_object->is_empty();
	}

	public function get_tag_part()
	{
		$tag_part = "";
		if ($this->db_object->get_tag())
		{
			$tag_part = $this->db_object->get_val_text();
		}
		return $tag_part;
	}

	/**
	 * wenn das eigene attibut ein label hat, dass in der label_list
	 * enthalten ist und der eigene wert nicht leer ist,
	 * wird der wert in der label_list hinterlegt
	 */
	public function fill_label_list( $label_list )
	{
		if (
			(($label = $this->db_object->get_label()) != 0)
			&&
			($label_list->label_exists( $label ))
			)
		{
			$value = $this->db_object->get_val_text();
			if (strlen($value)>0) $label_list->set_label_value( $label, $value );
		}
		return;
	}

	/**
	 * liefert den argument-namen fuer das attribute in formularen
	 * 
	 * der name wird aus einem prefix sowie der ID des CI-Type-Attributes
	 * zusammengesetzt, diese ID steht zu jedem Zeitraum zur verfuegung,
	 * dagegen ist die cia_id nicht zu jedem zeitpunkt eindeutig und daher
	 * ungeeignet
	 * 
	 * @return	string	argument-name
	 */
	protected function get_arg_name()
	{
		$arg_name = sprintf("val%d", $this->db_object->get_ta_id() );
		return $arg_name;
	}

	/**
	 * liefert den string zur darstellung des obligate-flags
	 */
	protected function get_oblig_string()
	{
		if ($this->db_object->get_oblig()) return "*";
		return "";
	}

	/**
	 * liefert den string zur darstellung des tag-flags
	 */
	protected function get_tag_string()
	{
		if ($this->db_object->get_tag()) return "#";
		return "";
	}

	/**
	 * get format string from db_object or datatype default 
	 */
	protected function get_format()
	{
		$format = $this->db_object->get_format();
		if (strlen($format)==0) $format="%s";
		return $format;
	}

	/**
	 * get string with formated value
	 */
	protected function get_formated_value()
	{
		$format = $this->get_format();
		$formated_value = sprintf( $format, $this->db_object->get_val_text() );
		return $formated_value;
	}

	protected function set_value ( $_value )
	{
	}

	/**
	 * stellt das attr als formular-feld dar
	 */
	public function display_form()
	{
		if ($this->db_object->get_input() == ATTR_INPUT_FORM)
		{
			$this->display_form_impl();
		}
		else
		{
			$this->display_impl();
		}

		return;
	}

	public function is_visible( $attr_visible=10 )
	{
		return ($this->db_object->get_visible() <= $attr_visible );
	}

	public function display()
	{
		$this->display_impl();
		return;
	}

	public function display_list()
	{
		printf( "%s: %s<br>",
			$this->db_object->get_title(), htmlentities( $this->db_object->get_val_text() ) );
		return;
	}

	public function valid_for_insert()
	{
		return ($this->db_object->is_valid() && $this->datatype_valid());
	}

	public function valid_for_update()
	{
		return ($this->db_object->is_valid() && $this->datatype_valid());
	}

	protected function display_impl()
	{
		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			$this->db_object->get_title(), htmlentities( $this->get_formated_value() ) );
		return;
	}

	protected function display_form_impl()
	{
		$val = html_entity_decode( $this->db_object->get_val_text(), ENT_QUOTES, 'ISO-8859-1' );
		$this->display_form_attr_title();

		printf( "<td class='f_value'><input type='text' name='%s' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			$this->get_arg_name(), 50, 200, $val );

		return;
	}

	protected function datatype_valid()
	{
		return true;
	}
	
	protected function create_db_object( $_id )
	{
		$this->db_object = new db_attr( $_id );
	}

	protected function display_form_attr_title()
	{
		$help_text = $this->db_object->get_desc();
		$help_icon = "";
		if (strlen($help_text)!=0) $help_icon = icon_display( ICON_HELP_BUBBLE, $help_text ); 
		printf( "<tr><td class='f_oblig'>%s %s</td><td class='f_attr' title='%s'>%s : %s</td>",
			$this->get_oblig_string(),
			$this->get_tag_string(),
			$help_text,
			$this->db_object->get_title(),
			$help_icon
			);
		return;
	}

}

/**
 * abgeleitete klasse fuer datatype int
 */
class attr_int extends attr
{
	/**
	 * ueberladen aus den argumenten
	 */
	public function load_from_args()
	{
		$_value = http_get_value( $this->get_arg_name(), $this->db_object->get_val_int() );
		$this->db_object->set_val_int( $_value );
		$this->db_object->set_val_text( sprintf( "%d", $_value ) );

		return;
	}

	/**
	 * load value from implicit function
	 */
	public function load_from_func( &$item_processor )
	{
		if ($this->db_object->get_input() == ATTR_INPUT_FUNC)
		{
			$_value = module_list::get_input_value( $this->db_object->get_label(), $item_processor);
			$this->db_object->set_val_int( $_value );
			$this->db_object->set_val_text( sprintf( "%d", $_value ) );
		}

		return;	
	}

	/**
	 * load value from array, delivered from CSV import
	 * 
	 * the array has ta_id => value mappings
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function load_from_csv( &$ta_val_map )
	{
		$_ta_id = $this->db_object->get_ta_id();
		if (array_key_exists( $_ta_id, $ta_val_map))
		{
			$_value = $ta_val_map[ $_ta_id ];
			$this->db_object->set_val_int( $_value );
			$this->db_object->set_val_text( sprintf( "%d", $_value ) );
		}
		return;
	}

	protected function datatype_valid()
	{
		$value = $this->db_object->get_val_int();
		if ((string)((int)$value) != $value)
		{
			throw new core_data_exception( "$value is not an Integer!");
		}
		return true;
	}

	/**
	 * get format string from db_object or datatype default 
	 */
	protected function get_format()
	{
		$format = $this->db_object->get_format();
		if (strlen($format)==0) $format="%d";
		return $format;
	}

	/**
	 * get string with formated value
	 */
	protected function get_formated_value()
	{
		$format = $this->get_format();
		$formated_value = sprintf( $format, $this->db_object->get_val_int() );
		return $formated_value;
	}


	/**
	 * stellt das attr als formular-feld dar
	 */
	protected function display_form_impl()
	{
		$this->display_form_attr_title();

		printf( "<td class='f_value'><input type='text' name='%s' size='%d' maxlength='%d' value='%d'></td></tr>\n",
			$this->get_arg_name(), 50, 200, $this->db_object->get_val_int() );

		return;
	}

}

/**
 * abgeleitete klasse fuer datatype float
 */
class attr_float extends attr
{
	/**
	 * ueberladen aus den argumenten
	 */
	public function load_from_args()
	{
		$_value = http_get_value( $this->get_arg_name(), $this->db_object->get_val_float() );
		$this->db_object->set_val_float( $_value );
		$this->db_object->set_val_text( sprintf( "%f", $_value ) );

		return;
	}

	/**
	 * load value from implicit function
	 */
	public function load_from_func( &$item_processor )
	{
		if ($this->db_object->get_input() == ATTR_INPUT_FUNC)
		{
			$_value = module_list::get_input_value( $this->db_object->get_label(), $item_processor);
			$this->db_object->set_val_float( $_value );
			$this->db_object->set_val_text( sprintf( "%f", $_value ) );
		}

		return;	
	}

	/**
	 * load value from array, delivered from CSV import
	 * 
	 * the array has ta_id => value mappings
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function load_from_csv( &$ta_val_map )
	{
		$_ta_id = $this->db_object->get_ta_id();
		if (array_key_exists( $_ta_id, $ta_val_map))
		{
			$_value = $ta_val_map[ $_ta_id ];
			$this->db_object->set_val_float( $_value );
			$this->db_object->set_val_text( sprintf( "%f", $_value ) );
		}
		return;
	}

	/**
	 * stellt das attr als formular-feld dar
	 */
	protected function display_form_impl ()
	{
		$this->display_form_attr_title();

		printf( "<td class='f_value'><input type='text' name='%s' size='%d' maxlength='%d' value='%f'></td></tr>\n",
			$this->get_arg_name(), 50, 200, $this->db_object->get_val_float() );

		return;
	}
	
	protected function datatype_valid()
	{
		$value = $this->db_object->get_val_int();
		if ((string)((float)$value) != $value)
		{
			throw new core_data_exception( "$value is not a Float!");
		}
		return true;
	}

	/**
	 * get format string from db_object or datatype default 
	 */
	protected function get_format()
	{
		$format = $this->db_object->get_format();
		if (strlen($format)==0)
		{
			if (($precision = $this->db_object->get_datatype_sub()) > 0)
			{
				$format = "%." . $precision . "f";
			}
			else
			{
				$format="%f";
			}
		}
		return $format;
	}

	/**
	 * get string with formated value
	 */
	protected function get_formated_value()
	{
		$format = $this->get_format();
		$formated_value = sprintf( $format, $this->db_object->get_val_float() );
		return $formated_value;
	}


}

/**
 * abgeleitete klasse fuer datatype date
 */
class attr_date extends attr
{
	/**
	 * ueberladen aus den argumenten
	 */
	public function load_from_args()
	{
		$new_date = new core_date( $this->db_object->get_val_date() );
		$_year   = http_get_value( $this->get_arg_name() . "y", $new_date->get_year() );
		$new_date->set_year( $_year );
		$_month  = http_get_value( $this->get_arg_name() . "m", $new_date->get_month() );
		$new_date->set_month( $_month );
		$_day    = http_get_value( $this->get_arg_name() . "d", $new_date->get_day() );
		$new_date->set_day( $_day );
		if ($new_date->is_empty())
		{
			$this->db_object->set_val_text( "" );
		}
		else
		{
			$this->db_object->set_val_date( $new_date->get() );
			$this->db_object->set_val_text( $new_date->get() );
		}
		return;
	}

	/**
	 * load value from implicit function
	 */
	public function load_from_func( &$item_processor )
	{
		if ($this->db_object->get_input() == ATTR_INPUT_FUNC)
		{
			$_value = module_list::get_input_value( $this->db_object->get_label(), $item_processor);
			$new_date = new core_date( $_value );
			if ($new_date->is_empty())
			{
				$this->db_object->set_val_text( "" );
			}
			else
			{
				$this->db_object->set_val_date( $new_date->get() );
				$this->db_object->set_val_text( $new_date->get() );
			}
		}

		return;	
	}

	/**
	 * load value from array, delivered from CSV import
	 * 
	 * the array has ta_id => value mappings
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function load_from_csv( &$ta_val_map )
	{
		$_ta_id = $this->db_object->get_ta_id();
		if (array_key_exists( $_ta_id, $ta_val_map))
		{
			$_value = $ta_val_map[ $_ta_id ];
			$new_date = new core_date( $_value );
			if ($new_date->is_empty())
			{
				$this->db_object->set_val_text( "" );
			}
			else
			{
				$this->db_object->set_val_date( $new_date->get() );
				$this->db_object->set_val_text( $new_date->get() );
			}
		}
		return;
	}

	/**
	 * darstellung im formular
	 */
	protected function display_form_impl()
	{
		$new_date = new core_date( $this->db_object->get_val_text() );
		$default_string="";
		if (! $this->db_object->get_oblig()) $default_string="-";
		
		$this->display_form_attr_title();

		printf( "<td class='f_value'>" );
		$new_date->cat_year_segment( 10, 10, $this->get_arg_name() . "y", $default_string );
		printf( " - " );
		$new_date->cat_month_segment( $this->get_arg_name() . "m", FALSE, $default_string );
		printf( " - " );
		$new_date->cat_day_segment( $this->get_arg_name() . "d", $default_string );
		printf( "</td>\n" );

		return;		
	}

}

/**
 * abgeleitete klasse fuer datatype time
 */
class attr_time extends attr
{
	/**
	 * ueberladen aus den argumenten
	 */
	public function load_from_args()
	{
		$new_time = new core_time( $this->db_object->get_val_time() );
		$_hour   = http_get_value( $this->get_arg_name() . "h", $new_time->get_hour() );
		$new_time->set_hour( $_hour );
		$_min    = http_get_value( $this->get_arg_name() . "m", $new_time->get_min() );
		$new_time->set_min( $_min );
		$_sec    = http_get_value( $this->get_arg_name() . "s", $new_time->get_sec() );
		$new_time->set_sec( $_sec );
		if ($new_time->is_empty())
		{
			$this->db_object->set_val_text( "" );
		}
		else
		{		
			$this->db_object->set_val_time( $new_time->get() );
			$this->db_object->set_val_text( $new_time->get() );
		}
		return;
	}

	/**
	 * load value from implicit function
	 */
	public function load_from_func( &$item_processor )
	{
		if ($this->db_object->get_input() == ATTR_INPUT_FUNC)
		{
			$_value = module_list::get_input_value( $this->db_object->get_label(), $item_processor);
			$new_time = new core_time( $_value );
			if ($new_time->is_empty())
			{
				$this->db_object->set_val_text( "" );
			}
			else
			{
				$this->db_object->set_val_time( $new_time->get() );
				$this->db_object->set_val_text( $new_time->get() );
			}
		}

		return;	
	}

	/**
	 * load value from array, delivered from CSV import
	 * 
	 * the array has ta_id => value mappings
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function load_from_csv( &$ta_val_map )
	{
		$_ta_id = $this->db_object->get_ta_id();
		if (array_key_exists( $_ta_id, $ta_val_map))
		{
			$_value = $ta_val_map[ $_ta_id ];
			$new_time = new core_time( $_value );
			if ($new_time->is_empty())
			{
				$this->db_object->set_val_text( "" );
			}
			else
			{
				$this->db_object->set_val_time( $new_time->get() );
				$this->db_object->set_val_text( $new_time->get() );
			}
		}
		return;
	}

	/**
	 * darstellung im formular
	 */
	protected function display_form_impl()
	{
		$new_time = new core_time( $this->db_object->get_val_text() );
		$default_string="";
		if (! $this->db_object->get_oblig()) $default_string="-";

		$this->display_form_attr_title();

		printf( "<td class='f_value'>" );
		$new_time->cat_hour_segment( $this->get_arg_name() . "h", $default_string );
		printf( " : " );
		$new_time->cat_min_segment( $this->get_arg_name() . "m", $default_string );
		printf( " : " );
		$new_time->cat_sec_segment( $this->get_arg_name() . "s", $default_string );
		printf( "</td>\n" );

		return;		
	}

}

/**
 * abgeleitete klasse fuer datatype textblock
 */
class attr_textblock extends attr
{
	/**
	 * darstellung im formular
	 */
	protected function display_form_impl()
	{
		$val = html_entity_decode( $this->db_object->get_val_text(), ENT_QUOTES, 'ISO-8859-1' );

		$this->display_form_attr_title();

		printf( "<td class='f_value'><textarea name='%s' cols='%d' rows='%d'>%s</textarea></td></tr>\n",
			$this->get_arg_name(), 50, 5, $val );

		return;		
	}

	protected function display_impl()
	{
		$c_text = $this->db_object->get_val_text();
		$c_text = htmlentities( $c_text, ENT_QUOTES, 'ISO-8859-1' );
		$c_text = str_replace( "\n", "<br>", $c_text );
		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			$this->db_object->get_title(), $c_text );
		return;
	}

}

/**
 * abgeleitete klasse fuer datatype url
 */
class attr_url extends attr
{
	protected function display_impl()
	{
		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>",
			$this->db_object->get_title() ); 
		
		if (strlen($this->db_object->get_val_text())!=0)
		{
			printf("<a href='%s' target='_blank' title='%s'>%s</a>",
			htmlentities( $this->db_object->get_val_text() ), 
			"Open the URL in a seperate window",
			htmlentities( $this->db_object->get_val_text() ) );
		}
			
		printf( "</td></tr>\n" );
		return;
	}

}

/**
 * abgeleitete klasse fuer datatype catalog
 */
class attr_catalog extends attr
{
	/**
	 * ueberladen aus den argumenten
	 */
	public function load_from_args()
	{
		$_cate_id = http_get_int( $this->get_arg_name(), $this->db_object->get_val_int() );
		if ($_cate_id == -1)
		{
			$_cate_val_new = http_get_string( $this->get_arg_name() . "new", "" );
			$_cate_id = cat_entry_list::get_id_by_title( $this->db_object->get_datatype_sub(), $_cate_val_new );
			if ($_cate_id == 0)
			{
				try
				{
					$new_cate = new db_cat_entry();
					$new_cate->set_cat_id( $this->db_object->get_datatype_sub() );
					$new_cate->set_title( $_cate_val_new );
//echo "<pre>";
//print_r( $new_cate );
//echo "</pre>\n";			
					$new_cate->db_insert();
					$_cate_id = $new_cate->get_id();
				}
				catch (core_db_exception $e)
				{
					$this->add_message(get_string('DBError_dsc')." ('" . $e->get_message() . "')");
				}
				catch (core_data_exception $e)
				{
					$this->add_message(get_string('ValidationError_dsc')." ('" . $e->get_message() . "')");
				}
			}
		}

		$_cate_val = cat_entry_list::get_title( $_cate_id );

		$this->db_object->set_val_int( $_cate_id );
		$this->db_object->set_val_text( $_cate_val );

		return;
	}

	/**
	 * load value from implicit function
	 */
	public function load_from_func( &$item_processor )
	{
		if ($this->db_object->get_input() == ATTR_INPUT_FUNC)
		{
			$_value = module_list::get_input_value( $this->db_object->get_label(), $item_processor);
			$_cate_id = cat_entry_list::get_id_by_title( $this->db_object->get_datatype_sub(), $_value );
			if ($_cate_id == 0)
			{
				try
				{
					$new_cate = new db_cat_entry();
					$new_cate->set_cat_id( $this->db_object->get_datatype_sub() );
					$new_cate->set_title( $_value );
//echo "<pre>";
//print_r( $new_cate );
//echo "</pre>\n";			
					$new_cate->db_insert();
					$_cate_id = $new_cate->get_id();
				}
				catch (core_db_exception $e)
				{
					$this->add_message(get_string('DBError_dsc')." ('" . $e->get_message() . "')");
				}
				catch (core_data_exception $e)
				{
					$this->add_message(get_string('ValidationError_dsc')." ('" . $e->get_message() . "')");
				}
			}

			$_cate_val = cat_entry_list::get_title( $_cate_id );
	
			$this->db_object->set_val_int( $_cate_id );
			$this->db_object->set_val_text( $_cate_val );
		}

		return;	
	}


	/**
	 * load value from array, delivered from CSV import
	 * 
	 * the array has ta_id => value mappings
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function load_from_csv( &$ta_val_map )
	{
		$_ta_id = $this->db_object->get_ta_id();
		if (array_key_exists( $_ta_id, $ta_val_map))
		{
			$_value = $ta_val_map[ $_ta_id ];
			$_cate_id = cat_entry_list::get_id_by_title( $this->db_object->get_datatype_sub(), $_value );
			if ($_cate_id == 0)
			{
				try
				{
					$new_cate = new db_cat_entry();
					$new_cate->set_cat_id( $this->db_object->get_datatype_sub() );
					$new_cate->set_title( $_value );
//echo "<pre>";
//print_r( $new_cate );
//echo "</pre>\n";			
					$new_cate->db_insert();
					$_cate_id = $new_cate->get_id();
				}
				catch (core_db_exception $e)
				{
					$this->add_message(get_string('DBError_dsc')." ('" . $e->get_message() . "')");
				}
				catch (core_data_exception $e)
				{
					$this->add_message(get_string('ValidationError_dsc')." ('" . $e->get_message() . "')");
				}
			}

			$_cate_val = cat_entry_list::get_title( $_cate_id );
	
			$this->db_object->set_val_int( $_cate_id );
			$this->db_object->set_val_text( $_cate_val );
		}
		return;
	}

	/**
	 * darstellung im formular
	 */
	protected function display_form_impl()
	{
		$this->display_form_attr_title();
		$default = array();
		if (! $this->db_object->get_oblig()) $default[0] = "---";
		printf( "<td class='f_value'>" );
		cat_entry_list::getInstance( $this->db_object->get_datatype_sub() )->html_options(
			array( $this->db_object->get_val_int() ), array(), $this->get_arg_name(), $default );
		if (! cat_entry_list::getInstance( $this->db_object->get_datatype_sub() )->is_strict() )
		{
			printf( " <input type='text' name='%s' size='%d' maxlength='%d' value='%s'>\n",
				$this->get_arg_name() . "new", 50, 200, "" );
		}
		printf( "</td></tr>\n" );

		return;		
	}

	protected function display_impl()
	{
		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value menu'>",
			$this->db_object->get_title() );
		
		if ($this->db_object->get_val_int()!=0)
		{
			$cate = new db_cat_entry( $this->db_object->get_val_int() );
			
			printf( "&gt;&gt; %s",
				get_html_url(
					sprintf( "%s?cate_id=%d", CATE_CI_LIST_PHP, $this->db_object->get_val_int() ),
					$cate->get_styled_title(),
					FALSE,
					get_string('ViewAllCIsReferingthisCat_dsc')
				)
			);
		}

		printf( "</td></tr>\n" );
		
		return;
	}
	
}

/**
 * abgeleitete klasse fuer datatype CI-Referenz
 */
class attr_ci_ref extends attr
{
	/**
	 * ueberladen aus den argumenten
	 */
	public function load_from_args()
	{
		$_ci_id = http_get_int( $this->get_arg_name(), $this->db_object->get_val_int() );
		$_ci_val = ci_navi_list::get_title( $_ci_id );

		$this->db_object->set_val_int( $_ci_id );
		$this->db_object->set_val_text( $_ci_val );

		return;
	}

	/**
	 * load value from implicit function
	 */
	public function load_from_func( &$item_processor )
	{
		if ($this->db_object->get_input() == ATTR_INPUT_FUNC)
		{
			$_ci_id = module_list::get_input_value( $this->db_object->get_label(), $item_processor);
			$_ci_val = ci_navi_list::get_title( $_ci_id );
	
			$this->db_object->set_val_int( $_ci_id );
			$this->db_object->set_val_text( $_ci_val );
		}

		return;	
	}

	/**
	 * load value from array, delivered from CSV import
	 * 
	 * the array has ta_id => value mappings
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function load_from_csv( &$ta_val_map )
	{
		$_ta_id = $this->db_object->get_ta_id();
		if (array_key_exists( $_ta_id, $ta_val_map))
		{
			$_ci_id = $ta_val_map[ $_ta_id ];
			$_ci_val = ci_navi_list::get_title( $_ci_id );
	
			$this->db_object->set_val_int( $_ci_id );
			$this->db_object->set_val_text( $_ci_val );
		}
		return;
	}

	/**
	 * store value into csv array, at position ta_id
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function store_to_csv( &$ta_val_map )
	{
		$_ta_id = $this->db_object->get_ta_id();
		if (array_key_exists( $_ta_id, $ta_val_map))
		{
			$ta_val_map[ $_ta_id ] = $this->db_object->get_val_int();
		}
		return;
	}

	/**
	 * darstellung im formular
	 */
	protected function display_form_impl()
	{
		$this->display_form_attr_title();
		$default = array();
		if (! $this->db_object->get_oblig()) $default[0] = "---";
		printf( "<td class='f_value'>" );
		ci_cit_navi_list::getInstance( $this->db_object->get_datatype_sub() )->html_options(
			array( $this->db_object->get_val_int() ), array(), $this->get_arg_name(), $default );
		printf( "</td></tr>\n" );

		return;		
	}

	protected function display_impl()
	{
		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value menu'>",
			$this->db_object->get_title() );
			 
		if ($this->db_object->get_val_int()!=0)
		{
			$icon_size = get_property("list_icon_size");
			$ci = new db_typed_ci( $this->db_object->get_val_int() );
			$icon_url = icon_list::getInstance()->get_html_imx( $ci->get_icon(), $icon_size, $icon_size );

			printf( "&gt;&gt; %s",
				get_html_url(
					sprintf( "%s?ci_id=%d", CI_VIEW_PHP, $this->db_object->get_val_int() ),
					$icon_url . " " . $this->db_object->get_val_text(),
					FALSE,
					get_string('GotoViewThisCI_ttl')
					)
			);
		}

		printf( "</td></tr>\n" );
		return;
	}
	
}

/**
 * abgeleitete klasse fuer datatype IP-Addr
 */
class attr_ip_addr extends attr
{

	/**
	 * ueberladen aus den argumenten
	 */
	public function load_from_args()
	{
		$addr = new core_ipv4( $this->db_object->get_val_text() );

		$_a = http_get_value( $this->get_arg_name() . "a", $addr->get_a() );
		$addr->set_a( $_a );
		$_b = http_get_value( $this->get_arg_name() . "b", $addr->get_b() );
		$addr->set_b( $_b );
		$_c = http_get_value( $this->get_arg_name() . "c", $addr->get_c() );
		$addr->set_c( $_c );
		$_d = http_get_value( $this->get_arg_name() . "d", $addr->get_d() );
		$addr->set_d( $_d );

		$this->db_object->set_val_text( $addr->get() );
		return;
	}

	/**
	 * load value from implicit function
	 */
	public function load_from_func( &$item_processor )
	{
		if ($this->db_object->get_input() == ATTR_INPUT_FUNC)
		{
			$_value = module_list::get_input_value( $this->db_object->get_label(), $item_processor);
			$addr = new core_ipv4( $_value );

			$this->db_object->set_val_text( $addr->get() );
		}

		return;	
	}

	/**
	 * load value from array, delivered from CSV import
	 * 
	 * the array has ta_id => value mappings
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function load_from_csv( &$ta_val_map )
	{
		$_ta_id = $this->db_object->get_ta_id();
		if (array_key_exists( $_ta_id, $ta_val_map))
		{
			$_value = $ta_val_map[ $_ta_id ];
			$addr = new core_ipv4( $_value );

			$this->db_object->set_val_text( $addr->get() );
		}
		return;
	}

	/**
	 * stellt das attr als formular-feld dar
	 */
	protected function display_form_impl()
	{
		$addr = new core_ipv4( $this->db_object->get_val_text() );
		
		$this->display_form_attr_title();

		printf( "<td class='f_value'>" .
			"<input type='text' name='%s' size='%d' maxlength='%d' value='%d'> . " .
			"<input type='text' name='%s' size='%d' maxlength='%d' value='%d'> . " .
			"<input type='text' name='%s' size='%d' maxlength='%d' value='%d'> . " .
			"<input type='text' name='%s' size='%d' maxlength='%d' value='%d'>" .
			"</td></tr>\n",
			$this->get_arg_name() . "a", 3, 3, $addr->get_a(),
			$this->get_arg_name() . "b", 3, 3, $addr->get_b(),
			$this->get_arg_name() . "c", 3, 3, $addr->get_c(),
			$this->get_arg_name() . "d", 3, 3, $addr->get_d()
			);

		return;
	}
	
}

/**
 * factory
 */
function & create_attr( $data )
{
	$attr = false;
	if (! array_key_exists( 'ta_datatype', $data )) return $attr;

	switch ($data['ta_datatype'])
	{
		case DT_INT:
		{
			$attr = new attr_int( $data );
			break;
		}
		case DT_FLOAT:
		{
			$attr = new attr_float( $data );
			break;
		}
		case DT_DATE:
		{
			$attr = new attr_date( $data );
			break;
		}
		case DT_TIME:
		{
			$attr = new attr_time( $data );
			break;
		}
		case DT_TEXTBLOCK:
		{
			$attr = new attr_textblock( $data );
			break;
		}
		case DT_TEXT:
		{
			$attr = new attr( $data );
			break;
		}
		case DT_URL:
		{
			$attr = new attr_url( $data );
			break;
		}
		case DT_CATALOG:
		{
			$attr = new attr_catalog( $data );
			break;
		}
		case DT_CI_REF:
		{
			$attr = new attr_ci_ref( $data );
			break;
		}
		case DT_IP_ADDR:
		{
			$attr = new attr_ip_addr( $data );
			break;
		}
	}
	return $attr;
}
