<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
 * base class for managing graphical boxes
 */
abstract class box
{
	// position
	protected $x			= 0;
	protected $y			= 0;
	// size
	protected $size_x		= 0;
	protected $size_y		= 0;
	// padding
	protected $pad_x		= 0;
	protected $pad_y		= 0;
	// alignment
	protected $align_x		= BOX_ALIGN_ADAPT;
	protected $align_y		= BOX_ALIGN_ADAPT;
	// content pos
	protected $c_x			= 0;
	protected $c_y			= 0;
	// content size
	protected $c_size_x		= 0;
	protected $c_size_y		= 0;
	// sichtbarkeit	
	protected $visible		= TRUE;
	// click map support
	protected $click_map	= FALSE;
	protected $click_url	= "";
	protected $click_title	= "";
	
	public function __construct ()
	{
		return;
	}
	
	// get
	
	public function get_x ()
	{
		return $this->x;
	}
	
	public function get_y ()
	{
		return $this->y;
	}

	public function get_size_x ()
	{
		return $this->size_x;
	}

	public function get_size_y ()
	{
		return $this->size_y;
	}

	public function get_pad_x ()
	{
		return $this->pad_x;
	}

	public function get_pad_y ()
	{
		return $this->pad_y;
	}

	/**
	 * liefert ausrichtug in y
	 */
	public function get_align_x ()
	{
		return $this->align_x;
	}

	/**
	 * liefert ausrichtug in y
	 */
	public function get_align_y ()
	{
		return $this->align_y;
	}

	/**
	 * liefert position des content in x
	 */
	public function get_c_x ()
	{
		return $this->c_x;
	}
	
	/**
	 * liefert position des content in y
	 */
	public function get_c_y ()
	{
		return $this->c_y;
	}
	
	/**
	 * liefert size des content in x
	 */
	public function get_c_size_x ()
	{
		return $this->c_size_x;
	}
	
	/**
	 * liefert size des content in y
	 */
	public function get_c_size_y ()
	{
		return $this->c_size_y;
	}
	
	public function is_visible ()
	{
		return $this->visible;
	}

	/**
	 * liefert die linke obere und rechte untere ecke als
	 * array
	 */
	public function get_box()
	{
		$box = array( $this->x, $this->y, $this->x + $this->size_x, $this->y + $this->size_y );
		return $box;
	}

	// set
	
	/**
	 * setzt position links oben, x-koord
	 */
	public function set_x ( $_x )
	{
		$this->x = $_x;
	}

	/**
	 * setzt position links oben, y-koord
	 */
	public function set_y ( $_y )
	{
		$this->y = $_y;
	}

	/**
	 * setzt position links oben
	 */
	public function set_pos ( $_x, $_y )
	{
//echo "DEBUG: box::set_pos( $_x, $_y )<br>\n";
		$this->set_x( $_x );
		$this->set_y( $_y );
		return;
	}
	
	/**
	 * setzt ausdehnung in x-richtung
	 */
	public function set_size_x ( $_size_x )
	{
		if ($this->get_align_x() == BOX_ALIGN_ADAPT) return;
		if ($_size_x < 0) return;
		$this->size_x = $_size_x;
		$this->resize();
	}

	/**
	 * setzt ausdehnung in y-richtung
	 */
	public function set_size_y ( $_size_y )
	{
		if ($this->get_align_y() == BOX_ALIGN_ADAPT) return;
		if ($_size_y < 0) return;
		$this->size_y = $_size_y;
		$this->resize();
	}

	/**
	 * setzt ausdehnung
	 */
	public function set_size ( $_size_x, $_size_y )
	{
		$this->set_size_x( $_size_x );
		$this->set_size_y( $_size_y );
		return;
	}
	
	/**
	 * setzt inneren abstand in x-richtung 
	 */
	public function set_pad_x ( $_pad_x )
	{
		if ($_pad_x < 0) return;
		$this->pad_x = $_pad_x;
		$this->resize();
	}

	/**
	 * setzt inneren abstand in y-richtung 
	 */
	public function set_pad_y ( $_pad_y )
	{
		if ($_pad_y < 0) return;
		$this->pad_y = $_pad_y;
		$this->resize();
	}

	/**
	 * setzt inneren abstand fuer beide richtungen
	 * 
	 * wird der zweite parameter nicht uebergeben, werden beide paddings
	 * mit dem ersten wert gesetzt 
	 */
	public function set_padding ( $_pad_x, $_pad_y=NULL )
	{
		if ($_pad_y == NULL) $_pad_y = $_pad_x;
		$this->set_pad_x ( $_pad_x );
		$this->set_pad_y ( $_pad_y );
	}

	/**
	 * setzt innere ausrichtung in x-richtung
	 */
	public function set_align_x ( $_align_x )
	{
		if (! in_array( $_align_x, array( BOX_ALIGN_LEFT, BOX_ALIGN_CENTER, BOX_ALIGN_RIGHT, BOX_ALIGN_ADAPT))) return;
		$this->align_x = $_align_x;
		$this->resize();
	}

	/**
	 * setzt innere ausrichtung in y-richtung
	 */
	public function set_align_y ( $_align_y )
	{
		if (! in_array( $_align_y, array( BOX_ALIGN_TOP, BOX_ALIGN_CENTER, BOX_ALIGN_BOTTOM, BOX_ALIGN_ADAPT))) return;
		$this->align_y = $_align_y;
		$this->resize();
	}

	/**
	 * setzt innere ausrichtung
	 */
	public function set_align ( $_align_x, $_align_y )
	{
		$this->set_align_x( $_align_x );
		$this->set_align_y( $_align_y );
	}

	/**
	 * setzt die innere ausrichtung in beiden richtungen,
	 * wenn das argument TRUE ist: adapt
	 * sonst: links, oben  
	 * 
	 */
	public function set_autospan ( $_autospan )
	{
		if ($_autospan)
		{
			$this->set_align( BOX_ALIGN_ADAPT, BOX_ALIGN_ADAPT );
		}
		else
		{
			$this->set_align( BOX_ALIGN_LEFT, BOX_ALIGN_TOP );
		}
	}

	/**
	 * setzt box sichtbar
	 */
	public function set_visible()
	{
		$this->visible = TRUE;
	}
	
	/**
	 * setzt box unsichtbar
	 */
	public function set_invisible()
	{
		$this->visible = FALSE;
	}
	
	/**
	 * setzt die click map unterstuetzung
	 */
	public function set_click ( $_url="", $_title="" )
	{
		$this->click_map = TRUE;
		$this->click_url = $_url;
		$this->click_title = $_title;
	}

	// actions
	
	public function shift_pos ( $_dx, $_dy )
	{
		$this->set_x( $this->get_x() + $_dx );
		$this->set_y( $this->get_y() + $_dy );
	}

	protected function resize()
	{
		$this->resize_content();

		// inneren offset bestimmen
		switch ($this->get_align_x())
		{
			case BOX_ALIGN_LEFT:
			{
				$this->c_x = $this->get_pad_x();
				break;
			}
			case BOX_ALIGN_CENTER:
			{
				$this->c_x = floor( ($this->get_size_x() - $this->get_c_size_x()) / 2 );
				break;
			}
			case BOX_ALIGN_RIGHT:
			{
				$this->c_x = $this->get_szie_x() - $this->get_c_size_x() - $this->get_pad_x();
				break;
			}
			case BOX_ALIGN_ADAPT:
			{
				// ausdehnung der box richtet sich nach ausdehnung des content
				$this->size_x = $this->get_c_size_x() + 2 * $this->get_pad_x();
				// position left
				$this->c_x = $this->get_pad_x();
				break;
			}
		}			
		switch ($this->get_align_y())
		{
			case BOX_ALIGN_TOP:
			{
				$this->c_y = $this->get_pad_y();
				break;
			}
			case BOX_ALIGN_CENTER:
			{
				$this->c_y = floor( ($this->get_size_y() - $this->get_c_size_y()) / 2 );
				break;
			}
			case BOX_ALIGN_BOTTOM:
			{
				$this->c_y = $this->get_size_y() - $this->get_c_size_y() - $this->get_pad_y();
				break;
			}
			case BOX_ALIGN_ADAPT:
			{
				// ausdehnung der box richtet sich nach ausdehnung des content
				$this->size_y = $this->get_c_size_y() + 2 * $this->get_pad_y();
				// position top
				$this->c_y = $this->get_pad_y();
				break;
			}
		}			

		return;
	}
	
	/**
	 * aktualisiert die werte c_size_x (breite des content)
	 * und c_size_y (hoehe des content)
	 */
	protected function resize_content()
	{
		return;
	}


	// draw	
	
	public function draw ( $image, $map_file, $offset_x=0, $offset_y=0 )	
	{
		if ($this->is_visible())
		{
			$ox = $offset_x + $this->x;
			$oy = $offset_y + $this->y;
//echo "DEBUG: " . get_class($this) . "::draw( $offset_x, $offset_y ): $ox, $oy / $this->size_x, $this->size_y <br>\n";
			$this->draw_impl( $image, $ox, $oy );
			$this->map_impl( $map_file, $ox, $oy );
		}
		return;
	}
	
	protected function draw_impl ( $image, $ox, $oy )
	{
//		$color_grey = ImageColorAllocate ($image, 127, 127, 127);
//		imagerectangle( $image, $ox, $oy, $ox + $this->size_x - 1, $oy + $this->size_y - 1, $color_grey );
	}

	protected function map_impl ( $map_file, $ox, $oy )
	{
		if ($this->click_map)
		{
			$url_str = "";
			if (strlen($this->click_url)!=0) $url_str=" href='" . $this->click_url . "'";
			$title_str = "";
			if (strlen($this->click_title)!=0) $title_str=" title='" . $this->click_title . "'";
			
			fwrite( $map_file, sprintf("<area shape='rect' coords='%d,%d,%d,%d'%s title='%s' alt='%s'>\n",
				$ox, $oy, $ox + $this->size_x, $oy + $this->size_y,
				$url_str, $this->click_title, $this->click_title )
				);
		}
	}

}

/**
 * klasse zur verwaltung von verschachtelten boxen 
 */
class box_group extends box
{
	protected $childs		= array();
	
	public function __construct ( $_sx=0, $_sy=0, $_autospan=TRUE )
	{
		parent::__construct();
		$this->set_autospan( $_autospan );
		$this->set_size( $_sx, $_sy );
	}
	
	public function count()
	{
		return count($this->childs);
	}

	/**
	 * fuegt box als child ein, die position von box bleibt unveraendert 
	 */	
	public function add ( $box )
	{
		$this->childs[] = $box;
		$this->resize();
		return;
	}

	/**
	 * fuegt box als child ein, dabei erfolgt eine statische
	 * ausrichtung von box innerhalb des eigenen ranges
	 */
	public function add_static ( $box, $align_x, $align_y )
	{
		$this->childs[] = $box;
		$x = 0;
		$y = 0;
		switch ($align_x)
		{
			case BOX_ALIGN_LEFT:
			{
				$x = 0;
				break;
			}
			case BOX_ALIGN_CENTER:
			{
				$x = floor( ($this->get_size_x() - $box->get_size_x()) / 2 );
				break;
			}
			case BOX_ALIGN_RIGHT:
			{
				$x = $this->get_size_x() - $box->get_size_x();
				break;
			}
			default:
			{
				trigger_error( "invalid alignment", E_ERROR );
			}
		}
		switch ($align_y)
		{
			case BOX_ALIGN_TOP:
			{
				$y = 0;
				break;
			}
			case BOX_ALIGN_CENTER:
			{
				$y = floor( ($this->get_size_y() - $box->get_size_y()) / 2 );
				break;
			}
			case BOX_ALIGN_BOTTOM:
			{
				$y = $this->get_size_y() - $box->get_size_y();
				break;
			}
			default:
			{
				trigger_error( "invalid alignment", E_ERROR );
			}
		}
		
//echo "DEBUG: box::set_pos( $x, $y )<br>\n";
		$box->set_pos( $x, $y );
		return;
	}

	/**
	 * fuegt box als child ein, dabei wird box an den schon vorhandenen
	 * childs ausgerichtet
	 */
	public function add_dynamic ( $box, $align_x, $align_y )
	{
		$cover_box = $this->get_covered_box();
		$this->childs[] = $box;
		$x = 0;
		$y = 0;
		switch ($align_x)
		{
			case BOX_ALIGN_LEFT:
			{
				$x = $cover_box[0] - $this->get_pad_x() - $box->get_size_x();
				break;
			}
			case BOX_ALIGN_CENTER:
			{
				$x = floor( ($cover_box[0] + $cover_box[2] - $box->get_size_x()) / 2 ); 
				break;
			}
			case BOX_ALIGN_RIGHT:
			{
				$x = $cover_box[2] + $this->get_pad_x();
				break;
			}
			default:
			{
				trigger_error( "invalid alignment", E_ERROR );
			}
		}
		switch ($align_y)
		{
			case BOX_ALIGN_TOP:
			{
				$y = $cover_box[1] - $this->get_pad_y() - $box->get_size_y();
				break;
			}
			case BOX_ALIGN_CENTER:
			{
				$y = floor( ($cover_box[1] + $cover_box[3] - $box->get_size_y()) / 2 ); 
				break;
			}
			case BOX_ALIGN_BOTTOM:
			{
				$y = $cover_box[3] + $this->get_pad_y();
				break;
			}
			default:
			{
				trigger_error( "invalid alignment", E_ERROR );
			}
		}
		
		// setzen
		if ($x < 0) $x = 0;
		if ($y < 0) $y = 0;
		$box->set_pos( $x, $y );

		// autospan
		$this->resize();

		return;
	}

	/**
	 * fuegt box als child ein, dabei wird box an den schon vorhandenen
	 * childs ausgerichtet (wie bei add_dynamic)
	 * 
	 * zusaetzlich wird die ausdehnung von box in die entsprechenden 
	 * richtungen ausgedehnt
	 */
	public function add_float ( $box, $align_x, $align_y )
	{
		$cover_box = $this->get_covered_box();

		// hier nur ausdehnung bestimmen und setzen
		$sx = 0;
		$sy = 0;
		
		if (count($cover_box)==0)
		{
			$sx = $this->get_size_x();
			$sy = $this->get_size_y();
		}
		else
		{
			switch ($align_x)
			{
				case BOX_ALIGN_LEFT:
				{
					$sx = $cover_box[0] - $this->get_pad_x();
					break;
				}
				case BOX_ALIGN_CENTER:
				{
					$sx = $this->get_size_x() - 2 * $this->get_pad_x(); 
					break;
				}
				case BOX_ALIGN_RIGHT:
				{
					$sx = $this->get_size_x() - $cover_box[2] - $this->get_pad_x();
					break;
				}
				default:
				{
					trigger_error( "invalid alignment", E_ERROR );
				}
			}
			switch ($align_y)
			{
				case BOX_ALIGN_TOP:
				{
					$sy = $cover_box[1] - $this->get_pad_y();
					break;
				}
				case BOX_ALIGN_CENTER:
				{
					$sy = $this->get_size_y() - 2 * $this->get_pad_y(); 
					break;
				}
				case BOX_ALIGN_BOTTOM:
				{
					$sy = $this->get_size_y() - $cover_box[3] - $this->get_pad_y();
					break;
				}
				default:
				{
					trigger_error( "invalid alignment", E_ERROR );
				}
			}
		}
//echo "DEBUG: box::set_size( $sx, $sy )<br>\n";
		$box->set_size( $sx, $sy );

		// und nun dynamisch einsetzen
		$this->add_dynamic( $box, $align_x, $align_y );

		return;
	}

	public function draw ( $image, $map_file, $offset_x=0, $offset_y=0 )	
	{
//echo "DEBUG: box_group::draw<br>\n";
		if ($this->is_visible())
		{
			$ox = $offset_x + $this->x;
			$oy = $offset_y + $this->y;
//echo "DEBUG: " . get_class($this) . "::draw( $offset_x, $offset_y ): $ox, $oy / $this->size_x, $this->size_y <br>\n";
	
			foreach ($this->childs as $child)
			{
				$child->draw( $image, $map_file, $ox, $oy );
			}
		}
		parent::draw( $image, $map_file, $offset_x, $offset_y );
		return;
	}


	/**
	 * aktualisiert die werte cx (breite des content)
	 * und cy (hoehe des content)
	 */
	protected function resize_content()
	{
		$cover_box = $this->get_covered_box();
		$this->c_size_x = $cover_box[2] - $cover_box[0] + 1;
		$this->c_size_y = $cover_box[3] - $cover_box[1] + 1;
		return;
	}

	protected function get_covered_box ()
	{
		if ($this->count() == 0)
		{
			$cover_box = array( 0, 0, 0, 0 );
		}
		else
		{
			$cover_box = array( $this->get_size_x(), $this->get_size_y(), 0, 0 );
			foreach ($this->childs as $child)
			{
				$box = $child->get_box();
				if ($box[0] < $cover_box[0]) $cover_box[0] = $box[0];
				if ($box[1] < $cover_box[1]) $cover_box[1] = $box[1];
				if ($box[2] > $cover_box[2]) $cover_box[2] = $box[2];
				if ($box[3] > $cover_box[3]) $cover_box[3] = $box[3];
			}
		}
//echo "DEBUG: box_group::get_covered_box(): $cover_box[0], $cover_box[1], $cover_box[2], $cover_box[3]<br>\n";	
		return $cover_box; 
	}
}

/**
 * klasse zur darstellung eines rechtecks
 */
class box_rectangle extends box
{
	protected $frame_color	= NULL;
	
	public function __construct ( $_sx, $_sy, $_frame_color=NULL )
	{
		parent::__construct();
		$this->set_align( BOX_ALIGN_LEFT, BOX_ALIGN_TOP );
		$this->set_size( $_sx, $_sy );
		$this->frame_color = $_frame_color;
	}
	
	protected function draw_impl ( $image, $ox, $oy )
	{
//echo "DEBUG: box_rectangle::draw_impl<br>\n";
		if ($this->frame_color != NULL)
		{
			imagerectangle( $image, $ox, $oy, $ox + $this->size_x - 1, $oy + $this->size_y - 1, $this->frame_color );
		}
	}
}

/**
 * klasse zur darstellung eines ausgefuellten rechtecks
 */
class box_rectangle_filled extends box_rectangle
{
	protected $back_color	= NULL;
	
	public function __construct ( $_sx, $_sy, $_frame_color=NULL, $_back_color=NULL )
	{
		parent::__construct( $_sx, $_sy, $_frame_color );
		$this->back_color = $_back_color;
	}

	protected function draw_impl ( $image, $ox, $oy )
	{
		if ($this->back_color != NULL)
		{
			imagefilledrectangle( $image, $ox, $oy, $ox + $this->size_x - 1, $oy + $this->size_y - 1, $this->back_color );
		}
		parent::draw_impl( $image, $ox, $oy );		
	}
}

/**
 * klasse zur darstellung von ft text
 */
class box_ft_text extends box
{
	protected $text			= "";
	protected $font			= "";
	protected $font_size	= 0;
	protected $color		= 0;
	
	public function __construct ( $_text, $_font, $_font_size, $_color )
	{
		parent::__construct();
		$this->color = $_color;
		$this->set_font( $_font, $_font_size );
		$this->set_text( $_text );
	}
	
	public function set_font ( $_font, $_font_size )
	{
		$this->font = $_font;
		$this->font_size = $_font_size;
		$this->resize();		
		return;
	}
	
	public function set_text ( $_text )
	{
		$this->text = $_text;
		$this->resize();		
		return;
	}

	protected function draw_impl ( $image, $ox, $oy )
	{
		Imagefttext( $image, $this->font_size, 0,
			$ox + $this->c_x, $oy + $this->c_y + $this->c_size_y,
			$this->color, $this->font,
			$this->text);

		parent::draw_impl( $image, $ox, $oy );
		return;
	}

	/**
	 * aktualisiert die werte cx (breite des content)
	 * und cy (hoehe des content)
	 */
	protected function resize_content()
	{
		// text ausdehnung
		$ftbox = imageftbbox( $this->font_size, 0, $this->font, $this->text );
		$text_sx = abs( $ftbox[2] - $ftbox[0] );
		$text_sy = abs( $ftbox[3] - $ftbox[5] );
//echo "DEBUG: box_tf_text::resize_content(): $text_sx, $text_sy<br>\n";
		$this->c_size_x = $text_sx;
		$this->c_size_y = $text_sy;
		return;
	}

}

/**
 * klasse zur darstellung eines icons
 */
class box_icon extends box_rectangle_filled
{
	protected $icon_name	= "";
	protected $icon_size	= 0;
	protected $icon			= FALSE;
	
	public function __construct ( $_icon_name, $_icon_size, $_autospan=TRUE, $_frame_color=NULL, $_back_color=NULL )
	{
		parent::__construct( 0, 0, $_frame_color, $_back_color );
		$this->set_autospan( $_autospan );
		$this->set_icon( $_icon_name, $_icon_size );
	}
	
	public function set_icon( $_icon_name, $_icon_size )
	{
		$this->icon = icon_list::getInstance()->get_icon_image( $_icon_name, $_icon_size );
		if ($this->icon !== FALSE)
		{
			$this->icon_name = $_icon_name;
			$this->icon_size = $_icon_size;
			
			$this->resize();
		}

		return;		
	}

	protected function resize_content ()
	{
		if ($this->icon !== FALSE)
		{
			$this->c_size_x = imagesx( $this->icon );
			$this->c_size_y = imagesy( $this->icon );
		}
		return;
	}

	protected function draw_impl ( $image, $ox, $oy )
	{
		parent::draw_impl( $image, $ox, $oy );

		// icon darstellen wenn:
		// - gueltige resource
		// - genuegend platz ist		
		if (
			($this->icon !== FALSE)
			&&
			($this->get_c_size_x() <= $this->get_size_x())
			&&
			($this->get_c_size_y() <= $this->get_size_y())
		   )
		{
			imagecopy( $image, $this->icon, $ox + $this->c_x, $oy + $this->c_y, 0, 0, $this->c_size_x, $this->c_size_y );
		}
		return;
	}
}

/**
 * klasse zur darstellung eines ci
 */
class box_ci extends box_group
{
	
	public function __construct ( $_padding, $_icon_name, $_icon_size, $_text, $_font, $_font_size, $_color )
	{
		parent::__construct( 0, 0, TRUE );
		$this->set_padding( $_padding );
		
		// icon
		if (strlen($_icon_name)!=0)
		{
			$icon_box = new box_icon( $_icon_name, $_icon_size );
			$this->add_dynamic( $icon_box, BOX_ALIGN_RIGHT, BOX_ALIGN_BOTTOM );
		}
		
		// text
		$text_box = new box_ft_text( $_text, $_font, $_font_size, $_color );
		$this->add_dynamic( $text_box, BOX_ALIGN_RIGHT, BOX_ALIGN_CENTER );
		
		return;
	}
}

/**
 * klasse zur darstellung eines rasters
 * 
 */
class box_grid extends box_rectangle_filled
{
	protected $gx_size	= 0;
	protected $gx_count	= 0;
	protected $gy_size	= 0;
	protected $gy_count	= 0;
	
	public function __construct ( $_gx_size, $_gx_count, $_gy_size, $_gy_count, $_frame_color, $_back_color )
	{
		$this->gx_size  = $_gx_size;
		$this->gx_count = $_gx_count;
		$this->gy_size  = $_gy_size;
		$this->gy_count = $_gy_count;
		
		parent::__construct( $this->gx_size * $this->gx_count, $this->gy_size * $this->gy_count, $_frame_color, $_back_color );
	}
	
	protected function draw_impl ( $image, $ox, $oy )
	{
		parent::draw_impl( $image, $ox, $oy );
		// raster
		for ($i=0; $i<$this->gx_count; $i++)
		{
			$x1 = $ox + $i * $this->gx_size;
			$x2 = $x1 + $this->gx_size - 1;
			for ($j=0; $j<$this->gy_count; $j++)
			{
				$y1 = $oy + $j * $this->gy_size;
				$y2 = $y1 + $this->gy_size - 1;

				imagefilledrectangle( $image, $x1, $y1, $x2, $y2, $this->back_color );
				imagerectangle( $image, $x1, $y1, $x2, $y2, $this->frame_color );
			}
		}
		
	}

}

/**
 * klasse zur darstellung eines racks
 * 
 */
class box_rack extends box_rectangle_filled
{
	protected $rz_size	= 0;
	protected $rz_count	= 0;
	
	public function __construct ( $_sx, $_rz_size, $_rz_count, $_frame_color, $_back_color )
	{
		$this->rz_size  = $_rz_size;
		$this->rz_count = $_rz_count;
		
		parent::__construct( $_sx, $this->rz_size * $this->rz_count, $_frame_color, $_back_color );
	}
	
	protected function draw_impl ( $image, $ox, $oy )
	{
		parent::draw_impl( $image, $ox, $oy );

		// rack-raster
		for ($i=5; $i<$this->rz_count; $i=$i+5)
		{
			$y1 = $oy + $this->size_y - (($i-5) * $this->rz_size) - 1;
			$y2 = $oy + $this->size_y - ($i * $this->rz_size);

			imagefilledrectangle( $image, $ox, $y1, $ox + $this->size_x - 1, $y2, $this->back_color );
			imagerectangle( $image, $ox, $y1, $ox + $this->size_x - 1, $y2, $this->frame_color );
		}
	}

}


/**
 * klasse zur darstellung eines devices im rack
 * 
 */
class box_device extends box_group
{
	protected $rz_size	= 0;
	protected $rz_count	= 0;
	
	/**
	 * erzeugt ein device fuer eine rack-darstellung
	 * 
	 * @param	$_device_sx		Breite des Device in Pixel
	 * @param	$_device_he		Hoehe des Device in HE
	 * @param	$_device_pos	Position des Device im Rack
	 * @param	$_rack_he		Hoehe des Rack in HE
	 * @param	$_he_size		Hoehe eines HE in Pixel
	 * @param	$_icon_name		Icon des Device
	 * @param	$_icon_size		Size des Icons
	 * @param	$_text			Beschriftung des Device
	 * @param	$_font			Font die Beschrift
	 * @param	$_text_color	Farbe fuer die Beschriftung
	 * @param	$_frame_color	Farbe fuer den Rahmen
	 * @param	$_back_color	Farbe fuer den Hintergrund	
	 */
	public function __construct ( $_device_sx, $_device_he, $_device_pos, $_rack_he, $_he_size, $_icon_name, $_icon_size, $_text, $_font, $_font_size, $_text_color, $_frame_color=NULL, $_back_color=NULL )
	{
		$_device_sy = $_device_he * $_he_size;
		$_device_pos_y = ($_rack_he - $_device_pos - $_device_he) * $_he_size; 

		// group
		parent::__construct( $_device_sx, $_device_sy, FALSE );
		$this->shift_pos( 0, $_device_pos_y );
		
		// hintergrund
		$back_box = new box_rectangle_filled( $_device_sx, $_device_sy, $_frame_color, $_back_color );
		$this->add( $back_box );

		// ci
		if (strlen($_text)!=0)
		{
			if ($_device_sy < $_icon_size) $_icon_name = "";
			$ci_box = new box_ci( 3, $_icon_name, $_icon_size, $_text, $_font, $_font_size, $_text_color );
			$this->add_static( $ci_box, BOX_ALIGN_LEFT, BOX_ALIGN_CENTER );
		}
	}
	
}
