<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_api.inc.php';
require_once 'core/core_html_form.inc.php';
require_once 'tables/db_cat.inc.php';
require_once 'includes/entity_processor.inc.php';

class cat extends entity_processor
{
	/**
	 * attribute des catalogs aus argumenten laden
	 * 
	 * ueberlaedt die felder des DB-Objektes mit den
	 * werten aus den argumenten, dazu muss allerdings
	 * das argument "cat_form" gesetzt sein 
	 * 
	 * @return	none
	 */	
	public function load_from_args ()
	{
		$_form = http_get_int("cat_form", 0);
		if ($_form == 0) return;

		$_title = http_get_string("cat_title", $this->db_object->get_title() );
		$this->db_object->set_title( $_title );

		$_desc = http_get_string("cat_desc", $this->db_object->get_desc() );
		$this->db_object->set_desc( $_desc );

		$_strict = http_get_int("cat_strict", 0 );
		$_strict = ($_strict == 1);
		$this->db_object->set_strict( $_strict );

		return;
	}

	public function display_form ( $title, $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );
	
		html_table_header( 25, 150, 625 );
	
		html_table_section( $title, get_empty_menu() );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='cat_id' value='%s'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='cat_form' value='1'></td></tr>\n" );
	
		html_oblig_row( '*', get_string('Title_ttl') );
		printf( "<td class='f_value'><input type='text' name='cat_title' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 200, html_entity_decode( $this->db_object->get_title(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '', get_string('Description_ttl') );
		printf( "<td class='f_value'><textarea name='cat_desc' cols='%d' rows='%d'>%s</textarea></td></tr>\n",
			50, 5, html_entity_decode( $this->db_object->get_desc(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '*', get_string('Strict_ttl') );
		print( "<td class='f_value'>" );
		html_form_checkbox( $this->db_object->get_strict(), "cat_strict" );
		print( "</td></tr>\n" );
	
		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
		printf( "</form>\n" );	
		printf( "</div>\n" );

	}

	public function display_question ( $action )
	{
		global $referer_script_name;

		$yes_args = new html_form_args();
		$yes_args->add( "cat_id", $this->db_object->get_id() );
	
		$no_args = new html_form_args();
		
		printf( "<div id='question'>\n" );
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('DeleteCatalog_ttl'), get_empty_menu() );
		html_table_footer();

		html_form_question( 0, 0, 
			sprintf( get_string('WantDeleteCatalog_dsc'), $this->db_object->get_title() ),
			get_string('YES_btn'), $action, $yes_args,
			get_string('NO_btn'), $referer_script_name, $no_args );
		printf( "</div>\n" );
			
		return;
	}

	// action support

	public function delete_pre ()
	{
		// loeschen der catalog entries
		$db = core_db::getInstance();
		$query = "DELETE FROM cat_entry WHERE cate_cat_id='".$this->db_object->get_id()."'";
		$result = $db->query( $query );
	}

	public function valid_for_delete ()
	{
		$db = core_db::getInstance();
		$query = "SELECT COUNT(*) FROM type_attr " .
			"WHERE ta_datatype='".DT_CATALOG."' " .
			"AND ta_datatype_sub='".$this->db_object->get_id()."'";
		$result = $db->query( $query ); 
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if (($count = $db->result( $result, 0 )) != 0)
		{
			throw new core_data_exception( get_string('CatalogUsedby_dsc')." $count ".get_string('CITypeAttributes_dsc'));
		}
		
		return TRUE;
	}

	// helper
	
	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$title = sprintf( get_string('Catalog_ttl')." '%s'", $this->db_object->get_title() );
		return $title;
	}


	protected function create_db_object ( $_id )
	{
		$this->db_object = new db_cat( $_id );
		return;
	}
}
