<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_api.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_html_form.inc.php';
require_once 'tables/db_cat.inc.php';
require_once 'tables/db_cat_entry.inc.php';
require_once 'includes/entity_processor.inc.php';

class cat_entry extends entity_processor
{
	public function get_cat_id ()
	{
		return $this->db_object->get_cat_id();
	}
	
	
	/**
	 * attribute des Catalog-Entries aus argumenten laden
	 * 
	 * ueberlaedt die felder des DB-Objektes mit den
	 * werten aus den argumenten, dazu muss allerdings
	 * das argument "cate_form" gesetzt sein 
	 * 
	 * @return	none
	 */	
	public function load_from_args ()
	{
		// nur aus args ueberladen, wenn noch nicht gespeichert
		if ($this->get_id()==0)
		{
			$_cat_id = http_get_int("cate_cat_id", $this->db_object->get_cat_id() );
			$this->db_object->set_cat_id( $_cat_id );
		}

		if (($_form = http_get_int("cate_form", 0)) == 0) return;

		$_title = http_get_string("cate_title", $this->db_object->get_title() );
		$this->db_object->set_title( $_title );

		$_icon = http_get_string("cate_icon", $this->db_object->get_icon() );
		$this->db_object->set_icon( $_icon );

		$_fg_color = http_get_string("cate_fg_color", $this->db_object->get_fg_color() );
		$this->db_object->set_fg_color( $_fg_color );

		$_bg_color = http_get_string("cate_bg_color", $this->db_object->get_bg_color() );
		$this->db_object->set_bg_color( $_bg_color );

		return;
	}

	// advanced get
	
	/**
	 * replay a string with a short description of this entry
	 */
//	public function get_synopse ()
//	{
//		$cat = new db_cat( $this->db_object->get_cat_id() );
//		$synopse = sprintf( "Catalog '%s' / Entry '%s'", $cat->get_title(), $this->db_object->get_title() );
//		return $synopse;		
//	}


	// display methods

	/**
	 * display synopse of this catalog entry
	 */
//	public function display_synopse ( $show_menu=FALSE )
//	{
//		$menu = & get_empty_menu();
//		if ($show_menu) $menu = & design_cate_view_menu( $this->get_id() );
//		$this->html_synopse_section( $menu );
//		
//		return;
//	}
//

	public function display_form ( $title, $action, $action_text )
	{
		$cat = new db_cat( $this->db_object->get_cat_id() );

		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );

		html_table_header( 25, 150, 625 );
	
		html_table_section( $title, get_empty_menu() );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='cate_id' value='%s'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='cate_cat_id' value='%s'></td></tr>\n",
			$this->db_object->get_cat_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='cate_form' value='1'></td></tr>\n" );
	
		html_oblig_row( '*', get_string('Title_ttl') );
		printf( "<td class='f_value'><input type='text' name='cate_title' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 200, html_entity_decode( $this->db_object->get_title(), ENT_QUOTES, 'ISO-8859-1' ) );

		html_oblig_row( '*', get_string('ForegroundColor_ttl') );
		printf( "<td class='f_value'>" );
		$enum_colors = get_enum_to_array("enum_colors"); 
		cat_array( $enum_colors, $this->db_object->get_fg_color(), -1, "cate_fg_color", "" );
		printf( "</td></tr>\n" );

		html_oblig_row( '*', get_string('BackgroundColor_ttl') );
		printf( "<td class='f_value'>" );
		cat_array( $enum_colors, $this->db_object->get_bg_color(), -1, "cate_bg_color", "" );
		printf( "</td></tr>\n" );

		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );

		html_table_footer();

		printf( "</form>\n" );
		printf( "</div>\n" );
	
	}

	public function display_icon_form ( $action )
	{
		printf( "<div id='form'>\n" );
		html_table_header( 25, 150, 625 );
	
		html_table_section( get_string('ChooseIcon_ttl'), get_empty_menu() );
		html_table_info( 0, 0, get_string('ChooseIcon_dsc'));
		printf( "<tr><td colspan='3'>" );
		icon_list::getInstance()->html_chooser(
			sprintf( "%s?cate_form=2&amp;cate_id=%d&amp;cate_icon=", $action, $this->db_object->get_id() ),
			get_string('NoIcon_dsc'),
			""
			);
		printf( "</td></tr>\n" );
		html_table_footer();
		printf( "</div>\n" );
	}

	public function display_question ( $action )
	{
		global $referer_script_name;

		$yes_args = new html_form_args();
		$yes_args->add( "cate_id", $this->db_object->get_id() );
	
		$no_args = new html_form_args();
		$no_args->add( "cat_id", $this->db_object->get_cat_id() );

		$cat = new db_cat( $this->db_object->get_cat_id() );

		printf( "<div id='question'>\n" );
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('DeleteCatalogEntry_ttl'), get_empty_menu() );
		html_table_footer();

		html_form_question( 0, 0, 
			get_string('WantDeleteEntry_dsc'),
			get_string('YES_btn'), $action, $yes_args,
			get_string('NO_btn'), $referer_script_name, $no_args );
		printf( "</div>\n" );

		return;
	}

	// action support

	public function valid_for_delete ()
	{
		$db = core_db::getInstance();
		$query = "SELECT COUNT(*) FROM attr, type_attr " .
			"WHERE a_val_int=".$this->db_object->get_id()." " .
			"AND a_ta_id=ta_id " .
			"AND ta_datatype=".DT_CATALOG." " .
			"AND ta_datatype_sub=".$this->db_object->get_cat_id()."";
		$result = $db->query( $query ); 
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( get_string('UnexpectedNumberOfRows_dsc') );
		}
		if (($count = $db->result( $result, 0 )) != 0)
		{
			throw new core_data_exception( get_string('CatalogEntryUsed_dsc')." $count ".get_string('CIAttributes_dsc'));
		}
		
		return TRUE;
	}

	// helper

	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$cat = new db_cat( $this->db_object->get_cat_id() );
		$title = sprintf( "%s / %s: '%s' / '%s'",
			get_string('Catalog_ttl'), get_string('Entry_ttl'),
			$cat->get_title(), $this->db_object->get_title()
		);
		return $title;
	}

	/**
	 * get a HTML string including the icon for the
	 * synopse section
	 * 
	 * @return		string		HTML code including an icon 
	 */
	protected function get_synopse_icon_html ()
	{
		$icon_html = icon_list::getInstance()->get_html_imx( $this->db_object->get_icon() );
		return $icon_html;
	}

	/**
	 * get a string with the information for the upper right
	 * corner in the synopse section
	 * 
	 * @return	string	information
	 */
	protected function get_synopse_info ()
	{
		$info = "";
		return $info;
	}

	protected function create_db_object ( $_id )
	{
		$this->db_object = new db_cat_entry( $_id );
		return;
	}

}
