<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_list.inc.php';
require_once 'tables/db_cat.inc.php';
require_once 'tables/db_cat_entry.inc.php';
require_once 'includes/navi_list.inc.php';
require_once 'includes/cat_entry.inc.php';
require_once 'includes/cat_list.inc.php';

/**
 * list of catalog entries
 */
class cat_entry_list extends navi_list
{
	static private $instance = array();
	protected $cat_id = 0;
	protected $strict = FALSE;

	static public function &getInstance( $_cat_id )
	{
		if (! array_key_exists( $_cat_id, self::$instance )) self::$instance[ $_cat_id ] = new cat_entry_list( $_cat_id );
		return self::$instance[ $_cat_id ];
	}
	
	protected function __construct ( $_cat_id )
	{
		// store catalog id
		$this->cat_id = $_cat_id;

		// set strict mode from catalog
		$db = core_db::getInstance();
		$query = "SELECT cat_strict FROM cat WHERE cat_id='$_cat_id'";
		$result = $db->query( $query );
		$this->strict = $db->result( $result, 0 );
		
		parent::__construct();
	}

	protected function get_navi_args ()
	{
		$navi_args = "&amp;cat_id=" . $this->cat_id;
		return $navi_args;
	}

	protected function get_dont_load_keys ()
	{
		$key_array = parent::get_dont_load_keys();
		$key_array[] = "cat_id";
		$key_array[] = "strict";
		return $key_array;
	}

	protected function post_load ()
	{
		$this->page_size = get_property("page_rows");
		parent::post_load();
	}

	protected function get_session_store_name ()
	{
		return sprintf( "cate_list_%d", $this->cat_id );
	} 

	protected function get_element_name ()
	{
		return get_string('Entries_ttl');
	} 

	public function is_strict()
	{
		return $this->strict;
	}
	
	public function html_options ( $ref_id, $excl_id, $arg_name, $extras, $form_name="", $multiple=FALSE )
	{
		if (! is_array($ref_id)) trigger_error( "ref_id ".get_string('IsNotAnArray_dsc'), E_USER_ERROR );
		if (! is_array($excl_id)) trigger_error( "excl_id ".get_string('IsNotAnArray_dsc'), E_USER_ERROR );
		if (! is_array($extras)) trigger_error( "extras ".get_string('IsNotAnArray_dsc'), E_USER_ERROR );

		// strict
		if (! $this->strict) $extras[ -1 ] = ">>> ".get_string('EnterNewOne_dsc')." >>>"; 

		parent::html_options( $ref_id, $excl_id, $arg_name, $extras, $form_name );

		return;
	}

	public function display_form ( $title, $action, $action_text )
	{
		$cat = new db_cat( $this->cat_id );
		
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post' enctype='multipart/form-data'>\n",
			$action );
	
		html_table_header( 25, 150, 625 );
	
		html_table_section( $title, get_empty_menu() );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='cate_cat_id' value='%s'></td></tr>\n",
			$this->cat_id );
		printf( "<tr><td colspan='3'><input type='hidden' name='cat_form' value='2'></td></tr>\n" );
	
		html_oblig_row( '*', $title );
		printf( "<td class='f_value'><input type='file' class='b_action' name='cat_file' size='%d' maxlength='%d' accept='text/*'></td></tr>\n",
			50, 100000 );

		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );

		html_table_footer();
		printf( "</form>\n" );	
		printf( "</div>\n" );
	}

	/**
	 * import entries from uploaded CSV-File
	 * 
	 * a line in csv file must be formated this way:
	 * <title>,<icon-name>,<foreground-color>,<background-color>
	 * 
	 * while only the <title> element is oblique, all others are optional
	 * 
	 * @param		$message		message string, insert info about upload/import processing
	 * @return		int				Return Code
	 */
	public function import_csv ( &$message )
	{
		// get form arg
		$cat_form = http_get_int( "cat_form", 0 );
		if ($cat_form == 0) throw new core_arg_exception( get_string('IllegalImportRequest_dsc') );

		// get file arg
		$cat_file = $_FILES['cat_file']['tmp_name'];
		if (strlen($cat_file)==0)
		{
			$message = get_string('NoFileForUpload_dsc');
			return 1;
		}
		
		// check upload status
		$err_file = $_FILES['cat_file']['error'];
		if ($err_file!=0)
		{
			$message = get_string('ErrorWhileUploading_dsc');
			return $err_file;
		}

		// check for correct upload file
		if (! is_uploaded_file( $cat_file )) throw new core_arg_exception( get_string('IllegalImportRequest_dsc') );

		// read lines from file
		$lines = file ($cat_file);

		// import lines as entries
		$import_count = 0;
		$rc = 0;
		foreach ($lines as $line_num => $line)
		{
			$line_array = explode( '#', trim($line) );
			$line_entry = $line_array[0];
			if (strlen($line_entry)!=0)
			{
				$entry_elements = explode( ',', $line_entry );
				if ((count($entry_elements)>0) && (strlen($entry_elements[0])>0))
				{
					// import as entry
					$_POST['cate_form'] = 2;
					$_POST['cate_cat_id'] = $this->cat_id;
					$_POST['cate_title'] = htmlspecialchars($entry_elements[0]);
					if (isset($entry_elements[1]) && (strlen($entry_elements[1])>0)) $_POST['cate_icon'] = htmlspecialchars($entry_elements[1]); 
					if (isset($entry_elements[2]) && (strlen($entry_elements[2])>0)) $_POST['cate_fg_color'] = (int)$entry_elements[2]; 
					if (isset($entry_elements[3]) && (strlen($entry_elements[3])>0)) $_POST['cate_bg_color'] = (int)$entry_elements[3]; 
					$cate = new cat_entry();
					$cate->load_from_args();
					if (! $cate->action_insert()) $rc = 1;
					$import_count++;
				}
			}
		}
		$message = sprintf( "$import_count ".get_string('EntriesImportedFromCSV_dsc') );

		// delete import file
		unlink( $cat_file );

		return $rc;
	}

	/**
	 * export entries to downloaded CSV-File
	 * 
	 * this method writes the csv lines directly to output device
	 * 
	 * @return	none
	 */
	public function export_csv()
	{
		$db		= core_db::getInstance();
		$query	= $this->get_query();
		$result = $db->query( $query[0] );
		$num    = $db->num_rows( $result );

		// write header as comment lines
		$title = cat_list::get_title( $this->cat_id );
		printf( "# PHP-CMDB Version %s\n", get_property("app_version") );
		printf( "# Date/Time: %s\n", date("r") );
		printf( "#\n" );
		printf( "# CATALOG: %s\n", $title );
		printf( "# ENTRIES: %d\n", $num );
		printf( "#\n" );
		printf( "# FIELDS: <title>,<icon>,<foreground-color>,<background-color>\n" );
		printf( "#\n" );

		// write entries
		for ($i=0; $i<$num; $i++)
		{
			$data = $db->fetch($result);

			printf( "%s,%s,%d,%d\n",
				$data['cate_title'],
				$data['cate_icon'],
				$data['cate_fg_color'],
				$data['cate_bg_color']
			);
		}

		return;
	}

	protected function get_id_name ()
	{
		return "cate_id";
	}

	public function show_page ()
	{
		html_table_header( 730, 70 );
		switch ($this->get_list_mode())
		{
			case LIST_MODE_USE:
			{
?>
<tr class='r_head'>
<?php
		printf( "	<td class='c_text r_head'>".get_string('Title_ttl')."</td>\n" );
		printf( "	<td class='c_action r_head'>".get_string('Actions_ttl')."</td>\n" );
?>
</tr>
<?php
				break;
			}
			case LIST_MODE_MANAGE:
			{
?>
<tr class='r_head'>
<?php
		printf( "	<td class='c_text r_head'>".get_string('Title_ttl')."</td>\n" );
		printf( "	<td class='c_action r_head'>".get_string('Actions_ttl')."</td>\n" );
?>
</tr>
<?php
				break;
			}
		}
		parent::show_page();
	
		html_table_footer();

		return;
	}	

	protected function show_item ( $i )
	{
		$cate = & $this->at( $i );
		html_table_row();

		switch ($this->get_list_mode())
		{
			case LIST_MODE_USE:
			{
				printf( "<td class='c_text'>%s</td>" .
					"<td class='c_action'></td>" .
					"</tr>\n",
					get_html_url(
						sprintf( "%s?cate_id=%d", CATE_CI_LIST_PHP, $cate->get_id()), 
						$cate->get_styled_title()
						)
					);
				break;
			}
			case LIST_MODE_MANAGE:
			{
				printf( "<td class='c_text'>%s</td>" .
					"<td class='c_action'>%s %s</td></tr>\n",
					$cate->get_styled_title(), 
					icon_display_url(CATE_EDIT_PHP . "?cate_id=" . $cate->get_id(), ICON_EDIT, get_string('EditCatalogEntry_dsc') ),
					icon_display_url(CATE_DELETE_PHP . "?cate_id=" . $cate->get_id(), ICON_DELETE, get_string('DeleteCatalogEntry_dsc') )
					);
				break;
			}
		}

	}

	protected function get_query()
	{
		$query = array();
		$query[] = "SELECT * FROM cat_entry " .
			"WHERE cate_cat_id=" . $this->cat_id . " " .
			"ORDER BY cate_title";
		return $query;
	}
	
	protected function create_object ( $_id )
	{
		$db_object = new db_cat_entry( $_id ); 
		return $db_object;
	}
		
	static public function get_title ( $_id )
	{
		$db     = core_db::getInstance();
		$query  = "SELECT cate_title FROM cat_entry WHERE cate_id='$_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)<1) return "";
		return $db->result( $result, 0 );
	}

	static public function get_id_by_title ( $_cat_id, $_title )
	{
		$db     = core_db::getInstance();
		$query  = "SELECT cate_id FROM cat_entry " .
			"WHERE cate_cat_id='$_cat_id' " .
			"AND cate_title='$_title'";
		$result = $db->query( $query );
		if ($db->num_rows($result)<1) return 0;
		return $db->result( $result, 0 );
	}

}
