<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'tables/db_cat.inc.php';
require_once 'includes/navi_list.inc.php';

class cat_list extends navi_list
{
	static private $instance = false;
	
	static public function &getInstance()
	{
		if (self::$instance === false) self::$instance = new cat_list();
		return self::$instance;
	}

	protected function __construct ()
	{
		$this->page_size = get_property("page_rows");
		parent::__construct();
	}

	protected function get_session_store_name ()
	{
		return "cat_list";
	} 

	protected function get_element_name ()
	{
		return get_string('Catalog_ttl');
	} 
	
	protected function get_id_name ()
	{
		return "cat_id";
	}


	public function show_page ()
	{
		html_table_header( 25, 225, 500, 50 );
		switch ($this->get_list_mode())
		{
			case LIST_MODE_USE:
			{
?>
<tr class='r_head'>
	<td class='c_action r_head'>&nbsp;</td>
<?php
		printf( "	<td class='c_text r_head'>".get_string('Catalog_ttl')."</td>\n" );
		printf( "	<td class='c_text r_head'>".get_string('Description_ttl')."</td>\n" );
		printf( "	<td class='c_action r_head'>".get_string('Actions_ttl')."</td>\n" );
?>
</tr>
<?php
				break;
			}
			case LIST_MODE_MANAGE:
			{
?>
<tr class='r_head'>
	<td class='c_action r_head' title='Strict'>S</td>
<?php
		printf( "	<td class='c_text r_head'>".get_string('Catalog_ttl')."</td>\n" );
		printf( "	<td class='c_text r_head'>".get_string('Description_ttl')."</td>\n" );
		printf( "	<td class='c_action r_head'>".get_string('Actions_ttl')."</td>\n" );
?>
</tr>
<?php
				break;
			}
		}
		
		parent::show_page();
		html_table_footer();

		return;
	}	

	protected function show_item ( $i )
	{
		$cat = & $this->at( $i );

		html_table_row();

		switch ($this->get_list_mode())
		{
			case LIST_MODE_USE:
			{		
				printf( "<td class='c_action'></td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_action'></td>" .
					"</tr>\n",
					get_html_url(
						sprintf( "%s?cat_id=%d", CATE_LIST_PHP, $cat->get_id() ),
						$cat->get_title()
						),
					$cat->get_desc()
					);
				break;
			}
			case LIST_MODE_MANAGE:
			{		
				printf( "<td class='c_action'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_action'>%s %s</td>" .
					"</tr>\n",
					html_status_checkbox( $cat->get_strict() ),
					get_html_url(
						sprintf("%s?cat_id=%d", CATE_INDEX_PHP, $cat->get_id() ),
						$cat->get_title(),
						FALSE,
						get_string('DisplayAndManageCatalogEntries_dsc') ),
					$cat->get_desc(),
					icon_display_url(CAT_EDIT_PHP . "?cat_id=" . $cat->get_id(), ICON_EDIT, get_string('EditCatalog_dsc') ),
					icon_display_url(CAT_DELETE_PHP . "?cat_id=" . $cat->get_id(), ICON_DELETE, get_string('DeleteCatalog_dsc') )
					);
				break;
			}
		}
	}

	protected function get_query ()
	{
		$query = array();
		$query[] = "SELECT * FROM cat ORDER BY cat_title"; 
		return $query;
	}
	
	protected function create_object ( $_id )
	{
		$db_object = new db_cat( $_id ); 
		return $db_object;
	}
	
	static public function get_title ( $_id )
	{
		$db     = core_db::getInstance();
		$query  = "SELECT cat_title FROM cat WHERE cat_id='$_id'";
		$result = $db->query( $query );
		
		return $db->result( $result, 0 );
	}

	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$title = get_string('AllCatalogs_ttl');
		return $title;
	}

}
