<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_api.inc.php';
require_once 'core/core_html_form.inc.php';
require_once 'core/core_html_icon.inc.php';
require_once 'tables/db_ci.inc.php';
require_once 'tables/db_user.inc.php';
require_once 'includes/ci_type_list.inc.php';
require_once 'includes/attr.inc.php';
require_once 'includes/relation_peer_list.inc.php';
require_once 'includes/relation_peer_imx.inc.php';
require_once 'includes/comment_list.inc.php';
require_once 'includes/icon_list.inc.php';
require_once 'includes/user_list.inc.php';
require_once 'includes/item_processor.inc.php';
require_once 'includes/module_base.inc.php';

require_once 'includes/graphviz_image.inc.php';
require_once 'includes/graphviz_pdf.inc.php';
require_once 'includes/graphviz_tracker.inc.php';

class ci extends item_processor
{
	protected $ci_type		= null;
	protected $ext_form		= 0;

	public function __construct ( $_id=0, $_cit_id=0, $_arg_name="", $_cit_arg_name="" )
	{
		parent::__construct( $_id, $_arg_name );
		if ($this->get_item_type_id() == 0)
		{
			if (($_cit_id==0) && (strlen($_cit_arg_name)>0))
			{
				$_cit_id = http_get_int( $_cit_arg_name, 0 );
			}
			$this->db_object->set_cit_id( $_cit_id );
		}
		$this->load_ci_type();
		$this->create_attr_list();
	}

	/**
	 * ueberlaedt die felder des DB-Objektes mit den
	 * werten aus den argumenten, dazu muss allerdings
	 * das argument "ci_form" gesetzt sein 
	 * 
	 * @return	none
	 */	
	public function load_from_args ()
	{
		$_form = http_get_int("map_form", 0);
		if ($_form == 1)
		{
			$map_class = http_get_string('ci_map_class', $this->get_map_class() );
			$this->set_map_class( $map_class );
		}

		$_form = http_get_int("attr_visible_form", 0);
		if ($_form == 1)
		{
			$attr_visible = http_get_string('ci_attr_visible', $this->get_attr_visible() );
			$this->set_attr_visible( $attr_visible );
		}

		$_form = http_get_int("rel_visible_form", 0);
		if ($_form == 1)
		{
			$rel_visible = http_get_string('ci_rel_visible', $this->get_rel_visible() );
			$this->set_rel_visible( $rel_visible );
		}

		$this->ext_form = http_get_int("ext_form", $this->ext_form);
		if ($this->ext_form != 0)
		{
			$ext_depth = http_get_string('ext_rd', $this->get_ext_depth() );
			$this->set_ext_depth( $ext_depth );

			$ext_size = http_get_int('ext_size', 0 );
			$this->set_ext_size( $ext_size );

			$ext_cit = http_get_array('ext_cit');
			if ($ext_cit == null ) $ext_cit = array( 0 );
			$this->set_ext_cit( $ext_cit );

			$ext_restyle = http_get_int('ext_restyle');
			$this->set_ext_restyle( $ext_restyle );
		}

		$_form = http_get_int("ci_form", 0);
		switch ($_form)
		{
			case 1:
			case 2:
			{
				parent::load_from_args ();
				$_clone_id = http_get_string( "ci_clone_id", $this->get_clone_id() );
				$this->set_clone_id( $_clone_id );
				$_icon = http_get_string( "ci_icon", $this->db_object->get_icon() );
				$this->db_object->set_icon( $_icon );
				break;
			}
			case 3:
			{
				$_icon = http_get_string( "ci_icon", $this->db_object->get_icon() );
				$this->db_object->set_icon( $_icon );
				break;
			}
		}

		return;
	}

	// get

	/**
	 * liefert den classtype code
	 */
	protected function get_classtype ()
	{
		return CLASS_CI;
	}

	/**
	 * liefert die ID des item_types
	 */
	public function get_item_type_id ()
	{
		return $this->db_object->get_cit_id();
	}

	/**
	 * liefert den title
	 */
	public function get_title ()
	{
		$item_title = parent::get_title();
		if (is_object($this->db_object)) $item_title = $this->db_object->get_title();
		return $item_title; 
	}

	/**
	 * liefert den title des item_types
	 */
	public function get_item_type_title ()
	{
		$item_type_title = parent::get_item_type_title();
		if (is_object($this->ci_type)) $item_type_title = $this->ci_type->get_title();
		return $item_type_title; 
	}

	public function get_icon ()
	{
		// eigenes
		$_icon = $this->db_object->get_icon();
		
		// falls leer, dann vom ci-type
		if (strlen($_icon)==0)
		{
			if (is_object($this->ci_type)) $_icon = $this->ci_type->get_icon();
		}
		
		return $_icon; 
	}

	public function get_last_user ()
	{
		$_user_name = "UNKNOWN";
		$_user  = user_list::getInstance()->get( $this->db_object->get_last_uid() );
		if ( is_object( $_user ) ) $_user_name = $_user->get_name();
		return $_user_name;
	}
	
	/**
	 * get attr_visible from properties 
	 */
	public function get_attr_visible()
	{
		$attr_visible = get_user_property( "ci_attr_visible", 0, 0, 10 );
		return $attr_visible;
	}

	/**
	 * get rel_visible from properties 
	 */
	public function get_rel_visible()
	{
		$rel_visible = get_user_property( "ci_rel_visible", 0, 0, REL_VISIBLE_YES );
		return $rel_visible;
	}

	/**
	 * get map_class from properties 
	 */
	public function get_map_class()
	{
		$map_class = get_user_property( "ci_map_class", $this->get_item_type_id(), $this->get_id(), CI_MAP_CLASS_DEFAULT );

		$class_array = module_list::get_display_class( $this );
		if (! array_key_exists( $map_class, $class_array ))
		{
			// fallback to default, if $value is not supported
			$map_class = CI_MAP_CLASS_DEFAULT;
			delete_user_property( "ci_map_class", $this->get_item_type_id(), $this->get_id() );
		}

		return $map_class;
	}

	/**
	 * get ext_depth from properties 
	 */
	public function get_ext_depth()
	{
		$ext_depth = get_user_property( "ci_ext_depth", 0, 0, 1 );
		return $ext_depth;
	}

	/**
	 * get ext_size from properties 
	 */
	public function get_ext_size()
	{
		$ext_size = get_user_property( "ci_ext_size", 0, 0, 1 );
		return $ext_size;
	}

	/**
	 * get ext_restyle from properties 
	 */
	public function get_ext_restyle()
	{
		$ext_restyle = get_user_property( "graphviz_restyle", 0, 0, 0 );
		return ($ext_restyle == 1);
	}

	/**
	 * get ext_cit from properties 
	 */
	public function get_ext_cit()
	{
		if (! isset($_SESSION['ci_ext_cit'])) $_SESSION['ci_ext_cit'] = array();	
		return $_SESSION['ci_ext_cit'];
	}

	// set

	/**
	 * setzt den titel des objektes
	 */
	protected function set_title ( $_title )
	{
		$this->db_object->set_title( $_title );
	}
	
	/**
	 * set attr_visible
	 */
	protected function set_attr_visible( $attr_visible )
	{
		set_user_property( "ci_attr_visible", $attr_visible, 0, 0 );
		return;
	}

	/**
	 * set rel_visible
	 */
	protected function set_rel_visible( $rel_visible )
	{
		set_user_property( "ci_rel_visible", $rel_visible, 0, 0 );
		return;
	}

	/**
	 * set a new map_class for this ci
	 */
	protected function set_map_class( $map_class )
	{
		$class_array = module_list::get_display_class( $this );
		if (! array_key_exists( $map_class, $class_array )) return;

		// default value, delete in property
		if ($map_class == CI_MAP_CLASS_DEFAULT)
		{
			delete_user_property( "ci_map_class", $this->get_item_type_id(), $this->get_id() );
			return;
		}

		$current_map_class = get_user_property( "ci_map_class", $this->get_item_type_id(), $this->get_id(), CI_MAP_CLASS_DEFAULT );
		if ($current_map_class != $map_class)
		{
			set_user_property( "ci_map_class", $map_class, $this->get_item_type_id(), $this->get_id() );
		}
		return;
	}
	
	/**
	 * set a new ext_depth
	 */
	protected function set_ext_depth( $ext_depth )
	{
		set_user_property( "ci_ext_depth", $ext_depth, 0, 0 );
		return;
	}

	/**
	 * set ext_size
	 */
	protected function set_ext_size( $ext_size )
	{
		set_user_property( "ci_ext_size", $ext_size, 0, 0 );
		return;
	}

	/**
	 * set ext_restyle
	 */
	protected function set_ext_restyle( $ext_restyle )
	{
		set_user_property( "graphviz_restyle", $ext_restyle, 0, 0 );
		return;
	}

	/**
	 * set ext_cit
	 */
	protected function set_ext_cit( $cit_array )
	{
		$_SESSION['ci_ext_cit'] = $cit_array;
		return;
	}

	// display

	public function display_form ( $title, $action1, $action1_text, $action2, $action2_text )
	{
		if ($this->get_item_type_id() == 0)
		{
			$this->display_form1( get_string('SelectCIType_ttl'), $action1, $action1_text );
		}
		else
		{
			$this->display_form2( $title, $action2, $action2_text );
		}
		return;
	}

	/**
	 * formular zur auswahl des CI-Types
	 */
	protected function display_form1 ( $title, $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post' name='ci_xxx'>\n",
			$action );

		html_table_header( 25, 150, 625 );
	
		html_table_section( get_string('CI_ttl') . ": $title", get_empty_menu() );
	
		printf( "<tr><td colspan='3'>" );
		printf( "<input type='hidden' name='ci_id' value='%d'>",
			$this->db_object->get_id() );
		printf( "<input type='hidden' name='ci_clone_id' value='%d'>",
			 $this->get_clone_id());
		printf( "<input type='hidden' name='ci_icon' value='%s'>",
			$this->db_object->get_icon() );
		printf( "<input type='hidden' name='ci_form' value='1'>" );
		printf( "</td></tr>\n" );
		
		html_oblig_row( '*', get_string('CIType_ttl') );
		printf( "<td class='f_value'>" );
		ci_type_list::getInstance()->html_options(
			array($this->db_object->get_cit_id()),
			array(),
			'ci_cit_id',
			array( 0 => "--- ".get_string('SelectPlease')." ---" ),
			"ci_xxx"
		);
		printf( "</td></tr>\n" );
		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();

		printf( "</form>\n" );
		printf( "</div>\n" );
	}

	/**
	 * form to enter attribute values
	 * 
	 * @param	string		$title			title of form
	 * @param	string		$action			destination URL
	 * @param	string		$action_text	submit button label
	 */
	protected function display_form2 ( $title, $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );

		html_table_header( 25, 150, 625 );
	
		$menu = & get_empty_menu();
		if (($ci_id = $this->db_object->get_id())!=0)
		{
			$menu = & browse_ci_form_menu( $ci_id );
		}
		else
		{
			if ($this->get_clone_id()!=0) $menu = & browse_ci_form_menu( $this->get_clone_id() );
		} 
	
		html_table_section( $title, $menu );
	
		printf( "<tr><td colspan='3'>" );
		printf( "<input type='hidden' name='ci_id' value='%d'>",
			$this->db_object->get_id() );
		printf( "<input type='hidden' name='ci_cit_id' value='%d'>",
			$this->db_object->get_cit_id() );
		printf( "<input type='hidden' name='ci_clone_id' value='%d'>",
			 $this->get_clone_id());
		printf( "<input type='hidden' name='ci_icon' value='%s'>",
			$this->db_object->get_icon() );
		printf( "<input type='hidden' name='ci_form' value='2'>" );
		printf( "</td></tr>\n" );

		// all my attributes
		$this->attr_display_form();

		// clone options
		if ($this->get_clone_id() != 0)
		{
			html_table_section( get_string('Options_ttl'), get_empty_menu() );

			html_oblig_row( '*', get_string('CloneRelations_ttl') );
			printf( "<td class='f_value'>" );
			html_form_checkbox( TRUE, "ci_clone_relations" );
			printf( "</td></tr>\n" );
		}

		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
	
		printf( "</form>\n" );
		printf( "</div>\n" );
	}

	public function display_icon_form ( $action )
	{
		printf( "<div id='form'>\n" );
		html_table_header( 25, 150, 625 );
	
		$menu = & get_empty_menu();
		if (($ci_id = $this->db_object->get_id())!=0) $menu = & browse_ci_form_menu( $ci_id ); 
	
		html_table_section( get_string('ChooseIcon_ttl'), $menu );
		html_table_info( 0, 0, get_string('ClickOnIconToSelect_dsc'));
		printf( "<tr><td colspan='3'>" );
		icon_list::getInstance()->html_chooser(
			sprintf( "%s?ci_form=3&amp;ci_id=%d&amp;ci_icon=", $action, $this->db_object->get_id() ),
			get_string('UseIconFromCIType_dsc')
			);
		printf( "</td></tr>\n" );
	
		html_table_footer();	
		printf( "</div>\n" );
	}

	/**
	 * loesch-abfrage
	 */
	public function display_question ( $action )
	{
		global $referer_script_name;

		$yes_args = new html_form_args();
		$yes_args->add( "ci_id", $this->db_object->get_id() );
	
		$no_args = new html_form_args();
		$no_args->add( "ci_id", $this->db_object->get_id() );
		
		printf( "<div id='question'>\n" );
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('DeleteCI_ttl'), browse_ci_form_menu( $this->get_id() ) );
		html_table_footer();

		html_form_question( 0, 0, 
			get_string('WantDeleteCI_dsc'),
			get_string('YES_btn'), $action, $yes_args,
			get_string('NO_btn'), $referer_script_name, $no_args );
		printf( "</div>\n" );
			
		return;
	}

	// display methoden

	/**
	 * display the attributes of the CI
	 * 
	 * @return	none
	 */
	public function display_attributes()
	{
		printf( "<div id='attributes'>\n" );
		$this->html_attr_section( get_string('Attributes_ttl'), browse_ci_attr_menu( $this->get_id() ) );

		// alle meine attribute
		html_table_header( 25, 150, 625 );
		$this->attr_display( $this->get_attr_visible() );

		// change info
		printf( "<tr><td class='changeinfo' colspan='3'>".get_string('LastChangeAt_dsc')." %s ".get_string('By_dsc')." %s</td></tr>\n",
			$this->db_object->get_last_change(), $this->get_last_user() );

		html_table_footer();
		printf( "</div>\n" );

		return;
	}

	/**
	 * display filter form for extended map
	 */
	public function display_extended_form ()
	{
		// setup size array
		$map_sx = get_property("map_sx");
		$map_sy = get_property("map_sy");

		$ext_sx = get_property("ext_sx");
		$ext_sy = get_property("ext_sy");
		
		$size_array = array(
			0 => "Auto",
			1 => sprintf( "%d x %d", $map_sx, $map_sy ),
			2 => sprintf( "%d x %d", $ext_sx, $ext_sy )
		);


		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			CI_VIEW_EXT_PHP );

		html_table_header( 25, 150, 225, 25, 150, 225 );

		html_table_section( "Filter", browse_ci_filter_menu( $this->get_id() ) );
	
		printf( "<tr><td colspan='6'><input type='hidden' name='ci_id' value='%s'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='6'><input type='hidden' name='ext_form' value='1'></td></tr>\n" );

		html_oblig_row( '*', 'Relation-Depth' );
		printf( "<td class='f_value'>" );
		$enum_relation_depth = get_enum_to_array("enum_relation_depth"); 
		cat_array( $enum_relation_depth, $this->get_ext_depth(), -2, "ext_rd", "All", "", -1 );
		printf( "</td>" );
		
		printf( "<td></td>" );
		print( "<td class='f_flag'>Size: " );
		cat_array( $size_array, $this->get_ext_size(), -1, "ext_size" );
		print( "</td>" );
		print( "<td class='f_flag'>Rescale Style: " );
		html_form_checkbox( $this->get_ext_restyle(), "ext_restyle" );
		print( "</td></tr>\n" );

		$cit_list = ci_type_list::getInstance();
		$cit_array = (array)$this->get_ext_cit();
		html_oblig_row( '*', 'CI-Types' );
		printf( "<td class='f_value' colspan='4'>" );
		$cit_list->html_checkbox( $cit_array, -1, "ext_cit" );
		printf( "</td></tr>\n" );
		printf( "<tr><td colspan='2'></td>");
?>
<td colspan='4'>
<input type='button' class='b_search' value='All'
onclick='for (var i = 5; i < (this.form.elements.length - 3); i++) {this.form.elements[i].checked = true;}'>
<input type='button' class='b_search' value='Clear '
onclick='for (var i = 5; i < (this.form.elements.length - 3); i++) {this.form.elements[i].checked = false;}'>
</td></tr>
<?php

		html_table_ruler();
	
		printf( "<tr><td colspan='6'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			get_string('Display_dsc') );

		html_table_footer();

		printf( "</form>\n" );
		printf( "</div>\n" );

		return;
	}

	public function display_relations_map ()
	{
		// section title
		printf( "<div id='map'>\n" );
		$this->html_map_section( get_string('RelationMap_ttl'), browse_ci_map_menu( $this->get_id() ) );

		// class name
		$imx = create_map_class( $this->get_id() );
		if ($imx !== FALSE)
		{
			printf( "<img src='%s?ci_id=%d' border='0' alt='%s' usemap='#map_locations'>", CI_VIEW_IMX_PHP, $this->get_id(), get_string('MapOfRelations_dsc') );
			$imx->get_map();
		}
		printf( "</div>\n" );

		return;
	}

	public function display_extended_map ()
	{
		if ($this->ext_form == 2)
		{
			// first call
			printf( "<div id='map'>\n" );
			html_table_header( 600, 200 );
			html_table_info( 0, 0, get_string('ChooseYourFilterSetup_dsc') );
			html_table_footer();
			printf( "</div>\n" );
			return;
		}
		
		switch ($this->get_ext_size())
		{
			case 1:
			{
				$sx = get_property("map_sx");
				$sy = get_property("map_sy");
				break;
			}
			case 2:
			{
				$sx = get_property("ext_sx");
				$sy = get_property("ext_sy");
				break;
			}
			default:
			{
				$sx = 2500;
				$sy = 2500;
			}
		}
		
		// create image
		$g_image = new graphviz_image( $sx, $sy,
			get_property("ext_font"), get_property("ext_font_size"),
			get_property("ext_padding"), get_property("ext_arrow"),
			get_property("ext_icon_size")
			);

		$g_tracker = new graphviz_tracker( $this->get_id(), $this->get_ext_depth(), $g_image );

		// load cit-filter-array only, if not all ci-types are in
		$cit_array = (array)$this->get_ext_cit();
		if (ci_type_list::getInstance()->count() != count($cit_array))
		{
			$g_tracker->add_ci_type_filter( $cit_array );
		}
		
		// run tracker and put all found CIs and relations into the image
		$g_tracker->run();
		
		// image processing
		$g_image->process( $this->get_id() );
		
		// display

		printf( "<div id='map'>\n" );
		html_table_header( 600, 200 );
		html_table_section(
			sprintf( get_string('ExtentedRelationMap_ttl')." (".get_string('CIs_ttl').": %d / ".get_string('Relations_ttl').".: %d)", $g_image->get_ci_count(), $g_image->get_relation_count()),
			browse_ci_ext_menu( $this->get_id() )
			);
		html_table_footer();
		printf( "<img src='%s?ext=1' border='0' usemap='#map_locations' alt='relations to other CIs'>",
			CI_VIEW_IMX_PHP );
		$g_image->get_map();
		printf( "</div>\n" );

		return;
	}

	public function display_extended_pdf ()
	{
		$sx			= 33.11;	// DIN A0
		$sy			= 46.81;
		$text_size	= 24;		// pixel
		$padding	= 0.5;		// inch
		$arrow		= 12;		// pixel
		$icon_size	= 24;		// pixel

		// create pdf
		$g_image = new graphviz_pdf( 
			get_property("pdf_sx"), get_property("pdf_sy"),
			get_property("pdf_font"), get_property("pdf_font_size"), 
			get_property("pdf_padding"), get_property("pdf_arrow"),
			get_property("pdf_icon_size")
			);

		$g_tracker = new graphviz_tracker( $this->get_id(), $this->get_ext_depth(), $g_image );

		// load cit-filter-array only, if not all ci-types are in
		$cit_array = (array)$this->get_ext_cit();
		if (ci_type_list::getInstance()->count() != count($cit_array))
		{
			$g_tracker->add_ci_type_filter( $cit_array );
		}

		// run tracker and put all found CIs and relations into the pdf
		$g_tracker->run();
		
		// pdf processing
		$g_image->process( $this->get_id() );

		// display pdf
		$g_image->display();

		return;
	}

	public function display_relations ()
	{
		printf( "<div id='relations'>\n" );
		$this->display_relation_list();
		$this->display_relation_counter();
		printf( "</div>\n" );
		
		return;
	}

	/**
	 * zeigt die beziehungen des CI an (tabellarisch)
	 */
	public function display_relation_list ()
	{
		$show_attr = http_get_int( "show_attr", 0 );
		$liste = relation_peer_list::getInstance( $this->get_id() );
		$count = $liste->count();
		
		$title = sprintf( get_string('Relations_ttl')." (".get_string('Total_ttl').": %d)", $count );
//		html_table_section( $title, browse_ci_relation_menu( $this->get_id(), $show_attr ) );
		$this->html_rel_section( $title, browse_ci_relation_menu( $this->get_id() ));
		
		if (($rel_visible = $this->get_rel_visible()) != REL_VISIBLE_NO)
		{
			// tabellen kopf
			html_table_header( 25, 150, 250, 325, 50 );
?>
<tr class='r_head'>
	<td class='c_action r_head'>&nbsp;</td>
<?php
printf( "	<td class='c_text r_head'>%s</td>\n", get_string('Relation_ttl') );
printf( "	<td class='c_text r_head'>%s / %s</td>\n", get_string('CI_ttl'), get_string('CIType_ttl') );
printf( "	<td class='c_text r_head'>%s</td>\n", get_string('Description_ttl') );
printf( "	<td class='c_action r_head'>%s</td>\n", get_string('Actions_ttl') );
?>
</tr>
<?php
	
			html_table_row_reset( array("r_normal","r_normal2") );
			$icon_size  = get_property("list_icon_size");
			if ($count == 0)
			{
				html_empty_row( 0, 1 );
			}
			else
			{
				// all my relations
				for ($i=0; $i<$count; $i++ )
				{
					$relation_peer = & $liste->at( $i );
					$relation_desc = htmlentities( $relation_peer->get_desc() );
					$relation_desc = str_replace( "\n", "<br>", $relation_desc );
					
					html_table_row();
					printf( "<td class='c_action'></td>" .
						"<td class='c_text'>%s</td>" .
						"<td class='c_text'>%s</td>" .
						"<td class='c_text'>%s</td>" .
						"<td class='c_action'>%s %s</td>".
						"</tr>\n",
						$relation_peer->get_peer_text(),
						get_html_url(
							sprintf("%s?ci_id=%d", CI_VIEW_PHP, $relation_peer->get_ci_id()),
							icon_list::getInstance()->get_html_imx( $relation_peer->get_icon(), $icon_size, $icon_size, TRUE ) . " " . $relation_peer->get_ci_title() . " / " . $relation_peer->get_cit_title(),
							FALSE, "Display CI"
						),
						$relation_desc,
						icon_display_url(R_EDIT_PHP . "?r_form=3&amp;r_id=" . $relation_peer->get_id() . "&amp;r_ci_id=" . $this->db_object->get_id(), ICON_EDIT, get_string('EditRelation_dsc') ),
						icon_display_url(R_DELETE_PHP . "?r_id=" . $relation_peer->get_id() . "&amp;r_ci_id=" . $this->db_object->get_id(), ICON_DELETE, get_string('DeleteRelation_dsc') )
						);
	
					// show attributes
					if ($rel_visible == REL_VISIBLE_WITH_ATTR)
					{
						$r = new relation( $relation_peer->get_id(), 0, $this->get_id() );
						$r->attr_list( 5, 3, 1 );
					}
				}
			}
	
			html_table_footer();

		}
		return;
	}

	/**
	 * zeigt die anzahl der beziehungstypen des CI an (tabellarisch)
	 */
	public function display_relation_counter ()
	{
		if (($rel_visible = $this->get_rel_visible()) != REL_VISIBLE_NO)
		{
			$db = core_db::getInstance();
	
			// 
			$query = "SELECT DISTINCT(rt_id), rt_to2_text AS text, cit_title, cit_icon, COUNT(*) AS count " .
				"FROM relation, relation_type, ci_type " .
				"WHERE r_ci1_id='" . $this->db_object->get_id() . "' " .
				"AND r_rt_id=rt_id " .
				"AND rt_cit2_id=cit_id " .
				"GROUP BY rt_id " .
				"ORDER BY rt_id";
			$result1 = $db->query( $query );
			$count1 = $db->num_rows( $result1 );
			
			$query = "SELECT DISTINCT(rt_id), rt_to1_text AS text, cit_title, cit_icon, COUNT(*) AS count " .
				"FROM relation, relation_type, ci_type " .
				"WHERE r_ci2_id='" . $this->db_object->get_id() . "' " .
				"AND r_rt_id=rt_id " .
				"AND rt_cit1_id=cit_id " .
				"GROUP BY rt_id " .
				"ORDER BY rt_id";
			$result2 = $db->query( $query );
			$count2 = $db->num_rows( $result2 );
			
			html_table_header( 25, 150, 250, 325, 50 );
		
			html_table_section( get_string('RelationCounter_ttl'), get_empty_menu() );
			
			// tabellen kopf
?>
<tr class='r_head'>
	<td class='c_action r_head'>&nbsp;</td>
<?php
printf( "	<td class='c_text r_head'>%s</td>\n", get_string('Relation_ttl') );
printf( "	<td class='c_text r_head'>%s</td>\n", get_string('CIType_ttl') );
printf( "	<td class='c_text r_head'>%s</td>\n", get_string('Counter_ttl') );
?>
	<td class='c_action r_head'>&nbsp;</td>
</tr>
<?php
		
			html_table_row_reset( array("r_normal","r_normal2") );
			$icon_size  = get_property("list_icon_size");
			if (($count1 + $count2) == 0)
			{
				html_empty_row( 0, 1 );
			}
			else
			{
				for ($i=0; $i<$count1; $i++ )
				{
					$data = $db->fetch_object( $result1 );
					if (is_object($data))
					{
						html_table_row();
						printf( "<td class='c_action'></td>" .
							"<td class='c_text'>%s</td>" .
							"<td class='c_text'>%s</td>" .
							"<td class='c_text'>%d</td>" .
							"<td class='c_action'></td>".
							"</tr>\n",
							$data->text,
							icon_list::getInstance()->get_html_imx( $data->cit_icon, $icon_size, $icon_size, TRUE ) . " " . $data->cit_title,
							$data->count
							);
					}
				}
				for ($i=0; $i<$count2; $i++ )
				{
					$data = $db->fetch_object( $result2 );
					if (is_object($data))
					{
						html_table_row();
						printf( "<td class='c_action'></td>" .
							"<td class='c_text'>%s</td>" .
							"<td class='c_text'>%s</td>" .
							"<td class='c_text'>%d</td>" .
							"<td class='c_action'></td>".
							"</tr>\n",
							$data->text,
							icon_list::getInstance()->get_html_imx( $data->cit_icon, $icon_size, $icon_size, TRUE ) . " " . $data->cit_title,
							$data->count
							);
					}
				}
			}
	
			html_table_footer();
		}
		return;
	}

	/**
	 * zeigt die kommentare zum CI an
	 */
	public function display_comments ()
	{
		$liste = comment_list::getInstance( $this->get_id() );
		$count = $liste->count();

		printf( "<div id='comments'>\n" );
		html_table_header( 25, 725, 50 );
		$title = sprintf( get_string('Comments_ttl')." (".get_string('Total_ttl').": %d)", $count );
		html_table_section( $title, browse_ci_comment_menu( $this->get_id() ) );
		html_table_footer();
				
		$liste->show_page();
		printf( "</div>\n" );

		return;
	}

	// action methoden

	public function action_update_icon ()
	{
		parent::action_update();

		return ($this->get_status() == STATUS_OK);
	}

	// action support

	public function delete_pre ()
	{
		$db = core_db::getInstance();

		// delete all relations
		$query = "SELECT r_id FROM relation " .
			"WHERE r_ci1_id='" . $this->db_object->get_id() . "' " .
			"OR r_ci2_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query );
		$num = $db->num_rows( $result );
		for ($i=0; $i<$num; $i++)
		{
			$r_id = $db->result( $result, $i );
			$r = new relation( $r_id, 0, $this->db_object->get_id() );
			$r->action_delete();
		}

		// delete kommentare
		$query = "DELETE FROM comment " .
			"WHERE c_ci_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query );

	}

	// helper

	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$title = sprintf( get_string('CI_ttl')." / ".get_string('CIType_ttl').": %s / %s", $this->get_title(), $this->get_item_type_title() );
		return $title;
	}

	/**
	 * get a HTML string including the icon for the
	 * synopse section
	 * 
	 * @return		string		HTML code including an icon 
	 */
	protected function get_synopse_icon_html ()
	{
		$icon_html = icon_list::getInstance()->get_html_imx( $this->get_icon() );
		return $icon_html;
	}

	/**
	 * get a string with the information for the upper right
	 * corner in the synopse section
	 * 
	 * @return	string	information
	 */
	protected function get_synopse_info ()
	{
		$relation_count = relation_peer_list::getInstance( $this->get_id() )->count();
		$comment_count = comment_list::getInstance( $this->get_id() )->count();

		$info = sprintf( "%s %d | %s %d", 
			icon_display( ICON_EQUATION, get_string('Relations_ttl') ), $relation_count,
			icon_display( ICON_COMMENT, get_string('Comments_ttl') ), $comment_count
			);

		return $info;
	}



	protected function create_db_object ( $_id )
	{
		$this->db_object = new db_ci( $_id );
		return;
	}
	
	private function load_ci_type ()
	{
		if (($cit_id = $this->get_item_type_id()) != 0) $this->ci_type = new db_ci_type( $cit_id );
	}

	private function html_attr_section ( $text, &$menu )
	{
		html_table_header( 600, 200 );
		printf( "<tr>" .
			"<td class='section'>%s</td>" .
			"<td class='session'><form action='%s' name='attr_visible' method='post'><input type='hidden' name='ci_id' value='%s'>".get_string('Visibility_ttl').": ",
			$text,
			CI_VIEW_PHP,
			$this->db_object->get_id()
			);

		$enum_attr_visible = get_enum_to_array("enum_attr_visible"); 
		cat_array( $enum_attr_visible, $this->get_attr_visible(), -1, "ci_attr_visible", get_string('none'), "attr_visible" );
		printf( "<input type='hidden' name='attr_visible_form' value='1'>" );
		printf("</form></td></tr>\n" );

		printf( "  <tr><td colspan='2' class='menu'>" );
		if ( $menu->get_count() > 0)
		{
			printf( "&gt;&gt; %s",
				$menu->get_menu_string() );
		}
		printf( "</td></tr>\n" );
		html_table_footer();
	}

	private function html_map_section ( $text, &$menu )
	{
		$map_class = $this->get_map_class();
		$class_array = module_list::get_display_class( $this );

		html_table_header( 600, 200 );
		printf( "<tr>" .
			"<td class='section'>%s</td>" .
			"<td class='session'><form action='%s' name='map_class' method='post'><input type='hidden' name='ci_id' value='%s'> ",
			$text,
			CI_VIEW_PHP,
			$this->db_object->get_id()
			);
		if (count($class_array)>1)
		{
			cat_array( $class_array, $map_class, -1, "ci_map_class", "", "map_class" );
			printf( "<input type='hidden' name='map_form' value='1'>" );
		}
		printf("</form></td></tr>\n" );

		printf( "  <tr><td colspan='2' class='menu'>" );
		if ( $menu->get_count() > 0)
		{
			printf( "&gt;&gt; %s",
				$menu->get_menu_string() );
		}
		printf( "</td></tr>\n" );
		html_table_footer();
	
		return;
	}

	private function html_rel_section ( $text, &$menu )
	{
		html_table_header( 600, 200 );
		printf( "<tr>" .
			"<td class='section'>%s</td>" .
			"<td class='session'><form action='%s' name='rel_visible' method='post'><input type='hidden' name='ci_id' value='%s'>".get_string('Visibility_ttl').": ",
			$text,
			CI_VIEW_PHP,
			$this->db_object->get_id()
			);

		$enum_rel_visible = get_enum_to_array("enum_rel_visible"); 
		cat_array( $enum_rel_visible, $this->get_rel_visible(), -1, "ci_rel_visible", get_string('none'), "rel_visible" );
		printf( "<input type='hidden' name='rel_visible_form' value='1'>" );
		printf("</form></td></tr>\n" );

		printf( "  <tr><td colspan='2' class='menu'>" );
		if ( $menu->get_count() > 0)
		{
			printf( "&gt;&gt; %s",
				$menu->get_menu_string() );
		}
		printf( "</td></tr>\n" );
		html_table_footer();
	}


}
