<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'tables/db_typed_ci.inc.php';
require_once 'includes/ci_navi_list.inc.php';
require_once 'includes/icon_list.inc.php';

/**
 * list of CI's of a given type
 */
class ci_cit_navi_list extends ci_navi_list
{
	/**
	 * array to cache lists by cit_id
	 * @var	array
	 */
	static private $instance	= array();

	/**
	 * ID of CI-type
	 * @var	int
	 */
	protected $search_cit_id	= 0;
	
	/**
	 * get list from cache or create a new one
	 * 
	 * @param	int		$_cit_id			ID of CI-type
	 * @return	none
	 */
	static public function &getInstance( $_cit_id = 0 )
	{
		if (! array_key_exists( $_cit_id, self::$instance ))
		{
			self::$instance[ $_cit_id ] = new ci_cit_navi_list( $_cit_id );
		}
		
		return self::$instance[ $_cit_id ];
	}

	/**
	 * initialize list
	 * 
	 * @param	int		$_cit_id			ID of CI-type
	 * @return	none
	 */
	protected function __construct( $_cit_id=0 )
	{
		$this->search_cit_id = $_cit_id;
		parent::__construct();
	}
	
	/**
	 * get name, under which the list is stored in session array
	 * 
	 * @return	string
	 */
	protected function get_session_store_name()
	{
		return "cit";
	} 

	/**
	 * get ID of CI-type
	 * 
	 * @return	int							ID of CI-type
	 */
	public function get_search_cit_id()
	{
		return $this->search_cit_id;
	}

	/**
	 * get query to load items from database
	 * 
	 * @return	string						SQL query
	 */
	protected function get_query()
	{
		$query = array();

		$query[] = "SELECT * FROM ci, ci_type " .
			"WHERE ci_cit_id=cit_id " .
			"AND cit_id='$this->search_cit_id' " .
			"ORDER BY ci_title, cit_title";

		return $query;
	}

	// helper

	protected function _load()
	{
		$_cit_id = $this->search_cit_id;
		
		parent::_load();

		// overload from args
		$_cit_id = http_get_int( 'cit_id', $_cit_id );
		if ($_cit_id != 0)
		{
			$this->reset();
			$this->search_cit_id = $_cit_id;
		}

		return;
	}
	
}
