<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_api.inc.php';
require_once 'core/core_html_form.inc.php';
require_once 'tables/db_ci_type.inc.php';
require_once 'includes/icon_list.inc.php';
require_once 'includes/design.inc.php';
require_once 'includes/entity_processor.inc.php';

class ci_type extends entity_processor
{
	/**
	 * attribute des CI-Types aus argumenten laden
	 * 
	 * ueberlaedt die felder des DB-Objektes mit den
	 * werten aus den argumenten, dazu muss allerdings
	 * das argument "cit_form" gesetzt sein 
	 * 
	 * @return	none
	 */	
	public function load_from_args ()
	{
		$_form = http_get_int("cit_form", 0);
		switch ($_form)
		{
			case 1:
			{
				$_title = http_get_string("cit_title", $this->db_object->get_title() );
				$this->db_object->set_title( $_title );

				$_desc = http_get_string("cit_desc", $this->db_object->get_desc() );
				$this->db_object->set_desc( $_desc );

				break;
			}
			case 2:
			{
				$_icon = http_get_string("cit_icon", $this->db_object->get_icon() );
				$this->db_object->set_icon( $_icon );

				break;
			}
		}

		return;
	}

	// advanced get
	
	// formulare

	public function display_form ( $title, $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );
		html_table_header( 25, 150, 625 );
	
		html_table_section( $title, design_cit_form_menu($this->db_object->get_id()) );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='cit_id' value='%s'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='cit_form' value='1'></td></tr>\n" );
	
		html_oblig_row( '*', get_string('Title_ttl') );
		printf( "<td class='f_value'><input type='text' name='cit_title' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 200, html_entity_decode( $this->db_object->get_title(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '', get_string('Description_ttl') );
		printf( "<td class='f_value'><textarea name='cit_desc' cols='%d' rows='%d'>%s</textarea></td></tr>\n",
			50, 5, html_entity_decode( $this->db_object->get_desc(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );
	
	}

	public function display_icon_form ( $action )
	{
		printf( "<div id='form'>\n" );

		html_table_header( 25, 150, 625 );
	
		html_table_section( get_string('ChooseIcon_ttl'), design_cit_form_menu($this->db_object->get_id()) );
		html_table_info( 0, 0, get_string('ClickOnIconToSelect_dsc'));
		printf( "<tr><td colspan='3'>" );
		icon_list::getInstance()->html_chooser(
			sprintf( "%s?cit_form=2&amp;cit_id=%d&amp;cit_icon=", $action, $this->db_object->get_id() ),
			get_string('NoIcon_dsc'),
			""
			);
		printf( "</td></tr>\n" );
		html_table_footer();
		printf( "</div>\n" );
	
	}

	public function display_question ( $action )
	{
		global $referer_script_name;

		$yes_args = new html_form_args();
		$yes_args->add( "cit_id", $this->db_object->get_id() );
	
		$no_args = new html_form_args();
		
		printf( "<div id='question'>\n" );
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('DeleteCIType_ttl'), design_cit_form_menu($this->db_object->get_id()) );
		html_table_footer();

		html_form_question( 0, 0, 
			get_string('WantDeleteCIType_dsc'),
			get_string('YES_btn'), $action, $yes_args,
			get_string('NO_btn'), $referer_script_name, $no_args );
		printf( "</div>\n" );

		return;
	}

	// display methoden

	/**
	 * zeigt die specification des relation-types an
	 */
	public function display_spec ()
	{
		printf( "<div id='spec'>\n" );
		html_table_header( 25, 150, 625 );

		html_table_section( get_string('Specification_dsc'), design_cit_spec_menu( $this->get_id() ) );

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('Title_ttl'), $this->db_object->get_title() ); 

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('Description_ttl'), $this->db_object->get_desc() ); 

		html_table_footer();
		printf( "</div>\n" );

		return;
	}

	/**
	 * zeigt die attribute des relation-types an
	 */
	public function display_attributes ()
	{
		$type_attr_list = &type_attr_list::getInstance( CLASS_CI, $this->get_id() );
		$type_attr_count = $type_attr_list->count();

		printf( "<div id='attributes'>\n" );
		html_table_header( 50, 20, 20, 20, 30, 200, 310, 100, 50 );
		html_table_section( get_string('Attributes_ttl')." (".get_string('Total_ttl').": $type_attr_count)", design_ta_index_menu( CLASS_CI, $this->get_id() ) );
?>
		<tr class='r_head'>
<?php
		printf( "			<td class='c_action r_head'>".get_string('Order_ttl')."</td>\n" );
		printf( "			<td class='c_action r_head' title='".get_string('Obligate_ttl')."'>O</td>\n" );
		printf( "			<td class='c_action r_head' title='".get_string('Unique_ttl')."'>U</td>\n" );
		printf( "			<td class='c_action r_head' title='".get_string('Tag_ttl')."'>T</td>\n" );
		printf( "			<td class='c_text r_head'>".get_string('Vis_dsc')."</td>\n" );
		printf( "			<td class='c_text r_head'>".get_string('Title_ttl')."</td>\n" );
		printf( "			<td class='c_text r_head'>".get_string('Description_ttl')."</td>\n" );
		printf( "			<td class='c_text r_head'>".get_string('Type_ttl')."</td>\n" );
		printf( "			<td class='c_action r_head'>".get_string('Action_ttl')."</td>\n" );
?>
		</tr>
<?php
		if ($type_attr_count == 0)
		{
			html_empty_row( 0, 0 );
		}
		else
		{ 
			html_table_row_reset( array("r_normal","r_normal2") );
			for ($i=0; $i<$type_attr_count; $i++)
			{
				$ta = & $type_attr_list->at( $i );
		
				html_table_row();
				printf( "<td class='c_action'>%s %s</td>" .
					"<td class='c_action'>%s</td>" .
					"<td class='c_action'>%s</td>" .
					"<td class='c_action'>%s</td>" .
					"<td class='c_text'>%d</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_action'>%s %s</td></tr>\n",
					icon_display_url(TA_UP_RUN_PHP . "?ta_id=" . $ta->get_id(), ICON_UP, get_string('MoveUp_dsc') ),
					icon_display_url(TA_DOWN_RUN_PHP . "?ta_id=" . $ta->get_id(), ICON_DOWN, get_string('MoveDown_dsc') ),
					html_status_checkbox( $ta->get_oblig() ),
					html_status_checkbox( $ta->get_unique() ),
					html_status_checkbox( $ta->get_tag() ),
					$ta->get_visible(), 
					$ta->get_title(), 
					$ta->get_desc(),
					get_enum_to_string( "enum_datatypes", $ta->get_datatype() ),
					icon_display_url(TA_EDIT_PHP . "?ta_id=" . $ta->get_id(), ICON_EDIT, get_string('EditCITypeAttribute_dsc') ),
					icon_display_url(TA_DELETE_PHP . "?ta_id=" . $ta->get_id(), ICON_DELETE, get_string('DeleteCITypeAttribute_dsc') )
					);
			}
		}
		html_table_footer();
		printf( "</div>\n" );

		return;
	}

	// action support

	/**
	 * necessary actions before deleting the CI-Type
	 * 
	 * @return	none
	 */
	public function delete_pre ()
	{
		$db = core_db::getInstance();

		// delete all CSV-elements and CSV-profiles for this CI-Type
		$query = "DELETE FROM csv_element " .
			"WHERE ce_cp_id in (SELECT cp_id FROM csv_profile WHERE cp_cit_id=32)";
		$result = $db->query( $query );

		$query = "DELETE FROM csv_profile " .
			"WHERE cp_cit_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query );

		// delete all attributes of this CI-Type
		$query = "DELETE FROM type_attr " .
			"WHERE ta_classtype='" . CLASS_CI . "' " .
			"AND ta_classtype_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query );

	}

	/**
	 * check, that all required conditions for deleting
	 * are fullfilled
	 * 
	 * method reply TRUE, if CI-Type can be deleted without problems,
	 * otherwise FALSE 
	 * 
	 * @return	boolean
	 */
	public function valid_for_delete ()
	{
		$db = core_db::getInstance();
		$query = "SELECT COUNT(*) FROM ci " .
			"WHERE ci_cit_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query ); 
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if (($count = $db->result( $result, 0 )) != 0)
		{
			throw new core_data_exception( get_string('CITypeUsedBy_dsc')." $count ".get_string('CIS_dsc'));
		}
		
		return TRUE;
	}

	// helper
	
	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$title = sprintf( get_string('CIType_ttl').": '%s'", $this->db_object->get_title() );
		return $title;
	}

	/**
	 * get a HTML string including the icon for the
	 * synopse section
	 * 
	 * @return		string		HTML code including an icon 
	 */
	protected function get_synopse_icon_html ()
	{
		$icon_html = icon_list::getInstance()->get_html_imx( $this->db_object->get_icon() );
		return $icon_html;
	}

	protected function create_db_object ( $_id )
	{
		$this->db_object = new db_ci_type( $_id );
		return;
	}

}
