<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'tables/db_ci_type.inc.php';
require_once 'includes/navi_list.inc.php';

class ci_type_list extends navi_list
{
	static private $instance = false;
	
	static public function getInstance()
	{
		if (self::$instance === false) self::$instance = new ci_type_list();
		return self::$instance;
	}

	protected function __construct ()
	{
		$this->page_size = get_property("page_rows");
		parent::__construct();
	}

	protected function get_session_store_name ()
	{
		return "ci_type_list";
	} 

	protected function get_element_name ()
	{
		return get_string('CIType_ttl');
	} 
	
	protected function get_id_name ()
	{
		return "cit_id";
	}

	public function show_page ()
	{
		html_table_header( 200, 550, 50 );
		switch ($this->get_list_mode())
		{
			case LIST_MODE_USE:
			{
?>
<tr class='r_head'>
<?php
printf("	<td class='c_text r_head'>".get_string('CIType_ttl')."</td>\n");
printf("	<td class='c_text r_head'>".get_string('Description_ttl')."</td>\n");
printf("	<td class='c_action r_head'>".get_string('Actions_ttl')."</td>\n");
?>
	</tr>
<?php
				break;
			}
			case LIST_MODE_MANAGE:
			{
?>
<tr class='r_head'>
<?php
printf("	<td class='c_text r_head'>".get_string('CIType_ttl')."</td>\n");
printf("	<td class='c_text r_head'>".get_string('Description_ttl')."</td>\n");
printf("	<td class='c_action r_head'>".get_string('Actions_ttl')."</td>\n");
?>
</tr>
<?php
				break;
			}
		}
		
		parent::show_page();
		html_table_footer();

		return;
	}	

	protected function show_item ( $i )
	{
		$ci_type = & $this->at( $i );
		$icon_size  = get_property("list_icon_size");

		html_table_row();

		switch ($this->get_list_mode())
		{
			case LIST_MODE_USE:
			{		
				printf(
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_action'></td>" .
					"</tr>\n",
					get_html_url(
						sprintf( "%s?cit_id=%d", CI_LIST_PHP, $ci_type->get_id()),
						icon_list::getInstance()->get_html_imx( $ci_type->get_icon(), $icon_size, $icon_size, TRUE ) . " " . $ci_type->get_title()
						),
					$ci_type->get_desc()
					);
				break;
			}
			case LIST_MODE_MANAGE:
			{		
				printf(
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_action'>%s %s</td>" .
					"</tr>\n",
					get_html_url(
						sprintf( "%s?cit_id=%d", CIT_VIEW_PHP, $ci_type->get_id()),
						icon_list::getInstance()->get_html_imx( $ci_type->get_icon(), $icon_size, $icon_size, TRUE ) . " " . $ci_type->get_title(),
						FALSE,
						get_string('DisplayAndManageCIType_dsc')
						),
					$ci_type->get_desc(),
					icon_display_url(CIT_EDIT_PHP . "?cit_id=" . $ci_type->get_id(), ICON_EDIT, get_string('EditCIType_dsc') ),
					icon_display_url(CIT_DELETE_PHP . "?cit_id=" . $ci_type->get_id(), ICON_DELETE, get_string('DeleteCIType_dsc') )
					);
				break;
			}
		}
	}

	protected function get_query ()
	{
		$query = array();
		$query[] = "SELECT * FROM ci_type ORDER BY cit_title"; 
		return $query;
	}
	
	protected function create_object ( $_id )
	{
		$db_object = new db_ci_type( $_id ); 
		return $db_object;
	}
		
	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$title = get_string('AllCITypes_ttl');
		return $title;
	}

} 
