<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_db_mysql.inc.php';
require_once 'includes/db_setup.inc.php';

/**
 * fallback form button
 */
function cmdb_setup_tryagain ( &$setup, $message )
{
	html_status( STATUS_ERROR, $message );
	html_table_line();
	printf( "<tr><td colspan='3'><form action='%s' method='post'></td></tr>\n",
		CMDB_SETUP_PHP );

	printf( "<tr><td colspan='3'>" .
		"<input type='hidden' name='db_form' value='0'>" .
		"<input type='hidden' name='db_host' value='%s'>" .
		"<input type='hidden' name='db_name' value='%s'>" .
		"<input type='hidden' name='db_user' value='%s'>" .
		"<input type='hidden' name='db_root_user' value='%s'>" .
		"<input type='hidden' name='cmdb_user' value='%s'>" .
		"</td></tr>\n",
		$setup->db_host,
		$setup->db_name,
		$setup->db_user,
		$setup->db_root_user,
		$setup->cmdb_user
		 );
	
	printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></form></td></tr>\n",
		"Try again..." );

	return;
}


/**
 * step 1: check prerequisites
 * 
 * @return	int		return code, 0 = Success
 */
function cmdb_setup_step1 ()
{
	$rc_global = 0;

	html_table_section( "Checking prerequisites", get_empty_menu() );
	
	// check php version
	if (version_compare( "5.0.0", PHP_VERSION ) <= 0)
	{
		html_status( STATUS_OK, "PHP-Version is " . PHP_VERSION );
	}
	else
	{
		html_status( STATUS_ERROR, "PHP-Version " . PHP_VERSION . " too old!" );
		$rc_global++;
	}
	
	// check extensions
	$needed_extensions = array( "exif", "gd", "session", "mysql" );
	foreach ( $needed_extensions as $extension )
	{
		if (extension_loaded($extension))
		{
			html_status( STATUS_OK, "Extension $extension loaded" );
		}
		else
		{
			html_status( STATUS_ERROR, "Extension $extension missing" );
			$rc_global++;
		}
	}

	// check gd capabilities
	if (function_exists( "gd_info" ))
	{
		$gd_data = gd_info();
//		$needed_gd_support = array('FreeType Support', 'T1Lib Support', 'PNG Support' );
		$needed_gd_support = array('FreeType Support', 'PNG Support' );
		foreach ( $needed_gd_support as $gd_support )
		{
			if ($gd_data[ $gd_support ])
			{
				html_status( STATUS_OK, "GD $gd_support available" );
			}
			else
			{
				html_status( STATUS_ERROR, "GD $gd_support missing" );
				$rc_global++;
			}
		}
	}
	else
	{
		html_status( STATUS_ERROR, "gd information not available" );
		$rc_global++;
	}
	
	// check temp directory
	$cache_dir = get_property_from_config("cache_dir");
	$cache_test = tempnam( $cache_dir, "cmdb_setup" );
	if (
		is_dir( $cache_dir)
		&&
		(($handle = fopen($cache_test, "w")) !== FALSE)
		)
	{
		fclose( $handle );
		unlink( $cache_test );
		html_status( STATUS_OK, "Cache directory '$cache_dir' exists and is writable" );
	}
	else
	{
		html_status( STATUS_ERROR, "Cache directory '$cache_dir' does not exists or is not writable!" );
		$rc_global++;
	}
	
	// check result
	if ($rc_global != 0)
	{
		html_table_line();
		html_status( STATUS_ERROR, "Please modify your setup to meet the required prerequisites and try again!", CMDB_SETUP_PHP );
	}

	return $rc_global;
}

/**
 * step 2: formular
 * 
 *
 * @return	int		return code, 0 = success 
 */
function cmdb_setup_step2 ( &$setup )
{
	$rc_global = 0;

	printf( "<form action='%s' method='post'>\n",
		CMDB_SETUP_PHP );

	html_table_section( "Initialise Database", get_empty_menu() );

	printf( "<tr><td colspan='3'><input type='hidden' name='db_form' value='1'></td></tr>\n" );

	// MySQL database setup

	html_table_info( 0, 0, "PHP-CMDB stores all it's information inside a MySQL database. " .
			"Please enter host, name, user and password for this dababase!" );

	html_oblig_row( '*', 'DB host' );
	printf( "<td class='f_value'><input type='text' name='db_host' size='%d' maxlength='%d' value='%s'></td></tr>\n",
		20, 100, $setup->db_host );

	html_oblig_row( '*', 'DB name' );
	printf( "<td class='f_value'><input type='text' name='db_name' size='%d' maxlength='%d' value='%s'></td></tr>\n",
		20, 20, $setup->db_name );

	html_oblig_row( '*', 'DB user' );
	printf( "<td class='f_value'><input type='text' name='db_user' size='%d' maxlength='%d' value='%s'></td></tr>\n",
		20, 20, $setup->db_user );

	html_oblig_row( '*', 'DB password' );
	printf( "<td class='f_value'><input type='password' name='db_pass' size='%d' maxlength='%d' value='%s'></td></tr>\n",
		20, 20, "" );

	html_oblig_row( '*', 'DB password' );
	printf( "<td class='f_value'><input type='password' name='db_pass_verify' size='%d' maxlength='%d' value='%s'> (repetition)</td></tr>\n",
		20, 20, "" );


	// MySQL root access

	html_table_info( 0, 0, "If the database and user for PHP-CMDB is not already created inside MySQL, setup can do it for you. " .
		"In this case, please enter the name and password for the MySQL root account!" );

	html_oblig_row( '', 'DB root user' );
	printf( "<td class='f_value'><input type='text' name='db_root_user' size='%d' maxlength='%d' value='%s'></td></tr>\n",
		20, 20, $setup->db_root_user );

	html_oblig_row( '', 'DB root password' );
	printf( "<td class='f_value'><input type='password' name='db_root_pass' size='%d' maxlength='%d' value='%s'></td></tr>\n",
		20, 20, "" );


	// CMDB initial user

	html_table_info( 0, 0, "To create an initial admin user inside PHP-CMDB, please fillout the following fields." );

	html_oblig_row( '*', 'CMDB user' );
	printf( "<td class='f_value'><input type='text' name='cmdb_user' size='%d' maxlength='%d' value='%s'></td></tr>\n",
		20, 20, $setup->cmdb_user );

	html_oblig_row( '*', 'CMDB password' );
	printf( "<td class='f_value'><input type='password' name='cmdb_pass' size='%d' maxlength='%d' value='%s'></td></tr>\n",
		20, 20, "" );

	html_oblig_row( '*', 'CMDB password' );
	printf( "<td class='f_value'><input type='password' name='cmdb_pass_verify' size='%d' maxlength='%d' value='%s'> (repetition)</td></tr>\n",
		20, 20, "" );

	html_table_ruler();

	printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></form>\n",
		get_string('Continue') . '...' );
	printf( "</form>\n" );	

	return $rc_global;
}

/**
 * step 3: verify formular data
 */
function cmdb_setup_step3 ( &$setup )
{
	$rc_global = 0;

	html_table_section( "Verify setup data", get_empty_menu() );

	if (strlen($setup->db_host)==0) $setup->db_host="localhost";

	if (strlen($setup->db_name)==0)
	{
		html_status( STATUS_ERROR, "Missing DB name!" );
		$rc_global++;
	}
		
	if (strlen($setup->db_user)==0)
	{
		html_status( STATUS_ERROR, "Missing DB user!" );
		$rc_global++;
	}
		
	if (strcmp($setup->db_pass, $setup->db_pass_verify)!=0)
	{
		html_status( STATUS_ERROR, "DB password and repetition not equal!" );
		$rc_global++;
	}

	if (
		(strlen($setup->cmdb_user)!=0)
		&&
		(strcmp($setup->cmdb_pass, $setup->cmdb_pass_verify)!=0)
		)
	{
		html_status( STATUS_ERROR, "CMDB password and repetition not equal!" );
		$rc_global++;
	}

	if ($rc_global != 0)
	{
		cmdb_setup_tryagain( $setup, "Verification error(s)!" );
	}
	else
	{
		html_status( STATUS_OK, "Verification completed!" );
	}

	return $rc_global;
}

/**
 * step 4: create database and user inside MySQL
 * 
 * @return	int		return code, 0 = success
 */
function cmdb_setup_step4 ( &$setup )
{
	$rc_global = 0;

	html_table_section( "Initialise database for PHP-CMDB", get_empty_menu() );

	if (strlen($setup->db_root_user)!=0)
	{
		// get name of cmdb_host to limit access rights inside mysql
		$cmdb_host = "localhost";
		if ($setup->db_host != $cmdb_host)
		{
			$cmdb_host = $_SERVER["HTTP_HOST"];
		}
		
		try 
		{
			// get connection
			$db_root = new core_db_mysql( $setup->db_host, $setup->db_root_user, $setup->db_root_pass, "mysql" );
	
			// create database
			$query = sprintf( "create database %s", $setup->db_name );
//			html_table_info( 0, 0, "DEBUG: " . $query );
			$result = $db_root->query( $query );

			// create user
			$query = sprintf( "grant all on %s.* to '%s'@'%s' identified by '%s'",
				$setup->db_name,
				$setup->db_user,
				$cmdb_host,
				$setup->db_pass
				);
//			html_table_info( 0, 0, "DEBUG: " . $query );
			$result = $db_root->query( $query );

			html_status( STATUS_OK, "New database and user created!" );

		}
		catch (Exception $e)
		{
			cmdb_setup_tryagain( $setup, sprintf( "Error while accessing MySQL! (%s / %d)", $e->get_message(), $e->get_code() ) );
			$rc_global++;
		}

	}

	return $rc_global;
}

/**
 * step 5: create database schema and initial CMDB user
 * 
 * @return	int		return code, 0 = success
 */
function cmdb_setup_step5 ( &$setup )
{
	$schema_file = '../resources/cmdb_schema.sql';
	$rc_global = 0;

	try 
	{
//echo "<pre>\n";

		// get connection
		$db_cmdb = new core_db_mysql( $setup->db_host, $setup->db_user, $setup->db_pass, $setup->db_name );

		// table attr
		db_setup::createTable( $db_cmdb, $schema_file, 'attr' );

		// table cat
		db_setup::createTable( $db_cmdb, $schema_file, 'cat' );

		// table cat_entry
		db_setup::createTable( $db_cmdb, $schema_file, 'cat_entry' );

		// table ci
		db_setup::createTable( $db_cmdb, $schema_file, 'ci' );

		// table ci_type
		db_setup::createTable( $db_cmdb, $schema_file, 'ci_type' );

		// table comment
		db_setup::createTable( $db_cmdb, $schema_file, 'comment' );

		// table csv_element
		db_setup::createTable( $db_cmdb, $schema_file, 'csv_element' );

		// table csv_profile
		db_setup::createTable( $db_cmdb, $schema_file, 'csv_profile' );

		// table property
		db_setup::createTable( $db_cmdb, $schema_file, 'property' );

		// table relation
		db_setup::createTable( $db_cmdb, $schema_file, 'relation' );

		// table relation_type
		db_setup::createTable( $db_cmdb, $schema_file, 'relation_type' );

		// table type_attr
		db_setup::createTable( $db_cmdb, $schema_file, 'type_attr' );

		// table user
		db_setup::createTable( $db_cmdb, $schema_file, 'user' );

		html_status( STATUS_OK, "Schema created!" );

		// create CMDB user
		if (strlen($setup->cmdb_user) != 0)
		{
			$query = sprintf( "INSERT INTO user " .
				"(u_login,u_passwd,u_role_id,u_auth_backend,u_active) VALUES " .
				"('%s','%s',%d,%d,%d)",
				$db_cmdb->escape_string($setup->cmdb_user),
				md5( $setup->cmdb_pass ),
				ROLE_ADMIN,
				AUTH_BACKEND_LOCAL,
				TRUE				
				);
			$result = $db_cmdb->query( $query );

			html_status( STATUS_OK, "CMDB user created!" );
		}

	}
	catch (Exception $e)
	{
		cmdb_setup_tryagain( $setup, sprintf( "Error while accessing MySQL! (%s / %d)", $e->get_message(), $e->get_code() ) );
		$rc_global++;
	}

	return $rc_global;
}

/**
 * step 6: write config
 * 
 * @return	int		return code, 0 = success
 */
function cmdb_setup_step6 ( &$setup )
{
	$rc_global = 0;

	// create config string
	$config_string = sprintf( 
		"<?php\n" . 		
		"\$g_db_host = '%s';\n" .
		"\$g_db_name = '%s';\n" .
		"\$g_db_user = '%s';\n" .
		"\$g_db_pass = '%s';\n" .
		"\n",
		$setup->db_host,
		$setup->db_name,
		$setup->db_user,
		$setup->db_pass
	);

	// 
	$config_dir = dirname( $_SERVER["SCRIPT_FILENAME"] );
	$config_file = $config_dir . "/" . CONFIG_INC_PHP;

	$config_handle  = @fopen($config_file, "w");
	if ($config_handle !== FALSE)
	{
		// write config directly
		fwrite( $config_handle, $config_string );		
		fclose( $config_handle );
		
		html_status( STATUS_OK, "Config written to file '$config_file'." );

	}
	else
	{
		// show config
		html_table_info( 0, 0, "Cannot write configuration directly to file '$config_file'. " .
			"Please create file manualy and copy the following lines into it!" );
		printf( "<tr><td></td><td colspan='2'><pre><code>\n%s</code></pre></td></tr>\n", htmlentities($config_string) );
		
	}

	return $rc_global;
}
