<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'includes/navi_list.inc.php';
require_once 'tables/db_comment.inc.php';

class comment_list extends navi_list
{
	static private $instance = array();
	protected $ci_id = 0;

	protected function __construct ( $_ci_id )
	{
		$this->ci_id = $_ci_id;
		parent::__construct();
	}
	
	static public function getInstance( $_ci_id )
	{
		if (! array_key_exists( $_ci_id, self::$instance )) self::$instance[ $_ci_id ] = new comment_list( $_ci_id );
		return self::$instance[ $_ci_id ];
	}
	
	protected function get_navi_args ()
	{
		$navi_args = "&amp;ci_id=" . $this->ci_id;
		return $navi_args;
	}

	protected function get_dont_load_keys ()
	{
		$key_array = parent::get_dont_load_keys();
		$key_array[] = "ci_id";
		return $key_array;
	}

	protected function post_load ()
	{
		$this->page_size = get_property("page_rows");
		parent::post_load();
	}

	protected function get_session_store_name ()
	{
		$txt = sprintf( "comment%d", $this->ci_id );
		return $txt;
	} 

	protected function get_element_name ()
	{
		return get_string('Comments_ttl');
	} 

	protected function get_id_name ()
	{
		return "c_id";
	}
	
	public function get_ci_id ()
	{
		return $this->ci_id;
	}

	protected function get_query ()
	{
		$query = array();
		$query[] = "SELECT * FROM comment " .
			"WHERE c_ci_id='$this->ci_id' " .
			"ORDER BY c_create_change DESC";
		return $query;
	}
	
	// show
	
	public function show_page ()
	{
		html_table_header( 25, 725, 50 );
		parent::show_page();
		html_table_footer();

		return;
	}

	protected function show_item ( $i )
	{
		$comment = & $this->at( $i );
		$c_text = htmlentities( $comment->get_text() );
		$c_text = str_replace( "\n", "<br>", $c_text );
		$c_user = get_string('Unknown_ttl');
		if (is_object($user = user_list::getInstance()->get( $comment->get_last_uid()))) $c_user = $user->get_name();
		
		printf( "<tr><td class='commenttext' colspan='2'>%s</td><td class='c_action'>%s %s</td></tr>\n",
			$c_text,
			icon_display_url(C_EDIT_PHP . "?c_id=" . $comment->get_id(), ICON_EDIT, get_string('EditComment_dsc') ),
			icon_display_url(C_DELETE_PHP . "?c_id=" . $comment->get_id(), ICON_DELETE, get_string('DeleteComment_dsc') )
			);
		printf( "<tr><td class='changeinfo' colspan='3'>created %s / ".get_string('LastChanged_dsc')." %s ".get_string('By_dsc')." %s</td></tr>\n", 
			$comment->get_create_change(),
			$comment->get_last_change(), $c_user );

	}

	// helper

	protected function create_object ( $_id )
	{
		$db_object = new db_comment( $_id ); 
		return $db_object;
	}

	protected function _load()
	{
		$_ci_id = $this->ci_id;
		parent::_load();
		if ($_ci_id != 0)
		{
			$this->reset();
			$this->ci_id = $_ci_id;
		}
		return;
	}
	
} 
 