<?php
/**
 * php-cmdb - a php based configuration management database
 *
 * Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de
 *
 * php-cmdb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * php-cmdb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category	php-cmdb
 * @package		core
 * @subpackage	base
 * @author		mscheibe
 */

// config_default_inc.php - default configuration values
//
// please read this file, to see which parameters and values
// exist and what they mean
//
// BUT PLEASE DO NOT MODIFY THIS FILE, to customize
// parameters use the file "../config_inc.php" instead!
//

// === strings ============================================

$s_enum_color_codes = '0:000000,1:000000,2:000080,3:0000FF,4:008000,5:008080,6:00FF00,7:00FFFF,8:800000,9:800080,10:808000,11:808080,12:C0C0C0,13:FF0000,14:FF00FF,15:FFFF00,16:FFFFFF';
$s_enum_relation_depth = '1:1,2:2,3:3,4:4,5:5,6:6,7:7,8:8,9:9,10:10';
$s_enum_attr_visible = '1:1,2:2,3:3,4:4,5:5,6:6,7:7,8:8,9:9,10:10';


// === functional setup ===================================

$g_cache_dir					= '';
if (strpos( php_uname(), 'indows' ) === FALSE)
{
	// on NON-windows systems we use the wellknown path
	$g_cache_dir				= '/tmp';
}
else
{
	// on windows systems, we try to read it from the environment
	// check this list of environment vars to fetch it 
	$env_tmp_list = array( 'TMP', 'TMPDIR', 'TEMP' );
	foreach ($env_tmp_list as $env_var)
	{
		if (
				(strlen($g_cache_dir) == 0) &&
				(($_tmp = getenv( $env_var )) !== FALSE)
			)
		{
			$g_cache_dir = $_tmp;
		}
	}
}

$g_icon_dir					= 'icons';

$g_cache_ico_prefix			= 'cmdb_ico_';

$g_map_prefix				= 'ci_map_';
$g_map_sx					= 640;
$g_map_sy					= 360;
$g_map_font					= 'libs/freefonts/transportm.ttf';
$g_map_font_size			= 10;
$g_map_padding				= 4;
$g_map_arrow				= 10;
$g_map_icon_size			= 24;

$g_ext_prefix				= 'ci_ext_';
$g_ext_sx					= 1240;
$g_ext_sy					= 1024;
$g_ext_font					= 'libs/freefonts/transportm.ttf';
$g_ext_font_size			= 10;
$g_ext_padding				= 4;
$g_ext_arrow				= 10;
$g_ext_icon_size			= 24;

$g_pdf_sx					= 33.11;		// inch, DIN A0
$g_pdf_sy					= 46.81;
$g_pdf_font					= 'libs/freefonts/transportm.afm';
$g_pdf_font_size			= 10;			// pixel, 24
$g_pdf_padding				= 4;			// pixel, 4
$g_pdf_arrow				= 10;			// pixel, 12
$g_pdf_icon_size			= 24;			// pixel, 24

$g_default_icon_name		= 'color_square_black.ico';
$g_default_icon_size		= 32;
$g_list_icon_size			= 16;

$g_title_wrap_width			= 15;

$g_display_login_errors		= FALSE;
$g_empty_passwd				= FALSE;

$g_protect_used_type_attr	= TRUE;
$g_delete_all_ci_of_type	= FALSE;

$g_last_change_items		= 10;
$g_last_change_comments		= 10;

$g_redirect_single_result	= FALSE;

// === graphviz ===========================================
//
$g_graphviz					= FALSE;
$g_graphviz_neato			= '/usr/bin/neato';
$g_graphviz_restyle			= FALSE;

// === trigger ============================================
//
$g_trigger					= FALSE;
$g_trigger_remote			= '127.0.0.1';

// === language ===========================================
//
$g_default_language			= 'english';


// === application information ============================
//
$g_app_stylesheet			= 'cmdb.css';
$g_app_logofile				= ''; 
$g_app_url					= 'http://www.nofiles.de/';
$g_app_favicon				= 'imx/cmdb_favicon.ico';
$g_app_name					= 'cmdb';
$g_app_version				= '0.7.3';		// do not edit, substituted while packaging
$g_app_copyright_holder		= 'nofiles';
$g_app_copyright_logofile	= 'imx/nofiles_1.png';
$g_app_copyright_period		= '2008 - 2012';

$g_customer_stylesheet		= '';

// === page layout configuration ==========================
//
$g_page_table_width			= 800;
$g_page_table_cellspace		= 0;
$g_page_table_cellpadding	= 1;
$g_page_rows				= 10;
$g_page_auto_refresh		= 1;

// === debug ==============================================
//
$g_debug					= FALSE;

// === database setup =====================================
//
$g_db_host					= '';
$g_db_name					= '';
$g_db_user					= '';
$g_db_pass					= '';

// === ldap setup =========================================

$g_auth_ldap				= FALSE;
$g_auth_ldap_server			= '';
$g_auth_ldap_port			= '';
$g_auth_ldap_user			= '';
$g_auth_ldap_pass			= '';
$g_auth_ldap_base			= '';
$g_auth_ldap_attr			= '';
$g_auth_ldap_auto_import	= FALSE;

/**
 * auth_ldap_group_attr
 * name of ldap attribute, which sepcifies membership 
 * in a ldap group
 * 
 * default: empty
 */
$g_auth_ldap_group_attr		= '';

/**
 * auth_ldap_membership_operator
 * 
 * to enable recursive group expansion in AD, use
 * the following value ':1.2.840.113556.1.4.1941:='
 * 
 * default: '='
 */
$g_auth_ldap_membership_operator = '=';

/**
 * auth_ldap_group_base
 * DN of the group, authorised users must be member of
 * 
 * if empty, no authorisation is used
 * 
 * default: empty
 */
$g_auth_ldap_group_base		= '';

/**
 * auth_ldap_role_map
 * array to map ROLE-IDs inside PHP-CMDB to LDAP-Groups
 * 
 * an example follows:
 * array(
 * 	ROLE_ADMIN => '<GROUP-DN>',
 * 	ROLE_USER_ADMIN => '<GROUP-DN>',
 * 	ROLE_DESIGNER => '<GROUP-DN>',
 * 	ROLE_CHANGE_MANAGER => '<GROUP-DN>',
 * 	ROLE_OPERATOR => '<GROUP-DN>',
 * 	ROLE_VIEWER => '<GROUP-DN>'
 * )
 * 
 * if a ROLE-ID is not included in the array or refers to a empty DN string,
 * this ROLE-ID will not be respected
 * 
 * if the array is empty, no role mapping is used
 * 
 * default: empty array
 */
$g_auth_ldap_role_map		= array();
