<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	processing
* @version		$Id: csv_processor.inc.php 367 2009-04-16 15:41:10Z micha $
*/

require_once 'core/core_processor.inc.php';
require_once 'tables/db_ci_type.inc.php';
require_once 'tables/db_type_attr.inc.php';
require_once 'includes/ci.inc.php';
require_once 'includes/csv_profile.inc.php';
require_once 'includes/csv_profile_list.inc.php';

/**
 * class to import/export CSV file using a CSV-profile
 */
class csv_processor extends core_processor
{
	protected $cit_id			= 0;
	protected $cp_id			= 0;
	protected $csv_file 		= "";
	protected $cp				= null;
	protected $sys_id_pos		= -1;
	protected $unique_pos		= -1;
	protected $pos_ta_map		= null;
	protected $count_lines		= 0;
	protected $count_comment	= 0;
	protected $count_error		= 0;
	protected $count_insert		= 0;
	protected $count_update		= 0;

	/**
	 * constructor
	 * 
	 * @return	none
	 */
	public function __construct()
	{
		return;
	}


	/**
	 * configure csv processor from http arguments
	 * 
	 * @return	boolean
	 */
	public function load_from_args()
	{
		// check form arg
		$csv_form = http_get_int( 'csv_form', 0 );
		
		switch ($csv_form)
		{
			case 0:
			{
				$this->cit_id = http_get_int( 'cit_id', $this->cit_id );
				break;
			}
			case CSV_FORM_IMPORT:
			{
				// get CSV file
				// get file arg
				$csv_file = $_FILES['csv_import_file']['tmp_name'];
				if (strlen($csv_file)==0)
				{
					$this->add_message( get_string('NoFileForUpload_dsc') );
					return FALSE;
				}
				
				// check upload status
				$err_file = $_FILES['csv_import_file']['error'];
				if ($err_file!=0)
				{
					$this->add_message( get_string('ErrorWhileUploadingFile_dsc') );
					return FALSE;
				}
		
				// check for correct upload file
				if (! is_uploaded_file( $csv_file )) throw new core_arg_exception( get_string('IllegalImportRequest_dsc') );
				
				$this->set_file( $csv_file );
		
				// get CSV profile 
				$cp_id = http_get_int( 'cp_id', 0 );
				if ($cp_id != 0)
				{
					// found CSV profile ID
					$this->set_cp_id( $cp_id );
				}

				// get CIT ID
				$this->cit_id = http_get_int( 'cit_id', $this->cit_id );

				break;
			}
			case CSV_FORM_EXPORT:
			{
				// get CSV profile 
				$cp_id = http_get_int( 'cp_id', 0 );
				if ($cp_id != 0)
				{
					// found CSV profile ID
					$this->set_cp_id( $cp_id );
				}

				// get CIT ID
				$this->cit_id = http_get_int( 'cit_id', $this->cit_id );

				break;
			}
			default:
			{
				throw new core_arg_exception( get_string('IllegalImportRequest_dsc') );
			}
		}
		
		return TRUE;
	}

	// set methods

	/**
	 * set name of csv file
	 * 
	 * @param	string	$_file				name of csv file
	 * @return	boolean
	 */
	public function set_file( $_file )
	{
		if (strlen($_file)==0) return FALSE;
		if (! is_file($_file)) return FALSE;
		
		$this->csv_file = $_file;
		return TRUE;
	}

	/**
	 * set csv profile by ID
	 * 
	 * @param	int		$_id				ID of csv profile
	 * @return	boolean
	 */
	public function set_cp_id( $_id )
	{
		$this->cp_id = $_id;
		$this->cp = new csv_profile( $_id );
		if ($this->cp->get_id() == 0)
		{
			$this->cp_id = 0;
			$this->cp = null;
		}
		return TRUE;
	}

	// display

	/**
	 * display HTML form to configure import
	 * 
	 * @param	string	$action		page to process form data
	 * @param	string	$action_text
	 */
	public function display_import_form( $action, $action_text )
	{
		printf( "<div id='form'>\n" );		
		printf( "<form action='%s' method='post' enctype='multipart/form-data'>\n",
			$action );
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('ImportCIs_ttl'), browse_ci_list_menu( $this->cit_id ) );
		if (csv_profile_list::getInstance( $this->cit_id )->count() == 0)
		{
			// no CSV profile available for this CI-Type
			html_table_info( 0, 0, get_string('PleaseCreateCSVProfile_dsc') );
		}
		else
		{
		
			printf( "<tr><td colspan='3'><input type='hidden' name='cit_id' value='%s'></td></tr>\n",
				$this->cit_id );
			printf( "<tr><td colspan='3'><input type='hidden' name='csv_form' value='%d'></td></tr>\n",
				CSV_FORM_IMPORT );
		
			html_oblig_row( '*', 'CSV-Profile' );
			printf( "<td class='f_value'>" );
			csv_profile_list::getInstance( $this->cit_id )->html_options( array($this->cp_id), array(), "cp_id", array( 0 => "--- ".get_string('SelectPlease')." ---" ) );
			printf( "</td></tr>\n" );
	
			html_oblig_row( '*', 'CSV-File' );
			printf( "<td class='f_value'><input type='file' class='b_action' name='csv_import_file' size='%d' maxlength='%d' accept='text/*'></td></tr>\n",
				50, 100000 );
	
			html_table_ruler();
		
			printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
				$action_text );
	
		}
		html_table_footer();
		printf( "</form>\n" );	
		printf( "</div>\n" );
		return;
	}

	/**
	 * display HTML form to configure export
	 * 
	 * @param	string	$action		page to process form data
	 * @param	string	$action_text
	 */
	public function display_export_form( $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post' enctype='multipart/form-data'>\n",
			$action );
	
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('ExportCIs_ttl'), browse_ci_list_menu( $this->cit_id ) );

		if (csv_profile_list::getInstance( $this->cit_id )->count() == 0)
		{
			// no CSV profile available for this CI-Type
			html_table_info( 0, 0, get_string('PleaseCreateCSVProfile_dsc') );
		}
		else
		{
		
			printf( "<tr><td colspan='3'><input type='hidden' name='cit_id' value='%s'></td></tr>\n",
				$this->cit_id );
			printf( "<tr><td colspan='3'><input type='hidden' name='csv_form' value='%d'></td></tr>\n",
				CSV_FORM_EXPORT );
		
			html_oblig_row( '*', 'CSV-Profile' );
			printf( "<td class='f_value'>" );
			csv_profile_list::getInstance( $this->cit_id )->html_options( array($this->cp_id), array(), "cp_id", array( 0 => "--- ".get_string('SelectPlease')." ---" ) );
			printf( "</td></tr>\n" );
	
			html_table_ruler();
		
			printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
				$action_text );
	
		}
		html_table_footer();
		printf( "</form>\n" );	
		printf( "</div>\n" );
		return;
	}

	/**
	 * print out the content of csv file to stdout 
	 * 
	 * @return	none
	 */
	public function display_csv_file()
	{
		// read file
		$csv_content = file( $this->csv_file );
		
		// display file content
		for ($i=0; $i<count($csv_content); $i++)
		{
			print( $csv_content[ $i ] );
		}
		
		// delete file
		unlink( $this->csv_file );

		return;
	}

	// processing
	
	/**
	 * run the CSV import
	 * 
	 * @return	boolean
	 */
	public function action_import()
	{
		// 
		$this->configure_mapper();
		
		// check validation
		if ($this->get_status() == STATUS_OK)
		{
			// open file
			$csv_handle = fopen( $this->csv_file, 'r' );

			// loop csv file
			while ( ($line_array = fgetcsv ($csv_handle, 4096, $this->cp->get_separator(), $this->cp->get_delimiter())) !== FALSE )
			{
				// check field count
				if (($field_count = count($line_array)) != 0)
				{
					if ($line_array[0][0]=='#')
					{
						// is a comment line
						$msg_line = sprintf( '%s %d: %s',
								get_string('Line_dsc'),
								$this->count_lines + 1,
								get_string('CommentIgnored_dsc')
						);
						$this->count_comment++;
						$this->add_message( $msg_line, STATUS_OK );
					}
					else
					{
						if ($field_count == count($this->pos_ta_map))
						{
							// is a valid line 
							$this->process_line_array( $line_array );
						}
						else
						{
							// is an invalid line
							$msg_line = sprintf( '%s %d: %s (%s: %d, %s %d)',
								get_string('Line_dsc'),
								$this->count_lines + 1,
								get_string('InvalidCountElements_dsc'),
								get_string('Required_dsc'),
								count($this->pos_ta_map),
								get_string('Found_dsc'),
								$field_count
							);
							$this->count_error++;
							$this->add_message( $msg_line, STATUS_ERROR );
						}
					}
				}
				$this->count_lines++;
			}

			// close csv file
			fclose( $csv_handle );
			unlink( $this->csv_file );
			
			// complete
			$this->add_message( $this->get_process_counter(), $this->get_status() );
		}

		return ($this->get_status() == STATUS_OK);
	}

	/**
	 * run the CSV export 
	 * 
	 * @return	boolean
	 */
	public function action_export()
	{
		// 
		$this->configure_mapper();
		
		// check validation
		if ($this->get_status() == STATUS_OK)
		{
			$db			= core_db::getInstance();

			// cit
			$cit= new db_ci_type( $this->cit_id );
			
			// create and open file
			$this->csv_file = tempnam( "", "cmdb-csv-export" );
			$csv_handle = fopen( $this->csv_file, "w" );			

			// write header
			fwrite( $csv_handle, sprintf( "# PHP-CMDB Version %s\n", get_property("app_version") ) );
			fwrite( $csv_handle, sprintf( "# Date/Time: %s\n", date("r") ) );
			fwrite( $csv_handle, sprintf( "#\n" ) );
			fwrite( $csv_handle, sprintf( "# ID/CI-Type: %d / %s\n", $cit->get_id(), $cit->get_title() ) );
			fwrite( $csv_handle, sprintf( "# ID/CSV-Profile: %d / %s\n", $this->cp->get_id(), $this->cp->get_title() ) );
			fwrite( $csv_handle, sprintf( "#\n" ) );
			fwrite( $csv_handle, sprintf( "# FIELDS: %s\n", $this->get_pos_ta_description() ) );
			fwrite( $csv_handle, sprintf( "#\n" ) );
			
			// get all CIs of this type
			$query		= "SELECT ci_id FROM ci " .
				"WHERE ci_cit_id='" . $this->cit_id . "' " .
				"ORDER BY ci_title";
			$result		= $db->query( $query );
			$ci_count	= $db->num_rows( $result );

			// loop over all CIs
			for ($i=0; $i<$ci_count; $i++)
			{
				// load CI
				$ci_id = $db->result( $result, $i );
				$ci = new ci( $ci_id );
				
				// get attribute values from CI
				$ta_val_map = & $this->create_ta_val_map();
				$ci->store_to_csv( $ta_val_map );
				$ta_val_map[ CE_TA_SYSID ] = $ci_id;
				
				// write values to file
				$pos_val_map = $this->get_pos_val_map( $ta_val_map );
				fputcsv( $csv_handle, $pos_val_map, $this->cp->get_separator(), $this->cp->get_delimiter() );
				
				$this->count_lines++;
			}

			// close file
			fclose( $csv_handle);

			// complete
			$this->add_message( $this->get_process_counter(), STATUS_OK );

		}

		return ($this->get_status() == STATUS_OK);
	}

	// helper

	/**
	 * load pos_ta_map from csv profile to configure mapping between
	 * attributes of CI and field positions in CSV
	 * 
	 * @return	none
	 */
	protected function configure_mapper()
	{
		// check: profile is loaded
		if (! is_object( $this->cp))
		{
			$this->add_message(get_string('CVSProfileNotConfigured_dsc'));
			return;
		}
	
		// load map 
		$this->pos_ta_map = $this->cp->get_pos_ta_array();
		
		// get sys id pos
		$p = 0;
		while (($p < count($this->pos_ta_map)) && ($this->sys_id_pos == -1))
		{
			if ($this->pos_ta_map[$p] == CE_TA_SYSID) $this->sys_id_pos = $p;
			$p++;
		}

		// get unique pos
		if ($this->sys_id_pos == -1)
		{
			$this->unique_pos = $this->cp->get_unique_element_pos();
		}

		return;
	}

	/**
	 * create an array, with ta_id => "" for all type attribute IDs,
	 * required by the CSV profile
	 * 
	 * @return	array
	 */
	protected function &create_ta_val_map()
	{
		$ta_val_map = array();
		for ($pos=0; $pos<count($this->pos_ta_map); $pos++)		
		{
			if (($ta_id = $this->pos_ta_map[ $pos ]) >= 0) $ta_val_map[ $ta_id ] = "";
		}
		return $ta_val_map;
	}

	/**
	 * process the given line_array: identify the CI and than insert or update it
	 * 
	 * @param	array	&$line_array
	 * @return	none 
	 */
	protected function process_line_array( &$line_array )
	{		
		$db			= core_db::getInstance();
		$ci_id		= 0;
		$ci			= null;
		$ta_val_map	= array();

		// try to get CI from SYS-ID
		if (($ci_id == 0) && ($this->sys_id_pos >= 0))
		{
			$ci_id = (int) $line_array[ $this->sys_id_pos ];
		}	

		// try to get CI from unique and oblig attribute
		if (($ci_id == 0) && ($this->unique_pos >= 0))
		{
			$ta_id = $this->pos_ta_map[ $this->unique_pos ];
			$ta_value = $line_array[ $this->unique_pos ];
			$query = "SELECT a_classtype_id FROM attr " .
				"WHERE a_classtype='" . CLASS_CI . "' " .
				"AND a_ta_id='$ta_id' " .
				"AND a_val_text='$ta_value' ";
			$result = $db->query( $query );
			if ($db->num_rows( $result ) == 1)
			{
				$ci_id = $db->result( $result, 0 );
			}
		}

		// check
		if ($ci_id != 0)
		{
			$query = "SELECT COUNT(*) FROM ci WHERE ci_id='$ci_id'";
			$result = $db->query( $query );
			if ($db->result( $result, 0 ) != 1) $ci_id=0; 
		}

		// load ci
		$ci = new ci( $ci_id, $this->cit_id );

		// load ta_val_map (ta_id => value)
		for ($pos=0; $pos<count($line_array); $pos++)
		{
			$ta_id = $this->pos_ta_map[ $pos ];
			if ($ta_id >= 0) $ta_val_map[ $ta_id ] = $line_array[ $pos ];
		}
		
		// update CI data
		$ci->load_from_csv( $ta_val_map );

		// store CI
		if ($ci->get_id() == 0)
		{
			if (! $ci->action_insert())
			{
				$this->add_message( get_string('Line_dsc') . ($this->count_lines + 1) . ": ".get_string('ErrorFromCIInsert_dsc')." (" . $ci->get_last_message() . ")", $ci->get_status() );
				$this->count_error++;	
			}
			else
			{
				$this->add_message( get_string('Line_dsc') . ($this->count_lines + 1) . ": ".get_string('CIInsertSuccesfull_dsc')." (" . $ci->get_last_message() . ")", $ci->get_status() );
				$this->count_insert++;
			}
		}
		else
		{
			if (! $ci->action_update())
			{
				$this->add_message( get_string('Line_dsc') . ($this->count_lines + 1) . ": ".get_string('ErrorFromCIUpdate_dsc')." (" . $ci->get_last_message() . ")", $ci->get_status() );
				$this->count_error++;	
			}
			else
			{
				$this->add_message( get_string('Line_dsc') . ($this->count_lines + 1) . ": ".get_string('CIUpdateSuccessful_dsc')." (" . $ci->get_last_message() . ")", $ci->get_status() );
				$this->count_update++;
			}
		}

		return;
	}

	/**
	 * create an array with pos => value mapping from the
	 * ta_val_map and the pos_ta_map
	 * 
	 * @param	array	&$ta_val_map
	 * @return	array
	 */
	protected function &get_pos_val_map( &$ta_val_map )
	{
		$pos_val_map = array();
		for ($pos=0; $pos<count($this->pos_ta_map); $pos++)		
		{
			$ta_id = $this->pos_ta_map[ $pos ];
			$val = $ta_val_map[ $ta_id ];
			$pos_val_map[ $pos ] = $val;
		}
		return $pos_val_map;
	}

	/**
	 * reply a string with all field-names of a CSV-line for the given profile
	 * 
	 * @return	string
	 */
	protected function get_pos_ta_description()
	{
		$desc="";
		for ($pos=0; $pos<count($this->pos_ta_map); $pos++)		
		{
			if ($pos!=0) $desc = $desc . $this->cp->get_separator();
			$ta_id = $this->pos_ta_map[ $pos ];

			if ($ta_id == CE_TA_SYSID) $desc = $desc . "SYS-ID";
			if ($ta_id > 0)
			{
				$ta = new db_type_attr( $ta_id );
				$desc = $desc . $ta->get_title();
			}
		}
		return $desc;
	}

	/**
	 * reply a string with the current counters
	 * 
	 * @return	string
	 */
	protected function get_process_counter()
	{
		$msg = sprintf( "%s: %d (%s: %d / %s: %d / %s: %d  / %s: %d)",
			get_string('LinesProcessed_ttl'), 
			$this->count_lines,
			get_string('Inserts_ttl'), 
			$this->count_insert,
			get_string('Updates_ttl'), 
			$this->count_update,
			get_string('Comments_ttl'), 
			$this->count_comment,
			get_string('Errors_ttl'),
			$this->count_error );
		return $msg;
	}
	
}
