<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_api.inc.php';
require_once 'core/core_html_form.inc.php';
require_once 'tables/db_csv_profile.inc.php';
require_once 'includes/design.inc.php';
require_once 'includes/entity_processor.inc.php';
require_once 'includes/ci_type_list.inc.php';
require_once 'includes/csv_element_list.inc.php';

class csv_profile extends entity_processor
{
	protected $elements	= null;
	
	public function __construct ( $_id=0, $_arg_name="" )
	{
		parent::__construct( $_id, $_arg_name );
		$this->elements = new csv_element_list( $this->db_object->get_id() );

		return;
	}

	public function load_from_args ()
	{
		$_form = http_get_int("cp_form", 0);
		
		switch ($_form)
		{
			case 1:
			{
				// update spec
				if ($this->db_object->get_id() == 0)
				{
					$_cit_id = http_get_int("cp_cit_id", $this->db_object->get_cit_id() );
					$this->db_object->set_cit_id( $_cit_id );
				}
				
				$_title = http_get_string("cp_title", $this->db_object->get_title() );
				$this->db_object->set_title( $_title );
	
				$_desc = http_get_string("cp_desc", $this->db_object->get_desc() );
				$this->db_object->set_desc( $_desc );
	
				$_separator = http_get_string("cp_separator", $this->db_object->get_separator () );
				$this->db_object->set_separator( $_separator );
	
				$_delimiter = http_get_string("cp_delimiter", $this->db_object->get_delimiter() );
				$this->db_object->set_delimiter( $_delimiter );

				break;
			}
			case 2:
			{
				// update elements
				$c = $this->elements->count();
				for ($pos=0; $pos < $c; $pos++)
				{
					$pos_ta_id = http_get_int( "ce" . $pos, 0 );
					$pos_object = & $this->elements->at( $pos );
					$pos_object->set_ta_id( $pos_ta_id );
				}
			}
		}

		return;
	}

	// advanced get

	public function get_unique_element_pos ()
	{
		return $this->elements->get_unique_pos();
	}
	
	public function get_pos_ta_array ()
	{
		return $this->elements->get_pos_ta_array();
	}

	public function get_title ()
	{
		return $this->db_object->get_title();
	}

	public function get_separator ()
	{
		return $this->db_object->get_separator();
	}

	public function get_delimiter ()
	{
		return $this->db_object->get_delimiter();
	}

	// display methoden

	/**
	 * display HTML form for spec edit
	 * 
	 * @param	string		$title		form section title
	 * @param	string		$action		page to process form request
	 * @param	string		$action_text	text for form button
	 * @return	none
	 */
	public function display_form ( $title, $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );
		printf( "<input type='hidden' name='cp_id' value='%s'>\n",
			$this->db_object->get_id() );
		printf( "<input type='hidden' name='cp_form' value='1'>\n" );

		html_table_header( 25, 150, 625 );
		html_table_section( $title, design_cp_form_menu( $this->db_object->get_id()) );
	
		html_oblig_row( '*', get_string('CIType_ttl') );
		printf( "<td class='f_value'>" );
		if ($this->db_object->get_id() == 0)
		{
			ci_type_list::getInstance()->html_options( array( $this->db_object->get_cit_id() ), array(), "cp_cit_id", array( 0 => "--- Select please ---" ) );
		}
		else
		{
			$ci_type = ci_type_list::getInstance()->get( $this->db_object->get_cit_id() );
			printf( "%s", $ci_type->get_title() );
		}
		printf( "</td></tr>\n" );

		html_oblig_row( '*', get_string('Title_ttl') );
		printf( "<td class='f_value'><input type='text' name='cp_title' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 200, html_entity_decode( $this->db_object->get_title(), ENT_QUOTES, 'ISO-8859-1' ) );

		html_oblig_row( '', get_string('Description_ttl') );
		printf( "<td class='f_value'><textarea name='cp_desc' cols='%d' rows='%d'>%s</textarea></td></tr>\n",
			50, 5, html_entity_decode( $this->db_object->get_desc(), ENT_QUOTES, 'ISO-8859-1' ) );

		html_oblig_row( '*', get_string('FieldSeparator_ttl') );
		printf( "<td class='f_value'><input type='text' name='cp_separator' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			1, 1, html_entity_decode( $this->db_object->get_separator(), ENT_QUOTES, 'ISO-8859-1' ) );

		html_oblig_row( '*', get_string('StringDelimiter_ttl') );
		printf( "<td class='f_value'><input type='text' name='cp_delimiter' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			1, 1, html_entity_decode( $this->db_object->get_delimiter(), ENT_QUOTES, 'ISO-8859-1' ) );

		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );

		return;
	}
	
	/**
	 * display HTML form for delete question
	 * 
	 * @param	string		$action			page to process form request
	 * @param	string		$action_text	text for form button
	 * @return	none
	 */
	public function display_question ( $action )
	{
		global $referer_script_name;

		$yes_args = new html_form_args();
		$yes_args->add( "cp_id", $this->db_object->get_id() );
	
		$no_args = new html_form_args();
		
		printf( "<div id='question'>\n" );
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('DeleteCSVProfile_ttl'), design_cp_form_menu($this->db_object->get_id()) );
		html_table_footer();

		html_form_question( 0, 0, 
			get_string('WandDeleteCSVProfile_dsc'),
			get_string('YES_btn'), $action, $yes_args,
			get_string('NO_btn'), $referer_script_name, $no_args );
		printf( "</div>\n" );

		return;
	}

	/**
	 * display specification of CSV-profile
	 */
	public function display_spec ()
	{
		printf( "<div id='spec'>\n" );
		html_table_header( 25, 150, 625 );

		html_table_section( get_string('Specification_ttl'), design_cp_spec_menu( $this->get_id() ) );

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('CIType_ttl'), ci_type_list::getInstance()->get( $this->db_object->get_cit_id() )->get_title() ); 

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('Title_ttl'), $this->db_object->get_title() ); 

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('Description_ttl'), $this->db_object->get_desc() ); 

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('FieldSeparator_ttl'), $this->db_object->get_separator() ); 

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('StringDelimiter_ttl'), $this->db_object->get_delimiter() ); 

		html_table_footer();
		printf( "</div>\n" );

		return;
	}

	/**
	 * display elements of CSV-profile
	 */
	public function display_elements ()
	{
		$element_count = $this->elements->count();
		$ta_list = & type_attr_list::getInstance( $this->db_object->get_classtype(), $this->db_object->get_cit_id() );

		printf( "<div id='elements'>\n" );
		printf( "<form action='%s' method='post'>\n",
			CP_CE_UPDATE_RUN_PHP );
		printf( "<input type='hidden' name='cp_id' value='%s'>\n",
			$this->db_object->get_id() );
		printf( "<input type='hidden' name='cp_form' value='2'>\n" );

		$section_title = get_string('Elements_ttl') . " (" . get_string('Total_ttl') . ": " . $element_count . ")";
		html_table_header( 50, 100, 550, 100 );
		html_table_section( $section_title, design_ce_index_menu( $this->get_id() ) );

		print ("<tr class='r_head'>");
		printf("<td class='c_action r_head'>%s</td>", get_string('Order_ttl') );
		printf("<td class='c_action r_head'>%s</td>", get_string('Position_ttl') );
		printf("<td class='c_text r_head'>%s</td>", get_string('Attribute_ttl') );
		printf("<td class='c_action r_head'>%s</td>", get_string('Actions_ttl') );
		print ("</tr>\n");

		if ($element_count == 0)
		{
			html_empty_row( 0, 0 );
		}
		else
		{ 
			html_table_row_reset( array("r_normal","r_normal2") );
			for ($i=0; $i<$element_count; $i++)
			{
				$ce = & $this->elements->at( $i );
		
				html_table_row();
				printf( "<td class='c_action'>%s %s</td>" .
					"<td class='c_action'>%d</td>" .
					"<td class='c_text'>",
					icon_display_url( sprintf( "%s?cp_id=%d&amp;pos1=%d&amp;pos2=%d", CP_CE_SWAP_RUN_PHP, $this->db_object->get_id(), $i, $i-1 ), ICON_UP, get_string('MoveUp_dsc') ),
					icon_display_url( sprintf( "%s?cp_id=%d&amp;pos1=%d&amp;pos2=%d", CP_CE_SWAP_RUN_PHP, $this->db_object->get_id(), $i, $i+1 ), ICON_DOWN, get_string('MoveDown_dsc') ),
					$i
				);
				$ta_list->html_options( array( $ce->get_ta_id() ), array(), "ce" . $i, array( CE_TA_SYSID => "SYS-ID", CE_TA_IGNORE => "---" ) );
				printf( "</td>" .
					"<td class='c_action'>%s %s</td></tr>\n",
					icon_display_url( sprintf( "%s?cp_id=%d&amp;ce_id=%d", CP_CE_DELETE_RUN_PHP, $this->db_object->get_id(), $ce->get_id() ), ICON_DELETE, get_string('DeleteElement_dsc') ),
					icon_display_url( sprintf( "%s?cp_id=%d&amp;ce_pos=%d", CP_CE_CREATE_RUN_PHP, $this->db_object->get_id(), $i+1 ), ICON_ADD, get_string('AddElementBehind_dsc') )
					);
			}
		}
		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			get_string('Update_btn') );
		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );

		return;
	}

	// actions

	/**
	 * add new element at position $_pos in list
	 * 
	 * @param	integer		$_pos
	 */
	public function action_add_element ( $_pos )
	{
		$rc = FALSE;
		try {
			// create
			$ce = new db_csv_element();
			$ce->set_cp_id( $this->db_object->get_id() );
			$ce->db_insert();

			// insert into list
			$this->elements->add_to_list( $ce, $_pos );

			// update elements
			$this->elements->db_update();
			
			$rc = TRUE;
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		catch (core_data_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('ValidationError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		return $rc;
	}

	/**
	 * update content of all list elements to database
	 */
	public function action_update_elements ()
	{
		$rc = FALSE;
		try {
			// update elements
			$rc = $this->elements->db_update();
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		catch (core_data_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('ValidationError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		return $rc;
	}

	/**
	 * delete element with $ce_id from list
	 * 
	 * @param	integer		$ce_id
	 * @return	boolean		status
	 */
	public function action_delete_element ( $ce_id )
	{
		$rc = FALSE;
		try {
			$ce = & $this->elements->remove_from_list( $ce_id );
			
			// delete
			$ce->db_delete();

			// update elements
			$this->elements->db_update();
			
			$rc = TRUE;
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		catch (core_data_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('ValidationError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		return $rc;
	}

	/**
	 * swap elements at $_pos1 and $_pos2 in list
	 * 
	 * @param	integer		$_pos1
	 * @param	integer		$_pos2
	 * @return	boolean		status
	 */
	public function action_swap_elements ( $_pos1, $_pos2 )
	{
		$rc = FALSE;
		try {
			// swap elements
			$this->elements->swap( $_pos1, $_pos2 );

			// update
			$this->elements->db_update();

			$rc = TRUE;
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		catch (core_data_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('ValidationError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		return $rc;
	}

	// helper
	
	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$cit_title = "-";
		if ($this->db_object->get_cit_id() != 0)
		{
			$ci_type = ci_type_list::getInstance()->get( $this->db_object->get_cit_id() );
			$cit_title = $ci_type->get_title();
		}
		$title = sprintf( "%s: '%s - %s'", get_string('CSVProfile_ttl'), $cit_title, $this->db_object->get_title() );
		return $title;
	}

	/**
	 * get a HTML string including the icon for the
	 * synopse section
	 * 
	 * @return		string		HTML code including an icon 
	 */
	protected function get_synopse_icon_html ()
	{
		$icon_html = "";
		return $icon_html;
	}

	// pre- / post- actions
	
	/**
	 * post insert actions
	 * 
	 * 
	 */
	protected function insert_post ()
	{
		$ta_list = & type_attr_list::getInstance( $this->db_object->get_classtype(), $this->db_object->get_cit_id() );
		$ta_count = $ta_list->count();
		
		// add fields for all attributes
		for ($i=0; $i<$ta_count; $i++)
		{	
			// create
			$ce = new db_csv_element();
			$ce->set_cp_id( $this->db_object->get_id() );
			$ce->set_ta_id( $ta_list->at( $i )->get_id() );
			$ce->db_insert();

			// insert into list
			$this->elements->add_to_list( $ce, $i );
		}

		// add SYS-ID at top
		$ce = new db_csv_element();
		$ce->set_cp_id( $this->db_object->get_id() );
		$ce->set_ta_id( CE_TA_SYSID );
		$ce->db_insert();
		$this->elements->add_to_list( $ce, 0 );

		// update elements
		$this->elements->db_update();

		return;
	}

	/**
	 * pre delete actions
	 * 
	 * 
	 */
	public function delete_pre ()
	{
		// delete all elements of this profile
		$db = core_db::getInstance();
		$query = "DELETE FROM csv_element " .
			"WHERE ce_cp_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query );
		
		// destroy list
		$this->elements = null;
		
	}

	// validatoren fuer actions
	
	/**
	 * check validation for insert
	 * 
	 * reply already TRUE, otherwise throw an exception
	 * 
	 * @return	bool 
	 */
	public function valid_for_insert ()
	{
		return TRUE;
	}

	/**
	 * check validation for update
	 * 
	 * reply already TRUE, otherwise throw an exception
	 * 
	 * @return	bool 
	 */
	public function valid_for_update ()
	{
		return TRUE;
	}

	/**
	 * check validation for delete
	 * 
	 * reply already TRUE, otherwise throw an exception
	 * 
	 * @return	bool 
	 */
	public function valid_for_delete ()
	{
		return TRUE;
	}

	/**
	 * create a new object with $_id
	 * 
	 * @param	integer			$_id
	 * @return	db_csv_profile
	 */
	protected function create_db_object ( $_id )
	{
		$this->db_object = new db_csv_profile( $_id );
		return;
	}

}
