<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'tables/db_ci_type.inc.php';
require_once 'tables/db_csv_profile.inc.php';
require_once 'includes/navi_list.inc.php';

class csv_profile_list extends navi_list
{
	static private $instance = false;
	protected $cit_id = 0;
	
	static public function getInstance( $_cit_id = 0 )
	{
		if (self::$instance === false) self::$instance = new csv_profile_list( $_cit_id );
		return self::$instance;
	}

	protected function __construct ( $_cit_id = 0 )
	{
		$this->cit_id = $_cit_id;
		$this->page_size = get_property("page_rows");
		parent::__construct();
	}

	protected function get_session_store_name ()
	{
		return "csv_profile_list" . $this->cit_id;
	} 

	protected function get_element_name ()
	{
		return get_string('CSVProfile_ttl');
	} 
	
	protected function get_id_name ()
	{
		return "cp_id";
	}

	public function show_page ()
	{
		html_table_header( 100, 200, 450, 50 );
		switch ($this->get_list_mode())
		{
			case LIST_MODE_USE:
			{
			printf("<tr class='r_head'>\n");
			printf("	<td class='c_text r_head'>".get_string('CIType_ttl')."</td>\n");
			printf("	<td class='c_text r_head'>".get_string('Title_ttl')."</td>\n");
			printf("	<td class='c_text r_head'>".get_string('Description_ttl')."</td>\n");
			printf("	<td class='c_action r_head'>".get_string('Actions_ttl')."</td>\n");
			printf("</tr>\n");
				break;
			}
			case LIST_MODE_MANAGE:
			{
			printf("<tr class='r_head'>\n");
			printf("	<td class='c_text r_head'>".get_string('CIType_ttl')."</td>\n");
			printf("	<td class='c_text r_head'>".get_string('Title_ttl')."</td>\n");
			printf("	<td class='c_text r_head'>".get_string('Description_ttl')."</td>\n");
			printf("	<td class='c_action r_head'>".get_string('Actions_ttl')."</td>\n");
			printf("</tr>\n");
				break;
			}
		}
		
		parent::show_page();
		html_table_footer();

		return;
	}	

	protected function show_item ( $i )
	{
		$csv_profile = & $this->at( $i );
		$ci_type = new db_ci_type( $csv_profile->get_cit_id() );
		$icon_size  = get_property("list_icon_size");

		html_table_row();

		switch ($this->get_list_mode())
		{
			case LIST_MODE_USE:
			{		
				printf(
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_action'></td>" .
					"</tr>\n",
					get_html_url(
						sprintf( "%s?cit_id=%d", CI_LIST_PHP, $ci_type->get_id()),
						icon_list::getInstance()->get_html_imx( $ci_type->get_icon(), $icon_size, $icon_size, TRUE ) . " " . $ci_type->get_title()
						),
					$ci_type->get_desc()
					);
				break;
			}
			case LIST_MODE_MANAGE:
			{		
				printf(
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'><a href='%s'>%s</a></td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_action'>%s %s</td>" .
					"</tr>\n",
					icon_list::getInstance()->get_html_imx( $ci_type->get_icon(), $icon_size, $icon_size, TRUE ) . " " . $ci_type->get_title(),
					CP_VIEW_PHP . "?cp_id=" . $csv_profile->get_id(), $csv_profile->get_title(),
					$csv_profile->get_desc(),
					icon_display_url(CP_EDIT_PHP . "?cp_id=" . $csv_profile->get_id(), ICON_EDIT, get_string('EditCSVProfile_dsc') ),
					icon_display_url(CP_DELETE_PHP . "?cp_id=" . $csv_profile->get_id(), ICON_DELETE, get_string('DeleteCSVProfile_dsc') )
					);
				break;
			}
		}
	}

	protected function get_query ()
	{
		$query = array();
		if ($this->cit_id == 0)
		{
			$query[] = "SELECT * FROM csv_profile " .
				"ORDER BY cp_cit_id, cp_title";
		}
		else
		{
			$query[] = "SELECT * FROM csv_profile " .
				"WHERE cp_cit_id='" . $this->cit_id . "' " .
				"ORDER BY cp_cit_id, cp_title";
		}
		return $query;
	}
	
	protected function create_object ( $_id )
	{
		$db_object = new db_csv_profile( $_id ); 
		return $db_object;
	}
		
	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$title = get_string('AllCSVProfiles_ttl');
		return $title;
	}

} 
