<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
 * modul includes - set db schemata
 * 
 * @category	php-cmdb
 * @package		includes
 * @author		mscheibe
 */
class db_setup
{

	/**
	 * create table in schema
	 * 
	 * @param	core_db_mysql	$db			db object			
	 * @param	string	$schema_file		name of file with sql create statements
	 * @param	string	$table_name			name of table to create
	 * @return	none
	 */
	static public function createTable( core_db_mysql &$db, $schema_file, $table_name )
	{
		$create	= self::getCreateString( $schema_file, $table_name );
		if (strlen($create)==0)
			throw new exception( "No create statement foudn for table '$table_name'!" );

		$drop	= "DROP TABLE IF EXISTS `" . $table_name . "`";
		$result	= $db->query( $drop );
		$result = $db->query( $create );
		
		return;
	}

	/**
	 * fetch table create string from sql file
	 * 
	 * @param	string	$schema_file		name of file with sql create statements
	 * @param	string	$table_name			name of table to create
	 * @return	string
	 */
	static protected function getCreateString( $schema_file, $table_name )
	{
		$create_string = "";
		$table_tag = "`" . $table_name . "`";
		$found = false;
		$line_array = file( $schema_file );
		$line_count = count( $line_array );
		$line_num	= 0;
		while ($line_num < $line_count)
		{
			$line = trim( $line_array[ $line_num ]);
			if (! $found)
			{
				$line_tags = explode( " ", $line );
				if ( ($line_tags[0] == "CREATE") &&
					 ($line_tags[1] == "TABLE")
					)
				{
					switch (count($line_tags))
					{
						case 4:
						{
							if (($line_tags[2] == $table_tag) || ($line_tags[2] == $table_name))
					    	{
								$found = true;
								$create_string .= $line;
					    	}
							break;
						}
						case 7:
						{
							if (($line_tags[5] == $table_tag) || ($line_tags[5] == $table_name))
					    	{
								$found = true;
								$create_string .= $line;
					    	}
							break;
						}
					}
				}
			}
			else
			{
				if (($pos = strpos( $line, ";")) !== FALSE)
				{
					$line = substr( $line, 0, $pos );
					$line_num = $line_count;
				}	
				$create_string .= $line;
			}
			$line_num++;
		}
		return $create_string;
	}

}

