<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'tables/db_typed_ci.inc.php';
require_once 'tables/db_typed_relation.inc.php';

define ('GRAPHVIZ_DPI',			72 );

/*
 * base class for graph generation of CIs and relations with graphviz
 */
class graphviz
{
	/**
	 * list of all CIs
	 * 
	 * ci_list[<ci-id>] => db_typed_ci
	 */
	protected $ci_list				= array();
	
	/**
	 * list of all relations
	 * 
	 * r_list[<r-id>] => db_typed_relation
	 */
	protected $r_list				= array();

	/**
	 * scale factor
	 */
	protected $page_scale			= 1;
	
	/**
	 * page setup in inch
	 */
	protected $page_x_i				= 0;
	protected $page_y_i				= 0;
	protected $margin_i				= 0.5;
	protected $text_font			= "";

	/**
	 * page setup in pixel
	 */
	protected $page_x_p				= 0;
	protected $page_y_p				= 0;
	protected $size_x_p				= 0;
	protected $size_y_p				= 0;
	protected $margin_p				= 0;
	
	/**
	 * style values (base, in pixel) 
	 */
	protected $arrow_length_b		= 24;
	protected $padding_b			= 12;
	protected $text_size_b			= 24;
	protected $icon_size_b			= 24;

	/**
	 * style values (scaled, in pixel)
	 */
	protected $arrow_length_p		= 0;
	protected $padding_p			= 0;
	protected $text_size_p			= 0;
	protected $icon_size_p			= 0;

	/**
	 * box values
	 */
	protected $box_y_i				= 0;
	protected $box_y_p				= 0;

	/**
	 * center / root node
	 */
	protected $center_ci_id			= 0;
	

	public function __construct ()
	{
		return;
	}

	/**
	 * get the count of typed_ci objects in the image
	 */
	public function get_ci_count ()
	{
		return count($this->ci_list);
	}
	
	/**
	 * get the count of typed_relation objects in the image
	 */
	public function get_relation_count ()
	{
		return count($this->r_list);
	}

	/**
	 * get the scale factor from .plain-file
	 */
	public function get_page_scale ()
	{
		return $this->page_scale;
	}
	
	/**
	 * add a typed_ci object to the image
	 */
	public function add_typed_ci ( &$_typed_ci )
	{
		$this->ci_list[ $_typed_ci->get_id() ] = $_typed_ci;
		return;
	}

	/**
	 * add a typed_relation object to the image
	 */
	public function add_typed_relation ( &$_typed_relation )
	{
		$this->r_list[ $_typed_relation->get_id() ] = $_typed_relation;
		return;
	}

	/**
	 * get 
	 */
	public function get_box_y_p ()
	{
		return $this->box_y_p;
	}

	public function get_box_y_i ()
	{
		return $this->box_y_i;
	}

	public function get_box_x_p ( $_text )
	{
		$box_sx		= 3 * $this->padding_p + $this->get_text_x_p( $_text ) + $this->icon_size_p;
		return $box_sx;
	}

	public function get_box_x_i ( $_text )
	{
		$box_sx = $this->get_box_x_p( $_text );
		return  $box_sx / GRAPHVIZ_DPI / $this->page_scale;
	}


	/**
	 * process graph
	 */
	public function process ( $_center_ci_id=0 )
	{
		set_time_limit( 60 );

		// store id of center/root node
		$this->center_ci_id = $_center_ci_id;
		
		// get temp file names
		$session_id = core_session::getInstance()->get_session_id();
		$dot_filename	= sprintf( "%s/%s%s.dot", get_property("cache_dir"), get_property("ext_prefix"), $session_id ); 
		$plain_filename	= sprintf( "%s/%s%s.plain", get_property("cache_dir"), get_property("ext_prefix"), $session_id ); 

		// 
		$this->generate_dot_file( $dot_filename );

		//
		$this->process_dot_file( $dot_filename, $plain_filename );
		
		//
		$this->process_plain_file( $plain_filename );

		// delete temp files
//		unlink($dot_filename);
//		unlink($plain_filename);

		return;
	}

	// protecteds

	/**
	 * generate a .dot file for processing with graphviz 
	 */
	protected function generate_dot_file ( $filename )
	{
		// open file
		$dot_filehandle = fopen($filename, "w");
		
		// write graph configuration
		fwrite($dot_filehandle, "graph P {\n" );
		fwrite($dot_filehandle, sprintf("dpi=\"%d\";\n", GRAPHVIZ_DPI ) );
		fwrite($dot_filehandle, sprintf("margin=\"%.2f\";\n", $this->margin_i ) );

		fwrite($dot_filehandle, "overlap=\"false\";\n" );
		fwrite($dot_filehandle, "model=\"subset\";\n" );
//		fwrite($dot_filehandle, "epsilon=0.001;\n" );
//		fwrite($dot_filehandle, "maxiter=1000;\n" );
//		fwrite($dot_filehandle, "nodesep=\".5\";\n" );
//		fwrite($dot_filehandle, "sep=\".5\";\n" );
		
		// write all CIs from our list
		foreach ($this->ci_list as $id => $ci )
		{
			fwrite($dot_filehandle, sprintf( "%08d [shape=box,fixedsize=true,height=%.3f,width=%.3f,label=\"%s\"];\n",
				$ci->get_id(),
				$this->get_box_y_i(),
				$this->get_box_x_i( $ci->get_title() ),
				$ci->get_title() )
				);
		}
		
		// write all relations from our list
		foreach ($this->r_list as $id => $r)
		{
			fwrite($dot_filehandle, sprintf( "%08d -- %08d [dir=forward];\n",
				$r->get_ci1_id(),
				$r->get_ci2_id() )
				); 
		}
		
		// close
		fwrite($dot_filehandle, "}\n" );
		fclose($dot_filehandle);

		return;
	}

	/**
	 * processing .dot-file with graphviz and create a .plain-file
	 */
	protected function process_dot_file ( $dot_filename, $plain_filename )
	{
		// create shell-command
		$rc		= 0;
		$cmd	= sprintf( "\"%s\" -Tplain %s -o %s", get_property("graphviz_neato"), $dot_filename, $plain_filename );
		$output	= "";
		
		// run shell-commmand
		exec( $cmd, $output, $rc );

		return $rc;
	}

	/**
	 * process .plain-file, create and render image 
	 */
	protected function process_plain_file ( $_file )
	{
		$plain_array = file( $_file );
		foreach ($plain_array as $plain_line)
		{
			// use strtok instead of explode, to avoid empty fields
			$plain_items = array();
			$tok = strtok( $plain_line, ' ' );
			while ($tok !== false)
			{
				$plain_items[] = $tok;
				$tok = strtok( ' ' );
			}

			switch (trim($plain_items[0]))
			{
				case 'graph':
				{
					$this->process_line_graph( $plain_items );
					break;
				}
				case 'node':
				{
					$this->process_line_node( $plain_items );
					break;
				}
				case 'edge':
				{
					$this->process_line_edge( $plain_items );
					break;
				}
				case 'stop':
				{
					$this->process_line_node_post();
					$this->process_line_stop( $plain_items );
					break;
				}
			}
		}
		

		return;
	}
	
	/**
	 * process graph line from plain file
	 * 
	 * items[0]	= graph
	 * items[1]	= <scale>
	 * items[2] = <page size x>
	 * items[3] = <page size y>
	 * 
	 * example: graph 1.000 24.931 23.514
	 * 
	 * @param	array	$items				array with line elements
	 * @return	none
	 */
	protected function process_line_graph( $items )
	{
		if ($this->processing()) return;

		// read args
		$this->rescale_page( (float)$items[1], (float)$items[2], (float)$items[3] );	
		$this->rescale_size();
		$this->rescale_style();

		return;
	}

	/**
	 * process node line from plain file
	 * 
	 * items[0]	= node
	 * items[1]	= <name>
	 * items[2] = <center x>
	 * items[3] = <center y>
	 * items[4] = <size x>
	 * items[5] = <size y>
	 * 
	 * example: node 00000302  12.667 12.042 2.333 0.444 "SuSE on flat-server" solid box black lightgrey
	 * 
	 * 
	 * @param	array	$items				array with line elements
	 * @return	none
	 */
	protected function process_line_node( $items )
	{
		if (! $this->processing()) return;

		$ci_id = (int)$items[1];
		$ci = $this->ci_list[ $ci_id ];

		$ci->cx = (int)floor((float)$items[2] * $this->page_scale * GRAPHVIZ_DPI);
		$ci->cy = (int)floor((float)$items[3] * $this->page_scale * GRAPHVIZ_DPI);

		$ci->bx = (int)floor((float)$items[4] * $this->page_scale * GRAPHVIZ_DPI);
		$ci->by = (int)floor((float)$items[5] * $this->page_scale * GRAPHVIZ_DPI);

		return;
	}

	protected function process_line_node_post()
	{
		if (! $this->processing()) return;

		// all CIs from our list
		foreach ($this->ci_list as $id => $ci )
		{
			$cx = $ci->cx;
			$cy = $ci->cy;
	
			$bx = $ci->bx;
			$by = $ci->by;
			$hx = (int)floor($bx / 2);
			$hy = (int)floor($by / 2);

			$this->draw_node( $cx, $cy, $bx, $by, $hx, $hy, $id );
		}

		return;
	}

	/**
	 * process edge line from plain file
	 * 
	 * items[0]	= edge
	 * items[1]	= <name 1>
	 * items[2] = <name 2>
	 * items[3] = <number of points>
	 * items[4] = <point1 x>
	 * items[5] = <point1 y>
	 * items[6] = <point2 x>
	 * items[7] = <point2 y>
	 * 
	 * example: edge 00000302 00000303 4 13.125 12.264 14.833 13.111 20.778 15.986 22.694 16.903 solid black
	 * 
	 * @param	array	$items				array with line elements
	 * @return	none
	 */
	protected function process_line_edge( $items )
	{
		if (! $this->processing()) return;
		
		$count = (int)$items[3];
		$x1		= (int)floor((float)$items[4] * $this->page_scale * GRAPHVIZ_DPI);
		$y1		= (int)floor((float)$items[5] * $this->page_scale * GRAPHVIZ_DPI);
		$ii		= 6;
		for ($i=2; $i<=$count; $i++)
		{
			$x2	= (int)floor((float)$items[$ii++] * $this->page_scale * GRAPHVIZ_DPI);
			$y2	= (int)floor((float)$items[$ii++] * $this->page_scale * GRAPHVIZ_DPI);
			
			$this->draw_line(
				$this->margin_p + $x1, $this->margin_p + $y1,
				$this->margin_p + $x2, $this->margin_p + $y2,
				($i == $count)
				);

			$x1 = $x2;
			$y1 = $y2;
		}		
		
		return;
	}

	protected function process_line_stop ( $items )
	{
		if (! $this->processing()) return;

		return;
	}

	// protected
	
	/**
	 * setup page with inch based values
	 * 
	 * @param $_x_i			page width in inch
	 * @param $_y_i			page height in inch
	 * @param _margin_i		page margin in inch 
	 */
	protected function setup_page_by_inch ( $_x_i, $_y_i, $_margin_i )
	{
		$this->page_x_i = $_x_i;
		$this->page_y_i = $_y_i;
		$this->margin_i = $_margin_i;
		$this->setup_page_from_inch();
		return;
	}
	
	/**
	 * setup page with pixel based values
	 * 
	 * @param $_x_p			page width in pixel
	 * @param $_y_p			page height in pixel
	 * @param _margin_p		page margin in pixel
	 */
	protected function setup_page_by_pixel ( $_x_p, $_y_p, $_margin_p )
	{
		$this->page_x_p = $_x_p;
		$this->page_y_p = $_y_p;
		$this->margin_p = $_margin_p;
		$this->setup_page_from_pixel();
		return;
	}
	

	/**
	 * setup style values (all in pixel)
	 */
	protected function setup_style ( $_text_font, $_text_size, $_padding, $_arrow_length, $_icon_size )
	{
		$this->text_font		= $_text_font;

		$this->arrow_length_b	= $_arrow_length;
		$this->padding_b		= $_padding;
		$this->text_size_b		= $_text_size;
		$this->icon_size_b		= $_icon_size;

		$this->rescale_style();

		return;
	}	

	/**
	 * get width of text in pixel 
	 */
	protected function get_text_x_p ( $_text )
	{
		return 0;
	}

	/**
	 * get height of text in pixel 
	 */
	protected function get_text_y_p ( $_text )
	{
		return 0;
	}

	/**
	 * return true, if output device/resource for processing
	 * is usable
	 */
	protected function processing()
	{
		return FALSE;
	}

	protected function draw_node ( $_cx, $_cy, $_bx, $_by, $_hx, $_hy, $_id )
	{
		$ci = $this->ci_list[ $_id ];
		$ci_title = $ci->get_title();
		$ci_icon = $ci->get_icon();
		$icon = icon_list::getInstance()->get_icon_image( $ci_icon, $this->icon_size_b );
		$icon_ss = 1;

		return;
	}
	
	protected function draw_line ( $_x1, $_y1, $_x2, $_y2, $_last )
	{
		return;
	}

	/**
	 * rescale style values (in pixel) from base
	 */
	protected function rescale_style ()
	{
		if (get_property("graphviz_restyle"))
		{
			$this->arrow_length_p	= (int)floor($this->arrow_length_b * $this->page_scale);
			$this->padding_p		= (int)floor($this->padding_b * $this->page_scale);
			$this->text_size_p		= (int)floor($this->text_size_b * $this->page_scale);
			$this->icon_size_p		= (int)floor($this->icon_size_b * $this->page_scale);
		}
		else
		{
//			$this->arrow_length_p	= (int)floor($this->arrow_length_b * $this->page_scale);
			$this->arrow_length_p	= $this->arrow_length_b;
			$this->padding_p		= $this->padding_b;
			$this->text_size_p		= $this->text_size_b;
			$this->icon_size_p		= $this->icon_size_b;
		}

		$this->box_y_p			= 2 * $this->padding_p + max( $this->get_text_y_p('X'), $this->icon_size_p );
		$this->box_y_i			= $this->box_y_p / GRAPHVIZ_DPI / $this->page_scale;

		return;
	}

	// privates

	/**
	 * setup page values (in pixel) from inch values
	 */
	private function setup_page_from_inch ()
	{
		$this->page_x_p = (int)floor($this->page_x_i * GRAPHVIZ_DPI);
		$this->page_y_p = (int)floor($this->page_y_i * GRAPHVIZ_DPI);
		$this->margin_p = (int)floor($this->margin_i * GRAPHVIZ_DPI);
		
		$this->size_x_p = $this->page_x_p - 2 * $this->margin_p;
		$this->size_y_p = $this->page_y_p - 2 * $this->margin_p;

		return;
	}

	/**
	 * setup page values (in inch) from pixel values
	 */
	private function setup_page_from_pixel ()
	{
		$this->page_x_i = $this->page_x_p / GRAPHVIZ_DPI;
		$this->page_y_i = $this->page_y_p / GRAPHVIZ_DPI;
		$this->margin_i = $this->margin_p / GRAPHVIZ_DPI;
		
		$this->size_x_p = $this->page_x_p - 2 * $this->margin_p;
		$this->size_y_p = $this->page_y_p - 2 * $this->margin_p;

		return;
	}


	/**
	 * recalculate page_scale from arguments (layout result from graphviz-tools)
	 * 
	 * @param $_s			scale factor
	 * @param $_x			width in inch
	 * @param $_y			height in inch
	 */
	private function rescale_page ( $_s, $_x, $_y )
	{
		// rotate internal, if necessary
		if (($_x > $_y) && ($this->page_x_i < $this->page_y_i))
		{
			$z_i = $this->page_x_i;
			$this->page_x_i = $this->page_y_i;
			$this->page_y_i = $z_i;
		}

		// calc scale factor from external args to internal page
		$fx = ($this->page_x_i - 2 * $this->margin_i) / $_x;
		$fy = ($this->page_y_i - 2 * $this->margin_i) / $_y;
		$ff = min( $fx, $fy );

		// set page scale
		$this->page_scale = $ff * $_s;

//echo "DEBUG: rescale_page( $_s, $_x, $_y ): $this->page_scale<br>\n";
		
		return;
	}

	/**
	 * rescale page values (in pixel) from inch
	 */
	private function rescale_size ()
	{
		$this->page_x_p = (int)floor($this->page_x_i * GRAPHVIZ_DPI);
		$this->page_y_p = (int)floor($this->page_y_i * GRAPHVIZ_DPI);
		$this->margin_p = (int)floor($this->margin_i * GRAPHVIZ_DPI);
		
		$this->size_x_p = $this->page_x_p - 2 * $this->margin_p;
		$this->size_y_p = $this->page_y_p - 2 * $this->margin_p;
	
//echo "DEBUG: rescale_size(): $this->page_x_p x $this->page_y_p<br>\n";
	
		return;
	}

}
