<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'includes/graphviz.inc.php';
require_once 'includes/image_helper.inc.php';

/*
 * klasse zur erstellung eines GD-Images aus einer
 * graphviz plain datei
 */
class graphviz_image extends graphviz
{
	protected $image				= null;
	protected $color_back			= null;
	protected $color_text			= null;
	protected $color_red			= null;
	protected $color_grey			= null;
	protected $png_name				= "";
	protected $map_name				= "";
	protected $map_handle			= null;
	
	public function __construct (  $_px, $_py, $_text_font, $_text_size, $_padding, $_arrow_length, $_icon_size )
	{
		parent::__construct();
		$this->setup_page_by_pixel( $_px, $_py, $_padding );
		$this->setup_style( $_text_font, $_text_size, $_padding, $_arrow_length, $_icon_size );

		$this->create_names();

		return;
	}

	public function process ( $_center_ci_id=0 )
	{
		parent::process( $_center_ci_id );

		// store
		ImagePng( $this->image, $this->png_name );

		// close processing
		imagedestroy( $this->image );
		$this->image = null;

		return;
	}

	/**
	 * deliver image as png
	 */
	public function display ()
	{
		if (! $this->exists() ) return;

		// ausliefern
		header ("Content-type: image/png");
		readfile( $this->png_name );

		return;
	}
	
	/**
	 * deliver map as stream
	 */
	public function get_map ()
	{
		if (! $this->exists() ) return;

		// deliver
		readfile( $this->map_name );
		
		return;
	}

	// protecteds

	/**
	 * ermittelt, ob die image datei existiert
	 */
	protected function exists () 
	{
		clearstatcache();
		return (file_exists( $this->png_name ) && file_exists( $this->map_name ));
	}

	protected function process_line_graph ( $items )
	{
		if ($this->processing()) return;
		parent::process_line_graph( $items );

		// create image
		$this->image = imagecreatetruecolor( $this->page_x_p, $this->page_y_p );
		if ($this->image == null) return;
		
		$this->color_back = ImageColorAllocate ($this->image, 255, 255, 255);
		$this->color_text = ImageColorAllocate ($this->image,   0,   0,   0);
		$this->color_red  = ImageColorAllocate ($this->image, 127,   0,   0);
		$this->color_grey = ImageColorAllocate ($this->image, 127, 127, 127);
		imagefill( $this->image, 0, 0, $this->color_back );

		// create map file
		$this->map_handle = fopen( $this->map_name, "w" );
		fwrite( $this->map_handle, "<map name='map_locations'>\n" );

		return;
	}


	protected function process_line_stop ( $items )
	{
		if (! $this->processing()) return;

		// close map file
		fwrite( $this->map_handle, "</map>\n" );
		fclose( $this->map_handle );
		$this->map_handle = null;

		parent::process_line_stop( $items );

		return;
	}

	/**
	 * get width of text in pixel 
	 */
	protected function get_text_x_p ( $_text )
	{
		$box = imageftbbox( $this->text_size_p, 0, $this->text_font, $_text );
		$_width = abs( $box[2] - $box[0] );
		return $_width;
	}

	/**
	 * get height of text in pixel 
	 */
	protected function get_text_y_p ( $_text )
	{
		$box = imageftbbox( $this->text_size_p, 0, $this->text_font, $_text );
		$_height = abs( $box[3] - $box[5] );
		return $_height;
	}

	/**
	 * return true, if output device/resource for processing
	 * is usable
	 */
	protected function processing()
	{
		return ($this->image != null);
	}

	protected function draw_node ( $_cx, $_cy, $_bx, $_by, $_hx, $_hy, $_id )
	{
		$ci = $this->ci_list[ $_id ];
		$ci_title = $ci->get_title();
		$ci_icon = $ci->get_icon();

//		// draw icon and text
//		$ci_box = image_string_box (
//			$this->image,
//			$this->text_font,
//			$this->padding_p,
//			$_cx, $_cy,
//			$ci_title,
//			$ci_icon,
//			$this->color_text,
//			$this->color_grey,
//			$this->color_back,
//			$this->icon_size_p );

		// draw icon and text
		$ci_box = image_ft_box (
			$this->image,
			$this->text_font,
			$this->text_size_p,
			$this->padding_p,
			$_cx, $_cy,
			$ci_title,
			$ci_icon,
			$this->color_text,
			$this->color_grey,
			$this->color_back,
			$this->icon_size_p );

		// insert into map file
		fwrite( $this->map_handle, sprintf("<area shape='rect' coords='%d,%d,%d,%d' href='%s?ci_id=%d' alt='%s' title='%s'>\n",
			$ci_box[0], $ci_box[1], $ci_box[2], $ci_box[3], CI_VIEW_EXT_PHP, $_id, $ci_title, $ci_title ) );

		// draw frame
		if ($this->center_ci_id == $_id)
		{
			image_rectanglebox ( $this->image, $ci_box, $this->color_red );
		}

		return;
	}
	
	protected function draw_line ( $_x1, $_y1, $_x2, $_y2, $_last )
	{
		if ($_last)
		{
			image_arrow( $this->image, $_x1, $_y1, $_x2, $_y2, $this->arrow_length_p, $this->color_grey );
//			imageline( $this->image, $_x1, $_y1, $_x2, $_y2, $this->color_red );
		}
		else
		{
			imageline( $this->image, $_x1, $_y1, $_x2, $_y2, $this->color_grey );
		}

		return;
	}

	// privates

	/**
	 * liefert den vollstaendigen dateinamen
	 * des images im cache-verzeichnise
	 */
	private function create_names ()
	{
		$session_id = core_session::getInstance()->get_session_id();
		$this->png_name = sprintf( "%s/%s%s.png", get_property("cache_dir"), get_property("ext_prefix"), $session_id ); 
		$this->map_name = sprintf( "%s/%s%s.map", get_property("cache_dir"), get_property("ext_prefix"), $session_id ); 
		return;
	}


}
