<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'includes/graphviz.inc.php';
require_once 'includes/pdf_helper.inc.php';

/*
 * klasse zur erstellung einer PDF-Datei aus einer
 * graphviz plain datei
 */
class graphviz_pdf extends graphviz
{
	protected $pdf					= null;
	
	public function __construct (  $_px, $_py, $_text_font, $_text_size, $_padding, $_arrow_length, $_icon_size )
	{
		parent::__construct();
		$this->setup_page_by_inch( $_px, $_py, $_padding );
		$this->setup_style( $_text_font, $_text_size, $_padding, $_arrow_length, $_icon_size );

		return;
	}

	/**
	 * deliver image as pdf
	 */
	public function display ()
	{
		header ("Content-type: text/pdf");
		$this->pdf->stream();
		return;
	}

	// protecteds

	/**
	 * generate a .dot file for processing with graphviz 
	 */
	protected function generate_dot_file ( $filename )
	{
		// init pdf to help calculate box width
		$this->create_pdf();

		parent::generate_dot_file( $filename );
		
		$this->destroy_pdf();

		return;
	}

	protected function process_line_graph ( $items )
	{
		if ($this->processing()) return;
		parent::process_line_graph( $items );

		// create pdf
		$this->create_pdf();

		return;
	}

	/**
	 * get width of text in pixel 
	 */
	protected function get_text_x_p ( $_text )
	{
		if ($this->pdf == null) return 0;
		return $this->pdf->getTextWidth( $this->text_size_p, $_text );
	}

	/**
	 * get height of text in pixel 
	 */
	protected function get_text_y_p ( $_text )
	{
		if ($this->pdf == null) return $this->text_size_p;
		return $this->pdf->getFontHeight( $this->text_size_p );
	}

	/**
	 * return true, if output device/resource for processing
	 * is usable
	 */
	protected function processing()
	{
		return ($this->pdf != null);
	}

	protected function draw_node ( $_cx, $_cy, $_bx, $_by, $_hx, $_hy, $_id )
	{
		$ci = $this->ci_list[ $_id ];
		$ci_title = $ci->get_title();
		$ci_icon = $ci->get_icon();
		$icon = icon_list::getInstance()->get_icon_image( $ci_icon, $this->icon_size_b );
		$icon_ss = 1;

		// draw frame
		if ($this->center_ci_id == $_id)
		{
			$this->pdf->setStrokeColor( 1, 0, 0 );
			$this->pdf->rectangle(
				$this->margin_p + $_cx - $_hx,
				$this->margin_p + $_cy - $_hy,
				$_bx, $_by );
			$this->pdf->setStrokeColor( 0, 0, 0 );
		}
		else
		{
//			$this->pdf->setStrokeColor( 0, 0, 1 );
//			$this->pdf->rectangle(
//				$this->margin_p + $_cx - $_hx,
//				$this->margin_p + $_cy - $_hy,
//				$_bx, $_by );
//			$this->pdf->setStrokeColor( 0, 0, 0 );
		}

		// draw icon
		$this->pdf->addImage(
			$icon,
			$this->margin_p + $_cx - $_hx + $this->padding_p,
			$this->margin_p + $_cy - $_hy + $this->padding_p,
			$this->icon_size_p, $this->icon_size_p );

		// draw text
		$this->pdf->addText(
			$this->margin_p + $_cx - $_hx + $this->icon_size_p + 2 * $this->padding_p,
			$this->margin_p + $_cy - $_hy + $this->padding_p,
			$this->text_size_p, $ci_title );
	
		return;
	}
	
	protected function draw_line ( $_x1, $_y1, $_x2, $_y2, $_last )
	{
		if ($_last)
		{
			pdf_arrow( $this->pdf,
				$_x1, $_y1,
				$_x2, $_y2,
				$this->arrow_length_p );
//			$this->pdf->line( $_x1, $_y1, $_x2, $_y2 );
		}
		else
		{
			$this->pdf->line( $_x1, $_y1, $_x2, $_y2 );
		}

		return;
	}

	// privates

	/**
	 * create a pdf instance
	 */
	private function create_pdf ()
	{
		if ($this->processing()) return;
		
		// create pdf
		$this->pdf = new Cpdf( array( 0, 0, $this->page_x_p, $this->page_y_p ) );

		$this->pdf->setColor( 0, 0, 0 );
		$this->pdf->setStrokeColor( 0, 0, 0 );
		$this->pdf->setLineStyle( 0.1 );
		$this->pdf->selectFont($this->text_font,'WinAnsiEncoding');
		
		$this->rescale_style();

		return;		
	}
	
	/**
	 * delete pdf instance
	 */
	private function destroy_pdf ()
	{
		$this->pdf = null;
		return;
	}


}
