<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
 * draw a rectangle defined by the box
 * 
 * @param	$image		the image ressource
 * @param	$box		array: [0 => x1, 1 => y1, 2 => x2, 3 => y2]
 * @param	$color		color ressource of image
 */
function image_rectanglebox ( &$image, $box, $color )
{
	imagerectangle( $image, $box[0], $box[1], $box[2], $box[3], $color );
	return;
}

/**
 * draw a box with icon and text inside
 * 
 * @param $image		Image-Ressource
 * @param $font			Font for text
 * @param $font_size	Font-Size for text
 * @param $padding		distance between border and content (in pixel)
 * @param $cx, $cy		center coordinates of the box
 * @param $text			the text
 * @param $icon_name	name of icon, icon will be displayed leftside of text
 * 						(if icon name is not empty)
 * @param $text_color	text color
 * @param $box_color	frame color
 * @param $back_color	background color
 * @result array		array with coordinates of the drawed box (upper left and lower right corner)    
 * 
 */
function image_ft_box ( &$image, $font, $font_size, $padding, $cx, $cy, $text, $icon_name, $text_color, $box_color, $back_color, $icon_size )
{
	global $running_script_base;
	
	$box = array();
	
	// get icon and size
	$icon = icon_list::getInstance()->get_icon_image( $icon_name, $icon_size );
	if ($icon)
	{
		$icon_sx = $icon_size;
		$icon_sy = $icon_size;
	}
	$icon_sx2 = floor( $icon_sx / 2 );
	$icon_sy2 = floor( $icon_sy / 2 );

	// text size
	$ftbox = imageftbbox( $font_size, 0, $font, $text );
	$text_sx = abs( $ftbox[2] - $ftbox[0] );
	$text_sy = abs( $ftbox[3] - $ftbox[5] );
	$text_sx2	= floor( $text_sx / 2 );
	$text_sy2	= floor( $text_sy / 2 );

	// box size
	$box_sx		= $icon_sx + $text_sx;
	$box_sy		= max( $text_sy, $icon_sy );
	if ($icon) $box_sx = $box_sx + $padding;
	$box_sx2	= floor( $box_sx / 2 );
	$box_sy2	= floor( $box_sy / 2 );
	
	// box position
	$box[0]			= $cx - $box_sx2 - $padding;
	$box[2]			= $cx + $box_sx2 + $padding;
	$box[1]			= $cy - $box_sy2 - $padding;
	$box[3]			= $cy + $box_sy2 + $padding;

	// move box, if outside of image
	$sx				= imagesx( $image );
	$sy				= imagesy( $image );
	if ($box[0]<0)
	{
		$dx = abs($box[0]);
		$box[2] = $box[2] + $dx;
		$box[0] = 0;
	}
	if ($box[1]<0)
	{
		$dy = abs($box[1]);
		$box[3] = $box[3] + $dy;
		$box[1] = 0;
	}
	if ($box[2]>=$sx)
	{
		$dx = $sx - $box[2] - 1;
		$box[0] = $box[0] + $dx;
		$box[2] = $box[2] + $dx;
	}
	if ($box[3]>=$sy)
	{
		$dy = $sy - $box[3] - 1;
		$box[1] = $box[1] + $dy;
		$box[3] = $box[3] + $dy;
	}

	// re-calculate center
	$cx = floor( ($box[0] + $box[2]) / 2 );
	$cy = floor( ($box[1] + $box[3]) / 2 );

	// icon box
	if ($icon)
	{
		$icon_box[0] = $box[0] + $padding;
		$icon_box[1] = $cy - $icon_sy2;
		$icon_box[2] = $icon_box[0] + $icon_sx;
		$icon_box[3] = $icon_box[1] + $icon_sy;
	}

	// text box
	$text_box[0] = $box[0] + $padding + $icon_sx;
	if ($icon) $text_box[0] = $text_box[0] + $padding;
	$text_box[1] = $cy - $text_sy2;
	$text_box[2] = $text_box[0] + $text_sx;
	$text_box[3] = $text_box[1] + $text_sy;

	// fill background
//	imagefilledrectangle( $image, $box[0], $box[1], $box[2], $box[3], $back_color );

	// draw icon
	if ($icon)
	{
		imagecopy( $image, $icon, $icon_box[0], $icon_box[1], 0, 0, $icon_sx, $icon_sy );
	}

	// draw text
	Imagefttext( $image, $font_size, 0,
		$text_box[0], $text_box[1] + $font_size,
		$text_color, $font,
		$text);
	
	// draw frame
//	imagerectangle( $image, $box[0], $box[1], $box[2], $box[3], $box_color );
	
	return $box;
}

/**
 * liefert ein array mit den Endpunkten der Linie zwischen den Mittelpunkten 
 * der beiden Boxen
 * 
 * die Endpunkte liegen exaktz auf dem Rand der Boxen
 * 
 * @param	$box1		Koordinaten der Box (linke obere/rechte untere Ecke)
 * @param	$box2		Koordinaten der Box (linke obere/rechte untere Ecke)
 * @result	array		Koordinaten der Linie (Anfang, Ende)
 */

function get_arrow_from_string_boxes ( $box1, $box2, $shift=0, $h=15 )
{
	// wichtige werte
	$b1x = abs( $box1[2] - $box1[0] );
	$b1y = abs( $box1[3] - $box1[1] );
	$b1x2 = floor( $b1x / 2 );
	$b1y2 = floor( $b1y / 2 );
	$b1cx = floor( ($box1[0] + $box1[2]) / 2 );
	$b1cy = floor( ($box1[1] + $box1[3]) / 2 );

	$b2x = abs( $box2[2] - $box2[0] );
	$b2y = abs( $box2[3] - $box2[1] );
	$b2x2 = floor( $b2x / 2 );
	$b2y2 = floor( $b2y / 2 );
	$b2cx = floor( ($box2[0] + $box2[2]) / 2 );
	$b2cy = floor( ($box2[1] + $box2[3]) / 2 );

	$cx = $b2cx - $b1cx;
	$cy = $b2cy - $b1cy;

	// richtungen ermitteln
	$fx = 1;
	if ($cx != 0) $fx = $cx / abs( $cx );
	$fy = 1;
	if ($cy != 0) $fy = $cy / abs( $cy );

	// 
	$cx = abs( $cx );
	$cy = abs( $cy );

	$cx1 = $b1x2;
	$cx2 = $cx - $b2x2;

	$cy1 = $b1y2;
	$cy2 = $cy - $b2y2;

	// shift
	$shift_x = 0;
	$shift_y = 0;
	
	if ($shift > 0)
	{	
		$shift_f = 1;
		if ((floor($shift/2)*2 - $shift)!=0) $shift_f = -1;
		$ha = floor( $h / sqrt(5) );	// seitenlaenge pfeil

		if ($cx > $cy)
		{
			$shift_y = $shift * $ha * $shift_f; 
		}
		else
		{
			$shift_x = $shift * $ha * $shift_f;
		}
	}
		
	// === offsets box1 ===
	if ($cy != 0)
	{
		$b1xo = floor( $cy1 * $cx / $cy );
	}
	else
	{
		$b1xo = $cx1;
	}
	if ($cx != 0)
	{
		$b1yo = floor( $cx1 * $cy / $cx );
	}
	else
	{
		$b1yo = $cy1;
	}
	
	// shift
	$b1xo = $b1xo + $shift_x;
	$b1yo = $b1yo + $shift_y;

	// trim
	if ($b1xo > $cx1) $b1xo = $cx1;
	if ($b1yo > $cy1) $b1yo = $cy1;

	// === offset box2 ===
	if ($cy != 0)
	{
		$b2xo = floor( $cy2 * $cx / $cy );
	}
	else
	{
		$b2xo = $cx2;
	}
	if ($cx != 0)
	{
		$b2yo = floor( $cx2 * $cy / $cx );
	}
	else
	{
		$b2yo = $cy2;
	}
	
	// shift
	$b2xo = $b2xo + $shift_x;
	$b2yo = $b2yo + $shift_y;

	// trim
	if ($b2xo < $cx2) $b2xo = $cx2;
	if ($b2yo < $cy2) $b2yo = $cy2;

	// === zusammensetzen p1 ===
	$p1x = $b1cx + $b1xo * $fx;
	$p1y = $b1cy + $b1yo * $fy;

	// === zusammensetzen p2 ===
	$p2x = $b1cx + $b2xo * $fx;
	$p2y = $b1cy + $b2yo * $fy;

	// result
	$arrow = array( $p1x, $p1y, $p2x, $p2y );

	return $arrow;
}

/**
 * zeichnet eine linie mit dreieckigem gef�llten pfeil am endpunkt
 * 
 * @param	$image			Image-Resource
 * @param	$x0, $y0		Koordinaten des Anfangspunktes
 * @param	$x1, $y1		Koordinaten des Endpunktes
 * @param	$h				L�nge des Pfeils in Pixeln
 * @param	$line_color		Farbe
 */
function image_arrow ( &$image, $x0, $y0, $x1, $y1, $h, $line_color )
{
	// wichtige werte
	$lx = $x1 - $x0;	// linien-laenge projektion auf x
	$ly = $y1 - $y0;	// linien-laenge projektion auf y
	$ll = sqrt( $lx * $lx + $ly * $ly );	// laenge
	if ($ll == 0) return;
	
	$hx = floor( $lx * $h / $ll );	// pfeil-laenge projektion auf x
	$hy = floor( $ly * $h / $ll );	// pfeil-laenge projektion auf y
	$ha = floor( $h / sqrt(5) );	// seitenlaenge pfeil

	// offset
	if ($ly == 0)
	{
		$xo = 0;
		$yo = $ha;
	}
	else
	{
		if ($lx == 0)
		{
			$xo = $ha;
			$yo = 0;	
		}
		else
		{
			$wa = atan( $lx / $ly );
			$xo = floor( cos( $wa ) * $ha );
			$yo = floor( sin( $wa ) * $ha );
		}
	}
	
	// p5 koordinaten
	$x5 = $x1 - $hx;
	$y5 = $y1 - $hy;

	// p2 koordinaten
	$arrow[0] = $x1;
	$arrow[1] = $y1;

	// p3 koordinaten
	$arrow[2] = $x5 + $xo;
	$arrow[3] = $y5 - $yo;

	// p4 koordinaten
	$arrow[4] = $x5 - $xo;
	$arrow[5] = $y5 + $yo;

	// line
	imageline( $image, $x0, $y0, $x1, $y1, $line_color );
//	imageline( $image, $x0, $y0, $x5, $y5, $line_color );

//	imageline( $image, $x3, $y3, $x4, $y4, $line_color );
	imagefilledpolygon( $image, $arrow, 3, $line_color );
	
	return;
}
