<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
 * recursive echo of array elements
 * 
 * @param	array	&$a						Array by Reference
 * @param	int		$level					level (or dimension)
 * @return	none
 */
function array_display_helper( &$a, $level )
{
	foreach( $a as $key => $value )
	{
		if (is_array($value))
		{
			printf( "%s[%s]\n", $level, $key );
			array_display_helper( $value, $level . "  " );
		}
		else
		{
			printf( "%s[%s]=%s\n", $level, $key, $value );
		}
	}
	return;
}

/**
 * checks existence of $uid in ldap 
 * 
 * returns DN (Distinguished Name) if found, or empty string
 * 
 * @param	string		$uid				login
 * @return	array							user attributes from ldap  
 */
function ldap_api_check_uid( $uid )
{
	// check: using ldap enabled 
	if (! get_property('auth_ldap')) return;
	
	// ldap setup
	$sm_ldap_server  = get_property('auth_ldap_server');
	$sm_ldap_port    = get_property('auth_ldap_port');
	$sm_ldap_user    = get_property('auth_ldap_user');
	$sm_ldap_pass    = get_property('auth_ldap_pass');
	$sm_ldap_base    = get_property('auth_ldap_base');
	$sm_ldap_attr    = get_property('auth_ldap_attr');

	// search uid
	$sm_ldap_connect = ldap_connect( $sm_ldap_server, $sm_ldap_port);
	ldap_set_option( $sm_ldap_connect, LDAP_OPT_PROTOCOL_VERSION, 3);
	ldap_set_option( $sm_ldap_connect, LDAP_OPT_REFERRALS, 0);
	if (strlen($sm_ldap_user)==0)
	{
		// bind anonymous
		$sm_ldap_bind    = ldap_bind($sm_ldap_connect);
	}
	else
	{
		// bind as ldap-user
		$sm_ldap_bind    = ldap_bind($sm_ldap_connect, $sm_ldap_user, $sm_ldap_pass);
	}
	
	$sm_ldap_search_filter = "(&($sm_ldap_attr=" . ldap_api_escape($uid) . "))";
	$sm_ldap_search_attrs  = array( "*" );

	$sm_ldap_search  = ldap_search($sm_ldap_connect, $sm_ldap_base, $sm_ldap_search_filter, $sm_ldap_search_attrs );
	$sm_ldap_count   = ldap_count_entries($sm_ldap_connect, $sm_ldap_search);

	if ($sm_ldap_count==0) return;

	$sm_ldap_info=ldap_get_entries($sm_ldap_connect, $sm_ldap_search);
	
//echo "<pre>\n";
//print_r($sm_ldap_info);	
//echo "</pre>\n";
	
	ldap_unbind($sm_ldap_connect);
	
	return $sm_ldap_info[0];
}

/**
 * verify password by ldap-bind
 * 
 * @param	string		$dn					user DN
 * @param	string		$pass				password
 * @return	boolean							success
 */
function ldap_api_check_passwd( $dn, $pass )
{
	// check: using ldap enabled 
	if (! get_property('auth_ldap')) return FALSE;
	
	// ldap setup
	$sm_ldap_server  = get_property('auth_ldap_server');
	$sm_ldap_port    = get_property('auth_ldap_port');
	$sm_ldap_base    = get_property('auth_ldap_base');
	
	error_reporting(E_ERROR);
	$sm_ldap_connect = ldap_connect( $sm_ldap_server, $sm_ldap_port);
	ldap_set_option( $sm_ldap_connect, LDAP_OPT_PROTOCOL_VERSION, 3);
	ldap_set_option( $sm_ldap_connect, LDAP_OPT_REFERRALS, 0);

	$sm_ldap_bind    = ldap_bind( $sm_ldap_connect, $dn, $pass );
	if ($sm_ldap_bind)
	{
		ldap_unbind($sm_ldap_connect);
		return TRUE;
	}
	else
	{
//		echo ldap_errno($sm_ldap_connect) . "<br>\n";
//		echo ldap_error($sm_ldap_connect) . "<br>\n";
	}
	return FALSE;
}

/**
 * verify that the DN is member of a given ldap group
 * 
 * ldap group setup is given by application properties, if no
 * ldap group is given, we return true
 * 
 * @param	string		$user_dn			user DN
 * @return	boolean	 						success
 */
function ldap_api_check_group( $user_dn )
{
	// get ldap parameters 
	$sm_ldap_server  		= get_property('auth_ldap_server');
	$sm_ldap_port			= get_property('auth_ldap_port');
	$sm_ldap_user			= get_property('auth_ldap_user');
	$sm_ldap_pass			= get_property('auth_ldap_pass');

	$sm_ldap_group_base				= get_property('auth_ldap_group_base');
	$sm_ldap_group_attr				= get_property('auth_ldap_group_attr');
	$sm_ldap_membership_operator	= get_property('auth_ldap_membership_operator');

	// check: return true, if group-params not set
	//	
	if ((strlen($sm_ldap_group_base)==0) ||
		(strlen($sm_ldap_group_attr)==0)
	)
		return true;

	// connect to ldap
	$sm_ldap_connect = ldap_connect( $sm_ldap_server, $sm_ldap_port);
	ldap_set_option( $sm_ldap_connect, LDAP_OPT_PROTOCOL_VERSION, 3);
	ldap_set_option( $sm_ldap_connect, LDAP_OPT_REFERRALS, 0);
	$sm_ldap_bind    = ldap_bind($sm_ldap_connect, $sm_ldap_user, $sm_ldap_pass);
	
	// nothing found
	$sm_ldap_count	= 0;

	// search user-dn in group record
	$sm_ldap_search_filter = "(&($sm_ldap_group_attr$sm_ldap_membership_operator" . ldap_api_escape( $user_dn ) . "))";
	$sm_ldap_search_attrs  = array( $sm_ldap_group_attr );

	$sm_ldap_search  = ldap_search($sm_ldap_connect, $sm_ldap_group_base, $sm_ldap_search_filter, $sm_ldap_search_attrs );
	$sm_ldap_count   = ldap_count_entries($sm_ldap_connect, $sm_ldap_search);

	ldap_unbind($sm_ldap_connect);
	
	return ($sm_ldap_count!=0);
}

/**
 * return role ID for the given user DN based on a config file-defined
 * mapping to LDAP groups
 * 
 * if no mapping is found in LDAP for the user, der default_role_id
 * is returned
 * 
 * @param	string		$user_dn			user DN
 * @param	int			$default_role_id	current role id of the user
 * @return	int
 */
function ldap_api_get_role( $user_dn, $default_role_id )
{
	// get ldap parameters 
	$sm_ldap_server  		= get_property('auth_ldap_server');
	$sm_ldap_port			= get_property('auth_ldap_port');
	$sm_ldap_user			= get_property('auth_ldap_user');
	$sm_ldap_pass			= get_property('auth_ldap_pass');

	$sm_ldap_group_attr				= get_property('auth_ldap_group_attr');
	$sm_ldap_membership_operator	= get_property('auth_ldap_membership_operator');
	$sm_ldap_role_map				= get_property('auth_ldap_role_map');

	//	check: return default id, if role_params not set
	if ((strlen($sm_ldap_group_attr)==0) ||
		(strlen($sm_ldap_membership_operator)==0) ||
		(count($sm_ldap_role_map)==0)
	)
		return $default_role_id;

	// connect to ldap
	$sm_ldap_connect = ldap_connect( $sm_ldap_server, $sm_ldap_port);
	ldap_set_option( $sm_ldap_connect, LDAP_OPT_PROTOCOL_VERSION, 3);
	ldap_set_option( $sm_ldap_connect, LDAP_OPT_REFERRALS, 0);
	$sm_ldap_bind    = ldap_bind($sm_ldap_connect, $sm_ldap_user, $sm_ldap_pass);

	// sort array by keys descending, to start with the highest role
	krsort( $sm_ldap_role_map ); 

	// nothing found
	$found_id = ROLE_NONE;

	// search
	foreach ($sm_ldap_role_map as $role_id => $role_dn)
	{
		// read the group record from ldap
		$sm_ldap_search_filter = "($sm_ldap_group_attr$sm_ldap_membership_operator" . ldap_api_escape( $user_dn ) . ")";
	
		$sm_ldap_search  = ldap_read($sm_ldap_connect, $role_dn, $sm_ldap_search_filter );
		$sm_ldap_count   = ldap_count_entries($sm_ldap_connect, $sm_ldap_search);

		if($sm_ldap_count > 0)
		{
			// We've already done the filtering in the LDAP search.
			// Therefore, if a role was returned then this must be a match.
			$found_id = $role_id;
		}		

		// stop search if found
		if ($found_id != ROLE_NONE) 
			break;
	}

	ldap_unbind($sm_ldap_connect);
	
	// fall back to default, if nothing found
	if ($found_id == ROLE_NONE) $found_id=$default_role_id;

	return $found_id;
}

/**
 * return dn string from ldap-entries array depending of LDAP type (ADS or other)
 * 
 * @param	array		$ldap_info			Array with attributes
 * @return	string							user DN
 */
function ldap_api_get_dn( $ldap_info )
{
	$sm_ldap_attr    = get_property('auth_ldap_attr');
	if (($sm_ldap_attr == 'sAMAccount') && (isset($ldap_info['userprincipalname'][0])))
	{
		// Windows ADS
		$dn = $ldap_info['userprincipalname'][0];
	}
	else
	{
		// typical LDAP
		$dn = $ldap_info['dn'];		
	}
	return $dn;
}

/**
 * quote a string for ldap requests per RFC 2254
 * 
 * @param	string		$str				ldap string
 * @return	string							quoted ldap string	
 */
function ldap_api_escape( $str )
{
	$str_out = str_replace(
		array( '\\', ' ', '*', '(', ')' ),
		array( '\\5c', '\\20', '\\2a', '\\28', '\\29' ),
		$str
	); 

//	echo "ldap_api_escape( $str ): $str_out<br>\n";

	return $str_out; 
}
