<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'includes/ci.inc.php';
require_once 'includes/relation.inc.php';
require_once 'includes/ci_tracker.inc.php';

class module_label_list
{
	protected $labels		= array();
	protected $filled_count	= 0;

	public function __construct ( $label_array=null )
	{
		if ($label_array != null) $this->add_label_array( $label_array );
	}
	
	// set methoden

	/**
	 * setzt den wert des labels, wenn es vorhanden
	 * und noch nicht befuellt ist, der wert von
	 * filled_count wird entsprechend hoch gesetzt,
	 * die methode liefert TRUE
	 * 
	 * andernfalls FALSE
	 */
	public function set_label_value ( $label, $value )
	{
		if (
			! $this->label_exists( $label )
			||
			$this->label_filled( $label )
			)
		{
			return FALSE;
		}
		$this->labels[ $label ] = $value;
		$this->filled_count++;
		return TRUE;
	}

	// get methoden

	/**
	 * liefert die anzahl der labels in der liste
	 */
	public function count ()
	{
		return count( $this->labels );
	}

	/**
	 * liefert die anzahl der befuellten labels in
	 * der liste
	 */
	public function get_filled_count ()
	{
		return $this->filled_count;
	}

	/**
	 * liefert den wert des labels, bzw. null
	 * falls es nicht befuellt ist
	 */
	public function get_label_value ( $label )
	{
		if (! $this->label_exists( $label )) return null;
		return $this->labels[ $label ];
	}
	
	/**
	 * liefert TRUE, wenn das label in der liste
	 * vorhanden ist
	 * 
	 * sonst FALSE
	 */
	public function label_exists ( $label )
	{
		return array_key_exists( $label, $this->labels );
	}
	
	/**
	 * liefert TRUE, wenn das label in der liste
	 * vorhanden und befuellt ist
	 * 
	 * sonst FALSE
	 */
	public function label_filled ( $label )
	{
		return (
				$this->label_exists( $label )
				&&
				($this->labels[ $label ] != null)
				);
	}

	/**
	 * liefert TRUE, wenn alle labels befuellt sind
	 * 
	 * sonst FALSE
	 */
	public function completed ()
	{
		return ($this->count() == $this->get_filled_count());
	}

	// actions

	/**
	 * nimmt das label in der liste auf, falls es noch nicht
	 * vorhanden ist
	 */
	public function add_label ( $label )
	{
		if (array_key_exists( $label, $this->labels )) return;
		$this->labels[$label]=null;
		return;
	}
	
	/**
	 * nimmt alle label aus label_array auf
	 */
	public function add_label_array ( $label_array )
	{
		if (! is_array( $label_array )) return FALSE;
		foreach( $label_array as $key => $label)
		{
			$this->add_label( $label );
		}
		return TRUE;
	}

	/**
	 * entfernt das label aus der liste, falls es
	 * vorhanden ist
	 * 
	 * hatte das label befuellt ist (filled), dann
	 * wird auch der wert von filled_count entsprechend
	 * verringert
	 */
	public function remove_label ( $label )
	{
		if (! array_key_exists( $label, $this->labels )) return FALSE;
		if ($this->labels[$label]!=null) $this->filled_count--;
		unset($this->labels[$label]);
		return TRUE;
	}
}

/**
 * klasse zum fuellen einer extension_label_list ueber
 * mehrere CIs hinweg
 */
class label_list_tracker extends ci_tracker
{
	protected $label_list		= null;
	
	public function __construct ( $_ci_id, $_max_depth, $_label_list )
	{
		parent::__construct( $_ci_id, $_max_depth );
		$this->label_list = $_label_list;
	}
	
	protected function process_ci ( $_ci_id )
	{
		$ci	= new ci( $_ci_id );
		$ci->fill_label_list( $this->label_list );
		return (! $this->label_list->completed());
	}
	
	protected function process_relation ( $_r_id, $_ci_id )
	{
		$r	= new relation( $_r_id, 0, $_ci_id );
		$r->fill_label_list( $this->label_list );
		return (! $this->label_list->completed());
	}
	
}