<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_navi_list.inc.php';
require_once 'tables/db_typed_ci.inc.php';
require_once 'includes/icon_list.inc.php';

/**
 * abstract class to persist a core_navi_list inside session
 * and add display methods for page navigation
 */
abstract class navi_list extends core_navi_list
{
	protected	$list_mode		= LIST_MODE_USE;
	
	protected function __construct ()
	{
		$this->_load();
		$this->post_load();
		parent::__construct();
		$this->post_construct();
	}

	protected function post_load ()
	{
	}

	protected function post_construct ()
	{
		// load navi from args
		$_pn = http_get_int( 'pn', 0 );
		if ($_pn == 1) $this->set_page_num( 0 );
		if ($_pn == 2) $this->page_prev();
		if ($_pn == 3) $this->page_next();
		if ($_pn == 4) $this->set_page_num( $this->get_page_last() );

		$_p = http_get_int( 'p', -1 );
		if ($_p >= 0) $this->set_page_num( $_p );

//		$this->debug_info("navi_list::post_construct");
	}

	abstract protected function get_session_store_name (); 

	abstract protected function get_element_name (); 

	// list mode
	
	public function get_list_mode ()
	{
		return $this->list_mode;
	}
	
	public function set_list_mode ( $_mode )
	{
		if ($this->list_mode == $_mode) return TRUE;
		if (! in_array( $_mode, array( LIST_MODE_USE, LIST_MODE_MANAGE))) return FALSE;
		
		// set new mode
		$this->list_mode = $_mode;
		
		// reset list to begin
		$this->set_page_num( 0 );
		
		return TRUE;
	}


	// show page

	public function display_synopse ( $menu = false )
	{
		if ($menu === false) $menu = & get_empty_menu();
		$this->html_synopse_section( $menu );
		return;
	}
	
	public function show ( $menu=false )
	{
		
		$this->show_location( $menu );

		$this->show_page();

		return;
	}
		
	public function show_page ()
	{
		if ($this->count() == 0)
		{
			html_empty_row( 0, 0 );
		}
		else
		{
			html_table_row_reset( array("r_normal","r_normal2") );
			parent::show_page();
		}
		$this->_store();
	}

	protected function show_location ( $menu=false )
	{
		html_table_header( 500, 300 );

		if ($this->count()==0)
		{
			// location information
			printf( "<tr><td class='section'>%s (".get_string('Total_ttl').": %d)</td><td class='navigation'></td></tr>\n",
				$this->get_element_name(),
				$this->count()
				);
		}
		else
		{	
			printf( "<tr>" );
			// location information
			printf( "<td class='section'>%s: %d ... %d (".get_string('Total_ttl').": %d)</td>",
				$this->get_element_name(),
				$this->get_page_start() + 1,
				$this->get_page_end() + 1,
				$this->count()
				);

			// navi
			$this->show_navigation();
			printf( "</tr>" );
		}
		
		// menu
		if (($menu !== false) && is_object($menu) && ($menu->get_count() > 0) )
		{
			printf( "  <tr><td colspan='2' class='menu'>&gt;&gt; %s</td></tr>\n",
				$menu->get_menu_string() );
		}
		
		html_table_footer();
	}

	protected function show_navigation ()
	{
		define ('NAV_PAGE_SINGLE_RANGE',	10 );
		define ('NAV_PAGE_RANGE_LIMIT',		5 );
		define ('NAV_PAGE_MARGIN_SMALL',	1 );
		define ('NAV_PAGE_MARGIN_LARGE',	2 );

		printf( "<td class='navigation'>" );
		printf( "%s", icon_display_url( "?pn=2" . $this->get_navi_args(), ICON_PREV, "Prev", 0, 0, "Previous page") );
		
		if (($num = $this->get_page_last()) > NAV_PAGE_SINGLE_RANGE)
		{
			$pc = $this->get_page_num();
			$p5 = $num - NAV_PAGE_RANGE_LIMIT;
			
			if ($pc < NAV_PAGE_RANGE_LIMIT)
			{
				$this->show_nav_page_range( 0, NAV_PAGE_RANGE_LIMIT );
				printf( " ... ");
				$this->show_nav_page_range( $num - NAV_PAGE_MARGIN_LARGE, $num );
			}
			if ((NAV_PAGE_RANGE_LIMIT <= $pc) && ($pc <= $p5))
			{
				$this->show_nav_page_range( 0, NAV_PAGE_MARGIN_LARGE );
				printf( " ... ");
				$this->show_nav_page_range( $pc - NAV_PAGE_MARGIN_SMALL, $pc + NAV_PAGE_MARGIN_SMALL );
				printf( " ... ");
				$this->show_nav_page_range( $num - NAV_PAGE_MARGIN_LARGE, $num );
			}
			if ($p5 < $pc)
			{
				$this->show_nav_page_range( 0, NAV_PAGE_MARGIN_LARGE );
				printf( " ... ");
				$this->show_nav_page_range( $p5, $num );
			}
			
		}
		else
		{
			$this->show_nav_page_range( 0, $num );
		}

		printf( "%s", icon_display_url( "?pn=3" . $this->get_navi_args(), ICON_NEXT, "Next", 0, 0, "Next page") );

		printf( "</td>\n" );
		return;
	}

	// helper

	private function show_nav_page_range ( $p1, $p2 )
	{
		for ($i=$p1; $i<=$p2; $i++)
		{
			if ($i == $this->get_page_num())
			{
				printf( " %d ", $i+1 );
			}
			else
			{
				printf( " <a href='?p=%d%s'>%d</a> ", $i, $this->get_navi_args(), $i+1 );
			}
		}
		return;
	}

	protected function get_navi_args ()
	{
		$navi_args = "";
		return $navi_args;
	}

	protected function get_dont_load_keys ()
	{
		$key_array = array();
		$key_array[] = "object_list";
		$key_array[] = "index_list";
		return $key_array;
	}

	protected function _load()
	{
		global $_SESSION;

		$ss_name = $this->get_session_store_name();
//echo "DEBUG (navi_list::_load): session = $ss_name<br>\n";

		// load from session
		$dont_load_keys = $this->get_dont_load_keys();
		foreach ($this as $key => $value)
		{
			if (! in_array( $key, $dont_load_keys ))
			{
				if (isset($_SESSION[$ss_name][$key]))
				{
					$this->$key = $_SESSION[$ss_name][$key];
//echo "DEBUG (navi_list::_load): $key => " . $this->$key . "<br>\n";
				}
			}
		}
		return;
	}
	
	protected function _store()
	{
		global $_SESSION;

		$ss_name = $this->get_session_store_name();
		$dont_load_keys = $this->get_dont_load_keys();
		// store to session
		foreach ($this as $key => $value)
		{
			if (! in_array( $key, $dont_load_keys ))
			{
				$_SESSION[$ss_name][$key] = $value;
//echo "DEBUG (navi_list::_store): $key => $value<br>\n";
			}
		} 

		return;
	}

	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$title = "";
		return $title;
	}

	/**
	 * get a HTML string including the icon for the
	 * synopse section
	 * 
	 * @return		string		HTML code including an icon 
	 */
	protected function get_synopse_icon_html ()
	{
		$icon_html = "";
		return $icon_html;
	}

	/**
	 * get a string with the information for the upper right
	 * corner in the synopse section
	 * 
	 * @return	string	information
	 */
	protected function get_synopse_info ()
	{
		$info = "";
		return $info;
	}

	/**
	 * html output the synopse section
	 */
	protected function html_synopse_section ( &$menu )
	{
		$icon_html = $this->get_synopse_icon_html();
		$title = $this->get_synopse_title();
		$info = $this->get_synopse_info();

		printf( "<div id='synopse'>\n" );
		html_table_header( 50, 600, 150 );
		printf( "<tr>" .
			"<td rowspan='2'>%s</td>" .
			"<td class='section'>%s</td>" .
			"<td class='session'>%s</td>" .
			"</tr>\n",
			$icon_html,
			$title,
			$info
			);
		printf( "<tr><td colspan='2' class='menu'>" );
		if ( $menu->get_count() > 0)
		{
			printf( "&gt;&gt; %s",
				$menu->get_menu_string() );
		}
		printf( "</td></tr>\n" );
		html_table_footer();
		printf( "</div>\n" );
	
		return;
	}

}
