<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

// page_init_inc.php - central init of every page request

// include configurations
require_once 'includes/constants.inc.php';
require_once 'includes/config_default.inc.php';

if (
	is_file( CONFIG_INC_PHP )
	||
	is_link( CONFIG_INC_PHP )
)
{
	@include_once CONFIG_INC_PHP;
}
else
{
	// run setup
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 TRANSITIONAL//EN">
<html>
<head>
  <title>redirect</title>
  <meta http-equiv="refresh" content="0; URL=<?php echo CMDB_SETUP_PHP;?>">
</head>
<body>
</body>
</html>
<?php
	exit;
}

// who we are
$running_script_name = basename( $_SERVER['SCRIPT_FILENAME'] );
$running_script_base = dirname( $_SERVER['SCRIPT_FILENAME'] );
$referer_script_name = INDEX_PHP;
if (isset($_SERVER['HTTP_REFERER'])) $referer_script_name = basename( $_SERVER['HTTP_REFERER'] );

// include all core libs
require_once 'core/autoload.inc.php';
require_once 'core/core_api.inc.php';
require_once 'core/core_db.inc.php';
require_once 'core/core_html.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_arm.inc.php';

// include modules
require_once 'includes/module_base.inc.php';
require_once 'modules/modules.inc.php';

// error handling
set_exception_handler('core_exception_handler');

// inits
$rc=0;

// offline check
//
if ( file_exists( CMDB_OFFLINE_PHP ) )
{
	html_page_redirect( 0, CMDB_OFFLINE_PHP );
	exit;
}

// delete timed out sessions
//session_cleanup();

// load default language
$lang_file_name = lang_catalog::getFilename( get_property_from_config('default_language') );
require_once $lang_file_name;

// get session
//
$session = core_session::getInstance();
if (! $session->is_authenticated()) include 'includes/login.inc.php';

// load user language
$lang_file_name = lang_catalog::getFilename( get_property('default_language') );
require_once $lang_file_name;

// access and role check
// 
if (! core_arm::getInstance()->ac_permitted( get_url_to_ac( $running_script_name ) ))
{
	require_once 'includes/page_error.inc.php';
	
	html_error_page( get_string('ActionNotPermitted_dsc') );
	exit();
}	

//$session = null;
//if ($session == null) $session = new session();
//session_in();
