<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_api.inc.php';
require_once 'core/core_html_form.inc.php';
require_once 'core/core_processor.inc.php';
require_once 'tables/db_relation_type.inc.php';
require_once 'tables/db_relation.inc.php';
require_once 'tables/db_ci.inc.php';
require_once 'includes/browse.inc.php';
require_once 'includes/relation_type_peer_list.inc.php';
require_once 'includes/relation_type.inc.php';
require_once 'includes/relation_peer_imx.inc.php';
require_once 'includes/typed_ci_list.inc.php';
require_once 'includes/item_processor.inc.php';
require_once 'includes/module_base.inc.php';

class relation extends item_processor
{
	protected $r_form		= 0;
	protected $r_ci_id		= 0;
	protected $r_ci_pos		= 0;
	protected $r_peer_id	= 0;
	protected $r_ci_filter	= '';
	protected $ci			= false;
	protected $rt			= false;

	public function __construct ( $_id=0, $_ci_pos=0, $_ci_id=0, $_arg_name="", $_peer_id=0 )
	{
		parent::__construct( $_id, $_arg_name );
		$this->set_ci( $_ci_pos, $_ci_id );
		if ($_peer_id != 0) $this->set_peer_id( $_peer_id );
		$this->create_attr_list();
		return;
	} 

	/**
	 * attribute des Relation-Types aus argumenten laden
	 * 
	 * ueberlaedt die felder des DB-Objektes mit den
	 * werten aus den argumenten, dazu muss allerdings
	 * das argument "r_form" gesetzt sein 
	 * 
	 * @return	none
	 */	
	public function load_from_args ()
	{
		$this->r_form = http_get_int("r_form", $this->r_form);

		$_ci_filter = http_get_string( "r_ci_filter", $this->r_ci_filter );
		$this->set_ci_filter( $_ci_filter );

		$_rt_id = http_get_int("r_rt_id", $this->db_object->get_rt_id() );
		$this->set_rt_id( $_rt_id );

		$_peer_id = http_get_array( "r_peer_id" );
		if (! is_array($_peer_id))
		{
			$_peer_id = http_get_int("r_peer_id", $this->r_peer_id );
			$this->set_peer_id( $_peer_id );
		}
		
		$_r_desc = http_get_string("r_desc", $this->db_object->get_desc() );
		$this->db_object->set_desc( $_r_desc );

		parent::load_from_args();

//echo "<pre>";
//print_r( $this );
//echo "</pre>\n";

		return;
	}

	// get

	/**
	 * liefert den classtype code
	 */
	protected function get_classtype ()
	{
		return CLASS_REL;
	}

	/**
	 * liefert die ID des item_types
	 */
	public function get_item_type_id ()
	{
		return $this->db_object->get_rt_id();
	}

	/**
	 * liefert den title
	 */
	public function get_title ()
	{
		$item_title = parent::get_title();
		if (is_object($this->db_object)) $item_title = $this->db_object->get_title();
		return $item_title; 
	}

	/**
	 * liefert den title des item_types
	 */
	public function get_item_type_title ()
	{
		$item_type_title = parent::get_item_type_title();
		if (is_object($this->rt)) $item_type_title = $this->rt->get_title();
		return $item_type_title;
	}


	public function get_ci_id()
	{
		return $this->r_ci_id;
	}

	public function get_ci_pos()
	{
		return $this->r_ci_pos;
	}

	public function get_peer_id()
	{
		return $this->r_peer_id;
	}

	// set

	/**
	 * setzt den titel des objektes
	 */
	protected function set_title ( $_title )
	{
//		$this->db_object->set_title( $_title );
		return;
	}
	
	public function set_ci ( $_pos, $_id )
	{
		switch ($_pos)
		{
			case 0:
			{
				// aus der CI ID ermitteln
				if ($_id == 0)
				{
					throw new core_data_exception( "position and id not set for relation" );
				}
				if ($_id == $this->db_object->get_ci1_id()) $this->r_ci_pos = 1; 
				if ($_id == $this->db_object->get_ci2_id()) $this->r_ci_pos = 2; 

				break;
			}
			case 1:
			{
				if ($_id != 0)
				{
					$this->db_object->set_ci1_id( $_id );
				}
				$this->r_ci_pos = 1;
				break;
			}
			case 2:
			{
				if ($_id != 0)
				{
					$this->db_object->set_ci2_id( $_id );
				}
				$this->r_ci_pos = 2;
				break;
			}
			default:
			{
				throw new core_data_exception( "invalid position for relation" );
			}
		}

		// process
		switch ($this->r_ci_pos)
		{
			case 0:
			{
				throw new core_data_exception( "could not identifiy position for relation" );
				break;
			}
			case 1:
			{
				$this->r_ci_id = $this->db_object->get_ci1_id();
				$this->r_peer_id = $this->db_object->get_ci2_id();
				break;
			}
			case 2:
			{
				$this->r_ci_id = $this->db_object->get_ci2_id();
				$this->r_peer_id = $this->db_object->get_ci1_id();
				break;
			}
		}

		// load ci
		$this->ci = new db_ci ( $this->r_ci_id );

		return;
	}
	
	/**
	 * setzt den relation type
	 */
	protected function set_rt_id ( $_rt_id )
	{
		if ($this->db_object->get_rt_id() != $_rt_id )
		{
			$this->db_object->set_rt_id( $_rt_id );
			$this->rt = false;
		}

		if ($this->rt === false)
		{
			$this->load_rt();
			$this->create_attr_list();
		}

		return;
	}

	protected function set_peer_id ( $_peer_id )
	{
		$this->r_peer_id = $_peer_id;
		if ($this->r_ci_pos==1) $this->db_object->set_ci2_id( $_peer_id );
		if ($this->r_ci_pos==2) $this->db_object->set_ci1_id( $_peer_id );
		return;
	}

	protected function set_ci_filter( $_ci_filter )
	{
		$this->r_ci_filter = $_ci_filter;
		return;
	}

	// formular

	/**
	 * display form to create/modify relation(s)
	 * 
	 * the form is split into two steps, in step 1 (display_form1) the
	 * relation type will be selected, in step 2 (display_form2) the
	 * peer CI(s) are selected, based on the relation type from step 1
	 * 
	 * if "multiple" ist set TRUE, more than one peer can be selected, but
	 * in this case, its not possible, to unselect already selected peers,
	 * so please use MULTI only for create, not for edit!  
	 * 
	 * @param	string	$title			Formular title
	 * @param	string	$action1		form action for step 1
	 * @param	string	$action1_text	button text for step 1
	 * @param	string	$action2		form action for step 2
	 * @param	string	$action2_text	button text for step 2
	 * @param	bool	$multiple		set TRUE, to enable multiple selects
	 * 
	 */
	public function display_form ( $title, $action1, $action1_text, $action2, $action2_text, $multiple=FALSE )
	{
		if ($this->db_object->get_rt_id() == 0)
		{
			$this->display_form1( $title, $action1, $action1_text );
		}
		else
		{
			$this->display_form2( $title, $action2, $action2_text, $multiple );
		}
		return;
	}

	protected function display_form1 ( $title, $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post' name='r_xxx'>\n",
			$action );
		
		html_table_header( 25, 150, 625 );
	
		html_table_section( $title, browse_ci_relation_edit_menu( $this->r_ci_id, $this->r_ci_pos ) );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='r_id' value='%d'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_ci_id' value='%d'></td></tr>\n",
			$this->get_ci_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_ci_pos' value='%d'></td></tr>\n",
			$this->get_ci_pos() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_form' value='1'></td></tr>\n" );

		html_oblig_row( '*', get_string('RelationType_ttl') );
		printf( "<td class='f_value'>" );

		$rt_id=0;
		if ($this->rt !== false) $rt_id = $this->rt->get_id();
		relation_type_peer_list::getInstance($this->ci->get_cit_id(), $this->get_ci_pos())->html_options( 
			array( $rt_id ), array(), "r_rt_id", array( 0 => "--- ".get_string('SelectPlease')." ---" ), "r_xxx" );
		printf( "</td></tr>\n" );

		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );
	}

	protected function display_form2 ( $title, $action, $action_text, $multiple=FALSE )
	{
		printf( "<div id='form'>\n" );

		// formular upper part: relation type and CI filter

		printf( "<form action='%s' method='post' name='r_xxx'>\n",
			"" );

		html_table_header( 25, 150, 625 );
	
		html_table_section( $title, get_empty_menu() );

		printf( "<tr><td colspan='3'><input type='hidden' name='r_id' value='%d'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_ci_id' value='%d'></td></tr>\n",
			$this->get_ci_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_ci_pos' value='%d'></td></tr>\n",
			$this->get_ci_pos() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_rt_id' value='%d'></td></tr>\n",
			$this->db_object->get_rt_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_form' value='2'></td></tr>\n" );
	
		html_oblig_row( '*', get_string('RelationType_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			$this->rt->get_pos_text( $this->r_ci_pos ) );

		html_oblig_row( '', get_string('FilterPeerCIsWith_ttl') );
		printf( "<td class='f_value'>" );
		printf( "<input type='text' name='r_ci_filter' size=19  value='%s'  onchange='%s.submit()'>",
			htmlentities ( $this->r_ci_filter ),
			'r_xxx'
		);
		printf( "</td></tr>\n" );
		printf( "<tr><td></td><td></td><td class='tableinfo'>%s</td></tr>\n", get_string('FilterPeerCIsWith_note') );

		html_table_footer();
		printf( "</form>\n" );


		// formular lower part: peer CI, desc, relation attributes
		printf( "<form action='%s' method='post'>\n",
			$action );
	
		html_table_header( 25, 150, 625 );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='r_id' value='%d'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_ci_id' value='%d'></td></tr>\n",
			$this->get_ci_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_ci_pos' value='%d'></td></tr>\n",
			$this->get_ci_pos() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_rt_id' value='%d'></td></tr>\n",
			$this->db_object->get_rt_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='r_form' value='2'></td></tr>\n" );

		html_oblig_row( '*', get_string('PeerCI_ttl') );
		printf( "<td class='f_value'>" );
		typed_ci_list::getInstance( $this->rt->get_pos_peer_cit_id( $this->r_ci_pos ), $this->r_ci_filter )->html_options(
			array( $this->r_peer_id ), array(), "r_peer_id", array( 0 => "--- ".get_string('SelectPlease')." ---" ), "", $multiple );
		printf( "</td></tr>\n" );
		if ($multiple)
		{
			printf( "<tr><td></td><td></td><td class='tableinfo'>%s</td></tr>\n", get_string('PeerCI_note') );
		}

		html_oblig_row( '', 'Description' );
		printf( "<td class='f_value'><textarea name='r_desc' cols='%d' rows='%d'>%s</textarea></td></tr>\n",
			50, 5, html_entity_decode( $this->db_object->get_desc(), ENT_QUOTES, 'ISO-8859-1' ) );

		// alle meine attribute
		$this->attr_display_form();
	
		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );
	}

	public function display_question ( $action, $ci_id )
	{
		global $referer_script_name;

		$yes_args = new html_form_args();
		$yes_args->add( "r_id", $this->db_object->get_id() );
		$yes_args->add( "r_ci_id", $ci_id );
	
		$no_args = new html_form_args();
		$no_args->add( "ci_id", $ci_id );
		
		printf( "<div id='question'>\n" );
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('DeleteRelation_ttl'), browse_ci_filter_menu( $ci_id ) );
		html_table_footer();

		html_form_question( 0, 0, 
			sprintf( get_string('WantDeleteRelation_dsc') ),
			get_string('YES_btn'), $action, $yes_args,
			get_string('NO_btn'), $referer_script_name, $no_args );
		printf( "</div>\n" );
			
		return;
	}

	// actions

	public function insert_post ()
	{
		$imx = new relation_peer_imx( $this->get_ci_id() );
		$imx->flush();
		$imx = new relation_peer_imx( $this->get_peer_id() );
		$imx->flush();
	}

	public function update_pre ()
	{
		$imx = new relation_peer_imx( $this->db_object->get_ci1_id() );
		$imx->flush();
		$imx = new relation_peer_imx( $this->db_object->get_ci2_id() );
		$imx->flush();
	}

	public function delete_pre ()
	{
		$imx = new relation_peer_imx( $this->db_object->get_ci1_id() );
		$imx->flush();
		$imx = new relation_peer_imx( $this->db_object->get_ci2_id() );
		$imx->flush();
	}

	// helper

	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$peer_ci = new db_ci( $this->r_peer_id );
		
		$title = sprintf( get_string('Relation_ttl').": %s  - %s - %s",
			$this->ci->get_title(),
			$this->rt->get_pos_text( $this->r_ci_pos ),
			$peer_ci->get_title()
			);
		return $title;
	}

	protected function create_db_object ( $_id )
	{
		$this->db_object = new db_relation( $_id );
		$this->load_rt();
		return;
	}

	protected function load_rt ()
	{
		$this->rt = new relation_type( $this->db_object->get_rt_id() );
		return;
	}
}
