<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/*
Modifications:
Date		ID     	Author	Description
2010-07-24	MOD-001	HCK   	Optimize image labels for long text.

*/

require_once 'core/core_list.inc.php';
require_once 'includes/ci.inc.php';
require_once 'includes/relation_peer_list.inc.php';
require_once 'includes/image_helper.inc.php';

class relation_peer_imx
{
	protected $ci			= false;
	protected $peer_list	= false;
	protected $imx_name		= "";
	protected $map_name		= "";


	public function __construct ( $_ci_id )
	{
		$this->ci = new ci( $_ci_id );
		$this->create_names();
	}
	
	/**
	 * liefert das image als stream aus
	 */
	public function get_imx ()
	{
		// falls nicht vorhanden, dann erzeugen 
		if (! $this->exists() ) $this->create();

		// ausliefern
		header ("Content-type: image/png");
		readfile( $this->imx_name );
		
		return;
	}

	/**
	 * liefert die map als stream aus
	 */
	public function get_map ()
	{
		// falls nicht vorhanden, dann erzeugen 
		if (! $this->exists() ) $this->create();

		// ausliefern
		readfile( $this->map_name );
		
		return;
	}

	/**
	 * bereinigt die image datei
	 */
	public function flush ()
	{
		$this->destroy();
		return;
	}

	/**
	 * ermittelt, ob die image datei existiert
	 */
	protected function exists () 
	{
		clearstatcache();
		return (file_exists( $this->imx_name ) && file_exists( $this->map_name ));
	}

	protected function load_peer_list()
	{
		// falls nicht vorhanden, peer liste holen
		if ($this->peer_list === FALSE) $this->peer_list = relation_peer_list::getInstance( $this->ci->get_id() );
		
		return;
	}

	/**
	 * erzeugt das image neu und legt es im cache-verzeichnis ab
	 */
	protected function create ()
	{
		$this->load_peer_list();
		$count = $this->peer_list->count();

		$peer_boxes	= array();
		$peer_count = array();
		
		// map erzeugen
		$mp = fopen( $this->map_name, "w" );
		fwrite( $mp, "<map name='map_locations'>\n" );

		// bild erzeugen
		$sx = get_property("map_sx");
		$sy = get_property("map_sy");
		$im = imagecreatetruecolor( $sx, $sy );
		$color_back = ImageColorAllocate ($im, 255, 255, 255);
		$color_text = ImageColorAllocate ($im,   0,   0,   0);
		$color_red  = ImageColorAllocate ($im, 127,   0,   0);
		$color_grey = ImageColorAllocate ($im, 127, 127, 127);
		imagefill( $im, 0, 0, $color_back );
		
		// relevante werte
		$font			= get_property("map_font");
		$font_size		= get_property("map_font_size");
		$padding		= get_property("map_padding");
		$arrow_length	= get_property("map_arrow");
		$icon_size		= get_property("map_icon_size");

		$cx				= floor($sx / 2);
		$cy				= floor($sy / 2);
		$rx				= floor($cx * 0.85);
		$ry				= floor($cy * 0.9);

		// ci selbst eintragen
		$title_full = $this->ci->get_title() . " with " . $count . " Relation(s)";
		$title      = wordwrap( $this->ci->get_title(), get_property('title_wrap_width'), "\n", TRUE );
		$ci_box     = image_ft_box( $im, $font, $font_size, $padding, $cx, $cy, $title, $this->ci->get_icon(), $color_text, $color_red, $color_back, $icon_size );

		fwrite( $mp, sprintf("<area shape='rect' coords='%d,%d,%d,%d' href='%s?ci_id=%d' alt='%s' title='%s'>\n",
			 $ci_box[0], $ci_box[1], $ci_box[2], $ci_box[3], CI_VIEW_PHP, $this->ci->get_id(), $title_full, $title_full) );

		// peers eintragen
		if ($count > 0)
		{
			$delta		= floor( 360 / $count);
	
			for ($i=0; $i<$count; $i++ )
			{
				$relation_peer	= & $this->peer_list->at( $i );
				$peer_id		= $relation_peer->get_ci_id();
				$title			= wordwrap( $relation_peer->get_ci_title(), get_property('title_wrap_width'), "\n", TRUE );
				$title_full		= $this->ci->get_title() . " " . $relation_peer->get_peer_text() . " " . $title . " / " . $relation_peer->get_cit_title();
				if (strlen($relation_peer->get_desc())!=0) $title_full = $title_full . " (" . $relation_peer->get_desc() . ")";
				$icon			= $relation_peer->get_icon();
				$winkel			= deg2rad($delta * $i);
				$px				= floor( $rx * cos( $winkel ) );
				$py				= floor( $ry * sin( $winkel ) );

				if (! array_key_exists( $peer_id, $peer_boxes ))
				{
//					$peer_box = image_string_box( $im, $font_size, $padding, $cx + $px, $cy + $py, $title, $icon, $color_text, $color_grey, $color_back );
					$peer_box = image_ft_box( $im, $font, $font_size, $padding, $cx + $px, $cy + $py, $title, $icon, $color_text, $color_grey, $color_back, $icon_size );
					fwrite( $mp, sprintf("<area shape='rect' coords='%d,%d,%d,%d' href='%s?ci_id=%d' alt='%s' title='%s'>\n",
						 $peer_box[0], $peer_box[1], $peer_box[2], $peer_box[3], CI_VIEW_PHP, $relation_peer->get_ci_id(), $title_full, $title_full ) );
					$peer_boxes[ $peer_id ] = $peer_box;
					$peer_count[ $peer_id ] = 0;
				}
				else
				{
					$peer_count[ $peer_id ]++;
				}
				
				// arrow
				if ($relation_peer->get_ci_pos()==1)
				{
					$arrow = get_arrow_from_string_boxes( $ci_box, $peer_boxes[ $peer_id ], $peer_count[ $peer_id ], $arrow_length );
				}
				else
				{
					$arrow = get_arrow_from_string_boxes( $peer_boxes[ $peer_id ], $ci_box, $peer_count[ $peer_id ], $arrow_length );
				}
				image_arrow( $im, $arrow[0], $arrow[1], $arrow[2], $arrow[3], $arrow_length, $color_grey );

			}
		}

		// ausseren rahmen
//		imagerectangle( $im, 0, 0, $sx - 1, $sy - 1, $color_grey );

		// speichern
		ImagePng( $im, $this->imx_name );

		// map schliessen
		fwrite( $mp, "</map>\n" );
		fclose( $mp );

		return;
	}

	/**
	 * loescht die relation dateien im cache-verzeichnise
	 */
	protected function destroy ()
	{
		$cache_dir = get_property( "cache_dir" );
		$file_prefix = sprintf( "%s%d", get_property("map_prefix"), $this->ci->get_id() ); 
		$prefix_len = strlen( $file_prefix );
		$clear_count = 0;

		// check		
		if (! is_dir($cache_dir)) return;
		
		// oeffnen des verzeichnisses
	    if ($dh = opendir($cache_dir))
	    {
	        while (($file = readdir($dh)) !== false)
	        {
	        	if (strpos( $file, $file_prefix )!=0) continue;
	        	$file_name = $cache_dir . "/" . $file;
	        	if (is_file($file_name))
	        	{
	        		if (@unlink($file_name)) $clear_count++;
	        	}
			}
		    closedir($dh);
	    }

		return;
	}

	/**
	 * liefert einen postfix zum dateinamen, der das
	 * genutzte module beschreibt
	 */
	protected function get_module_postfix ()
	{
		return "";
	}

	/**
	 * liefert den vollstaendigen dateinamen
	 * des images im cache-verzeichnise
	 */
	private function create_names ()
	{
		$this->imx_name = sprintf( "%s/%s%d%s.png", get_property("cache_dir"), get_property("map_prefix"), $this->ci->get_id(), $this->get_module_postfix() ); 
		$this->map_name = sprintf( "%s/%s%d%s.map", get_property("cache_dir"), get_property("map_prefix"), $this->ci->get_id(), $this->get_module_postfix() ); 
		return;
	}

}

function clear_imx_cache ( &$message )
{
	$cache_dir = get_property( "cache_dir" );
	$map_prefix = get_property("map_prefix");
	$prefix_len = strlen( $map_prefix );
	$clear_count = 0;

	// check		
	if (! is_dir($cache_dir))
	{
		$message = "Cache directory not valid!";
		return STATUS_ERROR;
	}
	
	// oeffnen des verzeichnisses
    if ($dh = opendir($cache_dir))
    {
        while (($file = readdir($dh)) !== false)
        {
        	if (strpos( $file, $map_prefix )===false) continue;
        	if (strpos( $file, $map_prefix )!=0) continue;
        	$file_name = $cache_dir . "/" . $file;
        	if (is_file($file_name))
        	{
        		if (@unlink($file_name)) $clear_count++;
        	}
		}
	    closedir($dh);
    }
    $message = get_string('ClearMapCacheCompleted_dsc').", $clear_count ".get_string('FilesDeleted_dsc')."!";
    return STATUS_OK;
}

/**
 * factory function, liefert eine map-instance fuer das ci
 * 
 * dabei wird die zu nutzende klasse aus der session ermittelt.
 */
function create_map_class ( $ci_id )
{
	$ci = new db_ci( $ci_id );
	
	// class name
	$map_class = get_user_property( 'ci_map_class', $ci->get_cit_id(), $ci->get_id(), CI_MAP_CLASS_DEFAULT);
	if (!class_exists($map_class)) return FALSE;

//echo "DEBUG: create_map_class( $ci_id ): $map_class<br>\n";

	$imx = new $map_class( $ci_id );
	return $imx;
}
