<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_api.inc.php';
require_once 'core/core_html_form.inc.php';
require_once 'tables/db_relation_type.inc.php';
require_once 'includes/ci_type_list.inc.php';
require_once 'includes/type_attr_list.inc.php';
require_once 'includes/design.inc.php';
require_once 'includes/entity_processor.inc.php';

class relation_type extends entity_processor
{
	protected $cit1		= null;
	protected $cit2		= null;


	/**
	 * attribute des Relation-Types aus argumenten laden
	 * 
	 * ueberlaedt die felder des DB-Objektes mit den
	 * werten aus den argumenten, dazu muss allerdings
	 * das argument "rt_form" gesetzt sein 
	 * 
	 * @return	none
	 */	
	public function load_from_args ()
	{
		$_form = http_get_int("rt_form", 0);
		if ($_form == 0) return;

		$_cit1_id = http_get_int("rt_cit1_id", $this->db_object->get_cit1_id() );
		$this->db_object->set_cit1_id( $_cit1_id );

		$_cit2_id = http_get_int("rt_cit2_id", $this->db_object->get_cit2_id() );
		$this->db_object->set_cit2_id( $_cit2_id );

		$_to1_text = http_get_string("rt_to1_text", $this->db_object->get_to1_text() );
		$this->db_object->set_to1_text( $_to1_text );

		$_to2_text = http_get_string("rt_to2_text", $this->db_object->get_to2_text() );
		$this->db_object->set_to2_text( $_to2_text );

		$_desc = http_get_string("rt_desc", $this->db_object->get_desc() );
		$this->db_object->set_desc( $_desc );

		return;
	}

	// advanced get
	
	/**
	 * liefert die position in der relation
	 * 
	 * ermittelt, an welches "ende" der verbindung der
	 * CI-Type passt und liefert die entsprechende
	 * Nummer zurueck
	 * 
	 * @param	_cit_id		int		ID des CI-Types
	 * @return				int		1 oder 2
	 */
	public function get_pos ( $_cit_id )
	{
		$pos = 2;
		if (
			($_cit_id == $this->db_object->get_cit1_id())
			||
			(
				($_cit_id != $this->db_object->get_cit2_id())
				&&
				($this->db_object->get_cit1_id()==0)
			)
		)
		{
			$pos = 1;
		}

//echo "DEBUG relation_type::get_pos( $_cit_id ): $pos\n";

		return $pos;
	}

	public function get_pos_text ( $_pos )
	{
		$pos_text = "";
		if ($_pos == 1) $pos_text=$this->db_object->get_to2_text();
		if ($_pos == 2) $pos_text=$this->db_object->get_to1_text();
		return $pos_text;
	}

	public function get_pos_peer_cit_id ( $_pos )
	{
		if ($_pos == 1) $peer_id=$this->db_object->get_cit2_id();
		if ($_pos == 2) $peer_id=$this->db_object->get_cit1_id();
		return $peer_id;
	}

	// formulare

	public function display_form ( $title, $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );
		html_table_header( 25, 200, 575 );
	
		html_table_section( $title, design_rel_form_menu( $this->get_id() ) );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='rt_id' value='%s'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='rt_form' value='1'></td></tr>\n" );

		html_oblig_row( '*', get_string('CIType1_ttl') );
		printf( "<td class='f_value'>" );
		ci_type_list::getInstance()->html_options( array( $this->db_object->get_cit1_id() ), array(), "rt_cit1_id", array( 0 => "--- Select please ---" ) );
		printf( "</td></tr>\n" );

		html_oblig_row( '*', get_string('CIType2_ttl') );
		printf( "<td class='f_value'>" );
		ci_type_list::getInstance()->html_options( array( $this->db_object->get_cit2_id() ), array(), "rt_cit2_id", array( 0 => "--- Select please ---" ) );
		printf( "</td></tr>\n" );

	
		html_oblig_row( '*', get_string('RelationText1gt2_ttl') );
		printf( "<td class='f_value'><input type='text' name='rt_to2_text' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 200, html_entity_decode( $this->db_object->get_to2_text(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '*', get_string('RelationText1lt2_ttl') );
		printf( "<td class='f_value'><input type='text' name='rt_to1_text' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 200, html_entity_decode( $this->db_object->get_to1_text(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '', get_string('Description_ttl') );
		printf( "<td class='f_value'><textarea name='rt_desc' cols='%d' rows='%d'>%s</textarea></td></tr>\n",
			50, 5, html_entity_decode( $this->db_object->get_desc(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
		printf( "</form>\n" );	
		printf( "</div>\n" );
	
	}

	public function display_question ( $action )
	{
		global $referer_script_name;

		$yes_args = new html_form_args();
		$yes_args->add( "rt_id", $this->db_object->get_id() );
	
		$no_args = new html_form_args();
		$no_args->add( "rt_id", $this->db_object->get_id() );
		
		printf( "<div id='question'>\n" );
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('DeleteRelationType_ttl'), design_rel_form_menu( $this->db_object->get_id() ) );
		html_table_footer();

		html_form_question( 0, 0, 
			get_string('WandDeleteRelationType_dsc'),
			get_string('YES_btn'), $action, $yes_args,
			get_string('NO_btn'), $referer_script_name, $no_args );
		printf( "</div>\n" );
			
		return;
	}

	// display methoden

	/**
	 * zeigt die synopse des relation-types
	 */
	public function display_synopse ( $show_menu=FALSE )
	{
		$menu = & get_empty_menu();
		if ($show_menu) $menu = & design_rel_view_menu( $this->get_id() );
		
		$this->html_synopse_section( $menu );

		return;
	}

	/**
	 * zeigt die specification des relation-types an
	 */
	public function display_spec ()
	{
		printf( "<div id='spec'>\n" );
		html_table_header( 25, 200, 575 );

		html_table_section( get_string('Specification_ttl'), design_rel_view_spec_menu( $this->get_id() ) );

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('CIType1_ttl'), $this->cit1->get_title() ); 

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('CIType2_ttl'), $this->cit2->get_title() ); 

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('RelationText1gt2_ttl'), $this->db_object->get_to2_text() ); 

		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('RelationText1lt2_ttl'), $this->db_object->get_to1_text() ); 
	
		printf( "<tr><td></td><td class='f_attr'>%s:</td><td class='f_value'>%s</td></tr>\n",
			get_string('Description_ttl'), $this->db_object->get_desc() ); 
	
		// change info
//		printf( "<tr><td class='changeinfo' colspan='3'>last changed at %s by %s</td></tr>\n",
//			$this->db_object->get_last_change(), $this->get_last_user() );

		html_table_footer();
		printf( "</div>\n" );

		return;
	}

	/**
	 * zeigt die attribute des relation-types an
	 */
	public function display_attributes ()
	{
		$type_attr_list = & type_attr_list::getInstance( CLASS_REL, $this->get_id() );
		$type_attr_count = $type_attr_list->count();
		
		printf( "<div id='attributes'>\n" );
		html_table_header( 50, 20, 20, 20, 200, 340, 100, 50 );
		html_table_section( get_string('Attributes_ttl')." (".get_string('Total_ttl').": $type_attr_count)", design_ta_index_menu( CLASS_REL, $this->get_id() ) );
?>
		<tr class='r_head'>
			<td class='c_action r_head'>Order</td>
			<td class='c_action r_head' title='Obligate'>O</td>
			<td class='c_action r_head' title='Unique'>U</td>
			<td class='c_action r_head' title='Tag'>T</td>
			<td class='c_text r_head'>Title</td>
			<td class='c_text r_head'>Description</td>
			<td class='c_text r_head'>Type</td>
			<td class='c_action r_head'>Actions</td>
		</tr>
<?php
		if ($type_attr_count == 0)
		{
			html_empty_row( 0, 0 );
		}
		else
		{ 
			html_table_row_reset( array("r_normal","r_normal2") );
			for ($i=0; $i<$type_attr_count; $i++)
			{
				$ta = & $type_attr_list->at( $i );
		
				html_table_row();
				printf( "<td class='c_action'>%s %s</td>" .
					"<td class='c_action'>%s</td>" .
					"<td class='c_action'>%s</td>" .
					"<td class='c_action'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_action'>%s %s</td></tr>\n",
					icon_display_url(TA_UP_RUN_PHP . "?ta_id=" . $ta->get_id(), ICON_UP, get_string('MoveUp_dsc') ),
					icon_display_url(TA_DOWN_RUN_PHP . "?ta_id=" . $ta->get_id(), ICON_DOWN, get_string('MoveDown_dsc') ),
					html_status_checkbox( $ta->get_oblig() ),
					html_status_checkbox( $ta->get_unique() ),
					html_status_checkbox( $ta->get_tag() ),
					$ta->get_title(), 
					$ta->get_desc(),
					get_enum_to_string( "enum_datatypes", $ta->get_datatype() ),
					icon_display_url(TA_EDIT_PHP . "?ta_id=" . $ta->get_id(), ICON_EDIT, get_string('EditRelationTypeAttribute_dsc') ),
					icon_display_url(TA_DELETE_PHP . "?ta_id=" . $ta->get_id(), ICON_DELETE, get_string('DeleteRelationTypeAttribute_dsc') )
					);
			}
		}
		html_table_footer();
		printf( "</div>\n" );

		return;
	}


	// action support

	public function action_switch ()
	{
		$rc = FALSE;
		try {
			// args laden
			$this->load_from_args();		
			
			// valid check
			$this->valid_for_switch();
	
			// pre
			$this->switch_pre();

			// speichern
			$this->db_object->db_update();
			
			// post
			$this->switch_post();

			// message
			$this->add_message( get_string('SwitchSuccessful_dsc'), STATUS_OK );
			$rc = TRUE;
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		catch (core_data_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('ValidationError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		return $rc;
	}


	// pre- / post- actions
	
	protected function switch_pre ()
	{
		// swap cit
		$cit_swap = $this->db_object->get_cit1_id();
		$this->db_object->set_cit1_id( $this->db_object->get_cit2_id() );
		$this->db_object->set_cit2_id( $cit_swap ); 

		// swap text
		$text_swap = $this->db_object->get_to1_text();
		$this->db_object->set_to1_text( $this->db_object->get_to2_text() );
		$this->db_object->set_to2_text( $text_swap );
		
		return;
	}

	protected function switch_post ()
	{
		$db = core_db::getInstance();
		$query = "SELECT * FROM relation " .
			"WHERE r_rt_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query ); 
		if (($count = $db->num_rows( $result )) != 0)
		{
			for ($i=0; $i<$count; $i++)
			{
				$data = $db->fetch_object( $result, $i );
				$query = "UPDATE relation SET " .
					"r_ci1_id='$data->r_ci2_id', " .
					"r_ci2_id='$data->r_ci1_id' " .
					"WHERE r_id='$data->r_id' LIMIT 1";
				$update = $db->query( $query );
			}
		}

		return;
	}

	// validatoren fuer actions
	
	public function delete_pre ()
	{
		// loeschen der Type attribute
		$db = core_db::getInstance();
		$query = "DELETE FROM type_attr " .
			"WHERE ta_classtype='" . CLASS_REL . "' " .
			"AND ta_classtype_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query );
	}

	public function valid_for_delete ()
	{
		$db = core_db::getInstance();
		$query = "SELECT COUNT(*) FROM relation " .
			"WHERE r_rt_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query ); 
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if (($count = $db->result( $result, 0 )) != 0)
		{
			throw new core_data_exception( "Relation-Type used by $count relation(s)");
		}
		
		return TRUE;
	}

	public function valid_for_switch ()
	{
		return TRUE;
	}

	
	// helper

	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$title = sprintf( "%s: %s - %s - %s", get_string('RelationType_ttl'), $this->cit1->get_title(), $this->db_object->get_title(), $this->cit2->get_title() );
		return $title;
	}

	protected function create_db_object ( $_id )
	{
		$this->db_object = new db_relation_type( $_id );
		$this->update_cit();
		return;
	}

	protected function update_cit ()
	{
		$this->cit1	= new db_ci_type( $this->db_object->get_cit1_id() ); if (! is_object($this->cit1)) $this->cit1 = new db_ci_type();
		$this->cit2	= new db_ci_type( $this->db_object->get_cit2_id() ); if (! is_object($this->cit2)) $this->cit2 = new db_ci_type();
		return;		
	}

}
