<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'tables/db_relation_type.inc.php';
require_once 'includes/navi_list.inc.php';

class relation_type_list extends navi_list
{
	static private $instance = array();
	protected $cit_id = 0;
	protected $ci_types;

	static public function getInstance( $_cit_id = 0 )
	{
		if (! array_key_exists( $_cit_id, self::$instance )) self::$instance[ $_cit_id ] = new relation_type_list( $_cit_id );
		return self::$instance[ $_cit_id ];
	}
	
	protected function __construct ( $_cit_id )
	{
		$this->cit_id = $_cit_id;
		$this->ci_types = ci_type_list::getInstance();
		parent::__construct();
	}
	
	protected function get_navi_args ()
	{
		$navi_args = "&amp;cit_id=" . $this->cit_id;
		return $navi_args;
	}

	protected function get_dont_load_keys ()
	{
		$key_array = parent::get_dont_load_keys();
		$key_array[] = "cit_id";
		$key_array[] = "ci_types";
		return $key_array;
	}

	protected function post_load ()
	{
		$this->page_size = get_property("page_rows");
		parent::post_load();
	}

	protected function get_session_store_name ()
	{
		return sprintf( "relation_type_list_%d", $this->cit_id );
	} 

	protected function get_element_name ()
	{
		return get_string('RelationType_ttl');
	} 
	
	protected function get_id_name ()
	{
		return "rt_id";
	}
	
	public function show_page ()
	{
		html_table_header( 150, 200, 150, 230, 70 );
		switch ($this->get_list_mode())
		{
			case LIST_MODE_USE:
			{
				print ( "<tr class='r_head'>" );
				printf( "	<td class='c_text r_head'>%s</td>\n", get_string('CIType1_ttl'));
				printf( "	<td class='c_text r_head'>%s</td>\n", get_string('RelationType_ttl'));
				printf( "	<td class='c_text r_head'>%s</td>\n", get_string('CIType2_ttl'));
				printf( "	<td class='c_text r_head'>%s</td>\n", get_string('Description_ttl'));
				printf( "	<td class='c_action r_head'>%s</td>\n", get_string('Action_ttl'));
				print ( "</tr>" );
				break;
			}
			case LIST_MODE_MANAGE:
			{
				print ( "<tr class='r_head'>" );
				printf( "	<td class='c_text r_head'>%s</td>\n", get_string('CIType1_ttl'));
				printf( "	<td class='c_text r_head'>%s</td>\n", get_string('RelationType_ttl'));
				printf( "	<td class='c_text r_head'>%s</td>\n", get_string('CIType2_ttl'));
				printf( "	<td class='c_text r_head'>%s</td>\n", get_string('Description_ttl'));
				printf( "	<td class='c_action r_head'>%s</td>\n", get_string('Action_ttl'));
				print ( "</tr>" );
				break;
			}
		}
		
		parent::show_page();
		html_table_footer();

		return;
	}	

	protected function show_item ( $i )
	{
		$rt		= & $this->at( $i );
		$cit1	= $this->ci_types->get( $rt->get_cit1_id() ); if (! is_object($cit1)) $cit1 = new db_ci_type();
		$cit2	= $this->ci_types->get( $rt->get_cit2_id() ); if (! is_object($cit2)) $cit2 = new db_ci_type();
		$icon_size  = get_property("list_icon_size");

		html_table_row();

		switch ($this->get_list_mode())
		{
			case LIST_MODE_USE:
			{		
				break;
			}
			case LIST_MODE_MANAGE:
			{		
				printf( "<td class='c_text'>%s %s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_text'>%s %s</td>" .
					"<td class='c_text'>%s</td>" .
					"<td class='c_action'>%s %s %s</td>" .
					"</tr>\n",
					icon_list::getInstance()->get_html_imx( $cit1->get_icon(), $icon_size, $icon_size, TRUE ), $cit1->get_title(),
					get_html_url( RT_VIEW_PHP . "?rt_id=" . $rt->get_id(), $rt->get_title(), FALSE, get_string('ShowRelationType_dsc') ),
					icon_list::getInstance()->get_html_imx( $cit2->get_icon(), $icon_size, $icon_size, TRUE ), $cit2->get_title(),
					$rt->get_desc(),
					icon_display_url(RT_SWITCH_RUN_PHP . "?rt_id=" . $rt->get_id(), ICON_SWITCH, get_string('SwapDirection_dsc') ),
					icon_display_url(RT_EDIT_PHP . "?rt_id=" . $rt->get_id(), ICON_EDIT, get_string('EditRelationType_dsc') ),
					icon_display_url(RT_DELETE_PHP . "?rt_id=" . $rt->get_id(), ICON_DELETE, get_string('DeleteRelationType_dsc') )
					);
				break;
			}
		}
	}

	protected function get_query ()
	{
		$query = array();
		if ($this->cit_id==0)
		{
			$query[] = "SELECT * FROM relation_type, ci_type as cit1, ci_type as cit2 " .
				"WHERE rt_cit1_id=cit1.cit_id " .
				"AND rt_cit2_id=cit2.cit_id " .
				"ORDER BY cit1.cit_title, cit2.cit_title, rt_to1_text, rt_to2_text";
		}
		else
		{
			$query[] = "SELECT * FROM relation_type " .
				"WHERE rt_cit1_id='0' OR rt_cit1_id='$this->cit_id' " .
				"OR rt_cit2_id='0' OR rt_cit2_id='$this->cit_id' " .
				"ORDER BY rt_to1_text, rt_to2_text";
		} 
		return $query;
	}
	
	protected function cat_option ( &$db_object, $ref_id )
	{
		if ($this->cit_id==0)
		{
			cat_option( $db_object->get_id(), $ref_id, $db_object->get_to1_text() . "/" . $db_object->get_to2_text());
		}
		else
		{
			cat_option( $db_object->get_id(), $ref_id, $db_object->get_to1_text() . "/" . $db_object->get_to2_text());
		}
	}

	protected function create_object ( $_id )
	{
		$db_object = new db_relation_type( $_id ); 
		return $db_object;
	}

	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$title = get_string('AllRelationTypes_ttl');
		return $title;
	}


} 
