<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/*
Modifications:
Date		ID     	Author	Description
2010-07-23	MOD-001	HCK   	Added additional query to search attributes of relations.

*/

require_once 'tables/db_typed_ci.inc.php';
require_once 'includes/ci_navi_list.inc.php';
require_once 'includes/icon_list.inc.php';

/**
 * class for various search modes
 * 
 * manages search configuration, create search-statements
 * search configuration will be stored in session
 * 
 * singleton implementation
 */
class search_list extends ci_navi_list
{
	static private $instance	= false;
	protected $search_mode		= 0;		// search mode, see SEARCH_MODE_ constants
	protected $search_text		= "";		// search for this text
	protected $search_cit_id	= 0;		// limit search for this CI-TYPE
	protected $search_cat_id	= 0;		// limit search for this CATALOG
	protected $search_cate_id	= 0;		// limit search for this catalog entry
	protected $search_compare_operator	= 0;// limit search comparing number of relations
	protected $search_compare_value		= 0;// limit search comparing number of relations
	protected $relation_condition_array		= array();	

	static public function getInstance()
	{
		if (self::$instance === false) self::$instance = new search_list();
		return self::$instance;
	}
	
	protected function get_session_store_name ()
	{
		return "search";
	} 


	/**
	 * reply TRUE, if object with $_idx as ID is in the list, otherwise FALSE
	 * 
	 * @param 	integer	$_idx
	 * @return	boolean	
	 */
	public function object_exists ( $_idx )
	{
		// if $_idx is not in condition_array, we reply TRUE, to avoid from insert
		if (
			($this->search_mode == SEARCH_MODE_ADVANCED)
			&&
			($this->search_compare_operator != 0)
			&&		
			( ! in_array( $_idx, $this->relation_condition_array ) )
			) return TRUE;

		return parent::object_exists( $_idx );
	}

	public function get_search_mode()
	{
		return $this->search_mode;
	}

	public function get_search_text()
	{
		return $this->search_text;
	}

	public function get_search_cit_id()
	{
		return $this->search_cit_id;
	}

	public function get_search_cat_id()
	{
		return $this->search_cat_id;
	}

	public function get_search_cate_id()
	{
		return $this->search_cate_id;
	}

	public function get_search_compare_operator()
	{
		return $this->search_compare_operator;
	}

	public function get_search_compare_value()
	{
		return $this->search_compare_value;
	}

	protected function get_dont_load_keys ()
	{
		$key_array = parent::get_dont_load_keys();
		$key_array[] = "relation_condition_array";
		return $key_array;
	}

	/**
	 * create queries and put them in an array
	 * 
	 * @return	array		list of queries
	 */
	protected function get_query ()
	{
		$query = array();

		switch ($this->search_mode)
		{
			case SEARCH_MODE_SIMPLE:
			{
				if (strlen($this->search_text)!=0)
				{
					// search in ci title
					$query[]  = "SELECT ci.*, ci_type.* FROM ci, ci_type " .
						"WHERE ci_title like '%" . $this->search_text . "%' " .
						"AND ci_cit_id=cit_id " .
						"ORDER BY ci_title, cit_title";
		
					// search in ci attributes
					$query[]  = "SELECT DISTINCT(a_classtype_id), ci.*, ci_type.* FROM attr, ci, ci_type " .
						"WHERE a_val_text like '%" . $this->search_text . "%' " .
						"AND a_classtype='" . CLASS_CI . "' " .
						"AND a_classtype_id=ci_id " .
						"AND ci_cit_id=cit_id " .
						"GROUP BY a_classtype_id " .
						"ORDER BY ci_title, cit_title";
		
					// search in comments
					$query[]  = "SELECT ci.*, ci_type.* FROM comment, ci, ci_type " .
						"WHERE c_text like '%" . $this->search_text . "%' " .
						"AND c_ci_id=ci_id " .
						"AND ci_cit_id=cit_id " .
						"ORDER BY ci_title, cit_title";

					// search in relation names, regardless of relation direction
					$query[]  = "SELECT ci.*, ci_type.* FROM relation, ci, ci_type " .
						"WHERE r_desc like '%" . $this->search_text . "%' " .
						"AND (r_ci1_id=ci_id OR  r_ci2_id=ci_id) " .
						"AND ci_cit_id=cit_id " .
						"ORDER BY ci_title, cit_title";

					// search in relation attributes
					$query[]  = "SELECT ci.*, ci_type.* FROM relation, ci, ci_type " . 
						"WHERE ci_cit_id=cit_id " .
						"AND (r_ci1_id=ci_id OR r_ci2_id=ci_id) " .
						"AND  r_id IN (" .
							"SELECT a_classtype_id FROM attr " .
							"WHERE a_classtype=" . CLASS_REL . " " .
							"AND a_val_text LIKE '%" . $this->search_text . "%'" . 
						") " .
						"ORDER BY ci_title, cit_title";

				}
				break;
			}
			case SEARCH_MODE_ADVANCED:
			{
				// create sub-select
				// SELECT ci_id
				// FROM ci[, attr, type_attr] <$ss_fc_cat>
				// WHERE 1=1
				// AND ci_cit_id='search_cit_id' <$ss_wc_cit>
				// AND a_classtype=CLASS_CI
				//		AND a_classtype_id=ci_id
				//		AND a_ta_id=ta_id
				//		AND ta_datatype=DT_CATALOG
				//		AND ta_datatype_sub=search_cat_id
				//		[
				//			AND a_val_int=search_cate_id
				//		] 
				//		<$ss_wc_cat>

				// check: any choice given
				//
				if (
					(strlen($this->search_text)!=0)
					||
					($this->search_cit_id != 0)
					||
					($this->search_cat_id != 0)
					||
					($this->search_compare_operator != 0)
					)
				{
					// 
					$this->_load_conditions();
	
					// sub: where clause for CIT
					$ss_wc_cit = "";
					if ($this->search_cit_id != 0) $ss_wc_cit = "AND ci_cit_id='" . $this->search_cit_id . "' ";
					
					// sub: where clause for CAT
					$ss_fc_cat = "";
					$ss_wc_cat = "";
					if ($this->search_cat_id != 0)
					{ 
						$ss_fc_cat = ", attr, type_attr";
						$ss_wc_cat = "AND a_classtype='" . CLASS_CI . "' " .
							"AND a_classtype_id=ci_id " .
							"AND a_ta_id=ta_id " .
							"AND ta_datatype='" . DT_CATALOG . "' " .
							"AND ta_datatype_sub='" . $this->search_cat_id . "' ";
						if ($this->search_cate_id != 0)
						{
							$ss_wc_cat = $ss_wc_cat . "AND a_val_int='" . $this->search_cate_id . "' ";
						}	
					}
					
					// sub: complete
					$sub_select = "SELECT ci_id FROM ci" . $ss_fc_cat . " " .
						"WHERE 1=1 " . $ss_wc_cit . $ss_wc_cat;
	
					// where clause for search-text
					$wc_text = "AND ci_title like '%";
					if (strlen($this->search_text)!=0) $wc_text = $wc_text . $this->search_text . "%";
					$wc_text = $wc_text . "' ";
					
					// main: in ci title
					$query[]  = "SELECT ci.*, ci_type.* FROM ci, ci_type " .
						"WHERE ci_title like '%" . $this->search_text . "%' " .
						"AND ci_cit_id=cit_id " .
						"AND ci_id in (" . $sub_select . ") " .
						"ORDER BY ci_title, cit_title";
	
					// main: in ci attributes
					$query[] = "SELECT DISTINCT(a_classtype_id), ci.*, ci_type.* FROM attr, ci, ci_type " .
						"WHERE 1=1 " . $wc_text .
						"AND a_classtype='" . CLASS_CI . "' " .
						"AND a_classtype_id=ci_id " .
						"AND ci_cit_id=cit_id " .
						"AND ci_id in (" . $sub_select . ") " .
						"GROUP BY a_classtype_id " .
						"ORDER BY ci_title, cit_title";
					
					// main: in comments
					$query[]  = "SELECT ci.*, ci_type.* FROM comment, ci, ci_type " .
						"WHERE c_text like '%" . $this->search_text . "%' " .
						"AND c_ci_id=ci_id " .
						"AND ci_cit_id=cit_id " .
						"AND ci_id in (" . $sub_select . ") " .
						"ORDER BY ci_title, cit_title";

					// main: in relation names
					$query[]  = "SELECT ci.*, ci_type.* FROM relation, ci, ci_type " .
						"WHERE r_desc like '%" . $this->search_text . "%' " .
						"AND ( r_ci1_id=ci_id OR r_ci2_id=ci_id) " .
						"AND ci_cit_id=cit_id " .
						"AND ci_id in (" . $sub_select . ") " .
						"ORDER BY ci_title, cit_title";
					
					// main: in relation attributes
					$query[]  = "SELECT ci.*, ci_type.* FROM relation, ci, ci_type " . 
						"WHERE ci_cit_id=cit_id " .
						"AND ci_id in (" . $sub_select . ") " .
						"AND (r_ci1_id=ci_id OR r_ci2_id=ci_id) " .
						"AND  r_id IN (" .
							"SELECT a_classtype_id FROM attr " .
							"WHERE a_classtype=" . CLASS_REL . " " .
							"AND a_val_text LIKE '%" . $this->search_text . "%'" . 
						") " .
						"ORDER BY ci_title, cit_title";

				}

				break;
			}
			case SEARCH_MODE_CIT:
			{
				// search by CI-TYPE
				$query[] = "SELECT * FROM ci, ci_type " .
					"WHERE ci_cit_id=cit_id " .
					"AND cit_id='$this->search_cit_id' " .
					"ORDER BY ci_title, cit_title";
				break;
			}
			case SEARCH_MODE_CATE:
			{
				// search by catalog entry
				$query[] = "SELECT DISTINCT(ci_id), ci.*, ci_type.* FROM ci, ci_type, attr, type_attr " .
					"WHERE ta_datatype='" . DT_CATALOG . "' " .
					"AND a_ta_id=ta_id " .
					"AND a_val_int=$this->search_cate_id " .
					"AND a_classtype='" . CLASS_CI . "' " .
					"AND a_classtype_id=ci_id " .
					"AND ci_cit_id=cit_id " .
					"ORDER BY ci_title, cit_title";
				break;
			}
		}

//echo "<pre>\n";
//print_r( $query );
//echo "</pre>\n";

		return $query;
	}

	// set

	/**
	 * set search mode, the given search mode must be valid 
	 * 
	 * @param	$_mode		search mode (see SEARCH_MODE_ constants)
	 */
	protected function set_search_mode ( $_mode )
	{
		if (! in_array( $_mode, array(SEARCH_MODE_SIMPLE, SEARCH_MODE_ADVANCED, SEARCH_MODE_CIT, SEARCH_MODE_CATE) )) return;
		$this->reset();
		$this->search_mode = $_mode;
		return;
	}

	/**
	 * reset search setup
	 */
	protected function reset()
	{
		parent::reset();
		$this->search_mode		= 0;
		$this->search_text		= "";
		$this->search_cit_id	= 0;
		$this->search_cate_id	= 0;
		$this->search_compare_operator	= 0;
		$this->search_compare_value		= 0;

		return;
	}

	// helper

	/**
	 * load search setup from session and overload from args
	 */
	protected function _load()
	{
		parent::_load();

		// overload from args
		$_form = http_get_int( 's_form', 0 );
		switch ($_form)
		{
			case SEARCH_MODE_SIMPLE:
			{
				$this->set_search_mode( SEARCH_MODE_SIMPLE );
	
				$_text = http_get_string( 's_text', "" );
				$this->search_text = $_text;
				break; 
			}
			case SEARCH_MODE_ADVANCED:
			{
				$this->set_search_mode( SEARCH_MODE_ADVANCED );

				$_text = http_get_string( 's_text', "" );
				$this->search_text = $_text;

				$_cit_id = http_get_int( 's_cit_id', $this->search_cit_id );
				$this->search_cit_id = $_cit_id;

				$_cat_id = http_get_int( 's_cat_id', $this->search_cat_id );
				$this->search_cat_id = $_cat_id;

				if ($_cat_id == 0)
				{
					$_cate_id = 0;
				}
				else
				{	
					$_cate_id = http_get_int( 's_cate_id', $this->search_cate_id );
				}
				$this->search_cate_id = $_cate_id;

				$_compare_operator = http_get_int( 's_compare_operator', $this->search_compare_operator );
				$this->search_compare_operator = $_compare_operator;

				$_compare_value = http_get_int( 's_compare_value', $this->search_compare_value );
				$this->search_compare_value = $_compare_value;

				break;
			}
			default:
			{
				// by CI-Type
				$_cit_id = http_get_int( 'cit_id', 0 );
				if ($_cit_id != 0)
				{
					$this->set_search_mode( SEARCH_MODE_CIT );
					$this->search_cit_id = $_cit_id;
				}
				
				// by Catalog Entry
				$_cate_id = http_get_int( "cate_id", 0 );
				if ($_cate_id != 0)
				{
					$this->set_search_mode( SEARCH_MODE_CATE );
					$this->search_cate_id = $_cate_id;
				}
				
			}
		}

		return;
	}

	private function _load_conditions ()
	{
		if (($operator = $this->get_search_compare_operator()) == 0) return;
		$value = $this->get_search_compare_value();
		
		$db = core_db::getInstance();
		$query  = "SELECT ci_id, COUNT(r_id) AS count_r FROM ci LEFT JOIN relation " .
			"ON r_ci1_id=ci_id " .
			"XOR r_ci2_id=ci_id " .
			"XOR r_ci1_id is null " .
			"XOR r_ci2_id is null " .
			"GROUP BY ci_id";			
		$result = $db->query( $query );			
		$num = $db->num_rows( $result );
		switch ($this->get_search_compare_operator())
		{
			case 1:
			{
				// lower than
				for ($i=0; $i<$num; $i++)
				{
					$data = $db->fetch_object( $result );
					if ($data->count_r < $value) $this->relation_condition_array[] = $data->ci_id;
				}
				break;
			}
			case 2:
			{
				// equal
				for ($i=0; $i<$num; $i++)
				{
					$data = $db->fetch_object( $result );
					if ($data->count_r == $value) $this->relation_condition_array[] = $data->ci_id;
				}
				break;
			}
			case 3:
			{
				// greater than
				for ($i=0; $i<$num; $i++)
				{
					$data = $db->fetch_object( $result );
					if ($data->count_r > $value) $this->relation_condition_array[] = $data->ci_id;
				}
				break;
			}
		}
		
		return;
	}

}
