<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_api.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_html_form.inc.php';
require_once 'tables/db_classtype_fabric.inc.php';
require_once 'tables/db_type_attr.inc.php';
require_once 'includes/entity_processor.inc.php';
require_once 'includes/cat_list.inc.php';
require_once 'includes/ci_type_list.inc.php';
require_once 'includes/module_base.inc.php';

class type_attr extends entity_processor
{
	protected $form_status = 0;
	
	public function set_classtype ( $_classtype )
	{
		if ($this->db_object->get_id()==0) $this->db_object->set_classtype( $_classtype );
	}

	public function get_classtype ()
	{
		return $this->db_object->get_classtype();
	}

	public function get_classtype_id ()
	{
		return $this->db_object->get_classtype_id();
	}

	public function get_form_status ()
	{
		return $this->form_status;
	}

	/**
	 * liefert TRUE, wenn alle Angaben vollstaendig sind
	 * 
	 * die methode liefert FALSE, wenn:
	 * - der Type auf DT_CATALOG steht und der Subtype 0 ist
	 * - der Type auf DT_CI_REF steht und der Subtype 0 ist
	 */
	public function is_complete ()
	{
		if ( ($this->db_object->get_type()==DT_CATALOG) && ($this->db_object->get_datatype_sub()==0) ) return FALSE; 
		if ( ($this->db_object->get_type()==DT_CI_REF) && ($this->db_object->get_type_sub()==0) ) return FALSE; 

		return TRUE;
	}
	
	/**
	 * attribute des Type-Attr aus argumenten laden
	 * 
	 * ueberlaedt die felder des DB-Objektes mit den
	 * werten aus den argumenten, dazu muss allerdings
	 * das argument "ta_form" gesetzt sein 
	 * 
	 * @return	none
	 */	
	public function load_from_args ()
	{
		// nur aus args ueberladen, wenn noch nicht gespeichert
		if ($this->get_id()==0)
		{
			$_classtype = http_get_int("ta_classtype", $this->db_object->get_classtype() );
			$this->db_object->set_classtype( $_classtype );

			$_classtype_id = http_get_int("ta_classtype_id", $this->db_object->get_classtype_id() );
			$this->db_object->set_classtype_id( $_classtype_id );
		}

		$this->form_status = http_get_int("ta_form", $this->form_status);
		if ($this->form_status == 0) return;

		$_title = http_get_string("ta_title", $this->db_object->get_title() );
		$this->db_object->set_title( $_title );

		$_desc = http_get_string("ta_desc", $this->db_object->get_desc() );
		$this->db_object->set_desc( $_desc );

		$_visible = http_get_int("ta_visible", $this->db_object->get_visible() );
		$this->db_object->set_visible( $_visible );

		$_datatype = http_get_int("ta_datatype", $this->db_object->get_datatype() );
		$this->db_object->set_datatype( $_datatype );

		$_order = http_get_int("ta_order", $this->db_object->get_order() );
		$this->db_object->set_order( $_order );

		$_label = http_get_int("ta_label", $this->db_object->get_label() );
		$this->db_object->set_label( $_label );

		$_oblig = http_get_int("ta_oblig", 0 );
		$_oblig = ($_oblig == 1);
		$this->db_object->set_oblig( $_oblig );

		$_unique = http_get_int("ta_unique", 0 );
		$_unique = ($_unique == 1);
		$this->db_object->set_unique( $_unique );
		
		$_tag = http_get_int("ta_tag", 0 );
		$_tag = ($_tag == 1);
		$this->db_object->set_tag( $_tag );
		
		$_input = http_get_int("ta_input", 0 );
		$this->db_object->set_input( $_input );
		
		$_format = http_get_string("ta_format", $this->db_object->get_format() );
		$this->db_object->set_format( $_format );

		$_datatype_sub = http_get_int("ta_datatype_sub", $this->db_object->get_datatype_sub() );
		$this->db_object->set_datatype_sub( $_datatype_sub );

		return;
	}

	public function action_up ()
	{
		$rc = TRUE;
		try
		{
			// vorgaenger bestimmen
			if (($peer = & $this->get_prev_attribute()) == null) return; 
	
			// order swappen
			$swap = $peer->get_order();
			$peer->set_order( $this->db_object->get_order() );
			$this->db_object->set_order( $swap );
			
			// speichern
			$peer->db_update();
			$this->db_object->db_update();
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
			$rc = FALSE;
		}
		return $rc;
	}

	public function action_down ()
	{
		$rc = TRUE;
		try
		{
			// nachfolger bestimmen
			if (($peer = & $this->get_next_attribute()) == null) return; 
	
			// order swappen
			$swap = $peer->get_order();
			$peer->set_order( $this->db_object->get_order() );
			$this->db_object->set_order( $swap );
			
			// speichern
			$peer->db_update();
			$this->db_object->db_update();
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
			$rc = FALSE;
		}
		return $rc;
	}

	public function display_form ( $title, $action1, $action1_text, $action2, $action2_text )
	{
		switch ($this->get_form_status())
		{
			case 0:
			{
				$this->display_form1( $title, $action1, $action1_text );
				break;
			}
			case 1:
			{
				$this->display_form2( $title, $action2, $action2_text );
				break;
			}
		}
	}

	public function display_form1 ( $title, $action, $action_text )
	{
		global $s_enum_module_labels;
		$s_enum_module_labels = module_list::get_enum_label_string();
		
		$type = get_db_classtype_instance( $this->db_object->get_classtype(), $this->db_object->get_classtype_id() );

		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );
		html_table_header( 25, 150, 625 );
	
		html_table_section( $title, design_ta_form_menu( $this->db_object->get_classtype(), $type->get_id() ) );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='ta_id' value='%s'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='ta_classtype' value='%s'></td></tr>\n",
			$this->db_object->get_classtype() );
		printf( "<tr><td colspan='3'><input type='hidden' name='ta_classtype_id' value='%s'></td></tr>\n",
			$this->db_object->get_classtype_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='ta_form' value='1'></td></tr>\n" );
	
		html_oblig_row( '*', get_string('Title_ttl') );
		printf( "<td class='f_value'><input type='text' name='ta_title' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 200, html_entity_decode( $this->db_object->get_title(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '*', get_string('Visibility_ttl') );
		printf( "<td class='f_value'>" );
		$enum_attr_visible = get_enum_to_array("enum_attr_visible"); 
		cat_array( $enum_attr_visible, $this->db_object->get_visible(), -1, "ta_visible", "" );
		printf( "</td></tr>\n" );

		html_oblig_row( '*', get_string('Obligate_ttl') );
		print( "<td class='f_value'>" );
		html_form_checkbox( $this->db_object->get_oblig(), "ta_oblig" );
		print( "</td></tr>\n" );
	
		html_oblig_row( '*', get_string('Unique_ttl') );
		print( "<td class='f_value'>" );
		html_form_checkbox( $this->db_object->get_unique(), "ta_unique" );
		print( "</td></tr>\n" );
	
		html_oblig_row( '*', get_string('Tag_ttl') );
		print( "<td class='f_value'>" );
		html_form_checkbox( $this->db_object->get_tag(), "ta_tag" );
		print( "</td></tr>\n" );
	
		html_oblig_row( '', get_string('Description_ttl') );
		printf( "<td class='f_value'><textarea name='ta_desc' cols='%d' rows='%d'>%s</textarea></td></tr>\n",
			50, 5, html_entity_decode( $this->db_object->get_desc(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '*', get_string('Datatype_ttl') );
		printf( "<td class='f_value'>" );
		$enum_datatypes = get_enum_to_array("enum_datatypes"); 
		cat_array( $enum_datatypes, $this->db_object->get_datatype(), -1, "ta_datatype", "" );
		printf( "</td></tr>\n" );

		html_oblig_row( '', get_string('Format_ttl') );
		printf( "<td class='f_value'><input type='text' name='ta_format' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 100, html_entity_decode( $this->db_object->get_format(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		if (module_list::count()!=0)
		{
			html_oblig_row( '', get_string('Label_ttl') );
			printf( "<td class='f_value'>" );
			$enum_module_labels = get_enum_to_array("enum_module_labels"); 
			cat_array( $enum_module_labels, $this->db_object->get_label(), -1, "ta_label" );
			printf( "</td></tr>\n" );
		}

		html_oblig_row( '*', get_string('InputValue_ttl') );
		printf( "<td class='f_value'>" );
		$enum_input = get_enum_to_array("enum_input");
		cat_array( $enum_input, $this->db_object->get_input(), -1, "ta_input", "" );
		printf( "</td></tr>\n" );

		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );

		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );
	
	}

	public function display_form2 ( $title, $action, $action_text )
	{
		global $s_enum_module_labels;
		$s_enum_module_labels = module_list::get_enum_label_string();
		
		$type = get_db_classtype_instance( $this->db_object->get_classtype(), $this->db_object->get_classtype_id() );
		$type_name = get_enum_to_string( "enum_classtypes", $this->db_object->get_classtype() );

		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );
		printf( "<input type='hidden' name='ta_id' value='%s'>\n",
			$this->db_object->get_id() );
		printf( "<input type='hidden' name='ta_classtype' value='%s'>\n",
			$this->db_object->get_classtype() );
		printf( "<input type='hidden' name='ta_classtype_id' value='%s'>\n",
			$this->db_object->get_classtype_id() );
		printf( "<input type='hidden' name='ta_title' value='%s'>\n",
			$this->db_object->get_title() );
		printf( "<input type='hidden' name='ta_visible' value='%s'>\n",
			$this->db_object->get_visible() );
		printf( "<input type='hidden' name='ta_label' value='%s'>\n",
			$this->db_object->get_label() );
		printf( "<input type='hidden' name='ta_oblig' value='%d'>\n",
			$this->db_object->get_oblig() );
		printf( "<input type='hidden' name='ta_unique' value='%d'>\n",
			$this->db_object->get_unique() );
		printf( "<input type='hidden' name='ta_tag' value='%d'>\n",
			$this->db_object->get_tag() );
		printf( "<input type='hidden' name='ta_desc' value='%s'>\n",
			$this->db_object->get_desc() );
		printf( "<input type='hidden' name='ta_datatype' value='%s'>\n",
			$this->db_object->get_datatype() );
		printf( "<input type='hidden' name='ta_format' value='%s'>\n",
			$this->db_object->get_format() );
		printf( "<input type='hidden' name='ta_input' value='%d'>\n",
			$this->db_object->get_input() );
		printf( "<input type='hidden' name='ta_form' value='2'>\n" );

		html_table_header( 25, 150, 625 );
	
		html_table_section( $title, design_ta_form_menu( $this->db_object->get_classtype(), $type->get_id() ) );
	
	
		html_oblig_row( '*', get_string('Title_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			$this->db_object->get_title() );
	
		html_oblig_row( '*', get_string('Visibility_ttl') );
		printf( "<td class='f_value'>%d</td></tr>\n",
			$this->db_object->get_visible() );

		html_oblig_row( '*', get_string('Obligate_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			html_status_checkbox( $this->db_object->get_oblig() ) );

		html_oblig_row( '*', get_string('Unique_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			html_status_checkbox( $this->db_object->get_unique() ) );
	
		html_oblig_row( '*', get_string('Tag_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			html_status_checkbox( $this->db_object->get_tag() ) );
	
		html_oblig_row( '', get_string('Description_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			$this->db_object->get_desc() );
	
		html_oblig_row( '*', get_string('Datatype_ttl') );
		printf( "<td class='f_value'>%s",
			get_enum_to_string( "enum_datatypes", $this->db_object->get_datatype() ) );
//		if ($this->db_object->get_datatype() == DT_FLOAT)
//		{
//			$decimal = array( 0 => "arbitrary", 1 => "1", 2 => "2", 3 => "3", 4 => "4", 5 => "5", 6 => "6", 7 => "7", 8 => "8" );
//			printf( " &gt;&gt; decimal precision " );
//			cat_array( $decimal, $this->db_object->get_datatype_sub(), -1, "ta_datatype_sub" );
//		}
		if ($this->db_object->get_datatype() == DT_CATALOG)
		{
			printf( " &gt;&gt; " );
			cat_list::getInstance()->html_options( array( $this->db_object->get_datatype_sub() ), array(), "ta_datatype_sub", array() );
		}
		if ($this->db_object->get_datatype() == DT_CI_REF)
		{
			printf( " &gt;&gt; " );
			ci_type_list::getInstance()->html_options( array( $this->db_object->get_datatype_sub() ), array(), "ta_datatype_sub", array() );
		}
		printf( "</td></tr>\n" );

		html_oblig_row( '', get_string('Format_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			$this->db_object->get_format() );

		if (module_list::count()!=0)
		{
			html_oblig_row( '', get_string('Label_ttl') );
			printf( "<td class='f_value'>%s",
				get_enum_to_string( "enum_module_labels", $this->db_object->get_label() ) );
		}

		html_oblig_row( '*', get_string('InputValue_ttl') );
		printf( "<td class='f_value'>%s",
			get_enum_to_string( "enum_input", $this->db_object->get_input() ) );
		printf( "</td></tr>\n" );

		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );

		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );
	
	}

	public function display_question ( $action )
	{
		global $referer_script_name;

		$yes_args = new html_form_args();
		$yes_args->add( "ta_id", $this->db_object->get_id() );
	
		$no_args = new html_form_args();
		
		$type = get_db_classtype_instance( $this->db_object->get_classtype(), $this->db_object->get_classtype_id() );

		printf( "<div id='question'>\n" );
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('DeleteAttribute_ttl'), design_ta_form_menu( $this->db_object->get_classtype(), $type->get_id() ) );
		html_table_footer();

		html_form_question( 0, 0, 
			get_string('WantDeleteAttribute_dsc'),
			get_string('YES_btn'), $action, $yes_args,
			get_string('NO_btn'), $referer_script_name, $no_args );
		printf( "</div>\n" );

		return;
	}

	protected function & get_prev_attribute ()
	{
		$attr   = null;
		$db     = core_db::getInstance();
		$query  = "SELECT * FROM type_attr " .
			"WHERE ta_classtype='" . $this->db_object->get_classtype() . "' " .
			"AND ta_classtype_id='" . $this->db_object->get_classtype_id() . "' " .
			"AND ta_order<'" . $this->db_object->get_order() . "' " .
			"ORDER BY ta_order DESC " .
			"LIMIT 1";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1) return $attr;
		$data   = $db->fetch( $result );
		$attr   = new db_type_attr( $data['ta_id'] );
		$attr->load_from_array( $data );
		return $attr;
	}

	protected function & get_next_attribute ()
	{
		$attr   = null;
		$db     = core_db::getInstance();
		$query  = "SELECT * FROM type_attr " .
			"WHERE ta_classtype='" . $this->db_object->get_classtype() . "' " .
			"AND ta_classtype_id='" . $this->db_object->get_classtype_id() . "' " .
			"AND ta_order>'" . $this->db_object->get_order() . "' " .
			"ORDER BY ta_order ASC " .
			"LIMIT 1";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1) return $attr;
		$data   = $db->fetch( $result );
		$attr   = new db_type_attr( $data['ta_id'] );
		$attr->load_from_array( $data );
		return $attr;
	}

	// action support

	/**
	 * necessary actions before deleting the type attribute
	 * 
	 * @return	none
	 */
	public function delete_pre ()
	{
		$db = core_db::getInstance();

		// cleanup CSV-elements for this type attribute
		$query = "UPDATE csv_element SET ce_ta_id=0 " .
			"WHERE ce_ta_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query );
		 
		// delete attributes refering this type attribute
		$query = "DELETE FROM attr " .
			"WHERE a_ta_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query );
	}

	/**
	 * check, that all required conditions for deleting
	 * are fullfilled
	 * 
	 * method reply TRUE, if type attribute can be deleted without problems,
	 * otherwise FALSE 
	 * 
	 * @return	boolean
	 */
	public function valid_for_delete ()
	{
		$db = core_db::getInstance();
		$query = "SELECT COUNT(*) FROM attr " .
			"WHERE a_ta_id='" . $this->db_object->get_id() . "'";
		$result = $db->query( $query ); 
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if ( get_property("protect_used_type_attr") && (($count = $db->result( $result, 0 )) != 0) )
		{
			throw new core_data_exception( "Type Attribute used by $count Item(s)");
		}
		
		return TRUE;
	}

	protected function create_db_object ( $_id )
	{
		$this->db_object = new db_type_attr( $_id );
		return;
	}

}
