<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'core/core_api.inc.php';
require_once 'core/core_html_form.inc.php';
require_once 'tables/db_user.inc.php';
require_once 'includes/entity_processor.inc.php';
require_once 'includes/maintain.inc.php';
require_once 'lang/catalog.inc.php';

class user extends entity_processor
{
	/**
	 * attribute des users aus argumenten laden
	 * 
	 * ueberlaedt die felder des DB-Objektes mit den
	 * werten aus den argumenten, dazu muss allerdings
	 * das argument "u_form" gesetzt sein 
	 * 
	 * @return	none
	 */	
	public function load_from_args ()
	{
		$_form = http_get_int("u_form", 0);

		switch ($_form)
		{
			case 1:
			{
				// diese werte duerfen nur aus form 1 (admin) gesetzt werden
				$_role_id = http_get_int("u_role_id", $this->db_object->get_role_id() );
				$this->db_object->set_role_id( $_role_id );

				$_auth_backend = http_get_int("u_auth_backend", $this->db_object->get_auth_backend() );
				$this->db_object->set_auth_backend( $_auth_backend );

				$_active = http_get_int("u_active", 0 );
				$_active = ($_active == 1);
				$this->db_object->set_active( $_active );

			}
			case 2:
			{
				// diese werte duerfen aus form 1 (admin) und form 2 (user) gesetzt werden
				$_login = http_get_string("u_login", $this->db_object->get_login() );
				$this->db_object->set_login( $_login );

				$_last_name = http_get_string("u_last_name", $this->db_object->get_last_name() );
				$this->db_object->set_last_name( $_last_name );

				$_first_name = http_get_string("u_first_name", $this->db_object->get_first_name() );
				$this->db_object->set_first_name( $_first_name );

				$_email = http_get_string("u_email", $this->db_object->get_email() );
				$this->db_object->set_email( $_email );

				$_notify = http_get_int("u_notify", 0 );
				$_notify = ($_notify == 1);
				$this->db_object->set_notify( $_notify );

				break;
			}
			case 3:
			{
				// das passwort darf aus form 3 gesetzt werden, ohne current-check
				$_pw1 = http_get_passwd("u_pw1", "" );
				$_pw2 = http_get_passwd("u_pw2", "" );

				if (strcmp($_pw1, $_pw2)!=0)
				{
					throw new core_data_exception( "new password and repetition not equal" );
				}

				$this->db_object->set_passwd( $_pw1 );

				break;
			}
			case 4:
			{
				// das passwort darf aus form 4 gesetzt werden, mit current-check
				$pw_current = http_get_passwd("u_pw_current", "" );

				$_pw1 = http_get_passwd("u_pw1", "" );
				$_pw2 = http_get_passwd("u_pw2", "" );

				if (
					(strlen($pw_current)!=0)
					&&
					(strlen($_pw1)!=0)
					&&
					(strlen($_pw2)!=0)
					)
				{
					if (! $this->db_object->password_match($pw_current))
					{
						throw new core_data_exception( "current password does not match" );
					}
	
					if (strcmp($_pw1, $_pw2)!=0)
					{
						throw new core_data_exception( "new password and repetition not equal" );
					}
	
					$this->db_object->set_passwd( $_pw1 );
				}

				// e-mail adresse anpassen
				$_email = http_get_string("u_email", $this->db_object->get_email() );
				$this->db_object->set_email( $_email );

				// e-mail notify anpassen
				$_notify = http_get_int("u_notify", 0 );
				$_notify = ($_notify == 1);
				$this->db_object->set_notify( $_notify );

				break;
			}
			case 5:
			{
				// get account settings
				
				$_language = http_get_string("u_language", null );
				if ($_language != null) set_user_property("default_language", $_language, 0, 0 );

				$_page_rows = http_get_string("u_page_rows", NULL );
				if ($_page_rows != null) set_user_property("page_rows", (int)$_page_rows, 0, 0 );
				
				$_redirect_single_result = http_get_string("u_redirect_single_result", 0 );
				set_user_property("redirect_single_result", (int)$_redirect_single_result, 0, 0 );

				$_last_change_items = http_get_string("u_last_change_items", NULL );
				if ($_last_change_items != null) set_user_property("last_change_items", (int)$_last_change_items, 0, 0 );

				$_last_change_comments = http_get_string("u_last_change_comments", NULL );
				if ($_last_change_comments != null) set_user_property("last_change_comments", (int)$_last_change_comments, 0, 0 );
				
				$_page_auto_refresh = http_get_string("u_page_auto_refresh", NULL );
				if ($_page_auto_refresh != null) set_user_property("page_auto_refresh", (int)$_page_auto_refresh, 0, 0 );
				
				$this->add_message( get_string('UpdateSuccessful_dsc'), STATUS_OK );
				
				break;
			}
		}

		return;
	}

	public function display_form ( $title, $action, $action_text )
	{

		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );

		html_table_header( 25, 150, 625 );
		html_table_section( $title, get_empty_menu() );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='u_id' value='%s'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='u_form' value='1'></td></tr>\n" );
	
		html_oblig_row( '*', get_string('Login_ttl') );
		printf( "<td class='f_value'><input type='text' name='u_login' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 100, html_entity_decode( $this->db_object->get_login(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '*', get_string('Active_ttl') );
		print( "<td class='f_value'>" );
		html_form_checkbox( $this->db_object->get_active(), "u_active" );
		print( "</td></tr>\n" );

		html_oblig_row( '', get_string('LastName_ttl') );
		printf( "<td class='f_value'><input type='text' name='u_last_name' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 100, html_entity_decode( $this->db_object->get_last_name(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '', get_string('FirstName_ttl') );
		printf( "<td class='f_value'><input type='text' name='u_first_name' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 100, html_entity_decode( $this->db_object->get_first_name(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '*', get_string('Role_ttl') );
		printf( "<td class='f_value'>" );
		$enum_roles = get_enum_to_array("enum_roles");
		cat_array( $enum_roles, $this->db_object->get_role_id(), -1, "u_role_id", "" );
		printf( "</td></tr>\n" );
	
		html_oblig_row( '', get_string('Notify_ttl') );
		print( "<td class='f_value'>" );
		html_form_checkbox( $this->db_object->get_notify(), "u_notify" );
		print( "</td></tr>\n" );

		html_oblig_row( '', get_string('EMail_ttl') );
		printf( "<td class='f_value'><input type='text' name='u_email' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 100, html_entity_decode( $this->db_object->get_email(), ENT_QUOTES, 'ISO-8859-1' ) );
	
		html_oblig_row( '*', get_string('AuthBackend_ttl') );
		printf( "<td class='f_value'>" );
		$enum_auth_backend = get_enum_to_array("enum_auth_backend");
		cat_array( $enum_auth_backend, $this->db_object->get_auth_backend(), -1, "u_auth_backend", "" );
		printf( "</td></tr>\n" );
	
		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );
	
	}

	public function display_self_form ( $title, $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );
	
		html_table_header( 25, 200, 575 );
	
		html_table_section( $title, get_empty_menu() );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='u_id' value='%s'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='u_form' value='4'></td></tr>\n" );
	
		html_oblig_row( '*', get_string('Login_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			$this->db_object->get_login() );
	
		html_oblig_row( '*', get_string('Active_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			html_status_checkbox( $this->db_object->get_active() ) );

		html_oblig_row( '', get_string('LastName_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			$this->db_object->get_last_name() );
	
		html_oblig_row( '', get_string('FirstName_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			$this->db_object->get_first_name() );
	
		html_oblig_row( '*', get_string('Role_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n", 
			get_enum_to_string( "enum_roles", $this->db_object->get_role_id() ) );
	
		html_oblig_row( '', get_string('Notify_ttl') );
		print( "<td class='f_value'>" );
		html_form_checkbox( $this->db_object->get_notify(), "u_notify" );
		print( "</td></tr>\n" );

		html_oblig_row( '', get_string('EMail_ttl') );
		printf( "<td class='f_value'><input type='text' name='u_email' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 100, $this->db_object->get_email() );
	
		html_oblig_row( '*', get_string('AuthBackend_ttl') );
		printf( "<td class='f_value'>%s</td></tr>\n",
			get_enum_to_string( "enum_auth_backend", $this->db_object->get_auth_backend() ) );
	
		if ($this->db_object->get_auth_backend() == AUTH_BACKEND_LOCAL)
		{
			html_oblig_row( '*', get_string('CurrentPassword_ttl') );
			printf( "<td class='f_value'><input type='password' name='u_pw_current' size='%d' maxlength='%d' value='%s'></td></tr>\n",
				50, 100, "" );
		
			html_oblig_row( '*', get_string('NewPassword_ttl') );
			printf( "<td class='f_value'><input type='password' name='u_pw1' size='%d' maxlength='%d' value='%s'></td></tr>\n",
				50, 100, "" );
		
			html_oblig_row( '*', get_string('RepeatPassword_ttl') );
			printf( "<td class='f_value'><input type='password' name='u_pw2' size='%d' maxlength='%d' value='%s'></td></tr>\n",
				50, 100, "" );
		}

		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );	
	}

	public function display_settings_form ( $title, $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );
	
		html_table_header( 25, 300, 475 );
	
		html_table_section( $title, get_empty_menu() );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='u_id' value='%s'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='u_form' value='5'></td></tr>\n" );
	
		html_oblig_row( '', get_string('Language_ttl') );
		printf( "<td class='f_value'>" );
		$enum_language = lang_catalog::listAvailable(); 
		cat_array( $enum_language, get_user_property( 'default_language', 0, 0, get_property('default_language' )), -1, 'u_language' );
		printf( "</td></tr>\n" );

		html_oblig_row( '', get_string('RowsPerPage_ttl') );
		printf( "<td class='f_value'><input type='text' name='u_page_rows' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			5, 5, get_user_property( 'page_rows', 0, 0, get_property('page_rows' )) ); 
	
		html_oblig_row( '', get_string('FollowSingleResults_ttl') );
		print( "<td class='f_value'>" );
		html_form_checkbox( get_user_property( "redirect_single_result", 0, 0, get_property("redirect_single_result" )), "u_redirect_single_result" );
		print( "</td></tr>\n" );

		html_oblig_row( '', get_string('LastChangedCIs_ttl') );
		printf( "<td class='f_value'><input type='text' name='u_last_change_items' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			5, 5, get_user_property( 'last_change_items', 0, 0, get_property('last_change_items' )) ); 
	
		html_oblig_row( '', get_string('LastChangedComments_ttl') );
		printf( "<td class='f_value'><input type='text' name='u_last_change_comments' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			5, 5, get_user_property( 'last_change_comments', 0, 0, get_property('last_change_comments' )) ); 
	
		html_oblig_row( '', get_string('AutoRefreshAfterSuccess_ttl') );
		printf( "<td class='f_value'><input type='text' name='u_page_auto_refresh' size='%d' maxlength='%d' value='%s'> ".get_string('Seconds_ttl')."</td></tr>\n",
			2, 2, get_user_property( 'page_auto_refresh', 0, 0, get_property('page_auto_refresh' )) ); 

		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );	
	}

	public function display_pw_form ( $title, $action, $action_text )
	{
		printf( "<div id='form'>\n" );
		printf( "<form action='%s' method='post'>\n",
			$action );
	
		html_table_header( 25, 200, 575 );
	
		html_table_section( $title, get_empty_menu() );
	
		printf( "<tr><td colspan='3'><input type='hidden' name='u_id' value='%s'></td></tr>\n",
			$this->db_object->get_id() );
		printf( "<tr><td colspan='3'><input type='hidden' name='u_form' value='3'></td></tr>\n" );
	
		if ($this->db_object->get_auth_backend() != AUTH_BACKEND_LOCAL)
		{
			html_table_info( 0, 0, "This account is under control of another authentication backend!" );
		}
			
		html_oblig_row( '*', get_string('NewPassword_ttl') );
		printf( "<td class='f_value'><input type='password' name='u_pw1' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 100, "" );
	
		html_oblig_row( '*', get_string('RepeatPassword_ttl') );
		printf( "<td class='f_value'><input type='password' name='u_pw2' size='%d' maxlength='%d' value='%s'></td></tr>\n",
			50, 100, "" );
		
		html_table_ruler();
	
		printf( "<tr><td colspan='3'><input type='submit' class='b_action' value='%s'></td></tr>\n",
			$action_text );
	
		html_table_footer();
		printf( "</form>\n" );
		printf( "</div>\n" );
	}

	public function display_question ( $action )
	{
		global $referer_script_name;

		$yes_args = new html_form_args();
		$yes_args->add( "u_id", $this->db_object->get_id() );
	
		$no_args = new html_form_args();
		
		printf( "<div id='question'>\n" );
		html_table_header( 25, 150, 625 );
		html_table_section( get_string('DeleteUser_ttl'), get_empty_menu() );
		html_table_footer();

		html_form_question( 0, 0, 
			get_string('WantDeleteUser_dsc'),
			get_string('YES_btn'), $action, $yes_args,
			get_string('NO_btn'), $referer_script_name, $no_args );
		printf( "</div>\n" );
			
		return;
	}

	// action support

	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$_login = "";
		$_role = "";
		if ($this->db_object->get_id() != 0)
		{
			$_login = $this->db_object->get_login();
			$_role = get_enum_to_string( 'enum_roles', $this->db_object->get_role_id() );
		}  
			
		$title = sprintf( get_string('User_ttl').": $_login / ".get_string('Role_ttl').": $_role" );
		return $title;
	}

	protected function delete_pre ()
	{
		delete_all_property_in_db( $this->db_object->get_id() );
		return;
	}

	protected function create_db_object ( $_id )
	{
		$this->db_object = new db_user( $_id );
		return;
	}
}
