<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'tables/db_typed_ci.inc.php';
require_once 'includes/navi_list.inc.php';
require_once 'includes/icon_list.inc.php';

class user_navi_list extends navi_list
{
	static private $instance	= false;
	protected $search_text		= "";
	
	static public function getInstance()
	{
		if (self::$instance === false) self::$instance = new user_navi_list();
		return self::$instance;
	}

	protected function __construct ()
	{
		$this->page_size = get_property("page_rows");
		parent::__construct();
		return;
	}

	/**
	 * load search args
	 * 
	 */
	protected function post_load ()
	{
		$_form = http_get_int("u_form", 0);
		$_text = http_get_string("s_text", "" );
		if (($_form == 2) && (strlen($_text)!=0)) $this->search_text = $_text;
		return;
	}

	/**
	 * preserve search_text from store in session context
	 */
	protected function get_dont_load_keys ()
	{
		$key_array = parent::get_dont_load_keys();
		$key_array[] = "search_text";
		return $key_array;
	}

	protected function get_session_store_name ()
	{
		return "user";
	} 

	protected function get_element_name ()
	{
		return get_string('User_ttl');
	} 

	protected function get_id_name ()
	{
		return "u_id";
	}

	// show

	public function show_page ()
	{
		html_table_header( 25, 100, 200, 150, 25, 230, 70 );
?>
<tr class='r_head'>
<?php
printf( "	<td class='c_action r_head'>A</td>\n");
printf( "	<td class='c_text r_head'>".get_string('Login_ttl')."</td>\n");
printf( "	<td class='c_text r_head'>".get_string('Name_ttl')."</td>\n");
printf( "	<td class='c_text r_head'>".get_string('Role_ttl')."</td>\n");
printf( "	<td class='c_action r_head'>N</td>\n");
printf( "	<td class='c_text r_head'>".get_string('EMail_ttl')."</td>\n");
printf( "	<td class='c_action r_head'>".get_string('Action_ttl')."</td>\n");
?>
</tr>
<?php
		parent::show_page();
	
		html_table_footer();

		return;
	}
	
	protected function show_item ( $i )
	{
		$user = $this->at( $i );
		html_table_row();
		printf( "<td class='c_action'>%s</td>" .
			"<td class='c_text'>%s</td>" .
			"<td class='c_text'>%s, %s</td>" .
			"<td class='c_text'>%s</td>" .
			"<td class='c_action'>%s</td>" .
			"<td class='c_text'>%s</td>" .
			"<td class='c_action'>%s %s %s</td>" .
			"</tr>\n",
			html_status_checkbox( $user->get_active() ),
			$user->get_login(),
			$user->get_last_name(), $user->get_first_name(),
			get_enum_to_string( "enum_roles", $user->get_role_id() ),
			html_status_checkbox( $user->get_notify() ),
			$user->get_email(), 
			icon_display_url(U_PW_EDIT_PHP . "?u_id=" . $user->get_id(), ICON_KEY, get_string('ChangePassword_dsc') ),
			icon_display_url(U_EDIT_PHP . "?u_id=" . $user->get_id(), ICON_EDIT, get_string('EditUser_dsc') ),
			icon_display_url(U_DELETE_PHP . "?u_id=" . $user->get_id(), ICON_DELETE, get_string('DeleteUser_dsc') )
			);

	}

	protected function get_query ()
	{
		$wc = "";
		if (strlen($this->search_text)!=0)
		{
			$wc = "WHERE u_last_name like '" . $this->search_text . "%'";
		}

		$query = array();
		$query[] = "SELECT * FROM user $wc ORDER BY u_login";
		return $query;
	}

	// helper

	/**
	 * get a string with the title for the sysnopse section
	 * 
	 * @return		string		title for synopse
	 */
	protected function get_synopse_title ()
	{
		$title = get_string('AllUsers_ttl');
		return $title;
	}

	/**
	 * get a string with the information for the upper right
	 * corner in the synopse section
	 * 
	 * create a search form to filter the user_last_name
	 * 
	 * @return	string	information
	 */
	protected function get_synopse_info ()
	{
		$info = sprintf("<form action='%s' method='post' name='s_form'>" .
			"<input type='hidden' name='u_form' value='2'>" .
			"<input type='text' class='c_search' name='s_text' size='10' maxlength='200' value='%s' onchange='s_form.submit()'>" .
			"<input type='submit' class='b_search' value=".get_string('Search_btn')."></form>",
			U_INDEX_PHP,
			$this->search_text
			);
		return $info;
	}

	protected function create_object ( $_id )
	{
		$db_object = new db_user( $_id ); 
		return $db_object;
	}

	// static helpers
	
}
