<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
 * modul language - language catalog
 * 
 * @category	php-cmdb
 * @package		lang
 * @subpackage	base
 * @author		Dr. Michael Scheibe
 */

/**
 * multi language support
 *    
 * @category	php-cmdb
 * @package		lang
 * @subpackage	base
 * @author		Dr. Michael Scheibe
 */
class lang_catalog
{
	/**
	 * return an assoc. array with available languages
	 * as key-value-pairs, where key is the part of the
	 * filename in the lang directory and value is presented
	 * in the drop-down-list in user settings
	 * 
	 * @return	array 
	 */
	static public function &listAvailable()
	{
		$lang_array = array
		(
//			'aaa' 	=> 'AAA-Check',		// FOR TESTS ONLY !!!
			'english' 	=> 'English',
//			'french' 	=> 'Francais',
//			'russian' 	=> 'Russian',
//			'chinese' 	=> 'Chinese',
			'german' 	=> 'Deutsch',
//			'spanish' 	=> 'Spanish',
		);
		asort( $lang_array );
		return $lang_array;
	}

	/**
	 * return the relative filename under lang directory for 
	 * the given language key
	 * 
	 * if the file is not available, the fallback is returned
	 * 
	 * @param	string	$lang_key			language key
	 * @return	string  
	 */
	static public function getFilename( $lang_key )
	{
		$lang_file_name = "lang/strings_$lang_key.inc.php";
		if (! file_exists($lang_file_name))
		{
			$lang_file_name = 'lang/strings_english.inc.php';
		}
		return $lang_file_name;		
	}

}