<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'includes/page_init.inc.php';
require_once 'includes/maintain.inc.php';

// 
$db = core_db::getInstance();
$auto_correct = http_get_int( 'm_db_check_auto', 0 );
$rc_global = STATUS_OK;
$rc_counts = array( STATUS_OK => 0, STATUS_WARNING => 0, STATUS_ERROR => 0 );

function add_rc( $rc )
{
	global $rc_global;
	global $rc_counts;
	
	if ($rc > $rc_global) $rc_global = $rc;
	$rc_counts[ $rc ]++;
}


// page
html_page_header_maintain();

printf( "<div id='content'>\n" );
html_table_header( 100, 700 ); 

// table: cat_entry
html_table_section( get_string('CheckTable_ttl') . " 'cat_entry'", get_empty_menu() );
$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM cat_entry " .
			"LEFT JOIN cat ON cate_cat_id=cat_id " .
			"WHERE cat_id is NULL";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutCatalog_dsc') . ": $num" );
add_rc( $rc );

// table: type_attr
html_table_section( get_string('CheckTable_ttl') . " 'type_attr'", get_empty_menu() );
$rc		= STATUS_OK;
$query  = "SHOW TABLE STATUS LIKE 'type_attr'";
$result = $db->query( $query );
if (($num	= $db->num_rows( $result )) == 0) $rc = STATUS_ERROR;
html_status( $rc, get_string('TableExists_dsc') );
add_rc( $rc );

$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM type_attr " .
			"LEFT JOIN cat ON ta_datatype_sub=cat_id " .
			"WHERE ta_datatype='" . DT_CATALOG . "' " .
			"AND cat_id is NULL";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesOfCatalogTypeWithoutCatalog_dsc') . ": $num" );
add_rc( $rc );

$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM type_attr " .
			"LEFT JOIN ci_type ON ta_classtype_id=cit_id ".
			"WHERE cit_id is NULL " .
			"AND ta_classtype='" . CLASS_CI . "' "; 
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutCIType_dsc') . ": $num" );
add_rc( $rc );

$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM type_attr " .
			"LEFT JOIN relation_type ON ta_classtype_id=rt_id ".
			"WHERE rt_id is NULL " .
			"AND ta_classtype='" . CLASS_REL . "' "; 
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutRelationType_dsc') . ": $num" );
add_rc( $rc );

// table: ci
html_table_section( get_string('CheckTable_ttl') . " 'ci'", get_empty_menu() );
$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM ci " .
			"LEFT JOIN ci_type ON ci_cit_id=cit_id " .
			"WHERE cit_id is NULL ";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutCIType_dsc') . ": $num" );
add_rc( $rc );

// table: attr
html_table_section( get_string('CheckTable_ttl') . " 'attr'", get_empty_menu() );
$rc		= STATUS_OK;
$query  = "SHOW TABLE STATUS LIKE 'attr'";
$result = $db->query( $query );
if (($num	= $db->num_rows( $result )) == 0) $rc = STATUS_ERROR;
html_status( $rc, get_string('TableExists_dsc') );
add_rc( $rc );

$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM attr " .
			"LEFT JOIN ci ON a_classtype_id=ci_id " .
			"WHERE ci_id is NULL " .
			"AND a_classtype='" . CLASS_CI . "' ";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutCI_dsc') . ": $num" );
add_rc( $rc );

$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM attr " .
			"LEFT JOIN relation ON a_classtype_id=r_id " .
			"WHERE r_id is NULL " .
			"AND a_classtype='" . CLASS_REL . "' ";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutRelation_dsc') . ": $num" );
add_rc( $rc );

$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM attr " .
			"LEFT JOIN type_attr ON a_ta_id=ta_id " .
			"WHERE ta_id is NULL ";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutTypeAttribute_dsc') . ": $num" );
add_rc( $rc );


// table: relation_type
html_table_section( get_string('CheckTable_ttl') . " 'relation_type'", get_empty_menu() );
$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM relation_type " .
			"LEFT JOIN ci_type ON rt_cit1_id=cit_id " .
			"WHERE rt_cit1_id!=0 AND cit_id is NULL";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutCIType1ID_dsc') . ": $num" );
add_rc( $rc );

$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM relation_type " .
			"LEFT JOIN ci_type ON rt_cit2_id=cit_id " .
			"WHERE rt_cit2_id!=0 AND cit_id is NULL";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutCIType2ID_dsc') . ": $num" );
add_rc( $rc );

// table: relation
html_table_section( get_string('CheckTable_ttl') . " 'relation'", get_empty_menu() );
$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM relation " .
			"LEFT JOIN relation_type ON r_rt_id=rt_id " .
			"WHERE rt_id is NULL ";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutRelationType_dsc') . ": $num" );
add_rc( $rc );

$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM relation " .
			"LEFT JOIN ci ON r_ci1_id=ci_id " .
			"WHERE ci_id is NULL ";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutCI1ID_dsc') . ": $num" );
add_rc( $rc );

$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM relation " .
			"LEFT JOIN ci ON r_ci2_id=ci_id " .
			"WHERE ci_id is NULL ";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutCI2ID_dsc') . ": $num" );
add_rc( $rc );

// table: comment
html_table_section( get_string('CheckTable_ttl') . " 'comment'", get_empty_menu() );
$rc     = STATUS_OK;
$query  = "SELECT COUNT(*) FROM comment " .
			"LEFT JOIN ci ON c_ci_id=ci_id " .
			"WHERE ci_id is NULL ";
$result = $db->query( $query );
if (($num	= $db->result( $result, 0 ))!=0) $rc = STATUS_WARNING;
html_status( $rc, get_string('EntriesWithoutCI_dsc') . ": $num" );
add_rc( $rc );



// fertig
html_table_line();
html_status( $rc_global, sprintf( get_string('CheckCompleted_ttl')." (".get_string('Status_OK').": %d / ".get_string('Status_WARNING').": %d / ".get_string('Status_ERROR').": %s)", $rc_counts[STATUS_OK], $rc_counts[STATUS_WARNING], $rc_counts[STATUS_ERROR]) );

html_table_footer();
printf( "</div>\n" );

html_page_footer();

require_once 'includes/page_done.inc.php';
