<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'includes/page_init.inc.php';
require_once 'includes/maintain.inc.php';

// 
$db = core_db::getInstance();
$auto_correct = http_get_int( 'm_db_check_auto', 0 );
$rc_global = STATUS_OK;
$rc_counts = array( STATUS_OK => 0, STATUS_WARNING => 0, STATUS_ERROR => 0 );

function add_rc( $rc )
{
	global $rc_global;
	global $rc_counts;
	
	if ($rc > $rc_global) $rc_global = $rc;
	$rc_counts[ $rc ]++;
}

// page
html_page_header_maintain();

printf( "<div id='content'>\n" );
html_table_header( 100, 700 ); 

// step 1: create table type_attr
$query  = "SHOW TABLE STATUS LIKE 'type_attr'";
$result = $db->query( $query );
$num	= $db->num_rows( $result );
if ($num == 0)
{
	$query = "CREATE TABLE `type_attr` (" .
		"`ta_id` int(11) NOT NULL auto_increment," .
		"`ta_classtype` int(11) NOT NULL," .
		"`ta_classtype_id` int(11) NOT NULL," .
		"`ta_title` varchar(200) NOT NULL," .
		"`ta_desc` text NOT NULL," .
		"`ta_datatype` int(11) NOT NULL," .
		"`ta_datatype_sub` int(11) NOT NULL," .
		"`ta_order` int(11) NOT NULL," .
		"`ta_visible` int(11) NOT NULL DEFAULT '1'," .
		"`ta_label` int(11) NOT NULL," .
		"`ta_oblig` char(1) NOT NULL," .
		"`ta_unique` char(1) NOT NULL," .
		"`ta_tag` char(1) NOT NULL," .
		"`ta_format` varchar(100) NOT NULL," .
		"PRIMARY KEY  (`ta_id`)," .
		"KEY `ta_order` (`ta_order`)," .
		"KEY `ta_classtype` (`ta_classtype`)," .
		"KEY `ta_classtype_id` (`ta_classtype_id`)," .
		"FULLTEXT KEY `ta_title` (`ta_title`)," .
		"FULLTEXT KEY `ta_desc` (`ta_desc`)" .
		") DEFAULT CHARSET=utf8";

	$result = $db->query( $query );
	$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
	html_status( $rc, get_string('TableCreated_dsc') . ": 'type_attr' (MySQL: $db_rc('$db_msg'))" );
	add_rc( $rc );

	if ($rc == 0)
	{
		$query  = "SELECT * FROM ci_type_attr";
		$result = $db->query( $query );
		$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
		$num	= $db->num_rows( $result );
		for ($i=0; $i<$num; $i++)
		{	
			$data = $db->fetch_object( $result );
			$query_insert  = "INSERT INTO type_attr " .
				"VALUES (" .
				$data->cita_id . "," .
				CLASS_CI . "," .
				$data->cita_cit_id . ",'" .
				$db->escape_string($data->cita_title)."','".
				$db->escape_string($data->cita_desc)."',".
				$data->cita_type . "," .
				$data->cita_subtype . "," .
				$data->cita_order . "," .
				$data->cita_label . "," .
				$data->cita_oblig  . "," .
				$data->cita_unique . "," .
				$data->cita_tag . ")";
			$result_insert = $db->query( $query_insert );
			$rc = $db->errno_msg( $result, $db_rc, $db_msg );
			add_rc( $rc );
		}
		html_status( $rc, get_string('TableFilled_dsc') . ": 'type_attr' (MySQL: $db_rc('$db_msg'))" );
	}

	if ($rc == 0)
	{
		$query_drop  = "DROP TABLE `ci_type_attr`";
		$result_drop = $db->query( $query_drop );
		$rc = $db->errno_msg( $result_drop, $db_rc, $db_msg );
		add_rc( $rc );
		html_status( $rc, get_string('TableDropped_dsc') . ": 'ci_type_attr' (MySQL: $db_rc('$db_msg'))" );
	}
}
else
{
	html_status( 0, get_string('TableExists_dsc') . ": 'type_attr'" );
}

// step 2: update table type_attr
$query  = "SHOW COLUMNS FROM `type_attr` WHERE field='ta_visible'";
$result = $db->query( $query );
$num	= $db->num_rows( $result );
if ($num == 0)
{
	$query = "ALTER TABLE `type_attr` " .
		"ADD `ta_visible` INT NOT NULL DEFAULT '1'";
	$result = $db->query( $query );
	$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
	html_status( $rc, get_string('TableColumnUpdated_dsc') . ": 'type_attr'.'ta_visible' (MySQL: $db_rc('$db_msg'))" );
	add_rc( $rc );
}
else
{
	html_status( 0, get_string('TableColumnUpToDate_dsc') . ": 'type_attr'.'ta_visible'" );
}

$query  = "SHOW COLUMNS FROM `type_attr` WHERE field='ta_format'";
$result = $db->query( $query );
$num	= $db->num_rows( $result );
if ($num == 0)
{
	$query = "ALTER TABLE `type_attr` " .
		"ADD `ta_format` varchar(100) NOT NULL";
	$result = $db->query( $query );
	$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
	html_status( $rc, get_string('TableColumnUpdated_dsc') . "'type_attr'.'ta_format' (MySQL: $db_rc('$db_msg'))" );
	add_rc( $rc );
}
else
{
	html_status( 0, get_string('TableColumnUpToDate_dsc') . ": 'type_attr'.'ta_format'" );
}

$query  = "SHOW COLUMNS FROM `type_attr` WHERE field='ta_input'";
$result = $db->query( $query );
$num	= $db->num_rows( $result );
if ($num == 0)
{
	$query = "ALTER TABLE `type_attr` " .
		"ADD `ta_input` INT NOT NULL DEFAULT '0' AFTER `ta_tag`";
	$result = $db->query( $query );
	$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
	html_status( $rc, get_string('TableColumnUpdated_dsc') . ": 'type_attr'.'ta_input' (MySQL: $db_rc('$db_msg'))" );
	add_rc( $rc );
}
else
{
	html_status( 0, get_string('TableColumnUpToDate_dsc') . ": 'type_attr'.'ta_input'" );
}


// step 3: create table attr
$query  = "SHOW TABLE STATUS LIKE 'attr'";
$result = $db->query( $query );
$num	= $db->num_rows( $result );
if ($num == 0)
{
	$query = "CREATE TABLE `attr` (" .
		"`a_id` int(11) NOT NULL auto_increment," .
		"`a_classtype` int(11) NOT NULL," .
		"`a_classtype_id` int(11) NOT NULL," .
		"`a_ta_id` int(11) NOT NULL," .
		"`a_val_int` int(11) NOT NULL," .
		"`a_val_float` float NOT NULL," .
		"`a_val_date` date NOT NULL," .
		"`a_val_time` time NOT NULL," .
		"`a_val_text` text NOT NULL," .
		"PRIMARY KEY  (`a_id`)," .
		"KEY `a_ta_id` (`a_ta_id`)," .
		"KEY `a_classtype` (`a_classtype`)," .
		"KEY `a_classtype_id` (`a_classtype_id`)," .
		"FULLTEXT KEY `a_val_text` (`a_val_text`)" .
		") DEFAULT CHARSET=utf8";

	$result = $db->query( $query );
	$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
	html_status( $rc, get_string('TableCreated_dsc') . ": 'attr' (MySQL: $db_rc('$db_msg'))" );
	add_rc( $rc );

	if ($rc == 0)
	{
		$query  = "SELECT * FROM ci_attr";
		$result = $db->query( $query );
		$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
		$num	= $db->num_rows( $result );
		for ($i=0; $i<$num; $i++)
		{	
			$data = $db->fetch_object( $result );
			$query_insert  = "INSERT INTO attr " .
				"VALUES (".
				$data->cia_id . ",".
				CLASS_CI . ",".
				$data->cia_ci_id . ",".
				$data->cia_cita_id . ",".
				$data->cia_val_int . ",".
				$data->cia_val_float . ",'".
				$data->cia_val_date . "','".
				$data->cia_val_time . "','".
				$db->escape_string($data->cia_val_text)."')";
			$result_insert = $db->query( $query_insert );
			$rc = $db->errno_msg( $result, $db_rc, $db_msg );
			add_rc( $rc );
		}
		html_status( $rc, get_string('TableFilled_dsc') . ": 'attr' (MySQL: $db_rc('$db_msg'))" );
	}

	if ($rc == 0)
	{
		$query_drop  = "DROP TABLE `ci_attr`";
		$result_drop = $db->query( $query_drop );
		$rc = $db->errno_msg( $result_drop, $db_rc, $db_msg );
		add_rc( $rc );
		html_status( $rc, get_string('TableDropped_dsc') . ": 'ci_attr' (MySQL: $db_rc('$db_msg'))" );
	}
}
else
{
	html_status( 0, get_string('TableExists_dsc') . ": 'attr'" );
}

// step 4: create table property
$query  = "SHOW TABLE STATUS LIKE 'property'";
$result = $db->query( $query );
$num	= $db->num_rows( $result );
if ($num == 0)
{
	$query = "CREATE TABLE `property` (" .
		"`p_id` VARCHAR(64) NOT NULL ," .
		"`p_user_id` INT(11) NOT NULL," .
		"`p_type_id` INT(11) NOT NULL," .
		"`p_item_id` INT(11) NOT NULL," .
		"`p_value` VARCHAR(64) NOT NULL," .
		"PRIMARY KEY  (`p_id`, `p_user_id`, `p_type_id`, `p_item_id`) " .
		") DEFAULT CHARSET=utf8";

	$result = $db->query( $query );
	$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
	html_status( $rc, get_string('TableCreated_dsc') . ": 'property' (MySQL: $db_rc('$db_msg'))" );
	add_rc( $rc );

}
else
{
	html_status( 0, get_string('TableExists_dsc') . ": 'property'" );
}

// step 5: add fulltext to ci
$query  = "SHOW INDEX FROM ci WHERE key_name='ci_title'";
$result = $db->query( $query );
$num	= $db->num_rows( $result );
if ($num == 0)
{
	$query = "ALTER TABLE `ci` " .
		"ADD FULLTEXT (`ci_title`)"; 

	$result = $db->query( $query );
	$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
	html_status( $rc, get_string('TableUpdated_dsc') . ": 'ci' (MySQL: $db_rc('$db_msg'))" );
	add_rc( $rc );
}
else
{
	html_status( 0, get_string('TableUpToDate_dsc') . ": 'ci'" );
}

// step 6: update table cat_entry
$query  = "SHOW COLUMNS FROM `cat_entry` WHERE field='cate_icon'";
$result = $db->query( $query );
$num	= $db->num_rows( $result );
if ($num == 0)
{
	$query = "ALTER TABLE `cat_entry` " .
		"ADD `cate_icon` VARCHAR( 100 ) NOT NULL ," .
		"ADD `cate_fg_color` INT NOT NULL DEFAULT '0'," .
		"ADD `cate_bg_color` INT NOT NULL DEFAULT '0'";
	$result = $db->query( $query );
	$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
	html_status( $rc, get_string('TableUpdated_dsc') . ": 'cat_entry' (MySQL: $db_rc('$db_msg'))" );
	add_rc( $rc );
}
else
{
	html_status( 0, get_string('TableUpToDate_dsc') . ": 'cat_entry'" );
}

// step 7: create table csv_element
$query  = "SHOW TABLE STATUS LIKE 'csv_element'";
$result = $db->query( $query );
$num	= $db->num_rows( $result );
if ($num == 0)
{
	$query = "CREATE TABLE `csv_element` (".
		"`ce_id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,".
		"`ce_cp_id` INT NOT NULL ,".
		"`ce_ta_id` INT NOT NULL ,".
		"`ce_pos` INT NOT NULL".
		") DEFAULT CHARSET=utf8";
	$result = $db->query( $query );
	$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
	html_status( $rc, get_string('TableCreated_dsc') . ": 'csv_element' (MySQL: $db_rc('$db_msg'))" );
	add_rc( $rc );
}
else
{
	html_status( 0, get_string('TableExists_dsc') . ": 'csv_element'" );
}

// step 8: create table csv_profile
$query  = "SHOW TABLE STATUS LIKE 'csv_profile'";
$result = $db->query( $query );
$num	= $db->num_rows( $result );
if ($num == 0)
{
	$query = "CREATE TABLE `csv_profile` (".
		"`cp_id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,".
		"`cp_classtype` INT NOT NULL ,".
		"`cp_cit_id` INT NOT NULL ,".
		"`cp_title` VARCHAR( 200 ) NOT NULL ,".
		"`cp_desc` TEXT NOT NULL ,".
		"`cp_separator` VARCHAR( 1 ) NOT NULL ,".
		"`cp_delimiter` VARCHAR( 1 ) NOT NULL".
		") DEFAULT CHARSET=utf8";
	$result = $db->query( $query );
	$rc     = $db->errno_msg( $result, $db_rc, $db_msg );
	html_status( $rc, get_string('TableCreated_dsc') . ": 'csv_profile' (MySQL: $db_rc('$db_msg'))" );
	add_rc( $rc );
}
else
{
	html_status( 0, get_string('TableExists_dsc') . ": 'csv_profile'" );
}

// fertig
html_table_line();
html_status( $rc_global, sprintf( get_string('UpdateCompleted_dsc')." (".get_string('Status_OK').": %d / ".get_string('Status_WARNING').": %d / ".get_string('Status_ERROR').": %s)", $rc_counts[STATUS_OK], $rc_counts[STATUS_WARNING], $rc_counts[STATUS_ERROR]) );

html_table_footer();
printf( "</div>\n" );

html_page_footer();

require_once 'includes/page_done.inc.php';
