<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'includes/page_init.inc.php';
require_once 'includes/maintain.inc.php';
require_once 'includes/cmdb_setup.inc.php';
require_once 'includes/icon_list.inc.php';


/**
 * check php configuration 
 */
function setup_check_php ( $rc_global )
{
	html_table_section( "Check PHP configuration", get_empty_menu() );

	// web server software
	html_status( STATUS_OK, "Server Software is " . $_SERVER["SERVER_SOFTWARE"] );
	
	// memory
	html_status( STATUS_OK, "Memory limit is " . ini_get('memory_limit') );
	html_status( STATUS_OK, "Post max size is " . ini_get('post_max_size') );

	// exec time
	html_status( STATUS_OK, "Max. execution time is " . ini_get('max_execution_time') );
	
	// display errors
	if (! ini_get('display_errors'))
	{
		html_status( STATUS_OK, "display_errors is OFF" );
	}
	else
	{
		html_status( STATUS_WARNING, "display_errors is ON" );
		$rc_global++;
	}

	// display startup errors		
	if (! ini_get('display_startup_errors'))
	{
		html_status( STATUS_OK, "display_startup_errors is OFF" );
	}
	else
	{
		html_status( STATUS_WARNING, "display_startup_errors is ON" );
		$rc_global++;
	}

	// display startup errors		
	if (! ini_get('register_globals'))
	{
		html_status( STATUS_OK, "register_globals is OFF" );
	}
	else
	{
		html_status( STATUS_WARNING, "register_globals is ON" );
		$rc_global++;
	}


	return $rc_global;
}


/**
 * check CMDB configuration
 */
function setup_check_cmdb ( $rc_global )
{
	global $running_script_base;
	
	html_table_section( "Check CMDB configuration", get_empty_menu() );

	// check icon directory
	$icon_dir = icon_list::getInstance()->get_full_dir();
	$icon_count = icon_list::getInstance()->count();
	
	if (is_dir( $icon_dir))
	{
		html_status( STATUS_OK, "Icon directory '$icon_dir' exist, $icon_count icons found" );
	}
	else
	{
		html_status( STATUS_ERROR, "Icon directory '$icon_dir' does not exist!" );
		$rc_global++;
	}		

	// default icon
	$_default_icon_name = get_property( "default_icon_name" );
	$_default_icon_id	= icon_list::getInstance()->get_id( $_default_icon_name );
	
	if (strlen($_default_icon_name) != 0)
	{
		if ($_default_icon_id !== FALSE)
		{
			html_status( STATUS_OK, "Default icon '$_default_icon_name' exist" );
		}
		else
		{
			html_status( STATUS_ERROR, "Default icon '$_default_icon_name' does not exist!" );
			$rc_global++;
		}
	}
	else
	{
		html_status( STATUS_ERROR, "Default icon not defined!" );
		$rc_global++;
	}

	// icon size		
	$_default_icon_size = get_property( "default_icon_size" );
	if ($_default_icon_size > 0)
	{
		html_status( STATUS_OK, "Default icon size is $_default_icon_size" );
	}
	else
	{
		html_status( STATUS_ERROR, "Default icon size is $_default_icon_size" );
		$rc_global++;
	}
		
	$_map_icon_size = get_property( "map_icon_size" );
	if ($_map_icon_size > 0)
	{
		html_status( STATUS_OK, "Map icon size is $_map_icon_size" );
	}
	else
	{
		html_status( STATUS_ERROR, "Map icon size is $_map_icon_size" );
		$rc_global++;
	}

	$_ext_icon_size = get_property( "ext_icon_size" );
	if ($_ext_icon_size > 0)
	{
		html_status( STATUS_OK, "Ext icon size is $_ext_icon_size" );
	}
	else
	{
		html_status( STATUS_ERROR, "Ext icon size is $_ext_icon_size" );
		$rc_global++;
	}

	$_pdf_icon_size = get_property( "pdf_icon_size" );
	if ($_pdf_icon_size > 0)
	{
		html_status( STATUS_OK, "Pdf icon size is $_pdf_icon_size" );
	}
	else
	{
		html_status( STATUS_ERROR, "Pdf icon size is $_pdf_icon_size" );
		$rc_global++;
	}

	// check map font
	$_map_font = get_property( "map_font" );
	if ($_map_font[0]!='/') $_map_font = $running_script_base . "/" . $_map_font;
	if (is_file( $_map_font ))
	{
		html_status( STATUS_OK, "Map font '$_map_font' exist" );
	}
	else
	{
		html_status( STATUS_ERROR, "Map font '$_map_font' does not exist!" );
		$rc_global++;
	}
		
	// check ext font
	$_ext_font = get_property( "ext_font" );
	if ($_ext_font[0]!='/') $_ext_font = $running_script_base . "/" . $_ext_font;
	if (is_file( $_ext_font ))
	{
		html_status( STATUS_OK, "Ext font '$_ext_font' exist" );
	}
	else
	{
		html_status( STATUS_ERROR, "Ext font '$_ext_font' does not exist!" );
		$rc_global++;
	}
		
	// check pdf font
	$_pdf_font = get_property( "pdf_font" );
	if ($_pdf_font[0]!='/') $_pdf_font = $running_script_base . "/" . $_pdf_font;
	if (is_file( $_pdf_font ))
	{
		html_status( STATUS_OK, "Pdf font '$_pdf_font' exist" );
	}
	else
	{
		html_status( STATUS_ERROR, "Pdf font '$_pdf_font' does not exist!" );
		$rc_global++;
	}

	// check graphiv
	$_graphviz = get_property( "graphviz" );
	$_graphviz_neato = get_property( "graphviz_neato" );
	
	if ($_graphviz)
	{
		if (is_executable( $_graphviz_neato ))
		{
			html_status( STATUS_OK, "Graphviz enabled, executable '$_graphviz_neato' available" );
		}
		else
		{
			html_status( STATUS_ERROR, "Graphviz enabled, but executable '$_graphviz_neato' not available" );
			$rc_global++;
		}
	}
	else
	{
		html_status( STATUS_OK, "Graphviz not enabled" );
	}

	// check app_logofile
	$_app_logofile = get_property( "app_logofile" );
	if (strlen($_app_logofile)>0)
	{
		if (is_file( $_app_logofile ))
		{
			html_status( STATUS_OK, "Logofile configured and exist" );
		}
		else
		{
			html_status( STATUS_ERROR, "Logofile configured but does not exist!" );
			$rc_global++;
		}
		
	}
	else
	{
		html_status( STATUS_OK, "Logofile not configured" );		
	}
	
	// check customer_stylesheet
	$_customer_stylesheet = get_property( "customer_stylesheet" );
	if (strlen($_customer_stylesheet)>0)
	{
		if (is_file( $_customer_stylesheet ))
		{
			html_status( STATUS_OK, "customer stylesheet configured and exist" );
		}
		else
		{
			html_status( STATUS_ERROR, "customer stylesheet configured but does not exist!" );
			$rc_global++;
		}
		
	}
	else
	{
		html_status( STATUS_OK, "customer stylesheet not configured" );		
	}
	return $rc_global;
}

/**
 * check database configuration
 */
function setup_check_database ( $rc_global )
{
	global $running_script_base;
	
	html_table_section( "Check database configuration", get_empty_menu() );

	// connection
	$db = core_db::getInstance();
	$query  = "SHOW VARIABLES WHERE variable_name='version'";
	$result = $db->query( $query );
	$data = $db->fetch( $result );

	if ($data['Value']>="5.1.0")
	{
		html_status( STATUS_OK, "MySQL Version is " . $data['Value'] );		
	}
	else
	{
		html_status( STATUS_ERROR, "MySQL Version is " . $data['Value'] );		
		$rc_global++;
	}

	$query  = "SHOW VARIABLES WHERE variable_name='date_format'";
	$result = $db->query( $query );
	$num	= $db->num_rows( $result );
	$data = $db->fetch( $result );

	if ($data['Value']=='%Y-%m-%d')
	{
		html_status( STATUS_OK, "MySQL date format is " . $data['Value'] );		
	}
	else
	{
		html_status( STATUS_ERROR, "MySQL date format is " . $data['Value'] );		
		$rc_global++;
	}
	
	return $rc_global;
}

// 
$rc_global = STATUS_OK;


// page
html_page_header_maintain();

printf( "<div id='content'>\n" );

html_table_header( 100, 700 );

$rc_global = cmdb_setup_step1();
$rc_global = setup_check_php( $rc_global );
$rc_global = setup_check_cmdb( $rc_global );
$rc_global = setup_check_database( $rc_global );

// fertig
html_table_ruler();
if ($rc_global == 0)
{
	html_status( STATUS_OK, "Check completed!" );
}
else
{
	html_status( STATUS_WARNING, "Check completed with $rc_global warnings / errors!" );
}

html_table_footer();
printf( "</div>\n" );

html_page_footer();

require_once 'includes/page_done.inc.php';
