<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'includes/box.inc.php';
require_once 'includes/relation_peer_imx.inc.php';
require_once 'includes/module_label_list.inc.php';
require_once 'includes/ci.inc.php';

/**
 * class to draw DataCenter Rooms
 */
class dc_room_map extends relation_peer_imx
{
	protected function create ()
	{
		// setup values
		$font		= get_property("map_font");
		$font_size	= get_property("map_font_size");
		$padding	= get_property("map_padding");

		// get peer list
		$this->load_peer_list();
		$count = $this->peer_list->count();

		// get label data
		$dcr_label_list = new module_label_list( array(DC_ROOM_SIZE_X,DC_ROOM_SIZE_Y) );	 
		$this->ci->fill_label_list( $dcr_label_list );

		// create html-click-map file
		$mp = fopen( $this->map_name, "w" );
		fwrite( $mp, "<map name='map_locations'>\n" );

		// create image
		$sx = get_property("map_sx");
		$sy = get_property("map_sy");
		$im = imagecreatetruecolor( $sx, $sy );
		$color_back = ImageColorAllocate ($im, 255, 255, 255);
		$color_text = ImageColorAllocate ($im,   0,   0,   0);
		$color_red  = ImageColorAllocate ($im, 127,   0,   0);
		$color_grey = ImageColorAllocate ($im, 127, 127, 127);
		$color_grey_light = ImageColorAllocate ($im, 192, 192, 192);
		$color_blue = ImageColorAllocate ($im,   0,   0, 127);
		imagefill( $im, 0, 0, $color_back );

		// canvas
		$total_box = new box_group( $sx, $sy, FALSE );

		// ci 
		$icon_size = get_property("map_icon_size");
		$ci_box = new box_ci( $padding, $this->ci->get_icon(), $icon_size, $this->ci->get_title(), $font, $font_size, $color_text );
		$ci_box->set_click( "ci_view.php?ci_id=" . $this->ci->get_id() );
		$total_box->add_static( $ci_box, BOX_ALIGN_CENTER, BOX_ALIGN_TOP );

		// draw area
		$draw_box = new box_group( 0, 0, FALSE );
		$total_box->add_float( $draw_box, BOX_ALIGN_CENTER, BOX_ALIGN_BOTTOM );

		// get coordinates
		$ty = imagefontheight( $font );
		$rx_string = $dcr_label_list->get_label_value(DC_ROOM_SIZE_X);
		$ry_string = $dcr_label_list->get_label_value(DC_ROOM_SIZE_Y);
		
		$rx_elements = split( ",", $rx_string );
		$ry_elements = split( ",", $ry_string );
		
		$rx = count($rx_elements);
		$ry = count($ry_elements);

		if (($rx != 0) && ($ry != 0))
		{
			$r_sx_max = $draw_box->get_size_x() - $ty - $padding;
			$r_sy_max = $draw_box->get_size_y() - $ty - $padding;
	
			$fx = floor( $r_sx_max / $rx );
			$fy = floor( $r_sy_max / $ry );
			$f  = min( $fx, $fy );
			$r_sx = $f * $rx;
			$r_sy = $f * $ry;
	
			// frame box
			$f_sx = $r_sx + $ty + $padding;
			$f_sy = $r_sy + $ty + $padding;
	
			$frame_box = new box_group( $f_sx, $f_sy, FALSE );
			$draw_box->add_static( $frame_box, BOX_ALIGN_CENTER, BOX_ALIGN_CENTER);
	
			// raster box
			$r_box = new box_group( $r_sx, $r_sy, FALSE );
			$grid_box = new box_grid( $f, $rx, $f, $ry, $color_blue, $color_grey_light );
			$r_box->add_static( $grid_box, BOX_ALIGN_LEFT, BOX_ALIGN_TOP );
			$frame_box->add_static( $r_box, BOX_ALIGN_RIGHT, BOX_ALIGN_BOTTOM );
	
			// x-scale box
			$x_box = new box_group( $r_sx, $ty, FALSE );
			$frame_box->add_static( $x_box, BOX_ALIGN_RIGHT, BOX_ALIGN_TOP );
			for ($i=0; $i<$rx; $i++)
			{
				$txt_box = new box_ft_text( $rx_elements[$i], $font, $font_size, $color_text );
				$txt_box->set_align( BOX_ALIGN_CENTER, BOX_ALIGN_TOP );
				$txt_box->set_size( $f, $f );
				$txt_box->shift_pos( $i * $f, 0 );
				$x_box->add( $txt_box );
			}
	
			// y-scale box
			$y_box = new box_group( $ty, $r_sy, FALSE );
			$frame_box->add_static( $y_box, BOX_ALIGN_LEFT, BOX_ALIGN_BOTTOM );
			for ($i=0; $i<$ry; $i++)
			{
				$txt_box = new box_ft_text( $ry_elements[$i], $font, $font_size, $color_text );
				$txt_box->set_align( BOX_ALIGN_LEFT, BOX_ALIGN_CENTER );
				$txt_box->set_size( $f, $f );
				$txt_box->shift_pos( 0, $i * $f );
				$y_box->add( $txt_box );
			}
	
			// draw peers
			$rx_positions = array_flip( $rx_elements );
			$ry_positions = array_flip( $ry_elements );
			
			if ($count > 0)
			{
				for ($i=0; $i<$count; $i++ )
				{
					$relation_peer	= & $this->peer_list->at( $i );
					$peer_id		= $relation_peer->get_ci_id();
					$rll = new module_label_list( array(DC_RACK_POS_X,DC_RACK_POS_Y,DC_RACK_SIZE_X,DC_RACK_SIZE_Y) );
					
					$label_tracker	= new label_list_tracker( $peer_id, 5, $rll );
					$label_tracker->run();
					if ($rll->completed())
					{
						$title = $relation_peer->get_ci_title();
						$title_full	= $title . " / " . $relation_peer->get_cit_title();
	
						$rack_x_string = $rll->get_label_value( DC_RACK_POS_X );
						$rack_y_string = $rll->get_label_value( DC_RACK_POS_Y );
						
						$rack_x = 0;
						$rack_y = 0;
						if (array_key_exists( $rack_x_string, $rx_positions )) $rack_x = $rx_positions[ $rack_x_string ];
						if (array_key_exists( $rack_y_string, $ry_positions )) $rack_y = $ry_positions[ $rack_y_string ];
						
						$rack_sx = $rll->get_label_value( DC_RACK_SIZE_X );
						$rack_sy = $rll->get_label_value( DC_RACK_SIZE_Y );
	
						$rack_icon = $relation_peer->get_icon();
						$rack_box = new box_icon( $rack_icon, $icon_size, FALSE, $color_blue, $color_back );
						$rack_box->set_size( $rack_sx * $f, $rack_sy * $f );
						$rack_box->set_align( BOX_ALIGN_CENTER, BOX_ALIGN_CENTER );
						$rack_box->set_pos( $rack_x * $f, $rack_y * $f );
						$rack_box->set_click( "ci_view.php?ci_id=$peer_id", $title_full );
	
						$r_box->add( $rack_box );
					}
	
				}
			}
		}

		// output
		$total_box->draw( $im, $mp );

		// store
		ImagePng( $im, $this->imx_name );

		// close html-click-map file
		fwrite( $mp, "</map>\n" );
		fclose( $mp );

		return;
	}

	/**
	 * reply filename postfix, to describe the module
	 */
	protected function get_module_postfix ()
	{
		return "_dc_room";
	}

	
}

/**
 * class to draw DataCenter Racks
 */
class dc_rack_map extends relation_peer_imx
{
	protected function create ()
	{
		// setup values
		$font		= get_property("map_font");
		$font_size	= get_property("map_font_size");
		$padding	= get_property("map_padding");

		// get peer list
		$this->load_peer_list();
		$count = $this->peer_list->count();

		// get label data
		$dcr_label_list = new module_label_list( array(DC_RACK_SIZE_X,DC_RACK_SIZE_Y,DC_RACK_SIZE_Z,DC_RACK_BASE_Z) );
		$label_tracker	= new label_list_tracker( $this->ci->get_id(), 5, $dcr_label_list );
		$label_tracker->run();

		// create html-click-map file
		$mp = fopen( $this->map_name, "w" );
		fwrite( $mp, "<map name='map_locations'>\n" );

		// get coordinates
		$sx = get_property("map_sx");
		$sy = get_property("map_sy");	// will be adapted
		$icon_size = get_property("map_icon_size");
		$ty = imagefontheight( $font );

		// rack in standard-units
		$rack_x_qe = (int)$dcr_label_list->get_label_value(DC_RACK_SIZE_X);
		$rack_y_qe = (int)$dcr_label_list->get_label_value(DC_RACK_SIZE_Y);
		$rack_z_he = (int)$dcr_label_list->get_label_value(DC_RACK_SIZE_Z);
		if (($base_z_he = $dcr_label_list->get_label_value(DC_RACK_BASE_Z))==null) $base_z_he=0;
		$base_z_he = (int)$base_z_he;

		// rack in zoll
		$rack_x_zoll = $rack_x_qe * 19;
		$rack_y_zoll = $rack_y_qe * 19;
		$rack_z_zoll = $rack_z_he * 1.75;

		// positioning and scale
		$rx = 2.5 * $rack_x_zoll + $rack_y_zoll;
		$f  = floor( $sx / $rx );

		// rack in pixels
		$rack_x_pixel = floor( $rack_x_zoll * $f );
		$rack_y_pixel = floor( $rack_y_zoll * $f );

		$rack_z_he_pixel = floor( 1.75 * $f );
		$rack_z_pixel = floor( $rack_z_he_pixel * $rack_z_he );

		// calculate image height
		$sy = floor( $rack_z_pixel + $ty + max( $ty, $icon_size ) + 3 * $padding ); 

		// create image
		$im = imagecreatetruecolor( $sx, $sy );
		$color_back = ImageColorAllocate ($im, 255, 255, 255);
		$color_text = ImageColorAllocate ($im,   0,   0,   0);
		$color_red  = ImageColorAllocate ($im, 127,   0,   0);
		$color_grey = ImageColorAllocate ($im, 127, 127, 127);
		$color_grey_light = ImageColorAllocate ($im, 192, 192, 192);
		$color_blue = ImageColorAllocate ($im,   0,   0, 127);
		imagefill( $im, 0, 0, $color_back );

		// draw canvas
		$total_box = new box_group( $sx, $sy, FALSE );

		// ci 
		$ci_box = new box_ci( $padding, $this->ci->get_icon(), $icon_size, $this->ci->get_title(), $font, $font_size, $color_text );
		$ci_box->set_click( "ci_view.php?ci_id=" . $this->ci->get_id() );
		$total_box->add_static( $ci_box, BOX_ALIGN_CENTER, BOX_ALIGN_TOP );

		// draw area
		$draw_box = new box_group( 0, 0, FALSE );
		$total_box->add_float( $draw_box, BOX_ALIGN_CENTER, BOX_ALIGN_BOTTOM );

		// frame boxs
		$rear_frame = new box_group( $rack_x_pixel, $draw_box->get_size_y(), FALSE );
		$draw_box->add_static( $rear_frame, BOX_ALIGN_LEFT, BOX_ALIGN_BOTTOM);

		$side_frame = new box_group( $rack_y_pixel, $draw_box->get_size_y(), FALSE );
		$draw_box->add_static( $side_frame, BOX_ALIGN_CENTER, BOX_ALIGN_BOTTOM);

		$front_frame = new box_group( $rack_x_pixel, $draw_box->get_size_y(), FALSE );
		$draw_box->add_static( $front_frame, BOX_ALIGN_RIGHT, BOX_ALIGN_BOTTOM);

		// titles
		$rear_title = new box_ft_text( "rear", $font, $font_size, $color_text );
		$rear_title->set_align( BOX_ALIGN_CENTER, BOX_ALIGN_ADAPT );
		$rear_title->set_size( $rack_x_pixel, 0 );
		$rear_frame->add_static( $rear_title, BOX_ALIGN_CENTER, BOX_ALIGN_TOP );

		$side_title = new box_ft_text( "side", $font, $font_size, $color_text );
		$side_title->set_align( BOX_ALIGN_CENTER, BOX_ALIGN_ADAPT );
		$side_title->set_size( $rack_y_pixel, 0 );
		$side_frame->add_static( $side_title, BOX_ALIGN_CENTER, BOX_ALIGN_TOP );

		$front_title = new box_ft_text( "front", $font, $font_size, $color_text );
		$front_title->set_align( BOX_ALIGN_CENTER, BOX_ALIGN_ADAPT );
		$front_title->set_size( $rack_x_pixel, 0 );
		$front_frame->add_static( $front_title, BOX_ALIGN_CENTER, BOX_ALIGN_TOP );

		// rack boxes
		$rear_rack = new box_group( $rack_x_pixel, $rack_z_pixel, FALSE );
		$rear_box  = new box_rack( $rack_x_pixel, $rack_z_he_pixel, $rack_z_he, $color_blue, $color_grey_light );
		$rear_rack->add( $rear_box );
		$rear_frame->add_static( $rear_rack, BOX_ALIGN_CENTER, BOX_ALIGN_BOTTOM );

		$side_rack = new box_group( $rack_y_pixel, $rack_z_pixel, FALSE );
		$side_box  = new box_rack( $rack_y_pixel, $rack_z_he_pixel, $rack_z_he, $color_blue, $color_grey_light );
		$side_rack->add( $side_box );
		$side_frame->add_static( $side_rack, BOX_ALIGN_CENTER, BOX_ALIGN_BOTTOM );

		$front_rack = new box_group( $rack_x_pixel, $rack_z_pixel, FALSE );
		$front_box  = new box_rack( $rack_x_pixel, $rack_z_he_pixel, $rack_z_he, $color_blue, $color_grey_light );
		$front_rack->add( $front_box );
		$front_frame->add_static( $front_rack, BOX_ALIGN_CENTER, BOX_ALIGN_BOTTOM );

		// rear-scale
		$rear_box2 = $rear_rack->get_box();
		$side_box2 = $side_frame->get_box();
		$front_box2 = $front_frame->get_box();

		$rear_scale = new box_group( $side_box2[0] - $rear_box2[2], $rack_z_pixel, FALSE );
		$rear_scale->shift_pos( $rear_box2[2] + 1, $rear_box2[1] );
		$draw_box->add( $rear_scale );
		for ($i=0; $i<$rack_z_he; $i=$i+5)
		{
			$txt_box = new box_ft_text( ($base_z_he + $i), $font, $font_size, $color_text );
			$txt_box->set_align( BOX_ALIGN_CENTER, BOX_ALIGN_CENTER );
			$y1 = $rack_z_pixel - (($i+1) * $rack_z_he_pixel) + 1;

			$txt_box->set_size( $rear_scale->get_size_x(), $rack_z_he_pixel );
			$txt_box->shift_pos( 0, $y1 );
			$rear_scale->add( $txt_box );
		}

		// front scale
		$front_scale = new box_group( $front_box2[0] - $side_box2[2], $rack_z_pixel, FALSE );
		$front_scale->shift_pos( $side_box2[2] + 1, $rear_box2[1] );
		$draw_box->add( $front_scale );
		for ($i=0; $i<$rack_z_he; $i=$i+5)
		{
			$txt_box = new box_ft_text( ($base_z_he + $i), $font, $font_size, $color_text );
			$txt_box->set_align( BOX_ALIGN_CENTER, BOX_ALIGN_CENTER );
			$y1 = $rack_z_pixel - (($i+1) * $rack_z_he_pixel) + 1;

			$txt_box->set_size( $front_scale->get_size_x(), $rack_z_he_pixel );
			$txt_box->shift_pos( 0, $y1 );
			$front_scale->add( $txt_box );
		}

		// insert peers
		if ($count > 0)
		{
			// check overlaps
			$he_array = array();
			for ($j=0; $j<=$rack_z_he; $j++) $he_array[ $j ] = 0;

			// loop all peers
			for ($i=0; $i<$count; $i++ )
			{
				$relation_peer	= & $this->peer_list->at( $i );
				$peer_id		= $relation_peer->get_ci_id();

				$rll = new module_label_list( array(DC_DEVICE_POS_Z,DC_DEVICE_SIZE_Z) );
				$label_tracker	= new label_list_tracker( $this->ci->get_id(), 3, $rll );
				$label_tracker->run_relation( $relation_peer->get_id() );
				if ($rll->completed())
				{
					// yea, it's a device
					$title = $relation_peer->get_ci_title();
					$title_full	= $title . " / " . $relation_peer->get_cit_title();

					$device_pos = $rll->get_label_value( DC_DEVICE_POS_Z ) - $base_z_he;
					$device_size = $rll->get_label_value( DC_DEVICE_SIZE_Z );
					$device_icon = $relation_peer->get_icon();
					
					// inside rack range
					$_color_frame = $color_blue;
					if (
						($device_pos < 0)
						||
						(($device_pos + $device_size)>$rack_z_he)
						)
					{
						$_color_frame = $color_red;
					}

					// create box
					$device_box = new box_device( $rack_x_pixel,
						$device_size, $device_pos, $rack_z_he, $rack_z_he_pixel,
						$device_icon, $icon_size, $title, $font, $font_size, $color_text, $_color_frame, $color_back
						);
					$device_box->set_click( "ci_view.php?ci_id=$peer_id", $title_full );

					$front_rack->add( $device_box );
					$rear_rack->add( $device_box );
					
					// add to he_array
					for ($j=0; $j<$device_size; $j++)
					{
						$he = $device_pos + $j;
						if (isset($he_array[ $he ])) $he_array[ $he ]++;
					}
				}
				else
				{
					// perhaps a datacenter room
					$rll = new module_label_list( array(DC_ROOM_SIZE_X,DC_ROOM_SIZE_Y) );
					$label_tracker	= new label_list_tracker( $peer_id, 1, $rll );
					$label_tracker->run();
					if ($rll->completed())
					{
						// really, a datacenter room
						$title = $relation_peer->get_ci_title();
						$title_full	= $title . " / " . $relation_peer->get_cit_title();
	
						// ci 
						$rz_ci_box = new box_ci( $padding, $relation_peer->get_icon(), $icon_size, $title_full, $font, $font_size, $color_text );
						$rz_ci_box->set_click( "ci_view.php?ci_id=$peer_id" );
						$total_box->add_static( $rz_ci_box, BOX_ALIGN_RIGHT, BOX_ALIGN_TOP );
						
					}					
				}

			}
			
			// display overlays
			for ($he=$base_z_he; $he<=$rack_z_he; $he++)
			{
				if ($he_array[ $he ] > 1)
				{
					$overlap_box = new box_device( $rack_x_pixel,
						1, $he, $rack_z_he, $rack_z_he_pixel,
						"", $icon_size, "", $font, $font_size, $color_text, $color_red, $color_red
						);

					$front_rack->add( $overlap_box );
					$rear_rack->add( $overlap_box );

				}
			}	
		}

		// output
		$total_box->draw( $im, $mp );

		// store
		ImagePng( $im, $this->imx_name );

		// close html-click-map file
		fwrite( $mp, "</map>\n" );
		fclose( $mp );

		return;
	}

	/**
	 * reply filename postfix, to describe the module
	 */
	protected function get_module_postfix ()
	{
		return "_dc_rack";
	}

	
}
