<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'includes/module_base.inc.php';
require_once 'includes/module_label_list.inc.php';

// labels

define ('NW_DNS_IP',					101 );
define ('NW_DNS_FQDN',					102 );
define ('NW_DNS_DOMAIN_NAME',			103 );
define ('NW_DNS_HOST_NAME',				104 );

/**
 * management class for optional modul "network"
 */
class module_network extends module_base
{
	
	/**
	 * supported labels of this module
	 */
	public function get_enum_label_string ()
	{
		return NW_DNS_IP . ":Network-IP-Address," .
			NW_DNS_FQDN . ":Network-FQDN," .
			NW_DNS_DOMAIN_NAME . ":Network-Domainname," .
			NW_DNS_HOST_NAME . ":Network-Hostname";
	}
	
	public function get_display_class ( $ci )
	{
		$class_array = array();
		return $class_array;
	}

	public function get_input_value ( $label, &$item_processor )
	{
		$pre_value = "";
		
		switch ($label)
		{
			case NW_DNS_IP:
			{
				$_ip_addr = array();
				$nw_label_list = new module_label_list( array(NW_DNS_FQDN, NW_DNS_DOMAIN_NAME, NW_DNS_HOST_NAME) );	 
				$item_processor->fill_label_list( $nw_label_list );

				if (($nw_dns_fqdn = $nw_label_list->get_label_value( NW_DNS_FQDN )) != null)
				{
					// try to get IP for the FQDN
					$_ip_addr = gethostbynamel( $nw_dns_fqdn );
				}
				else
				{
					if (
						(($nw_dns_domain = $nw_label_list->get_label_value( NW_DNS_DOMAIN_NAME )) != null)
						&&
						(($nw_dns_host = $nw_label_list->get_label_value( NW_DNS_HOST_NAME )) != null)
						)
					{
						// try to get IP for host.domain
						$nw_dns_fqdn = $nw_dns_host . "." . $nw_dns_domain;
						$_ip_addr = gethostbynamel( $nw_dns_fqdn );
					}
				}
				
				// extract IPs from array
				if ($_ip_addr !== FALSE )
				{
					foreach( $_ip_addr as $_ip )
					{
						// skip localhost IPs
						if (substr( $_ip, 0, 4) != "127." ) $pre_value = $pre_value . $_ip . " ";
					}
				}

				break;
			}
		}

		return $pre_value;
	}

}

// register
$module = new module_network();
module_list::register( $module );
