<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_attr.inc.php 425 2010-04-24 16:57:17Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabellen attr (r/w) UND type_attr (ro) 
 * 
 * felder der tabelle attr (read/write zugriff)
 * 
 * a_id				int			primary key
 * a_classtype		int			code for classtype
 * a_classtype_id	int			foreign key auf class-instance
 * a_ta_id			int			foreign key auf type_attr
 * a_val_inc		int			value als INT
 * a_val_float		float		value als FLOAT
 * a_val_date		date		value als DATE
 * a_val_time		time		value als TIME
 * a_val_text		text		value als TEXT
 * 
 * felder der tabelle type_attr (read only zugriff)
 * 
 * ta_id			int			primary key
 * ta_classtype		int			class type (siehe Constanten CLASS_xxx)
 * ta_classtype_id	int			foreign key auf ci_type / rel_type / usw.
 * ta_title			string		Name/Bezeichner des CI-Type-Attributes
 * ta_desc			text		Beschreibung des CI-Type-Attributes
 * ta_datatype		int			daten typ
 * ta_datatype_sub	int			daten subtype
 * ta_order			int			ordnungs-nummer fuer reihenfolge
 * ta_visible		int			sichtbarkeits-nummer
 * ta_label			int			label fuer module
 * ta_oblig			bool		pflichtfeld: ja/nein
 * ta_unique		bool		feld ist unique innerhalb aller CIs des CI-Types: ja/nein
 * ta_tag			bool		wert des attributs wird in synopse des CI genutzt (ja/nein)
 * ta_input			int			art des input (siehe Constanten ATTR_INPUT_xxx) 
 * ta_format		string		formatierungs-vorlage fuer das attribut
 * 
 */
class db_attr extends core_db_object
{
	protected $a_id				= 0;
	protected $a_classtype		= 0;
	protected $a_classtype_id	= 0;
	protected $a_ta_id			= 0;
	protected $a_val_int		= 0;
	protected $a_val_float		= 0.0;
	protected $a_val_date		= "";
	protected $a_val_time		= "";
	protected $a_val_text		= "";

	protected $ta_id			= 0;
	protected $ta_classtype		= 0;
	protected $ta_classtype_id	= 0;
	protected $ta_title			= "";
	protected $ta_desc			= "";
	protected $ta_datatype		= DT_TEXT;
	protected $ta_datatype_sub	= 0;
	protected $ta_order			= 0;
	protected $ta_visible		= 1;
	protected $ta_label			= 0;
	protected $ta_oblig			= TRUE;
	protected $ta_unique		= FALSE;
	protected $ta_tag			= FALSE;
	protected $ta_input			= ATTR_INPUT_FORM;
	protected $ta_format		= "";

	public function __construct ( $_id = 0 )
	{
		parent::__construct ( $_id );
		return;
	}

	public function __clone ()
	{
		parent::__clone();
		$this->a_classtype_id = 0;
		if ($this->ta_unique)
		{
			switch ($this->ta_datatype)
			{
				case DT_TEXTBLOCK:
				case DT_TEXT:
				{
					$this->a_val_text = sprintf( get_string('CloneOf_dsc')." %s", $this->a_val_text );
					break;
				}		 
			}
		}
	}

	// set methoden (nur fuer felder von attr)

	public function set_id ( $_id )
	{
		$this->a_id = $_id;
		return TRUE;
	}

	public function set_classtype ( $_classtype )
	{
		$this->a_classtype = $_classtype;
		return TRUE;
	}

	public function set_classtype_id ( $_classtype_id )
	{
		if ($this->a_classtype_id == $_classtype_id) return TRUE;
		$this->a_classtype_id = $_classtype_id;
		$this->mark_modified();
		return TRUE;
	}

	public function set_ta_id ( $_id )
	{
		if ($this->a_ta_id == $_id) return TRUE;
		$this->a_ta_id = $_id;
		$this->mark_modified();
		return TRUE;
	}

	public function set_val_int ( $_value )
	{
		if ($this->a_val_int == $_value) return TRUE;
		$this->a_val_int = $_value;
		$this->mark_modified();
		return TRUE;
	}

	public function set_val_float ( $_value )
	{
		if ($this->a_val_float == $_value) return TRUE;
		$this->a_val_float = $_value;
		$this->mark_modified();
		return TRUE;
	}

	public function set_val_date ( $_value )
	{
		if ($this->a_val_date == $_value) return TRUE;
		$this->a_val_date = $_value;
		$this->a_val_text = $_value;
		$this->mark_modified();
		return TRUE;
	}

	public function set_val_time ( $_value )
	{
		if ($this->a_val_time == $_value) return TRUE;
		$this->a_val_time = $_value;
		$this->a_val_text = $_value;
		$this->mark_modified();
		return TRUE;
	}

	public function set_val_text ( $_value )
	{
		if ($this->a_val_text == $_value) return TRUE;
		$this->a_val_text = $_value;
		$this->mark_modified();
		return TRUE;
	}


	// get methoden (felder von attr)

	protected function get_id_name ()
	{
		return "a_id";
	}

	protected function get_table_name ()
	{
		return "attr";
	}

	public function get_id ()
	{
		return $this->a_id;
	}

	public function get_classtype ()
	{
		return $this->a_classtype;
	}

	public function get_classtype_id ()
	{
		return $this->a_classtype_id;
	}

	/**
	 * liefert die ID des Type-Attr
	 * 
	 * HINWEIS: es wird das feld des type_attr
	 * geliefert, weil dieses immer gesetzt ist
	 */
	public function get_ta_id ()
	{
//		return $this->a_ta_id;
		return $this->ta_id;
	}

	public function get_val_int ()
	{
		return $this->a_val_int;
	}

	public function get_val_float ()
	{
		return $this->a_val_float;
	}

	public function get_val_date ()
	{
		return $this->a_val_date;
	}

	public function get_val_time ()
	{
		return $this->a_val_time;
	}

	public function get_val_text ()
	{
		return $this->a_val_text;
	}

	// get methoden (felder von type_attr)

	public function get_ta_classtype ()
	{
		return $this->ta_classtype;
	}

	public function get_ta_classtype_id ()
	{
		return $this->ta_classtype_id;
	}

	public function get_title ()
	{
		return $this->ta_title;
	}

	public function get_desc ()
	{
		return $this->ta_desc;
	}

	public function get_datatype ()
	{
		return $this->ta_datatype;
	}

	public function get_datatype_sub ()
	{
		return $this->ta_datatype_sub;
	}

	public function get_order ()
	{
		return $this->ta_order;
	}

	public function get_visible ()
	{
		return $this->ta_visible;
	}

	public function get_label ()
	{
		return $this->ta_label;
	}

	public function get_oblig ()
	{
		return $this->ta_oblig;
	}

	public function get_unique ()
	{
		return $this->ta_unique;
	}

	public function get_tag ()
	{
		return $this->ta_tag;
	}

	public function get_input ()
	{
		return $this->ta_input;
	}

	public function get_format ()
	{
		return $this->ta_format;
	}

	// operators

	public function is_empty ()
	{
		return (strlen($this->a_val_text)==0);
	}

	// validation
	
	public function is_valid ()
	{
		$db		= core_db::getInstance();

		// plausi: ci_id gueltig
		$query = "";
		switch ($this->a_classtype)
		{
			case CLASS_CI:
			{
				$query	= "SELECT COUNT(*) FROM ci " .
					"WHERE ci_id='$this->a_classtype_id'";
				break;
			}
			case CLASS_REL:
			{
//				$query	= "SELECT COUNT(*) FROM relation " .
//					"WHERE r_id='$this->a_classtype_id'";
				break;
			}
		}
		if (strlen($query)!=0)
		{			
			$result = $db->query( $query );
			if ($db->num_rows($result)!=1)
			{
				throw new core_db_exception( get_string('UnexpectedNumberOfRows_dsc') );
			}
			if ($db->result( $result, 0 ) != 1)
			{
				throw new core_data_exception( get_string('InvalidIDofCT_dsc').": ".$this->a_classtype_id);
			}
		}

		// plausi: ta_id gueltig
		$query	= "SELECT COUNT(*) FROM type_attr " .
			"WHERE ta_id='$this->a_ta_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_db_exception( get_string('UnexpectedNumberOfRows_dsc') );
		}
		if ($db->result( $result, 0 ) != 1)
		{
			throw new core_data_exception( get_string('InvalidIDofTA_dsc').": ".$this->a_ta_id);
		}

		if ($this->ta_id != $this->a_ta_id)
		{
			throw new core_data_exception( get_string('IDofTANotEqual_dsc').": ".$this->ta_id." <> ".$this->a_ta_id);
		}

		// plausi: oblig
		if (($this->ta_oblig==1) && (strlen($this->a_val_text)==0))
		{
			throw new core_data_exception( get_string('Attribute_ttl')." ".$this->ta_title." ".get_string('ObligValueMustNotBeEmpty_dsc'));
		}

		// plausi: unique
		if (
			($this->ta_unique==1)
			&&
			(strlen($this->a_val_text)!=0)
			)
		{
			$query  = "SELECT COUNT(*) FROM attr " .
				"WHERE a_classtype='$this->a_classtype' " .
				"AND a_classtype_id!='$this->a_classtype_id' " .
				"AND a_ta_id='$this->a_ta_id' " .
				"AND a_val_text='".
				$db->escape_string($this->a_val_text)."'";
			$result = $db->query( $query );
			if ($db->num_rows($result)!=1)
			{
				throw new core_db_exception( get_string('UnexpectedNumberOfRows_dsc') );
			}
			if ($db->result( $result, 0 ) != 0)
			{
				throw new core_data_exception( get_string('Attribute_ttl')." ".$this->ta_title.": ".get_string('ValueMustBeUnique_dsc'));
			}
				
		}

		return TRUE;
	}

	// db actions low level

	/**
	 * inhalt des objektes aus der datenbank holen
	 */
	protected function db_load_impl ()
	{
		$db		= core_db::getInstance();
		
		// daten laden
		$query  = "SELECT * FROM attr, type_attr " .
			"WHERE a_ta_id=ta_id " .
			"AND a_id='$this->a_id'";
		$result = $db->query( $query );

		if ($db->num_rows($result)!=1)
		{
			throw new core_db_exception( get_string('UnexpectedNumberOfRows_dsc') );
		}

		$data= $db->fetch( $result );
		return $this->load_from_array( $data );
	}

	/**
	 * objekt in die DB einfuegen
	 */
	protected function db_insert_impl ()
	{
		// DB
		$db		= core_db::getInstance();

		$query	= "INSERT INTO attr " .
			"(a_classtype,a_classtype_id,a_ta_id,a_val_int,a_val_float,a_val_date,a_val_time,a_val_text) " .
			"VALUES " .
			"('$this->a_classtype','$this->a_classtype_id','$this->a_ta_id','$this->a_val_int','$this->a_val_float','" . $this->get_db_val_date() . "','" . $this->get_db_val_time() . "','" . $db->escape_string($this->a_val_text) . "')";
		$result = $db->query( $query );

		// id holen
		$this->a_id = $db->id();

		return TRUE;
	}
	
	/**
	 * objekt in der DB aktualisieren
	 */
	protected function db_update_impl ()
	{
		// DB
		$db		= core_db::getInstance();
		$query	= "UPDATE attr SET " .
			"a_classtype='$this->a_classtype', " .
			"a_classtype_id='$this->a_classtype_id', " .
			"a_ta_id='$this->a_ta_id', " .
			"a_val_int='$this->a_val_int', " .
			"a_val_float='$this->a_val_float', " .
			"a_val_date='" . $this->get_db_val_date() . "', " .
			"a_val_time='" . $this->get_db_val_time() . "', " .
			"a_val_text='".
			$db->escape_string($this->a_val_text)."' " .
			"WHERE a_id='$this->a_id' LIMIT 1";
//echo "DEBUG: $query<br>\n";
		$result = $db->query( $query );

		return TRUE;
	}
	
	/**
	 * objekt in der DB loeschen
	 */
	protected function db_delete_impl ()
	{
		$db		= core_db::getInstance();
		$query	= "DELETE FROM attr " .
			"WHERE a_id='$this->a_id' LIMIT 1";
		$result = $db->query( $query );

		return TRUE;
	}

	private function get_db_val_date()
	{
		if (strlen($this->a_val_date)==0)
		{
			return '0000-00-00';
		}
		return $this->a_val_date;
	} 

	private function get_db_val_time()
	{
		if (strlen($this->a_val_time)==0)
		{
			return '00:00:00';
		}
		return $this->a_val_time;
	} 

}
