<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_cat.inc.php 339 2009-04-10 12:39:09Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabelle cat
 * 
 * cat_id		int			primary key
 * cat_title	string		Name/Bezeichner des katalogs
 * cat_desc		text		Beschreibung des katalogs
 * cat_strict	bool		katalog ist strikt (ja/nein)
 * 
 * Strikte kataloge koennen nur durch die rolle "Designer"
 * erweitert werden, nicht-strikte kataloge koennen auch durch
 * die rolle "betreuer" erweitert werden
 * 
 */
 
class db_cat extends core_db_object
{
	protected $cat_id			= 0;
	protected $cat_title		= "";
	protected $cat_desc			= "";
	protected $cat_strict		= FALSE;
	
	// set methoden
	
	public function set_id ( $_id )
	{
		$this->cat_id = $_id;
		return TRUE;
	}

	public function set_title ( $_title )
	{
		if (strcmp($_title, $this->cat_title)==0) return TRUE;
		$this->cat_title = $_title;
		$this->mark_modified();
		return TRUE;
	}

	public function set_desc ( $_desc )
	{
		if (strcmp($_desc, $this->cat_desc)==0) return TRUE;
		$this->cat_desc = $_desc;
		$this->mark_modified();
		return TRUE;
	}

	public function set_strict ( $_strict )
	{
		if ($_strict == $this->cat_strict) return TRUE;
		$this->cat_strict = $_strict;
		$this->mark_modified();
		return TRUE;
	}

	// get methoden
	
	protected function get_id_name ()
	{
		return "cat_id";
	}

	protected function get_table_name ()
	{
		return "cat";
	}

	public function get_id ()
	{
		return $this->cat_id;
	}

	public function get_title ()
	{
		return $this->cat_title;
	}

	public function get_desc ()
	{
		return $this->cat_desc;
	}

	public function get_strict ()
	{
		return $this->cat_strict;
	}

	// validation
	
	public function is_valid ()
	{
		// plausi: title leer
		if (strlen($this->cat_title)==0)
		{
			throw new core_data_exception( "title must not be empty" );
		}

		// plausi: title schon vorhanden?
		$db		= core_db::getInstance();
		$query	= "SELECT COUNT(*) FROM cat " .
			"WHERE cat_id!='$this->cat_id' " .
			"AND cat_title='$this->cat_title'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}

		if ($db->result( $result, 0 ) != 0)
		{
			throw new core_data_exception( "title already used for another catalog" );
		}

		return TRUE;
	}

}
