<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_cat_entry.inc.php 339 2009-04-10 12:39:09Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabelle cat_entry
 * 
 * cate_id		int			primary key
 * cate_cat_id	int			fremdschluessel auf katalog
 * cate_title	string		text des katalog eintrags
 * cate_icon	string		(optional) icon name
 * cate_fg_color	int		foreground color
 * cate_bg_color	int		background color
 */
 
class db_cat_entry extends core_db_object
{
	protected $cate_id			= 0;
	protected $cate_cat_id		= 0;
	protected $cate_title		= "";
	protected $cate_icon		= "";
	protected $cate_fg_color	= COLOR_DEFAULT;
	protected $cate_bg_color	= COLOR_DEFAULT;
	
	// set methoden
	
	public function set_id ( $_id )
	{
		$this->cate_id = $_id;
		return TRUE;
	}

	public function set_cat_id ( $_cat_id )
	{
		$this->cate_cat_id = $_cat_id;
		return TRUE;
	}

	public function set_title ( $_title )
	{
		if (strcmp($_title, $this->cate_title)==0) return TRUE;
		$this->cate_title = $_title;
		$this->mark_modified();
		return TRUE;
	}

	public function set_icon ( $_icon )
	{
		if (strcmp($_icon, $this->cate_icon)==0) return TRUE;
		$this->cate_icon = $_icon;
		$this->mark_modified();
		return TRUE;
	}

	public function set_fg_color ( $_fg_color )
	{
		if (strcmp($_fg_color, $this->cate_fg_color)==0) return TRUE;
		$this->cate_fg_color = $_fg_color;
		$this->mark_modified();
		return TRUE;
	}

	public function set_bg_color ( $_bg_color )
	{
		if (strcmp($_bg_color, $this->cate_bg_color)==0) return TRUE;
		$this->cate_bg_color = $_bg_color;
		$this->mark_modified();
		return TRUE;
	}

	// get methoden
	
	protected function get_id_name ()
	{
		return "cate_id";
	}

	protected function get_table_name ()
	{
		return "cat_entry";
	}

	public function get_id ()
	{
		return $this->cate_id;
	}

	public function get_cat_id ()
	{
		return $this->cate_cat_id;
	}

	public function get_title ()
	{
		return $this->cate_title;
	}

	public function get_icon ()
	{
		return $this->cate_icon;
	}

	public function get_fg_color ()
	{
		return $this->cate_fg_color;
	}

	public function get_bg_color ()
	{
		return $this->cate_bg_color;
	}

	public function get_styled_title ()
	{
		$icon_size  = get_property('list_icon_size');
		$title = icon_list::getInstance()->get_html_imx( $this->get_icon(), $icon_size, $icon_size, FALSE ) . ' ' . $this->get_title();
		$style_str = '';
		if ($this->cate_fg_color != COLOR_DEFAULT)
		{
			$style_str = $style_str . 'color: #' . get_enum_to_string( 'enum_color_codes', $this->cate_fg_color ) . '; ';
		}
		if ($this->cate_bg_color != COLOR_DEFAULT)
		{
			$style_str = $style_str . 'background: #' . get_enum_to_string( 'enum_color_codes', $this->cate_bg_color ) . '; ';
		}
		if (strlen($style_str)!=0)
		{
			$title = "<span style='$style_str'>$title</span>";
		}
		return $title;
	}

	// validation
	
	public function is_valid ()
	{
		// plausi: cat_id
		$db		= core_db::getInstance();
		$query	= "SELECT COUNT(*) FROM cat " .
			"WHERE cat_id='$this->cate_cat_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if ($db->result( $result, 0 ) != 1)
		{
			throw new core_data_exception( "invalid catalog-type");
		}
		
		// plausi: title leer
		if (strlen($this->cate_title)==0)
		{
			throw new core_data_exception( "title must not be empty" );
		}

		// plausi: title schon vorhanden?
		$db		= core_db::getInstance();
		$query	= "SELECT COUNT(*) FROM cat_entry " .
			"WHERE cate_id!='$this->cate_id' " .
			"AND cate_cat_id='$this->cate_cat_id' " .
			"AND cate_title='".$db->escape_string($this->cate_title)."'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}

		if ($db->result( $result, 0 ) != 0)
		{
			throw new core_data_exception( "title already used for another catalog-entry" );
		}

		return TRUE;
	}

}
