<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_ci.inc.php 339 2009-04-10 12:39:09Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabelle ci
 * 
 * ci_id			int			primary key
 * ci_cit_id		int			fremdschluesser auf den CI-Type
 * ci_title			string		synopse des CI
 * ci_icon			string		Name der icon-datei
 * ci_last_uid		int			letzte aenderung durch user id
 * ci_last_changed	timestamp	letzte aenderung
 * 
 */
 
class db_ci extends core_db_object
{
	protected $ci_id			= 0;
	protected $ci_cit_id		= 0;
	protected $ci_title			= "";
	protected $ci_icon			= "";
	protected $ci_last_uid		= 0;
	protected $ci_last_change	= "";
	
	// set methoden
	
	public function set_id ( $_id )
	{
		$this->ci_id = $_id;
		return TRUE;
	}

	public function set_cit_id ( $_cit_id )
	{
		$this->ci_cit_id = $_cit_id;
		return TRUE;
	}

	public function set_title ( $_title )
	{
		if ($this->ci_title == $_title) return TRUE;
		$this->ci_title = $_title;
		$this->mark_modified();
		return TRUE;
	}

	public function set_icon ( $_icon )
	{
		if ($this->ci_icon == $_icon) return TRUE;
		$this->ci_icon = $_icon;
		$this->mark_modified();
		return TRUE;
	}

	// get methoden
	
	protected function get_id_name ()
	{
		return "ci_id";
	}

	protected function get_table_name ()
	{
		return "ci";
	}

	public function get_id ()
	{
		return $this->ci_id;
	}

	public function get_cit_id ()
	{
		return $this->ci_cit_id;
	}

	public function get_title ()
	{
		return $this->ci_title;
	}

	public function get_icon ()
	{
		return $this->ci_icon;
	}

	public function get_last_uid ()
	{
		return $this->ci_last_uid;
	}

	public function get_last_change ()
	{
		return $this->ci_last_change;
	}

	// validation
	
	public function is_valid ()
	{
		// plausi: cit_id
		$db		= core_db::getInstance();
		$query	= "SELECT COUNT(*) FROM ci_type " .
			"WHERE cit_id='$this->ci_cit_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if (($count = $db->result( $result, 0 )) != 1)
		{
			throw new core_data_exception( "invalid CI-Type: $this->ci_cit_id");
		}

		// plausi: title nicht leer
		if (strlen($this->ci_title)==0)
		{
			throw new core_data_exception( "title must not be empty" );
		}
		
		return TRUE;
	}

	// db actions low level

	/**
	 * objekt in die DB einfuegen
	 */
	protected function db_insert_impl ()
	{
		// last UID setzen
		$this->ci_last_uid = core_session::getInstance()->get_user_id();
		$this->ci_last_change = date("Y-m-d H:i:s");

		parent::db_insert_impl();

		return TRUE;
	}
	
	/**
	 * objekt in der DB aktualisieren
	 * 
	 * da sich an den feldern nicht mehr aendern kann, wird
	 * ci_last_change explizit gesetzt
	 */
	protected function db_update_impl ()
	{
		// last UID setzen
		$this->ci_last_uid = core_session::getInstance()->get_user_id();
		$this->ci_last_change = date("Y-m-d H:i:s");
		
		parent::db_update_impl();

		return TRUE;
	}
	
}
