<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_ci_type.inc.php 400 2009-07-04 07:53:55Z micha $
*/

require_once 'core/core_exception.inc.php';
require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabelle ci_type
 * 
 * cit_id			int			primary key
 * cit_title		string		Name/Bezeichner des CI-Types
 * cit_icon			string		Name der icon-datei
 * cit_desc			text		Beschreibung des CI-Types
 * cit_last_uid		int			letzte aenderung durch user id
 * cit_last_changed	timestamp	letzte aenderung
 * 
 */
class db_ci_type extends core_db_object
{
	protected $cit_id			= 0;
	protected $cit_title		= "";
	protected $cit_icon			= "";
	protected $cit_desc			= "";
	protected $cit_last_uid		= 0;
	protected $cit_last_changed	= "";

	// set methoden

	/**
	 * setzt die ID des CI-Types
	 */
	public function set_id ( $_id )
	{
		$this->cit_id = $_id;
		return TRUE;
	}

	/**
	 setzt den titel des CI-Types
	 */
	public function set_title ( $_title )
	{
		if (strcmp($_title, $this->cit_title)==0) return TRUE;
		$this->cit_title = $_title;
		$this->mark_modified();
		return TRUE;
	}
		
	/**
	 setzt den name des icons
	 */
	public function set_icon ( $_icon )
	{
		if (strcmp($_icon, $this->cit_icon)==0) return TRUE;
		$this->cit_icon = $_icon;
		$this->mark_modified();
		return TRUE;
	}
		
	/**
	 * setzt die beschreibung des CI-Types
	 */
	public function set_desc ( $_desc )
	{
		if (strcmp($_desc, $this->cit_desc)==0) return FALSE;
		$this->cit_desc = $_desc;
		$this->mark_modified();
		return TRUE;
	}

	// get methoden

	protected function get_id_name ()
	{
		return "cit_id";
	}

	protected function get_table_name ()
	{
		return "ci_type";
	}

	/**
	 * liefert die ID des CI-Types
	 */
	public function get_id ()
	{
		return $this->cit_id;
	}

	/**
	 * liefert den titel des CI-Types
	 */
	public function get_title ()
	{
		return $this->cit_title;
	}
	
	/**
	 * liefert das icon
	 */
	public function get_icon ()
	{
		return $this->cit_icon;
	}
	
	/**
	 * liefert die beschreibung des CI-Types
	 */
	public function get_desc ()
	{
		return $this->cit_desc;
	}
	
	/**
	 * liefert die UID des letzten bearbeiters des CI-Types
	 */
	public function get_last_uid ()
	{
		return $this->cit_last_uid;
	}

	/**
	 * liefert den zeitstempel der letzten bearbeitung des CI-Types
	 */
	public function get_last_changed ()
	{
		return $this->cit_last_changed;
	}

	// validation
	
	public function is_valid ()
	{
		// plausi: title leer
		if (strlen($this->cit_title)==0)
		{
			throw new core_data_exception( "title must not be empty" );
		}

		// plausi: title schon vorhanden?
		$db		= core_db::getInstance();
		$query	= "SELECT COUNT(*) FROM ci_type " .
			"WHERE cit_id!='$this->cit_id' " .
			"AND cit_title='$this->cit_title'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}

		if ($db->result( $result, 0 ) != 0)
		{
			throw new core_data_exception( "title already used for another CI-Type" );
		}
		
		return TRUE;
	}

	// db actions low level

	/**
	 * objekt in die DB einfuegen
	 */
	protected function db_insert_impl ()
	{
		// last UID setzen
		$this->cit_last_uid = core_session::getInstance()->get_user_id();
		$this->cit_last_changed = date("Y-m-d H:i:s");

		parent::db_insert_impl();

		return TRUE;
	}
	
	/**
	 * objekt in der DB aktualisieren
	 */
	protected function db_update_impl ()
	{
		// last UID setzen
		$this->cit_last_uid = core_session::getInstance()->get_user_id();
		$this->cit_last_changed = date("Y-m-d H:i:s");
		
		parent::db_update_impl();

		return TRUE;
	}

}
