<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_comment.inc.php 339 2009-04-10 12:39:09Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabelle comment
 * 
 * c_id				int			primary key
 * c_ci_id			int			fremdschluesser auf das CI
 * c_text			text		der kommentar
 * c_create_uid		int			erzeugt durch user id
 * c_last_uid		int			letzte aenderung durch user id
 * c_create_change	timestamp	erzeugt am/um
 * c_last_change	timestamp	letzte aenderung
 * 
 */
 
class db_comment extends core_db_object
{
	protected $c_id				= 0;
	protected $c_ci_id			= 0;
	protected $c_text			= "";
	protected $c_create_uid		= 0;
	protected $c_last_uid		= 0;
	protected $c_create_change	= "";
	protected $c_last_change	= "";
	
	// set methoden
	
	public function set_id ( $_id )
	{
		$this->c_id = $_id;
		return TRUE;
	}

	public function set_ci_id ( $_ci_id )
	{
		if ($this->c_id != 0) return FALSE;
		$this->c_ci_id = $_ci_id;
		return TRUE;
	}

	public function set_text ( $_text )
	{
		if ($this->c_text == $_text) return TRUE;
		if (strlen($_text)==0) return FALSE;
		$this->c_text = $_text;
		$this->mark_modified();
		return TRUE;
	}

	// get methoden
	
	protected function get_id_name ()
	{
		return "c_id";
	}

	protected function get_table_name ()
	{
		return "comment";
	}

	public function get_id ()
	{
		return $this->c_id;
	}

	public function get_ci_id ()
	{
		return $this->c_ci_id;
	}

	public function get_text ()
	{
		return $this->c_text;
	}

	public function get_create_uid ()
	{
		return $this->c_create_uid;
	}

	public function get_last_uid ()
	{
		return $this->c_last_uid;
	}

	public function get_create_change ()
	{
		return $this->c_create_change;
	}

	public function get_last_change ()
	{
		return $this->c_last_change;
	}

	// validation
	
	public function is_valid ()
	{
		// plausi: ci_id
		$db		= core_db::getInstance();
		$query	= "SELECT COUNT(*) FROM ci " .
			"WHERE ci_id='$this->c_ci_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if (($count = $db->result( $result, 0 )) != 1)
		{
			throw new core_data_exception( "invalid CI: $this->c_ci_id");
		}

		// plausi: text filled
		if (strlen($this->c_text)==0)
		{
			throw new core_data_exception( "text must not be empty!");
		}

		return TRUE;
	}

	// db actions low level


	/**
	 * objekt in die DB einfuegen
	 */
	protected function db_insert_impl ()
	{
		// last UID setzen
		$this->c_last_uid = core_session::getInstance()->get_user_id();
		$this->c_create_uid = $this->c_last_uid;
		$this->c_last_change = date("Y-m-d H:i:s");
		$this->c_create_change = $this->c_last_change;

		parent::db_insert_impl();

		return TRUE;
	}
	
	/**
	 * objekt in der DB aktualisieren
	 * 
	 * da sich an den feldern nicht mehr aendern kann, wird
	 * ci_last_change explizit gesetzt
	 */
	protected function db_update_impl ()
	{
		// last UID setzen
		$this->c_last_uid = core_session::getInstance()->get_user_id();
		$this->c_last_change = date("Y-m-d H:i:s");
		
		parent::db_update_impl();

		return TRUE;
	}
	
}
