<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_csv_element.inc.php 342 2009-04-10 17:15:07Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * access class for database table csv_element
 * 
 * ce_id			int			primary key
 * ce_cp_id			int			foreign key to csv_profile
 * ce_ta_id			int			foreign key to type attribute
 * ce_pos			int			position
 * 
 */
 
class db_csv_element extends core_db_object
{
	protected $ce_id			= 0;
	protected $ce_cp_id			= 0;
	protected $ce_ta_id			= 0;
	protected $ce_pos			= 0;
	
	// set methoden
	
	public function set_id ( $_id )
	{
		$this->ce_id = $_id;
		return TRUE;
	}

	public function set_cp_id ( $_cp_id )
	{
		if ($_cp_id == $this->ce_cp_id) return TRUE;
		$this->ce_cp_id = $_cp_id;
		$this->mark_modified();
		return TRUE;
	}

	public function set_ta_id ( $_ta_id )
	{
		if ($_ta_id == $this->ce_ta_id) return TRUE;
		$this->ce_ta_id = $_ta_id;
		$this->mark_modified();
		return TRUE;
	}

	public function set_pos ( $_pos )
	{
		if ($_pos == $this->ce_pos) return TRUE;
		$this->ce_pos = $_pos;
		$this->mark_modified();
		return TRUE;
	}

	// get methoden
	
	protected function get_id_name ()
	{
		return "ce_id";
	}

	protected function get_table_name ()
	{
		return "csv_element";
	}

	public function get_id ()
	{
		return $this->ce_id;
	}

	public function get_cp_id ()
	{
		return $this->ce_cp_id;
	}

	public function get_ta_id ()
	{
		return $this->ce_ta_id;
	}

	public function get_pos ()
	{
		return $this->ce_pos;
	}

	// list methods

	public function set_list_pos ( $_pos )
	{
		$this->set_pos( $_pos );
		return;
	}

	public function get_list_pos ()
	{
		return $this->get_pos();
	}

	// validation
	
	public function is_valid ()
	{
		$db		= core_db::getInstance();

		// plausi: cp_id
		$query	= "SELECT COUNT(*) FROM csv_profile " .
			"WHERE cp_id='$this->ce_cp_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if (($count = $db->result( $result, 0 )) != 1)
		{
			throw new core_data_exception( "invalid CSV-Profile: $this->ce_cp_id");
		}

		// plausi: ce_ta_id
		if ($this->ce_ta_id > 0)
		{
			$query	= "SELECT COUNT(*) FROM type_attr " .
				"WHERE ta_id='$this->ce_ta_id'";
			$result = $db->query( $query );
			if ($db->num_rows($result)!=1)
			{
				throw new core_mysql_exception( "unexpected number of rows" );
			}
			if (($count = $db->result( $result, 0 )) != 1)
			{
				throw new core_data_exception( "invalid type attribute: $this->ce_ta_id");
			}
		}

		return TRUE;
	}

}
