<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_csv_profile.inc.php 353 2009-04-13 12:55:01Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * access class for database table csv_profile
 * 
 * cp_id			int			primary key
 * cp_classtype		int			classtype identifier
 * cp_cit_id		int			foreignt key to CI type
 * cp_title			varchar		title of profile
 * cp_desc			text		description of profile
 * cp_separator		char		field separator
 * cp_delimiter		char		string delimiter
 * 
 */
 
class db_csv_profile extends core_db_object
{
	protected $cp_id			= 0;
	protected $cp_classtype		= CLASS_CI;
	protected $cp_cit_id		= 0;
	protected $cp_title			= "";
	protected $cp_desc			= "";
	protected $cp_separator		= ";";
	protected $cp_delimiter		= '"';
	
	// set methoden
	
	public function set_id ( $_id )
	{
		$this->cp_id = $_id;
		return TRUE;
	}

	public function set_classtype ( $_classtype )
	{
		if ($_classtype == $this->cp_classtype) return TRUE;
		$this->cp_classtype = $_classtype;
		$this->mark_modified();
		return TRUE;
	}

	public function set_cit_id ( $_cit_id )
	{
		if ($_cit_id == $this->cp_cit_id) return TRUE;
		$this->cp_cit_id = $_cit_id;
		$this->mark_modified();
		return TRUE;
	}

	public function set_title ( $_title )
	{
		if ($_title == $this->cp_title) return TRUE;
		$this->cp_title = $_title;
		$this->mark_modified();
		return TRUE;
	}

	public function set_desc ( $_desc )
	{
		if ($_desc == $this->cp_desc) return TRUE;
		$this->cp_desc = $_desc;
		$this->mark_modified();
		return TRUE;
	}

	public function set_separator ( $_separator )
	{
		if ($_separator == $this->cp_separator) return TRUE;
		$this->cp_separator = $_separator;
		$this->mark_modified();
		return TRUE;
	}

	public function set_delimiter ( $_delimiter )
	{
		if ($_delimiter == $this->cp_delimiter) return TRUE;
		$this->cp_delimiter = $_delimiter;
		$this->mark_modified();
		return TRUE;
	}


	// get methoden
	
	protected function get_id_name ()
	{
		return "cp_id";
	}

	protected function get_table_name ()
	{
		return "csv_profile";
	}

	public function get_id ()
	{
		return $this->cp_id;
	}

	public function get_classtype ()
	{
		return $this->cp_classtype;
	}

	public function get_cit_id ()
	{
		return $this->cp_cit_id;
	}

	public function get_title ()
	{
		return $this->cp_title;
	}

	public function get_desc ()
	{
		return $this->cp_desc;
	}

	public function get_separator ()
	{
		return $this->cp_separator;
	}

	public function get_delimiter ()
	{
		return $this->cp_delimiter;
	}

	// validation
	
	public function is_valid ()
	{
		$db		= core_db::getInstance();

		// plausi: title empty
		if (strlen($this->cp_title)==0)
		{
			throw new core_data_exception( "title must not be empty" );
		}

		// plausi: title unique?
		$query	= "SELECT COUNT(*) FROM csv_profile " .
			"WHERE cp_id!='$this->cp_id' " .
			"AND cp_cit_id='$this->cp_cit_id' " .
			"AND cp_title='$this->cp_title'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}

		if ($db->result( $result, 0 ) != 0)
		{
			throw new core_data_exception( "title already used for another CSV-Profile" );
		}

		// plausi: cp_classtype valid
		if (! in_array( $this->cp_classtype, array(CLASS_CI, CLASS_REL)))
		{
			throw new core_data_exception( "invalid classtype" );
		}
		
		// plausi: valid CI type
		$query	= "SELECT COUNT(*) FROM ci_type " .
			"WHERE cit_id='$this->cp_cit_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if (($count = $db->result( $result, 0 )) != 1)
		{
			throw new core_data_exception( "invalid CI-Type: $this->cp_cit_id");
		}

		return TRUE;
	}

}
