<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_relation.inc.php 339 2009-04-10 12:39:09Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabelle relation
 * 
 * r_id			int			primary key
 * r_rt_id		int			foreign key to relation type
 * r_ci1_id		int			foreign key to CI 1
 * r_ci2_id		int			foreign key to CI 2
 * r_desc		text		description
 * 
 */
 
class db_relation extends core_db_object
{
	protected $r_id				= 0;
	protected $r_rt_id			= 0;
	protected $r_ci1_id			= 0;
	protected $r_ci2_id			= 0;
	protected $r_desc			= "";
	
	// set methoden
	
	public function set_id ( $_id )
	{
		$this->r_id = $_id;
		return TRUE;
	}

	public function set_rt_id ( $_rt_id )
	{
		$this->r_rt_id = $_rt_id;
		return TRUE;
	}

	public function set_ci1_id ( $_ci1_id )
	{
		if ($_ci1_id == $this->r_ci1_id) return TRUE;
		$this->r_ci1_id = $_ci1_id;
		$this->mark_modified();
		return TRUE;
	}

	public function set_ci2_id ( $_ci2_id )
	{
		if ($_ci2_id == $this->r_ci2_id) return TRUE;
		$this->r_ci2_id = $_ci2_id;
		$this->mark_modified();
		return TRUE;
	}

	public function set_desc ( $_desc )
	{
		if (strcmp( $_desc, $this->r_desc )==0) return TRUE;
		$this->r_desc = $_desc;
		$this->mark_modified();
		return TRUE;
	}

	// get methoden
	
	protected function get_id_name ()
	{
		return "r_id";
	}

	protected function get_table_name ()
	{
		return "relation";
	}

	public function get_id ()
	{
		return $this->r_id;
	}

	public function get_rt_id ()
	{
		return $this->r_rt_id;
	}

	public function get_ci1_id ()
	{
		return $this->r_ci1_id;
	}

	public function get_ci2_id ()
	{
		return $this->r_ci2_id;
	}

	public function get_desc ()
	{
		return $this->r_desc;
	}

	// validation
	
	public function is_valid ()
	{
		$db		= core_db::getInstance();

		// plausi: rt_id
		$query  = "SELECT COUNT(*) FROM relation_type " .
			"WHERE rt_id='$this->r_rt_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if ($db->result( $result, 0 ) == 0)
		{
			throw new core_data_exception( "Relation-Type not valid" );
		}

		// plausi: CI 1
		$query  = "SELECT COUNT(*) FROM ci " .
			"WHERE ci_id='$this->r_ci1_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if ($db->result( $result, 0 ) == 0)
		{
			throw new core_data_exception( "CI 1 not valid" );
		}

		// plausi: CI 2
		$query  = "SELECT COUNT(*) FROM ci " .
			"WHERE ci_id='$this->r_ci2_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if ($db->result( $result, 0 ) == 0)
		{
			throw new core_data_exception( "CI 2 not valid" );
		}

		// plausi: schon vorhanden?
		$query	= "SELECT COUNT(*) FROM relation " .
			"WHERE r_id!='$this->r_id' " .
			"AND r_rt_id='$this->r_rt_id' " .
			"AND r_ci1_id='$this->r_ci1_id' " .
			"AND r_ci2_id='$this->r_ci2_id' " ;
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}

		if ($db->result( $result, 0 ) != 0)
		{
			throw new core_data_exception( "relation already available" );
		}

		return TRUE;
	}

}
