<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_relation_peer.inc.php 339 2009-04-10 12:39:09Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabellen 'relation', 'relation_type' und 'ci'
 * 
 * ci_id			int			primary key
 * ci_cit_id		int			fremdschluesser auf den CI-Type
 * ci_title			string		synopse des CI
 * ci_icon			string		Name der icon-datei
 * ci_last_uid		int			letzte aenderung durch user id
 * ci_last_changed	timestamp	letzte aenderung
 * 
 * r_id				int			primary key
 * r_rt_id			int			foreign key to relation type
 * r_ci1_id			int			foreign key to CI 1
 * r_ci2_id			int			foreign key to CI 2
 * r_desc		text		description
 * 
 * cit_id			int			primary key
 * cit_title		string		Name/Bezeichner des CI-Types
 * cit_icon			string		Name der icon-datei
 * cit_desc			text		Beschreibung des CI-Types
 * cit_last_uid		int			letzte aenderung durch user id
 * cit_last_changed	timestamp	letzte aenderung
 * 
 * WICHTIG: diese klasse dient nur zum LESEN
 * 
 */
 
class db_relation_peer extends core_db_object
{
	protected $base_ci_id		= 0;
	protected $base_ci_pos		= 0;
	
	protected $r_id				= 0;
	protected $r_rt_id			= 0;
	protected $r_ci1_id			= 0;
	protected $r_ci2_id			= 0;
	protected $r_desc			= "";

	protected $rt_id			= 0;
	protected $rt_cit1_id		= 0;
	protected $rt_cit2_id		= 0;
	protected $rt_to1_text		= "";
	protected $rt_to2_text		= "";
	protected $rt_desc			= "";

	protected $ci_id			= 0;
	protected $ci_cit_id		= 0;
	protected $ci_title			= "";
	protected $ci_icon			= "";
	protected $ci_last_uid		= 0;
	protected $ci_last_change	= "";
	
	protected $cit_id			= 0;
	protected $cit_title		= "";
	protected $cit_icon			= "";
	protected $cit_desc			= "";
	protected $cit_last_uid		= 0;
	protected $cit_last_changed	= "";

	public function __construct ( $_r_id, $_ci_id )
	{
		$this->base_ci_id = $_ci_id;
		parent::__construct ( $_r_id );
		$this->db_load();
		return;
	}

	// set methoden
	
	public function set_id ( $_id )
	{
		$this->r_id = $_id;
		return TRUE;
	}

	// get methoden

	protected function get_id_name ()
	{
		return "r_id";
	}

	protected function get_table_name ()
	{
		return "relation";
	}

	public function get_ci_pos ()
	{
		if ($this->base_ci_pos == 0)
		{
			if ($this->r_ci1_id == $this->base_ci_id) $this->base_ci_pos = 1;
			if ($this->r_ci2_id == $this->base_ci_id) $this->base_ci_pos = 2;
		}
		return $this->base_ci_pos;
	}
	
	public function get_id ()
	{
		return $this->r_id;
	}

	public function get_rt_id ()
	{
		return $this->r_rt_id;
	}

	public function get_ci1_id ()
	{
		return $this->r_ci1_id;
	}

	public function get_ci2_id ()
	{
		return $this->r_ci2_id;
	}

	public function get_desc ()
	{
		return $this->r_desc;
	}

	public function get_cit1_id ()
	{
		return $this->rt_cit1_id;
	}

	public function get_cit2_id ()
	{
		return $this->rt_cit2_id;
	}

	public function get_to1_text ()
	{
		return $this->rt_to1_text;
	}

	public function get_to2_text ()
	{
		return $this->rt_to2_text;
	}

	public function get_rt_desc ()
	{
		return $this->rt_desc;
	}

	public function get_ci_id ()
	{
		return $this->ci_id;
	}

	public function get_cit_id ()
	{
		return $this->ci_cit_id;
	}

	public function get_ci_title ()
	{
		return $this->ci_title;
	}

	public function get_ci_icon ()
	{
		return $this->ci_icon;
	}

	public function get_last_uid ()
	{
		return $this->ci_last_uid;
	}

	public function get_last_change ()
	{
		return $this->ci_last_change;
	}

	public function get_cit_title ()
	{
		return $this->cit_title;
	}
	
	public function get_cit_icon ()
	{
		return $this->cit_icon;
	}
	
	public function get_cit_desc ()
	{
		return $this->cit_desc;
	}
	
	public function get_cit_last_uid ()
	{
		return $this->cit_last_uid;
	}

	public function get_cit_last_changed ()
	{
		return $this->cit_last_changed;
	}

	// advanced get
	
	public function get_peer_text ()
	{
		$text = "";
		if ($this->get_ci_pos()==1) $text = $this->rt_to2_text;
		if ($this->get_ci_pos()==2) $text = $this->rt_to1_text;
		return $text;
	}

	public function get_self_text ()
	{
		$text = "";
		if ($this->get_ci_pos()==1) $text = $this->rt_to1_text;
		if ($this->get_ci_pos()==2) $text = $this->rt_to2_text;
		return $text;
	}

	public function get_icon ()
	{
		$_icon = $this->get_ci_icon();
		if (strlen($_icon)==0) $_icon = $this->get_cit_icon();
		return $_icon;
	}

	// validation
	
	public function is_valid ()
	{
		// plausi: cit_id
		$db		= core_db::getInstance();
		$query	= "SELECT COUNT(*) FROM ci_type " .
			"WHERE cit_id='$this->ci_cit_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if (($count = $db->result( $result, 0 )) != 1)
		{
			throw new core_data_exception( "invalid CI-Type: $this->ci_cit_id");
		}
		
		return TRUE;
	}

	// db actions low level

	/**
	 * inhalt des objektes aus der datenbank holen
	 */
	protected function db_load_impl ()
	{
		parent::db_load_impl();

		$db		= core_db::getInstance();
		
//		// relation laden
//		$query	= "SELECT * FROM relation WHERE r_id='$this->r_id'";
//		$result = $db->query( $query );
//		if ($db->num_rows($result)!=1) return;
//
//		$data= $db->fetch( $result );
//		$this->load_from_array( $data );
		
		// relation type laden
		$query	= "SELECT * FROM relation_type WHERE rt_id='$this->r_rt_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1) return;
		$data= $db->fetch( $result );
		foreach ($data as $key => $value)
		{
			$this->$key = $value;
		}

		// pos ermitteln
		if ($this->base_ci_id == $this->r_ci1_id)
		{
			$this->base_ci_pos = 1;
			$query = "SELECT * FROM ci WHERE ci_id='$this->r_ci2_id'";
		}
		if ($this->base_ci_id == $this->r_ci2_id)
		{
			$this->base_ci_pos = 2;
			$query = "SELECT * FROM ci WHERE ci_id='$this->r_ci1_id'";
		}
		if ($this->base_ci_pos == 0)
		{
			throw new core_data_exception( "CI not found in relation!");
		}
		
		// peer ci laden
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1) return;

		$data= $db->fetch( $result );
		foreach ($data as $key => $value)
		{
			$this->$key = $value;
		}
		
		// peer cit laden
		$query = "SELECT * FROM ci_type WHERE cit_id='$this->ci_cit_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1) return;

		$data= $db->fetch( $result );
		foreach ($data as $key => $value)
		{
			$this->$key = $value;
		}
		
		return;		
	}

	/**
	 * objekt in die DB einfuegen
	 */
	protected function db_insert_impl ()
	{
		return FALSE;
	}
	
	/**
	 * objekt in der DB aktualisieren
	 */
	protected function db_update_impl ()
	{
		return FALSE;
	}
	
	/**
	 * objekt in der DB loeschen
	 */
	protected function db_delete_impl ()
	{
		return FALSE;
	}

}
