<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_relation_type.inc.php 339 2009-04-10 12:39:09Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabelle relation_type
 * 
 * rt_id		int			primary key
 * rt_cit1_id	int			foreign key to CI-Type 1
 * rt_cit2_id	int			foreign key to CI-Type 2
 * rt_to1_text	text		auf 1. CI gerichteter Text
 * rt_to2_text	text		auf 2. CI gerichteter Text
 * rt_desc		text		Beschreibung des Relation-Types
 * 
 */
 
class db_relation_type extends core_db_object
{
	protected $rt_id			= 0;
	protected $rt_cit1_id		= 0;
	protected $rt_cit2_id		= 0;
	protected $rt_to1_text		= "";
	protected $rt_to2_text		= "";
	protected $rt_desc			= "";
	
	// set methoden
	
	public function set_id ( $_id )
	{
		$this->rt_id = $_id;
		return TRUE;
	}

	public function set_cit1_id ( $_cit1_id )
	{
		if ($_cit1_id == $this->rt_cit1_id) return TRUE;
		$this->rt_cit1_id = $_cit1_id;
		$this->mark_modified();
		return TRUE;
	}

	public function set_cit2_id ( $_cit2_id )
	{
		if ($_cit2_id == $this->rt_cit2_id) return TRUE;
		$this->rt_cit2_id = $_cit2_id;
		$this->mark_modified();
		return TRUE;
	}

	public function set_to1_text ( $_to1_text )
	{
		if (strcmp($_to1_text, $this->rt_to1_text)==0) return TRUE;
		$this->rt_to1_text = $_to1_text;
		$this->mark_modified();
		return TRUE;
	}

	public function set_to2_text ( $_to2_text )
	{
		if (strcmp($_to2_text, $this->rt_to2_text)==0) return TRUE;
		$this->rt_to2_text = $_to2_text;
		$this->mark_modified();
		return TRUE;
	}

	public function set_desc ( $_desc )
	{
		if (strcmp($_desc, $this->rt_desc)==0) return TRUE;
		$this->rt_desc = $_desc;
		$this->mark_modified();
		return TRUE;
	}

	// get methoden
	
	protected function get_id_name ()
	{
		return "rt_id";
	}

	protected function get_table_name ()
	{
		return "relation_type";
	}

	public function get_title ()
	{
		return $this->rt_to2_text . " / " . $this->rt_to1_text;
	}

	public function get_id ()
	{
		return $this->rt_id;
	}

	public function get_cit1_id ()
	{
		return $this->rt_cit1_id;
	}

	public function get_cit2_id ()
	{
		return $this->rt_cit2_id;
	}

	public function get_to1_text ()
	{
		return $this->rt_to1_text;
	}

	public function get_to2_text ()
	{
		return $this->rt_to2_text;
	}

	public function get_desc ()
	{
		return $this->rt_desc;
	}

	// validation
	
	public function is_valid ()
	{
		$db		= core_db::getInstance();

		// plausi: to1_text leer
		if (strlen($this->rt_to1_text)==0)
		{
			throw new core_data_exception( "to1_text must not be empty" );
		}

		// plausi: to2_text leer
		if (strlen($this->rt_to2_text)==0)
		{
			throw new core_data_exception( "to2_text must not be empty" );
		}

		// plausi: cit1 id ok
		$query  = "SELECT COUNT(*) FROM ci_type " .
			"WHERE cit_id='$this->rt_cit1_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if ($db->result( $result, 0 ) == 0)
		{
			throw new core_data_exception( "CI-Type 1 is not valid" );
		}
		
		// plausi: cit2 id ok
		$query  = "SELECT COUNT(*) FROM ci_type " .
			"WHERE cit_id='$this->rt_cit2_id'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}
		if ($db->result( $result, 0 ) == 0)
		{
			throw new core_data_exception( "CI-Type 2 is not valid" );
		}
		
		// plausi: schon vorhanden?
		$query	= "SELECT COUNT(*) FROM relation_type " .
			"WHERE rt_id!='$this->rt_id' " .
			"AND rt_cit1_id='$this->rt_cit1_id' " .
			"AND rt_cit2_id='$this->rt_cit2_id' " .
			"AND rt_to1_text='" . $db->escape_string($this->rt_to1_text) . "' " .
			"AND rt_to2_text='" . $db->escape_string($this->rt_to2_text) . "' ";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}

		if ($db->result( $result, 0 ) != 0)
		{
			throw new core_data_exception( "Relation-Type with equal combination of to1_text/to2_text already available" );
		}

		return TRUE;
	}

}
