<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_type_attr.inc.php 339 2009-04-10 12:39:09Z micha $
*/

require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabelle type_attr
 * 
 * ta_id			int			primary key
 * ta_classtype		int			class type (siehe Constanten CLASS_xxx)
 * ta_classtype_id	int			foreign key auf ci_type / rel_type / usw.
 * ta_title			string		Name/Bezeichner des CI-Type-Attributes
 * ta_desc			text		Beschreibung des CI-Type-Attributes
 * ta_datatype		int			daten typ
 * ta_datatype_sub	int			daten subtype
 * ta_order			int			ordnungs-nummer fuer reihenfolge
 * ta_visible		int			sichtbarkeits-nummer
 * ta_label			int			label fuer module
 * ta_oblig			bool		pflichtfeld: ja/nein
 * ta_unique		bool		feld ist unique innerhalb aller CIs des CI-Types: ja/nein
 * ta_tag			bool		wert des attributs wird in synopse des CI genutzt (ja/nein)
 * ta_input			int			art des input (siehe Constanten ATTR_INPUT_xxx) 
 * ta_format		string		formatierungs-vorlage fuer das attribut
 */
class db_type_attr extends core_db_object
{
	protected $ta_id			= 0;
	protected $ta_classtype		= 0;
	protected $ta_classtype_id	= 0;
	protected $ta_title			= "";
	protected $ta_desc			= "";
	protected $ta_datatype		= DT_TEXT;
	protected $ta_datatype_sub	= 0;
	protected $ta_order			= 0;
	protected $ta_visible		= 1;
	protected $ta_label			= 0;
	protected $ta_oblig			= TRUE;
	protected $ta_unique		= FALSE;
	protected $ta_tag			= FALSE;
	protected $ta_input			= ATTR_INPUT_FORM;
	protected $ta_format		= "";

	// set methoden

	public function set_id ( $_id )
	{
		$this->ta_id = $_id;
		return TRUE;
	}

	public function set_classtype ( $_classtype )
	{
		$this->ta_classtype = $_classtype;
		return TRUE;
	}

	public function set_classtype_id ( $_id )
	{
		if ($this->ta_classtype_id == $_id) return TRUE;
		$this->ta_classtype_id = $_id;
		$this->mark_modified();
		return TRUE;
	}

	public function set_title ( $_title )
	{
		if ($this->ta_title == $_title) return TRUE;
		$this->ta_title = $_title;
		$this->mark_modified();
		return TRUE;
	}

	public function set_desc ( $_desc )
	{
		if ($this->ta_desc == $_desc) return TRUE;
		$this->ta_desc = $_desc;
		$this->mark_modified();
		return TRUE;
	}

	public function set_datatype ( $_datatype )
	{
		if ($this->ta_datatype == $_datatype) return TRUE;
		$this->ta_datatype = $_datatype;
		$this->mark_modified();
		return TRUE;
	}

	public function set_datatype_sub ( $_datatype_sub )
	{
		if ($this->ta_datatype_sub == $_datatype_sub) return TRUE;
		$this->ta_datatype_sub = $_datatype_sub;
		$this->mark_modified();
		return TRUE;
	}

	public function set_order ( $_order )
	{
		if ($this->ta_order == $_order) return TRUE;
		$this->ta_order = $_order;
		$this->mark_modified();
		return TRUE;
	}

	public function set_visible ( $_visible )
	{
		if ($this->ta_visible == $_visible) return TRUE;
		$this->ta_visible = $_visible;
		$this->mark_modified();
		return TRUE;
	}

	public function set_label ( $_label )
	{
		if ($this->ta_label == $_label) return TRUE;
		$this->ta_label = $_label;
		$this->mark_modified();
		return TRUE;
	}

	public function set_oblig ( $_oblig )
	{
		if ($this->ta_oblig == $_oblig) return TRUE;
		$this->ta_oblig = $_oblig;
		$this->mark_modified();
		return TRUE;
	}

	public function set_unique ( $_unique )
	{
		if ($this->ta_unique == $_unique) return TRUE;
		$this->ta_unique = $_unique;
		$this->mark_modified();
		return TRUE;
	}

	public function set_tag ( $_tag )
	{
		if ($this->ta_tag == $_tag) return TRUE;
		$this->ta_tag = $_tag;
		$this->mark_modified();
		return TRUE;
	}

	public function set_input ( $_input )
	{
		if ($this->ta_input == $_input) return TRUE;
		$this->ta_input = $_input;
		$this->mark_modified();
		return TRUE;
	}

	public function set_format ( $_format )
	{
		if ($this->ta_format == $_format) return TRUE;
		$this->ta_format = $_format;
		$this->mark_modified();
		return TRUE;
	}

	// get methoden

	protected function get_id_name ()
	{
		return "ta_id";
	}

	protected function get_table_name ()
	{
		return "type_attr";
	}

	public function get_id ()
	{
		return $this->ta_id;
	}

	public function get_classtype ()
	{
		return $this->ta_classtype;
	}

	public function get_classtype_id ()
	{
		return $this->ta_classtype_id;
	}

	public function get_title ()
	{
		return $this->ta_title;
	}

	public function get_desc ()
	{
		return $this->ta_desc;
	}

	public function get_datatype ()
	{
		return $this->ta_datatype;
	}

	public function get_datatype_sub ()
	{
		return $this->ta_datatype_sub;
	}

	public function get_order ()
	{
		return $this->ta_order;
	}

	public function get_visible ()
	{
		return $this->ta_visible;
	}

	public function get_label ()
	{
		return $this->ta_label;
	}

	public function get_oblig ()
	{
		return $this->ta_oblig;
	}

	public function get_unique ()
	{
		return $this->ta_unique;
	}

	public function get_tag ()
	{
		return $this->ta_tag;
	}

	public function get_input ()
	{
		return $this->ta_input;
	}

	public function get_format ()
	{
		return $this->ta_format;
	}

	// validation
	
	public function is_valid ()
	{
		$db		= core_db::getInstance();

		// plausi: classtype gueltig, classtype_id gueltig
		$query = "";
		switch ($this->ta_classtype)
		{
			case CLASS_CI:
			{
				$query = "SELECT COUNT(*) FROM ci_type " .
					"WHERE cit_id='$this->ta_classtype_id'";
				break;
			}
			case CLASS_REL:
			{
				$query = "SELECT COUNT(*) FROM relation_type " .
					"WHERE rt_id='$this->ta_classtype_id'";
				break;
			}
		}
		if (strlen($query)==0)
		{
			throw new core_data_exception( "invalid classtype");
		}
				
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_db_exception( "unexpected number of rows" );
		}
		if ($db->result( $result, 0 ) != 1)
		{
			throw new core_data_exception( "invalid classtype_id");
		}

		// plausi: title nicht leer
		if (strlen($this->ta_title)==0)
		{
			throw new core_data_exception( "title must not be empty" );
		}

		// plausi: title nicht bereits vorhanden
		$query	= "SELECT COUNT(*) FROM type_attr " .
			"WHERE ta_id!='$this->ta_id' " .
			"AND ta_classtype='$this->ta_classtype' " .
			"AND ta_classtype_id='$this->ta_classtype_id' " .
			"AND ta_title='$this->ta_title'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_db_exception( "unexpected number of rows" );
		}
		if ($db->result( $result, 0 ) != 0)
		{
			throw new core_data_exception( "title already used for another type-attr of the same type-instance" );
		}

		// plausi: type gueltig
		if ( ($this->ta_datatype < DT_MIN) || (DT_MAX < $this->ta_datatype) )
		{
			throw new core_data_exception( "invalid datatype");
		}

		return TRUE;
	}

	// db actions low level

	/**
	 * objekt in die DB einfuegen
	 */
	protected function db_insert_impl ()
	{
		// DB
		$db		= core_db::getInstance();

		if ($this->ta_order == 0)
		{
			// implizit bestimmen
			$query  = "SELECT MAX(ta_order) FROM type_attr " .
				"WHERE ta_classtype='$this->ta_classtype' " .
				"AND ta_classtype_id='$this->ta_classtype_id' ";
			$result = $db->query( $query );
			$this->ta_order = $db->result( $result, 0 ) + 1;
		}

		parent::db_insert_impl();

		return TRUE;
	}

}
