<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	tables
* @version		$Id: db_user.inc.php 339 2009-04-10 12:39:09Z micha $
*/

require_once 'core/core_exception.inc.php';
require_once 'core/core_db.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_object.inc.php';

/**
 * zugriffs-klasse fuer DB-Tabelle ci_type
 * 
 * u_id				int			primary key
 * u_login			string		login name of user
 * u_last_name		string		last name of user
 * u_first_name		string		first name of user
 * u_email			string		mail address of user
 * u_passwd			string		password hash
 * u_role_id		int			foreign key to role
 * u_auth_backend	int			Identifier for Authentification Backend
 * u_active			bool		user is active 
 * u_notify			bool		email notification enabled
 * 
 */
class db_user extends core_db_object
{
	protected $u_id				= 0;
	protected $u_login			= "";
	protected $u_last_name		= "";
	protected $u_first_name		= "";
	protected $u_email			= "";
	protected $u_passwd			= "";
	protected $u_role_id		= 0;
	protected $u_auth_backend	= AUTH_BACKEND_LOCAL;
	protected $u_active			= FALSE;
	protected $u_notify			= FALSE;

	// set methoden

	public function set_id ( $_id )
	{
		$this->u_id = $_id;
		return TRUE;
	}

	public function set_login ( $_login )
	{
		if (strcmp($_login, $this->u_login)==0) return TRUE;
		$this->u_login = $_login;
		$this->mark_modified();
		return TRUE;
	}
		
	public function set_last_name ( $_last_name )
	{
		if (strcmp($_last_name, $this->u_last_name)==0) return TRUE;
		$this->u_last_name = $_last_name;
		$this->mark_modified();
		return TRUE;
	}
		
	public function set_first_name ( $_first_name )
	{
		if (strcmp($_first_name, $this->u_first_name)==0) return TRUE;
		$this->u_first_name = $_first_name;
		$this->mark_modified();
		return TRUE;
	}
		
	public function set_email ( $_email )
	{
		if (strcmp($_email, $this->u_email)==0) return TRUE;
		$this->u_email = $_email;
		$this->mark_modified();
		return TRUE;
	}
		
	public function set_passwd ( $_passwd )
	{
		$_pw_hash = md5( $_passwd );
		if (strcmp($_pw_hash, $this->u_passwd)==0) return TRUE;
		$this->u_passwd = $_pw_hash;
		$this->mark_modified();
		return TRUE;
	}
		
	public function set_role_id ( $_role_id )
	{
		if ($_role_id == $this->u_role_id) return TRUE;
		$this->u_role_id = $_role_id;
		$this->mark_modified();
		return TRUE;
	}
		
	public function set_auth_backend ( $_backend )
	{
		if ($_backend == $this->u_auth_backend) return TRUE;
		$this->u_auth_backend = $_backend;
		$this->mark_modified();
		return TRUE;
	}
		
	public function set_active ( $_active )
	{
		if ($_active == $this->u_active) return TRUE;
		$this->u_active = $_active;
		$this->mark_modified();
		return TRUE;
	}
		
	public function set_notify ( $_notify )
	{
		if ($_notify == $this->u_notify) return TRUE;
		$this->u_notify = $_notify;
		$this->mark_modified();
		return TRUE;
	}

	// get methoden

	protected function get_id_name ()
	{
		return "u_id";
	}

	protected function get_table_name ()
	{
		return "user";
	}

	public function get_id ()
	{
		return $this->u_id;
	}

	public function get_login ()
	{
		return $this->u_login;
	}

	public function get_last_name ()
	{
		return $this->u_last_name;
	}

	public function get_first_name ()
	{
		return $this->u_first_name;
	}

	public function get_email ()
	{
		return $this->u_email;
	}

	public function get_role_id ()
	{
		return $this->u_role_id;
	}

	public function get_auth_backend ()
	{
		return $this->u_auth_backend;
	}

	public function get_active ()
	{
		return $this->u_active;
	}

	public function get_notify ()
	{
		return $this->u_notify;
	}

	// advanced get
	
	public function password_match ( $_passwd )
	{
		if (strlen($this->u_passwd)==0) return TRUE;
		$_pw_hash = md5( $_passwd );
		return (strcmp($_pw_hash, $this->u_passwd)==0);
	}

	public function is_active ()
	{
		return ($this->u_active);
	}

	public function is_notify ()
	{
		return ($this->u_notify);
	}

	public function get_name ()
	{
		$name = sprintf( "%s %s", $this->get_first_name(), $this->get_last_name() );
		if (strlen($name)==1) $name = $this->get_login();
		return $name;
	}

	// validation
	
	public function is_valid ()
	{
		// plausi: title leer
		if (strlen($this->u_login)==0)
		{
			throw new core_data_exception( "login must not be empty" );
		}

		// plausi: login schon vorhanden?
		$db		= core_db::getInstance();
		$query	= "SELECT COUNT(*) FROM user " .
			"WHERE u_id!='$this->u_id' " .
			"AND u_login='$this->u_login'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_mysql_exception( "unexpected number of rows" );
		}

		if ($db->result( $result, 0 ) != 0)
		{
			throw new core_data_exception( "login already used for another user" );
		}

		// plausi: notify on with valid email
		if ($this->u_notify and (strlen($this->u_email)==0))
		{
			throw new core_data_exception( "email must be given, if notify is on" );
		}

		// plausi: role_id
		if (! in_array( $this->u_role_id, array(ROLE_VIEWER,ROLE_OPERATOR,ROLE_CHANGE_MANAGER,ROLE_DESIGNER,ROLE_USER_ADMIN,ROLE_ADMIN)))
		{
			throw new core_data_exception( "Role is not supported" );
		}
		
		// plausi: auth_backend
		if (! in_array( $this->u_auth_backend, array(AUTH_BACKEND_LOCAL,AUTH_BACKEND_LDAP)))
		{
			throw new core_data_exception( "Auth-Backend is not supported" );
		}
		
		return TRUE;
	}


}
