<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.


// include configurations
require_once 'includes/constants.inc.php';
require_once 'includes/config_default.inc.php';

if (
	is_file( CONFIG_INC_PHP )
	||
	is_link( CONFIG_INC_PHP )
)
{
	@include_once CONFIG_INC_PHP;
}
else
{
	die( 'configuration file not found!' );
}

// include all core libs
require_once 'core/autoload.inc.php';
require_once 'core/core_api.inc.php';
require_once 'core/core_db.inc.php';
require_once 'core/core_html.inc.php';
require_once 'core/core_session.inc.php';
require_once 'core/core_arm.inc.php';

// include modules
require_once 'includes/module_base.inc.php';
require_once 'modules/modules.inc.php';

// load default language
$lang_file_name = lang_catalog::getFilename( get_property_from_config('default_language') );
require_once $lang_file_name;

// error handling
set_exception_handler('core_exception_handler');

// inits
$rc=0;

// offline check
//
if ( file_exists( CMDB_OFFLINE_PHP ) )
{
	die( get_string('ServiceOffline_dsc') );
}


# check, that trigger is enabled
#
if (! get_property( 'trigger', false ))
{
	die( get_string('TriggerNotEnabled_dsc') );
}

# check remote ip
#
$remote_ip = $_SERVER['REMOTE_ADDR'];
if (strcmp( get_property('trigger_remote'), $remote_ip)!=0)
{
	die( get_string('RequestingNotAllowedForTrigger_dsc') . "( $remote_ip )" );
}

# run trigger for update value func
#
$db = core_db::getInstance();
$query = "SELECT DISTINCT(ci_id) FROM ci, attr, type_attr " .
	"WHERE a_classtype=" . CLASS_CI . " " .
	"AND a_classtype_id=ci_id " .
	"AND ta_id=a_ta_id " .
	"AND ta_input=1";
$result = $db->query( $query );
$num = $db->num_rows( $result );
for ($i=0; $i<$num; $i++)
{
	$ci_id = $db->result( $result, $i );
	$ci = new ci( $ci_id );
	$ci->load_from_func();

//	printf( "CI %s aktualisiert\n", $ci->getTitle() );
}
